#pragma rtGlobals=1		// Use modern global access method.
#Include <Remove Points> // This allows use of the remove NaNs function from wavemetrics

//Voir si borne1[counter] et borne2[counter] < peak_wave_ch1[counter]
Function MAS(peak_wave_ch1, Tags_ch1, True_wave_ch2, Amp_wave_ch1, peak_win_width, Bsl_width, AP_rise_time, Lag_to_peak, [lower_lim, upper_lim, compteur]) //peak_wave should be RealPeakPosition of ch1
	Wave peak_wave_ch1, Tags_ch1, True_wave_ch2, Amp_wave_ch1
	Variable peak_win_width, Bsl_width, AP_rise_time, Lag_to_peak // lag_to_peak is mean time interval between peak on ch1 and peak on ch2
	Variable lower_lim, upper_lim, compteur // parameters that should become optionnal
	
	
	//Set range indexing variables for future use
	Variable/G lower_limit_P, upper_limit_P// limits in point indexing ofanalysis on Action Potentials wave
	
	Variable random_window = 100// To be set in ms (for reading convenience)
	random_window *= 0.001//Converted in secs
	
	// finds the point in peak_wave_ch1 closest to lower_limit_P (point of the wave where analysis beggins)
	Lower_limit_P = X_finder(peak_wave_ch1, lower_lim)
	// Same thing for upper limit
	Upper_limit_P = X_finder(peak_wave_ch1, upper_lim)
	
	//Output waves
	Duplicate/O/R=[Lower_limit_P, upper_limit_P] peak_wave_ch1, TEMPO_Deltas
	TEMPO_Deltas = NaN //defaults value is NaN (used to remove points later)
	Make/O/N=(3 * numpnts(TEMPO_Deltas)) TEMPO_Random_noise
	TEMPO_Random_noise = NaN
	
	Variable counter, num_type1, num_type2, num_type3, num_type4 //used to remove points
	For (counter = Lower_limit_P; counter < (upper_limit_P + 1); counter += 1)

		If ((Tags_ch1[counter] == 1) && (numtype(Amp_wave_ch1[counter]) == 0)) // If event on chan 1 is selected and is not a NaN (you can also put a threshold here)
			
			//Data value
			TEMPO_Deltas[(counter - Lower_limit_P)] = Ratio_calculator(peak_wave_ch1, True_wave_ch2, counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, low_lim = lower_lim, up_lim = upper_lim)
			
			//noise value counter
			TEMPO_Random_noise[(counter - Lower_limit_P)] = Ratio_calculator(peak_wave_ch1, True_wave_ch2, counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, randomize = 1, low_lim = lower_lim, up_lim = upper_lim)
			//noise value counter'
			TEMPO_Random_noise[((counter - Lower_limit_P) + numpnts(TEMPO_Deltas))] = Ratio_calculator(peak_wave_ch1, True_wave_ch2, counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, randomize = 1, low_lim = lower_lim, up_lim = upper_lim)
			//noise value counter''
			TEMPO_Random_noise[((counter - Lower_limit_P) + (2 * numpnts(TEMPO_Deltas)))] = Ratio_calculator(peak_wave_ch1, True_wave_ch2, counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, randomize = 1, low_lim = lower_lim, up_lim = upper_lim)
			
			//Check the type of number stored at that step in each wave
			num_type1 = numtype(TEMPO_Deltas[(counter - Lower_limit_P)])
			num_type2 = numtype(TEMPO_Random_noise[(counter - Lower_limit_P)])
			num_type3 = numtype(TEMPO_Random_noise[((counter - Lower_limit_P) + numpnts(TEMPO_Deltas))])
			num_type4 = numtype(TEMPO_Random_noise[((counter - Lower_limit_P) + (2 * numpnts(TEMPO_Deltas)))])
			
			// If one of the numbers was not a normal number (NaN or Inf),  all other numbers at that step are set to NaN to be removed
			If ((num_type1 + num_type2 + num_type3 + num_type4) != 0)
				TEMPO_Deltas[(counter - Lower_limit_P)] = NaN
				TEMPO_Random_noise[(counter - Lower_limit_P)] = NaN
				TEMPO_Random_noise[((counter - Lower_limit_P) + numpnts(TEMPO_Deltas))] = NaN
				TEMPO_Random_noise[((counter - Lower_limit_P) + (2 * numpnts(TEMPO_Deltas)))] = NaN
			EndIf	
			
		EndIf
	
	EndFor
	
	RemoveNaNs(TEMPO_Deltas) ; RemoveNaNs(TEMPO_Random_noise)//Remove empty lines (when Tags_ch1 == 0), or one value was out of range

End




Function Ratio_calculator(peak_wave_ch1, True_wave_ch2, counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, [randomize, low_lim, up_lim])
	Wave peak_wave_ch1, True_wave_ch2
	Variable counter, Lag_to_peak, AP_rise_time, Bsl_width, peak_win_width, random_window, randomize, low_lim, up_lim
	//randomize should be mentionned if TEMPO_peak_position is expected to be random (optionnal)
	Variable the_ratio, TEMPO_peak_position, TEMPO_Bsl_mean, TEMPO_Peak_mean, rndm //TEMPO_peak_position is location of peak nb N on chan2
	
		//__________________________________check if we use the random peak_poistion___________________
		If (ParamIsDefault(randomize))
			TEMPO_peak_position = (peak_wave_ch1[counter] + Lag_to_peak)
			
		Else
			rndm = Abs(enoise(random_window)) // random_window in second will return rndm in seconds
			TEMPO_peak_position = ((peak_wave_ch1[counter] - AP_rise_time) - rndm) //True peak position minus the random nb (peak_win_width is used to prevent averaging the AP artifact)
		
		EndIf
		//___________________________________Check If values are in range_______________
		
		// ___________________________if not the ratio will be NaN_______________________
		// If we are at the end of the sweep (the peak location on ch2 would be out => value would be zero)
		If (((TEMPO_peak_position + (peak_win_width / 2)) > up_lim))
			The_ratio = NaN
			Nvar nb_values_inf_to_range
			nb_values_inf_to_range += 1
		// If the begginning of the baseline is before the begginning of the sweep
		ElseIf ((peak_wave_ch1[counter] - (Bsl_width + AP_rise_time)) < low_lim) // Begining of bsl range
			The_ratio = NaN
			Nvar nb_values_sup_to_range
			nb_values_sup_to_range += 1
		// If the value is in range
		Else // _____________________If we are the ratio will be computed________________
			WaveStats/Q/M=1/R=((peak_wave_ch1[counter] - (Bsl_width + AP_rise_time)), (peak_wave_ch1[counter] - AP_rise_time)) True_wave_ch2
			TEMPO_Bsl_mean = V_avg
			WaveStats/Q/M=1/R=((TEMPO_peak_position - (peak_win_width / 2)), TEMPO_peak_position + (peak_win_width / 2)) True_wave_ch2
			TEMPO_Peak_mean = V_avg
		
			The_ratio =  (TEMPO_Peak_mean - TEMPO_Bsl_mean)
			If (the_ratio == 0) // If the ratio is absolutely zero remove it (as it should not exist statistically, it means we average a null portion of wave)
				the_ratio =NaN
				Nvar nb_values_inf_to_range
				nb_values_inf_to_range += 1
			EndIf
		EndIf
					

	Return the_ratio
		
End

Function MAS_prompt(ChannelNb)
	Variable ChannelNb
	// ___________________________________________________Creting numeric and string variables____________________________________
	String peak_wave_ch1_str, True_Tags_ch1_str, True_tags_ch2_str, limits_waves_baseN_str, peak_wave_ch2_str, Ampl_wave_ch2_str, Ampl_wave_ch1_str
	//Variable peak_win_width, Bsl_width, AP_rise_time, Lag_to_peak, true_hist_bin_size, noise_hist_bin_size
	limits_waves_baseN_str = "borne"
	Variable TEMPO_limit_1, TEMPO_limit_2, Spacan_ampl // Range limits
	Variable spac_sum, spac_nb, lower_limit_P2, upper_limit_P2
	// To check the number of values out of range
	Variable/G nb_values_inf_to_range, nb_values_sup_to_range
	nb_values_inf_to_range = 0
	nb_values_sup_to_range = 0
	Variable max_noise, nb_bins_datas, nb_bins_noise
	Variable/G alpha
	Variable TEMPO_ampl, nb_succes, TEMPO_freq_spikes, TEMPO_ampl_fail
	Variable Lower_limit_P, Upper_limit_P
	Variable counterSweeps, nb_interSweeps_secs = 5
	Variable nb_interSweeps, nb_NaNs = 0
	
	
	//________________________________________________Globals________________________________________________________
	
			
	Peak_wave_ch1_str = "Occurrence" + num2str(ChannelNb)
	True_Tags_ch1_str = "Tags" + num2str(ChannelNb)
	NVar Peak_win_widthG = $("MASPeakWinWidth" + num2str(ChannelNb))
	NVar Bsl_widthG = $("MASBslWidth" + num2str(ChannelNb))
	NVar AP_Rise_TimeG = $("MASAPRiseTime" + num2str(ChannelNb))
	NVar Lag_To_PeakG = $("MASLagToPeak" + num2str(ChannelNb))
	NVar True_hist_bin_sizeG = $("MASTrueHistBinSize" + num2str(ChannelNb))
	NVar Noise_hist_bin_sizeG = $("MASNoiseHistBinSize" + num2str(ChannelNb))
	NVar hist_minG = $("MASHistMin" + num2str(ChannelNb))
	NVar hist_maxG = $("MASHistMax" + num2str(ChannelNb))
	// __________________________converting Variables units___________________________
	// Use of locals for conversion to avoid affecting the globals for next run
	Variable Peak_win_width, Bsl_width, Ap_rise_time, Lag_to_peak, true_hist_bin_size, noise_hist_bin_size
	Variable hist_min, hist_max
	//convert values from ms to seconds

	peak_win_width = peak_win_widthG * 0.001
	Bsl_width = Bsl_widthG * 0.001
	AP_rise_time = AP_rise_timeG * 0.001
	Lag_to_peak = Lag_to_peakG * 0.001
	
	//Convert values from pA to Ampers
	true_hist_bin_size = true_hist_bin_sizeG * 1e-12
	noise_hist_bin_size = noise_hist_bin_sizeG * 1e-12
	hist_min = hist_minG * 1e-12
	hist_max = hist_maxG * 1e-12
	
	//________________________set locals values
	nb_bins_datas = ((hist_max - hist_min) / true_hist_bin_size)
	nb_bins_noise = ((hist_max - hist_min) / noise_hist_bin_size)
	
	//_________________________________if limit waves are not found prompt for them______________________
	If ((waveExists($(limits_waves_baseN_str + "1")) != 1) || (waveExists($(limits_waves_baseN_str + "2")) != 1))
		DoAlert 1, "Create borne waves first"
		If (V_flag == 1)
			Make/O/N=0 $(limits_waves_baseN_str + "1"), $(limits_waves_baseN_str + "2")
			DoWindow/K LimitWaves
			Edit/k=1/N=LimitWaves $(limits_waves_baseN_str + "1"), $(limits_waves_baseN_str + "2")
			DoWindow/C LimitWaves
			PauseForUser LimitWaves
		Else
			Abort
		EndIf
	EndIf
	
	// _________________________________Same thing for the labels___________________________________
	If (WaveExists(borne_labels)) 
		Wave/T borne_labels
	Else
		DoAlert 1, "Edit borne_labels first ?"
		If (V_flag == 1)
			Make/O/T/N=(numpnts($(limits_waves_baseN_str + "1"))) borne_labels
			DoWindow/K LimitLabels
			Edit/k=1/N=LimitLabels borne_labels
			DoWindow/C LimitLabels
			PauseForUser LimitLabels
			Wave/T borne_labels
		Else
			Make/O/T/N=(numpnts(borne1)) borne_labels
			borne_labels = num2str(p)
		EndIf
	EndIf
	
	// ____________________________________Waves_________________________
	
	wave Les_bornes
	//convert prompted strings to SpAcAn type filenames (load channel 2 if ref is 1 and 1 if ref is 2)
	peak_wave_ch2_str = peak_wave_ch1_str[0, (strlen(peak_wave_ch1_str) - 2)] // Duplicates the base name
	True_tags_ch2_str = True_tags_ch1_str[0, strlen(True_tags_ch1_str) - 2]
	Ampl_wave_ch2_str = "RealAmp"
	Ampl_wave_ch1_str = "RealAmp"
	If (cmpstr(peak_wave_ch1_str[(strlen(peak_wave_ch1_str) - 1)], "1") == 0) //If last caracter of the AP wave is 1
		Svar Filename2
		Print "Target wave is : " + filename2
		Duplicate/O $Filename2, events_channel
		peak_wave_ch2_str += "2"
		true_tags_ch2_str += "2"
		Ampl_wave_ch2_str += "2"
		Ampl_wave_ch1_str += "1"
	ElseIf (cmpstr(peak_wave_ch1_str[(strlen(peak_wave_ch1_str) - 1)], "2") == 0) //If last caracter of the AP wave is 2
		Svar Filename1
		Print "Target wave is : " + filename1
		Duplicate/O $Filename1, events_channel
		peak_wave_ch2_str += "1"
		true_tags_ch2_str += "1"
		Ampl_wave_ch2_str += "1"
		Ampl_wave_ch1_str += "2"
	EndIf
	
	//Create local waves for the limits
	duplicate/O $(limits_waves_baseN_str + "1"), limit_wave_1
	duplicate/O $(limits_waves_baseN_str + "2"), limit_wave_2
	
	//Wave references
	Duplicate/O $peak_wave_ch1_str, peak_wave_ch1
	Duplicate/O $peak_wave_ch2_str, peak_wave_ch2
	Duplicate/O $True_tags_ch1_str, True_tags_ch1
	Duplicate/O $True_tags_ch2_str, True_tags_ch2
	Duplicate/O $Ampl_wave_ch1_str, Amp_wave_ch1
	Duplicate/O $Ampl_wave_ch2_str, Amp_wave_ch2
	
	//Creates the output waves
	Make/O/N=0 Deltas, Random_noise
	Make/O/N=(nb_bins_datas) Hist_Deltas
	Make/O/N=(nb_bins_noise) Hist_noise
	
	//____________________________Creating output matrix and waves____________________
	Make/O/N=((numpnts(limit_wave_1)), 10) MASMatriX = NaN //Creates the output stats wave with 9 columns
	// The default value is NaN to see where no calculus was performed
	
	// Setting of columns labels
	SetDimLabel 1, 0, borne1, MASMatriX
	SetDimLabel 1, 1, borne2, MASMatriX
	SetDimLabel 1, 2, failure_rate, MASMatriX
	SetDimLabel 1, 3, nb_events, MASMatriX
	SetDimLabel 1, 4, ampl, MASMatriX
	SetDimLabel 1, 5, SpAcAn_ampl, MASMatriX
	SetDimLabel 1, 6, ampl_fail, MASMatriX
	SetDimLabel 1, 7, spike_freq, MASMatriX
	SetDimLabel 1, 8, nb_spikes, MASMatriX
	SetDimLabel 1, 9, AmplFailSD, MASMatriX
	
	// Setting of rows labels
	Variable counter 
	For (counter = 0; counter < numpnts(borne_labels); counter += 1)
		SetDimLabel 0, counter, $(borne_labels[counter]), MASMatriX
	EndFor
	
	MASMatriX [] [0] = limit_wave_1[p] // Filling the two first columns
	MASMatriX [] [1] = limit_wave_2[p]
	
	//_______________________________________Part to supress intersweeps intervals duration in the caculation of AP freq___________________________
	
	Make/O/N=(DimSize(Les_Bornes,0)) sweep_limit1, sweep_limit2
	sweep_limit1[] = les_Bornes [p] [0]
	sweep_limit2[] = les_Bornes [p] [1]
	
	WaveStats/Q/M=1 sweep_limit1
	nb_NaNs = V_numnaNs
	WaveStats/Q/M=1 sweep_limit2
	nb_NANs += V_numNaNs
	If (nb_NaNs != 0) // If some NaN were found
		RemoveNaNs(sweep_limit1) // Remove them
		RemoveNaNs(sweep_limit2) // Remove them
	EndIf
	
	
	// __________________Necessary for stability against NaNs__________________________
	
	DoAlert 1, "If NaNs are found in the occurrence wave of ref channel \n procedure might be unstable \n do you want to automaticaly remove NaNs"
	
	If (V_flag == 1) // If yes is clicked
		ProcDeleteNaN(True_Tags_ch1_str) // This will untag events if they have a NaN occurence time on channel 1
	EndIf	
	
	//Check (probably useless)
	Wavestats/Q/M=1 Amp_wave_ch2
	If (V_numNans != 0)
		Abort "Warning ! NaNs detected in the channel 2 amplitude wave!"
	EndIf
	
	Wavestats/Q/M=1 True_tags_ch2
	If (V_numNans != 0)
		Abort "Warning ! NaNs detected in the channel 2 tags wave!"
	EndIf
	
	Wavestats/Q/M=1 True_tags_ch1
	If (V_numNans != 0)
		Abort "Warning ! NaNs detected in the channel 1 tags wave!"
	EndIf
	
	//___________________________loop for each sweep______________________________
	
	//Make/O/N=((numpnts(borne_labels)), 2) limit_P_store 
	For (counter = 0; counter < (numpnts(limit_wave_1)); counter += 1)
		TEMPO_limit_1 = limit_wave_1[counter] //beginning of the [counter] range (in sec)
		TEMPO_limit_2 = limit_wave_2[counter] //end of the [counter] range (in sec)
		MAS(peak_wave_ch1, True_Tags_ch1, events_channel, Amp_wave_ch1, peak_win_width, Bsl_width, AP_rise_time, Lag_to_peak, lower_lim = TEMPO_limit_1, upper_lim = TEMPO_limit_2, compteur = counter)
		Wave TEMPO_random_noise, TEMPO_Deltas // Import the wave generated by the function	
			
		Make/O/N=(nb_bins_datas) TEMPO_Hist_Deltas
		Make/O/N=(nb_bins_noise) TEMPO_Hist_noise
		//_______________Calculation w/ original SpAcAn method w/ Tags2 sync from Tags1
		Lower_limit_P2 = X_finder(peak_wave_ch2, TEMPO_limit_1)//On ch2
		Upper_limit_P2 = X_finder(peak_wave_ch2, TEMPO_limit_2)
		//Calculation of SpAcAn amplitude (w/  tags2 synchronysed)
		
		Amp_wave_ch2 *= True_tags_ch2 // Set amplitude of untagged events to 0
		
		spac_sum = Sum(Amp_wave_ch2, Lower_limit_P2, Upper_limit_P2)
		spac_nb = Sum(True_tags_ch2, Lower_limit_P2, Upper_limit_P2)
		Spacan_ampl = (spac_sum / spac_nb)
		
		//____________figures w/ MAS method_______________________________
		If (numpnts(TEMPO_Deltas) > 2) //Minimum required to do an histogram
			//________________________Calculation and plot of distribution________________________________
			//Part of function to do the histogram
			
			//Defines the limits of the histogram 
			Wavestats/Q TEMPO_Random_noise
			max_noise = V_max			

			//Creates the histogram wave and display
			
			Histogram/B={hist_min, true_hist_bin_size, nb_bins_datas} TEMPO_Deltas, TEMPO_Hist_Deltas
			Histogram/B={hist_min, noise_hist_bin_size, nb_bins_noise} TEMPO_Random_noise, TEMPO_Hist_noise
			
			// Define final and unique wave names
			String DataName = "Data_" + borne_labels[counter], NoiseName = "Noise_" + borne_labels[counter]
			Duplicate/O TEMPO_hist_Deltas, $DataName// Save the histogram waves
			Duplicate/O TEMPO_hist_noise, $NoiseName
			
			DoWindow/K TempHistName
			Display/K=1/N=TempHistName/B=amplitude/R=nb_events $DataName, $NoiseName
			SetAxis/W=TempHistName amplitude, -500e-12,50e-12
			DoWindow/K $("Hist" + borne_labels[counter])
			RenameWindow TempHistName, $("Hist" + borne_labels[counter])
			
			ModifyGraph mode=5, hbfill=2 // Curve properties
			ModifyGraph rgb($DataName)=(65535, 0, 0), rgb($NoiseName)=(0, 0, 65535) // Colors
			ModifyGraph freePos(amplitude)={0,nb_events}, freePos(nb_events)={0,kwFraction} // Axis
			Label nb_events "Nb events data"
			Label amplitude "Delta amplitude"
			ModifyGraph margin=40, axThick=2,font="DejaVu Sans"
			
			//Legend
			SetDrawEnv xcoord= rel,ycoord= rel,textrgb= (65280,0,0),fname= "DejaVu Sans", fstyle= 1
			DrawText 0.75,0.6,"Data"
			SetDrawEnv xcoord= rel,ycoord= rel,textrgb= (0,12800,52224),fname= "DejaVu Sans", fstyle= 1
			DrawText 0.75,0.5,"Noise"
			
			//______________Calculation of the alpha scaling factor to fit the noise wave to the data wave______
			
			// finds the point in peak_wave_ch1 closest to lower_limit_P (point of the wave where analysis beggins)
			Lower_limit_P = X_finder(peak_wave_ch1, TEMPO_limit_1)
			Upper_limit_P = X_finder(peak_wave_ch1, TEMPO_limit_2)
			
			alpha = hist_fit(TEMPO_hist_deltas, TEMPO_hist_noise, 0, max_noise) // We just fit the positive part of the noise curve
			TEMPO_hist_noise *= alpha // Scaling of the noise curve
			Duplicate/O TEMPO_hist_noise, $NoiseName // Overwrites the wave to update display after scaling
			
			alpha *=  ((numpnts(TEMPO_random_noise)) / (numpnts(TEMPO_Deltas))) //since we have several noise values per sample
			
			// ******************************************************************
			If (((numpnts(TEMPO_random_noise)) / (numpnts(TEMPO_Deltas))) != round((numpnts(TEMPO_random_noise)) / (numpnts(TEMPO_Deltas))))
				Abort
			EndIf
			//*****************************************************************
			
			//______________Filling the matriX w/ values____________________________________________
			MASMatriX [counter] [2] = alpha // Insert the failure ratio value in the stat matrix
			
			
			nb_succes = round((numpnts(TEMPO_Deltas) - (alpha * numpnts(TEMPO_Deltas)))) //number of successes
			
			WaveStats/Q TEMPO_Deltas
			TEMPO_ampl = (V_Sum / nb_succes)//Mean Amplitude without failures
			TEMPO_ampl_fail = V_Avg //Mean Amplitude w/ Failures
			MASMatriX [counter] [3] = V_npnts //number of events
			MASMatriX [counter] [9] = V_sdev
			//**************************************************Part to substract the intersweep intervals**********************************
			nb_interSweeps = 0 // Reset the variable
			
			For (counterSweeps = 0; counterSweeps < (numpnts(Sweep_limit2)); counterSweeps += 1)
				If ((Sweep_limit2[counterSweeps] > TEMPO_limit_1) && (Sweep_limit2[counterSweeps] < TEMPO_limit_2))
					If ((Sweep_limit1[counterSweeps + 1] > TEMPO_limit_1) && (Sweep_limit1[counterSweeps + 1] < TEMPO_limit_2)) // Used to lines to make it human readable
						nb_interSweeps += 1
					EndIf
				EndIf
			EndFor
			
			Print "Number of InterSweeps for range " + borne_labels[counter] + " = " + num2str(nb_interSweeps)
			TEMPO_freq_spikes = (Sum(True_tags_ch1, Lower_limit_P, Upper_limit_P) / ((TEMPO_limit_2 - TEMPO_limit_1) - (nb_interSweeps * nb_interSweeps_secs))) // Multiply by the nb of points inserted
			//**************************************************

			MASMatriX [counter] [4] = TEMPO_ampl //amplitude of current sweep without failures
			MASMatriX [counter] [5] = Spacan_ampl // Amplitude w/ SpAcAn tags2 synchronysed from tags1
			MASMatriX [counter] [6] = TEMPO_ampl_fail //amplitude of current sweep w/ failures
			MASMatriX [counter] [7] = TEMPO_freq_spikes //freq of AP in current sweep
			MASMatriX [counter] [8] = (Sum(True_tags_ch1, Lower_limit_P, Upper_limit_P)) // Number of actionpotentials in the sweep
						
		EndIf
		//_________________CreatingThe whole wave and killing locals______________________
		Print "the noise pnt nb for step " + num2str(counter) + "is : " + num2str(numpnts(TEMPO_random_noise))
		Print "the data pnt nb for step " + num2str(counter) + "is : " + num2str(numpnts(TEMPO_Deltas)) 
		Concatenate/O/NP {Random_noise, TEMPO_Random_noise}, Random_noise
		Concatenate/O/NP {Deltas, TEMPO_Deltas}, Deltas
		Killwaves TEMPO_Random_noise, TEMPO_Deltas, TEMPO_hist_deltas, TEMPO_hist_noise
	
	EndFor
	Killwaves Amp_wave_ch1 
	Killvariables Lower_limit_P, Upper_limit_P
	
	Edit/k=1 MASMatriX.ld //.ld is used to show labels
	
	//Defines the limits of the histogram 
	Wavestats/Q Random_noise
	max_noise = V_max

	Histogram/B={hist_min, true_hist_bin_size, nb_bins_datas} Deltas, Hist_Deltas
	Histogram/B={hist_min, noise_hist_bin_size, nb_bins_noise} Random_noise, Hist_noise
	
	DoWindow/K distribution_histo
	Display/K=1/N=distribution_histo/B=amplitude/R=nb_events_datas Hist_Deltas
	SetAxis/W=distribution_histo amplitude, -500e-12,50e-12
	AppendToGraph/W=distribution_histo/B=amplitude/L=nb_events_noise Hist_noise
	
	GetAxis/W=distribution_histo/Q nb_events_datas
	SetAxis/W=distribution_histo nb_events_noise, 0, V_max
	ModifyGraph/W=distribution_histo mode=5, hbfill=2 //Curve type 
	ModifyGraph/W=distribution_histo rgb(Hist_Deltas)=(65535, 0, 0), rgb(Hist_noise)=(0, 0, 65535) // Color
	ModifyGraph/W=distribution_histo freePos(amplitude)={0,nb_events_datas}, freePos(nb_events_datas)={0,kwFraction}, freePos(nb_events_noise)={0,kwFraction} // Axis

	Label/W=distribution_histo nb_events_datas "Nb events data"; Label/W=distribution_histo nb_events_noise "Nb events noise"
	Label/W=distribution_histo amplitude "Delta amplitude"
	ModifyGraph/W=distribution_histo margin=40, axThick=2,font="DejaVu Sans"
	
	//Scaling
	alpha = hist_fit(Hist_deltas, Hist_noise, 0, max_noise) // We just fit the positive part of the noise curve
	Hist_noise *= alpha // Scaling of the noise curve
	
	//Legend
	SetDrawEnv xcoord= rel,ycoord= rel,textrgb= (65280,0,0),fname= "DejaVu Sans", fstyle= 1
	DrawText 0.75,0.6,"Data"
	SetDrawEnv xcoord= rel,ycoord= rel,textrgb= (0,12800,52224),fname= "DejaVu Sans", fstyle= 1
	DrawText 0.75,0.5,"Noise"
	
	//Check the nb of values that would have been out of range (for curisosity)
	Print "Number of values inferior to range : " + num2str(nb_values_inf_to_range) + "\r Number of values superior to range : " + num2str(nb_values_sup_to_range)
	
	//Kill useless waves
	Killwaves events_channel, peak_wave_ch1, peak_wave_ch2, True_tags_ch1, True_tags_ch2, Amp_wave_ch2, limit_wave_1, limit_wave_2, Deltas, Random_noise
	KillVariables alpha
End

// Finds the location as a P index of the value in the Target_wave (X_value is specified because Tagrget_wave shall be monotonous)
//If X_value equidistant from two values, the lowest value is taken (except if U use the options
Function X_finder(Target_wave, X_value, [SupBorder, InfBorder]) //Put =1 if you want to use the optionnal parameter, SupBorder means take the point that is >= to X_value (1/X for InfBorder)
	Wave Target_wave
	Variable X_value
	Variable Supborder, InfBorder
	
	Variable P_value
	
	Duplicate/O Target_wave, delta_index_W
		
	delta_index_W -= X_value // fills the wave with the difference between each point and the value
	delta_index_W = Abs(delta_index_W) // trick to avoid negative values to be taken into account
	
	WaveStats/Q delta_index_W
	P_value = X2pnt(delta_index_W, V_minLoc) //returns the location in point of the minimum value in the wave
	Killwaves delta_index_W
	
	If (!ParamIsDefault(SupBorder)) // If parameter was used
		If (Target_wave[P_value] < X_value) // If value inferior increase by one
			P_value += 1
		EndIf
		For (; ((numtype(Target_wave[P_value]) == 2) && (P_value < numpnts(Target_wave))); ) // If value is NaN switch to next
			P_value += 1
		EndFor
	ElseIf (!(ParamIsDefault(InfBorder))) // If parameter was used
		If (Target_wave[P_value] > X_value) // If value superior decrease by one
			P_value -= 1
		EndIf
		For (; ((numtype(Target_wave[P_value]) == 2) && (P_value >= 0)); ) // If value is NaN switch to previous
			P_value -= 1
		EndFor
	EndIf
	
	Return P_value
End


Function hist_fit(wave_to_fit, wave_to_scale, begin, ending) //Take care begin and end are in X scaling (use pnt2X if you want to use point indexing)
	Wave wave_to_fit, wave_to_scale
	Variable begin, ending
	
	If (numpnts(wave_to_fit) != numpnts(wave_to_scale))
		Abort "Can't compute the Least Square Fit with two different distribution sizes"
	EndIf
	
	//The local variable for output
	variable my_coefficient
	
	Make/O/N=((ending - begin) + 1) products, squares
	
	products = wave_to_fit(begin + x) * wave_to_scale(begin + x)
	squares = wave_to_scale(begin + x) * wave_to_scale(begin + x)
	
	my_coefficient = (sum(products, 0, numpnts(products) - 1) / sum(squares, 0, numpnts(squares) - 1))
	Killwaves products, squares
	return my_coefficient
	//The least square difference equation is : sum ((wave_to_fit - wave_to_scale)) that we have to minimize
	// <=> wave_to_fit - 2*wave_to_fit_*wave_to_scale + wave_to_scale
	// With an @ scaling coefficient and the wave_to_fit as a constant, we have the relation :
	//cste - 2*alpha*wave_to_fit*wave_to_scale + alpha*wave_to_scale
	// Relation of the form C + Bx + Ax that will be minimum for  -(B + sqrt(delta)) / 2*A
	//We can also consider the null derivative : 2*wave_to_scale*alpha - 2*wave_to_fit*wave_to_scale = 0
	//with products and squares : alpha = 2*products / 2*squares = my_coefficient  

End