//////////////////////////////////////////////////////////////////
// This is SpAcAn 1.02.2 (Jan. 12, 2009)
// Project website: http://www.spacan.net
//////////////////////////////////////////////////////////////////

// WARNING. If you are running Igor PRO on a GNU/Linux system using WINE, and want to modify the code of SpAcAn,
// be aware that PopupMenu controls don't work. The alternative solution is to virtualize Windows...
// Wavemetrics does not seem to want to develop a Linux version of Igor PRO.

// CHANGE LOG OF VERSION 1.02.2:
// Adds a measure of the time of the "foot" of individual events (intersection of the baseline with the 
// 20%-80% rise time line) (function DETECT).
// Adds a button in the window DetectedEvents to superimpose tagged events. Events can be aligned
// on their "foot" time, half-rise time or peak time.
// fixes a bug in function LoadABF which was causing the function to abort.
// adds better graphs for Freq/Amp plots.
// Fixes a bug related to saving the analysis parameters in the wave "params".
// Some code cleanup.

// TODO:

// FIXME:
// In LoadABF, better if name proposed for saving is the same as the one from the file.

#pragma rtGlobals=1		// Use modern global access method.

MENU 	"SpAcAn"
	"Initialize SpAcAn", Initialize(0)
	//"------------------------"
	"Import abf file...", LoadABF()
	SubMenu "Bring to front..."
		"Main window", DoWindow/F MainWindow
		"Detected events, channel 1", DoWindow/F DetectedEvents1
		"Detected events, channel 2", DoWindow/F DetectedEvents2
	End
	"About SpAcAn...", About(3)
END

// ========== ABOUT SPACAN ==========
FUNCTION About(numero)
	Variable numero
	// Builds panel
	DoWindow/K Window_About; Display/W=(300,100,600,300) as "About SpAcAn..."; DoWindow/C Window_About
	NewPanel/K=1/N=Panel_About/HOST=Window_About/FG=(FL, FT, gRight, FB )/W=(1,1,1,1)
	ModifyPanel cbRGB=(65500,65500,65500)
	// Add Logo on top
	DrawPict/W=Window_About#Panel_About 145, 17, 1, 1, ProcGlobal#SpAcAnLogo
	// Close button
	Button ButtonCloseAbout pos={350,10},	size={40,20}, proc=ButtonProcCloseAbout, 	title="Close"
	// Text
	SetDrawEnv fsize= 14;DrawText 103,76,"Version 1.02.2 - Jan 11, 2010"
	SetDrawEnv fsize= 14;DrawText 10,115,"SpAcAn (Spontaneous Activity Analysis) is a collection of"
	SetDrawEnv fsize= 14;DrawText 10,135,"IGOR Pro functions for the detection and analysis of"
	SetDrawEnv fsize= 14;DrawText 10,155,"spontaneous activity in 1 or 2-channel electrophysiological"
	SetDrawEnv fsize= 14;DrawText 10,175,"recordings."
	SetDrawEnv fsize= 14;DrawText 10,195,"By Guillaume P. Dugu & Charly Rousseau, 2008."
	SetDrawEnv fsize= 14;DrawText 10,215,"Project website:"
	SetDrawEnv fsize= 14, fstyle=4;DrawText 115,215,"http://www.spacan.net/"
END

FUNCTION ButtonProcCloseAbout(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_About
END
// ========== END OF ABOUT SPACAN ==========

// ========== TABS HIDDING IN MAIN WINDOW ==========

// First group of tabs: detection parameters
FUNCTION MainWindowTabProc(CtrlName, TabNum) : TabControl
	String CtrlName
	Variable TabNum
	Variable isTab0= tabNum==0
	Variable isTab1= tabNum==1
	// To add new tabs:
	// Variable isTab2= tabNum==2
	// Variable isTab3= tabNum==3
	// Note: disable=0 means "show", disable=1 means "hide"
	
	// Detection params channel1
	ModifyControl SetVarBSLpnts1		disable= !isTab0	// hide if not Tab 0
	ModifyControl SetVarRTPnts1 		disable= !isTab0
	ModifyControl SetVarPpnts1 			disable= !isTab0
	ModifyControl DisplayBSLms1 		disable= !isTab0
	ModifyControl DisplayRTms1 		disable= !isTab0
	ModifyControl DisplayPms1 			disable= !isTab0
	ModifyControl SetWPD1 				disable= !isTab0
	ModifyControl SetThreshold1 		disable= !isTab0
	ModifyControl DW1 					disable= !isTab0
	ModifyControl NoEch1 				disable= !isTab0
	ModifyControl FreqEch1 				disable= !isTab0
	
	If (tabNum==0) // If the selected tab is the first
		NVar Diff1Clicked
		If (Diff1Clicked ==1) // If diff already performed
			ModifyControl Detect1 		disable = 0 	// Usable control
		Else
			ModifyControl Detect1 		disable = 2 	// Grayed control
		EndIf
	Else
		ModifyControl Detect1 			disable= 1	// hidden control
	EndIf
	
	// Part not in use but that may be usefull to call certain functions
	// If (TabNum == 0)
	//	SetActiveSubWindow MainWindow#MainPanel	
	// EndIf
		
	// Detection params channel2
	ModifyControl SetVarBSLpnts2 		disable= !isTab1	// hide if not Tab 1
	ModifyControl SetVarRTPnts2 		disable= !isTab1
	ModifyControl SetVarPpnts2 			disable= !isTab1
	ModifyControl DisplayBSLms2 		disable= !isTab1
	ModifyControl DisplayRTms2 		disable= !isTab1
	ModifyControl DisplayPms2 			disable= !isTab1
	ModifyControl SetWPD2 				disable= !isTab1
	ModifyControl SetThreshold2 		disable= !isTab1
	ModifyControl DW2 					disable= !isTab1
	ModifyControl NoEch2 				disable= !isTab1
	ModifyControl FreqEch2 				disable= !isTab1
	
	If (tabNum == 1) // If the selected tab is the second
		NVar Diff2Clicked
		If (Diff2Clicked == 1)// If diff already performed
			ModifyControl Detect2			disable = 0 // Usable control
		Else
			ModifyControl Detect2			disable = 2 // Grayed control
		EndIf
	Else
		ModifyControl Detect2 			disable= 1	// hidden control
	EndIf

	// Part not in use but that may be usefull to call certain functions
	// If (TabNum == 1)	
	// 	SetActiveSubWindow MainWindow#MainPanel
	// EndIf
	
	// ADD YOU TAB HERE

	return 0
END

// Second group of tabs: Plots and Correlogram
FUNCTION MainWindowTab2Proc(CtrlName, TabNum) : TabControl
	String CtrlName
	Variable TabNum
	Variable isTab0= tabNum==0
	Variable isTab1= tabNum==1
	// To add new tabs:
	// Variable isTab2= tabNum==2
	// Variable isTab3= tabNum==3
	// Note: disable=0 means "show", disable=1 means "hide"

	// Freq/Amp tab
	ModifyControl CheckBoxUseTags	disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonPlot 				disable= !isTab0
	ModifyControl SetFvsTBin 			disable= !isTab0
	ModifyControl SetAvsTBin 			disable= !isTab0
	ModifyControl SetAmpHistoBin 		disable= !isTab0

	// Part not in use but that may be usefull to call certain functions
	// If (TabNum == 0)	
	//	SetActiveSubWindow MainWindow#MainPanel
	// EndIf
	
	// Correlogram tab
	ModifyControl DoCorrelogram 				disable= !isTab1	// hide if not Tab 1
	ModifyControl SetCorrTimeWindow 		disable= !isTab1
	ModifyControl SetCorrBin 					disable= !isTab1
	ModifyControl CheckBoxUsePeakRef 		disable= !isTab1
	ModifyControl CheckBoxUseHRRef 		disable= !isTab1
	ModifyControl CheckBoxUsePeakTest 		disable= !isTab1
	ModifyControl CheckBoxUseHRTest 		disable= !isTab1
	ModifyControl RefCh1Box 					disable= !isTab1
	ModifyControl RefCh2Box						disable= !isTab1
	ModifyControl TestCh1Box 					disable= !isTab1
	ModifyControl TestCh2Box					disable= !isTab1
	ModifyControl RefTitleBox  					disable= !isTab1
	ModifyControl TestTitleBox  					disable= !isTab1
	ModifyControl CheckBoxCCTypeRaw		disable= !isTab1
	ModifyControl CheckBoxCCTypeShuff		disable= !isTab1
	ModifyControl CheckBoxCCTypeNorm		disable= !isTab1
	
	// Part not in use but that may be usefull to call certain functions
	// If (TabNum == 1)	
	//	SetActiveSubWindow MainWindow#MainPanel	
	// EndIf
	
	// ADD YOU TAB HERE

	return 0
END
// ========== END OF TABS HIDDING IN THE MAIN WINDOW ==========

// INITIALIZE: THE FUNCTION THAT INITIALIZES EVERYTHING AND DRAWS THE MAIN WINDOW
FUNCTION Initialize(numero)
	Variable numero
	
	DoWindow/K MainWindow
	DoWindow/K DetectedEvent1
	DoWindow/K DetectedEvent2
	DoWindow/K CrossCorrelogramGraph
	Newdatafolder/O root:spacan
	Killdatafolder root:spacan
	Newdatafolder/O/S root:spacan
	
	Variable/G NumChannels = 2
	Variable/G FDC1 = 0				// FDC1(2) = First Display Channel 1(2). Tells the functions ButtonLoadWave1 and ButtonLoadWave2 if this is the first time the waves are displayed (FDC1(2)=0).
	Variable/G FDC2 = 0
	Variable/G LoadedChannels = 0	// LoadedChannels =	0 if no wave loaded, 1 if one wave loaded, 2 if two waves loaded
	Variable/G Length = NaN			// The length of the recording
	Variable/G RefNum
	Variable/G BSLpnts1, RTpnts1, Ppnts1, BSLpnts2, RTpnts2, Ppnts2
	Variable/G BSLms1, RTms1, Pms1, BSLms2, RTms2, Pms2
	Variable/G Threshold1, Threshold2
	Variable/G SI1, SI2; SI1 = NaN; SI2 = NaN	// Sample Intervals of waves 1 and 2 are NaN; will be defined later, when loading the traces
	Variable/G C1E = 0; // C1E = 1 if channel 1 exists
	Variable/G C2E = 0; // C2E = 1 if channel 2 exists
	
	// Setting up the main window
	DoWindow/K MainWindow ; Display/W=(0,0,770,470) as "Main window"; DoWindow/C MainWindow
	NewPanel/K=1/N=MainPanel/HOST=MainWindow/FG=(FL, FT, gRight, FB )/W=(0,0,207,470) ; ModifyPanel cbRGB=(65500,65500,65500)
	
	Variable/G Ch1PosOffset = 208 // For programming purpose; easier to move the whole bloc (detection parameters) by changing this variable.
	// The following line creates the tab menu
	TabControl MainWindowTab,pos={7,Ch1PosOffset-27},size={257,165}, fstyle=1,proc=MainWindowTabProc, win=MainWindow#MainPanel, value= 0
	// The following two lines create the different tabs
	TabControl MainWindowTab,tabLabel(0)="Channel1",value= 0
	TabControl MainWindowTab,tabLabel(1)="Channel2",value= 0
	
	// The following line creates the tab menu
	Variable/G Tab2PosOffset = 379
	TabControl MainWindowTab2,pos={7,Tab2PosOffset},size={257,150}, fstyle=1,proc=MainWindowTab2Proc, win=MainWindow#MainPanel, value= 0
	// The following two lines create the different tabs
	TabControl MainWindowTab2,tabLabel(0)="Freq/Amp",value= 0
	TabControl MainWindowTab2,tabLabel(1)="Correlograms",value= 0
	
	// Add Logo
	DrawPict/W=MainWindow#MainPanel 80, 5, 1, 1, ProcGlobal#SpAcAnLogo
	
	// FIRST GROUP: loading and displaying waves.
	SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,70,"Controls"
	
	Variable/G ControlsPosOffset = 75
	Variable/G GridState1, GridState2; GridState1 = 0; GridState2 = 0
	Button		ButtonSync			pos={191,ControlsPosOffset},		size={36,40},		proc=ButtonSyncProc, 	title="Sync",											Disable=2
	Button 		LoadWave1 			pos={10,ControlsPosOffset},			size={45,20},		proc=ButtonLoadWave1,	title="Load 1"
	Button 		LoadWave2 			pos={10,ControlsPosOffset+20},		size={45,20},		proc=ButtonLoadWave2,	title="Load 2"		
	Button 		FileStart 				pos={55,ControlsPosOffset},			size={34,20},		proc=ButtonStart,			title="|<----",											Disable=2
	Button 		Leftward 				pos={89,ControlsPosOffset},			size={34,20},		proc=ButtonLeft,			title="<----",											Disable=2
	Button 		Rightward 			pos={123,ControlsPosOffset},		size={34,20},		proc=ButtonRight,			title="---->",											Disable=2
	Button 		FileEnd 				pos={157,ControlsPosOffset},		size={34,20},		proc=ButtonEnd,	 		title="---->|",											Disable=2
	Button 		FullScale 			pos={55,ControlsPosOffset+20},		size={68,20},		proc=ButtonFullScale,		title="Full Scale",									Disable=2
	Button 		Hexpand 				pos={123,ControlsPosOffset+20},	size={34,20},		proc=ButtonHexpand,		title="<---->",											Disable=2
	Button 		Hshrink 				pos={157,ControlsPosOffset+20},	size={34,20},		proc=ButtonHshrink, 		title=">----<",											Disable=2
	ValDisplay 	DisplayFL 			pos={120,ControlsPosOffset-21}, 	size={140,20}, 										title="Rec. length (s) ="
	Button		GridsMW				pos={227,ControlsPosOffset},		size={34,20},		proc=ButtonGridsMW,		title="Grid",											Disable=2
	Button		FlipWave				pos={227,ControlsPosOffset+20},	size={34,20},		proc=ButtonFlip,			title="Flip",											Disable=2
	ValDisplay 	DisplaySI1	 		pos={10,ControlsPosOffset+43}, 	size={250,20}, 										title="Sample Interval, channel 1 (s/pt) ="
	ValDisplay 	DisplaySI2	 		pos={10,ControlsPosOffset+60}, 	size={250,20}, 										title="Sample Interval, channel 2 (s/pt) ="
	Execute "Valdisplay DisplayFL value=root:spacan:Length"
	Execute "Valdisplay DisplaySI1 value=root:spacan:SI1"
	Execute "Valdisplay DisplaySI2 value=root:spacan:SI2"

	// SECOND GROUP, FIRST TAB: detection parameters for channel 1.
	Variable/G NumberofEventsch1 = NaN; Variable/G FreqEvents1 = NaN
	SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,175,"Detection parameters"
	Variable/G TWPD1 = NaN
	SetVariable 	SetVarBSLpnts1	pos={10,Ch1PosOffset},				size={147,20}, 		proc=SetBSLms,			title="Baseline (pnts) =",	 						limits={1,inf,2}, 	value = BSLpnts1
	SetVariable 	SetVarRTpnts1 		pos={10,Ch1PosOffset+17}, 			size={147,20},		proc=SetRTms,				title="Rise time (pnts) =", 	 						limits={1,inf,1}, 	value = RTpnts1
	SetVariable 	SetVarPpnts1		pos={10,Ch1PosOffset+34}, 			size={147,20}, 		proc=SetPms,				title="Peak (pnts) =", 								limits={1,inf,2}, 	value = Ppnts1
	Valdisplay 	DisplayBSLms1 	pos={163,Ch1PosOffset}, 			size={97,20},										title="<-> (ms)" 
	Valdisplay 	DisplayRTms1 		pos={163,Ch1PosOffset+17}, 		size={97,20},										title="<-> (ms)" 
	Valdisplay 	DisplayPms1 		pos={163,Ch1PosOffset+34}, 		size={97,20},										title="<-> (ms)" 
	SetVariable 	SetWPD1			pos={10,Ch1PosOffset+52},			size={250,10},										title="Window for peak detection (ms) =",		limits={0,inf,1},	value = TWPD1
	SetVariable 	SetThreshold1		pos={10,Ch1PosOffset+69},			size={147,10},										title="Threshold =",									limits={0,inf,0.1},value = Threshold1
	Button		DW1					pos={10,Ch1PosOffset+89},			size={52,20},		proc=ButtonDW1,			title="Diff",											Disable=2
	Button		Detect1				pos={10,Ch1PosOffset+109},		size={52,20},		proc=ButtonD1,				title="Detect",										Disable=2
	ValDisplay	NoEch1				pos={70,Ch1PosOffset+87},			size={190,20},										title="Number of events detected: "
	ValDisplay	FreqEch1			pos={70,Ch1PosOffset+104},		size={190,20},										title="Average frequency (Hz): "
	Execute "ValDisplay DisplayBSLms1 value = root:spacan:BSLms1"
	Execute "ValDisplay DisplayRTms1 value = root:spacan:RTms1"
	Execute "ValDisplay DisplayPms1 value = root:spacan:Pms1"
	Execute "ValDisplay NoEch1 value=root:spacan:NumberofEventsch1"
	Execute "ValDisplay FreqEch1 value=root:spacan:FreqEvents1"
	Variable/G Ch1Detected = 0	// This variable will tell us if the function ButtonD1 has been run.

	// SECOND GROUP, SECOND TAB: detection parameters for channel 2.
	Variable/G NumberofEventsch2 = NaN; Variable/G FreqEvents2 = NaN
	Variable/G TWPD2 = NaN
	Variable/G Ch2PosOffset = 208 // For programming purpose; easier to move the whole bloc by changing this variable.
	SetVariable 	SetVarBSLpnts2	pos={10,Ch2PosOffset},				size={147,20}, 		proc=SetBSLms,			title="Baseline (pnts) =",	 						limits={1,inf,2}, 	value = BSLpnts2
	SetVariable 	SetVarRTpnts2 		pos={10,Ch2PosOffset+17}, 			size={147,20},		proc=SetRTms,				title="Rise time (pnts) =", 	 						limits={1,inf,1}, 	value = RTpnts2
	SetVariable 	SetVarPpnts2		pos={10,Ch2PosOffset+34}, 			size={147,20}, 		proc=SetPms,				title="Peak (pnts) =", 								limits={1,inf,2}, 	value = Ppnts2
	Valdisplay 	DisplayBSLms2 	pos={163,Ch2PosOffset}, 			size={97,20},										title="<-> (ms)" 
	Valdisplay 	DisplayRTms2		pos={163,Ch2PosOffset+17}, 		size={97,20},										title="<-> (ms)" 
	Valdisplay 	DisplayPms2 		pos={163,Ch2PosOffset+34}, 		size={97,20},										title="<-> (ms)" 
	SetVariable 	SetWPD2			pos={10,Ch2PosOffset+52},			size={250,10},										title="Window for peak detection (ms) =",		limits={0,inf,1},	value = TWPD2
	SetVariable 	SetThreshold2		pos={10,Ch2PosOffset+69},			size={147,10},										title="Threshold =",									limits={0,inf,0.1},value = Threshold2
	Button		DW2					pos={10,Ch2PosOffset+89},			size={52,20},		proc=ButtonDW2,			title="Diff",											Disable=2
	Button		Detect2				pos={10,Ch2PosOffset+109},		size={52,20},		proc=ButtonD2,				title="Detect",										Disable=2
	ValDisplay	NoEch2				pos={70,Ch2PosOffset+87},			size={190,20},										title="Number of events detected: "
	ValDisplay	FreqEch2			pos={70,Ch2PosOffset+104},		size={190,20},										title="Average frequency (Hz): "
	Execute "ValDisplay DisplayBSLms2 value = root:spacan:BSLms2"
	Execute "ValDisplay DisplayRTms2 value = root:spacan:RTms2"
	Execute "ValDisplay DisplayPms2 value = root:spacan:Pms2"
	Execute "ValDisplay NoEch2 value=root:spacan:NumberofEventsch2"
	Execute "ValDisplay FreqEch2 value=root:spacan:FreqEvents2"
	MainWindowTabProc("MainWindowTab",0)
	Variable/G Ch2Detected = 0	// This variable will tell us if the function ButtonD2 has been run.

	// THIRD GROUP: freq/amp plots and correlograms
	SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,373,"Plots"
	Variable/G FvsTBin, AvsTBin, AmpHistoBin, UseTags; FvsTBin = 1; AvsTBin = 1; AmpHistoBin = NaN; UseTags = 0
	CheckBox	CheckBoxUseTags		pos={10,Tab2PosOffset+28},	size={20,20},		proc=ProcCheckBoxUseTags, 	title="Use tags"
	Button		ButtonPlot				pos={10,Tab2PosOffset+48},	size={52,20},		proc=ProcButtonPlot, 				title="Plot"
	SetVariable 	SetFvsTBin				pos={89,Tab2PosOffset+28},	size={170,20},												title="Bin freq. vs time (s) =", 		value = FvsTBin
	SetVariable 	SetAvsTBin				pos={89,Tab2PosOffset+45},	size={170,20},												title="Bin amp. vs time (s) =",		value = AvsTBin
	SetVariable 	SetAmpHistoBin			pos={89,Tab2PosOffset+62},	size={170,20},												title="Bin amp. histo. (--) =", 		value = AmpHistoBin
	Variable/G CorrCh1, CorrCh2, CorrMethod; CorrCh1 = 1; CorrCh2 = 1; CorrMethod = 1
	// Explanations: CorrCh1 takes the values 1 or 2 if "Channel 1" or "Channel 2" are selected, respectively, in the popup menu. Idem for CorrCh2
	// CorrMethod takes the value 1 if "Peak position" is selected, 2 if "Half rise time" is selected.
	Variable/G CorrTimeWindow, CorrBin; CorrTimeWindow = NaN; CorrBin = NaN
	// Here I replaced the drop-down menu (to choose the channels to use for the correlogram) by checkboxes, because the drop-down menus not to work with Wine.
	Variable/G RefCh1BoxVar = 1; Variable/G TestCh1BoxVar = 1; Variable/G RefCh2BoxVar = 0; Variable/G TestCh2BoxVar = 0
	TitleBox 		RefTitleBox  	frame=0, 		pos={10,Tab2PosOffset+51}, 		size={15, 20},											title="Ref:"
	TitleBox 		TestTitleBox  frame=0,	pos={10,Tab2PosOffset+74}, 		size={15, 20},											title="Test:"
	CheckBox 	RefCh1Box 					pos={50,Tab2PosOffset+51}, 		size={20,20},	proc=ProcToggleRefBox,			title="ch.1",  				value=1
	CheckBox 	RefCh2Box					pos={100, Tab2PosOffset+51}, 		size={20,20},	proc=ProcToggleRefBox,			title="ch.2",				value=0
	CheckBox 	TestCh1Box 				pos={50,Tab2PosOffset+74}, 		size={20,20},	proc=ProcToggleTestBox,		title="ch.1",				value=1
	CheckBox 	TestCh2Box					pos={100, Tab2PosOffset+74}, 		size={20,20},	proc=ProcToggleTestBox,		title="ch.2",				value=0
	Button		DoCorrelogram				pos={10,Tab2PosOffset+98},		size={52,20},	proc=ButtonDoCorrelogram,		title="Plot"
	SetVariable 	SetCorrTimeWindow		pos={10,Tab2PosOffset+28},		size={125,20},	 										title="Window (s) =",	value=CorrTimeWindow,		limits={0,inf,0.1}	
	SetVariable 	SetCorrBin					pos={150,Tab2PosOffset+28}, 		size={110,20},											title="Bin (s)",			value=CorrBin,					limits={0,inf,0.001}
	Variable/G RefUsePeak = 1; Variable/G RefUseHR = 0; Variable/G TestUsePeak = 1; Variable/G TestUseHR = 0
	CheckBox	CheckBoxUsePeakRef	pos={150,Tab2PosOffset+51},							proc=ProcToggleRef, 				title="peak", 				value = 1
	CheckBox	CheckBoxUseHRRef		pos={200,Tab2PosOffset+51},							proc=ProcToggleRef, 				title="half-rise", 			value= 0
	CheckBox	CheckBoxUsePeakTest	pos={150,Tab2PosOffset+74},							proc=ProcToggleTest, 			title="peak", 				value = 1
	CheckBox	CheckBoxUseHRTest		pos={200,Tab2PosOffset+74},							proc=ProcToggleTest, 			title="half-rise", 			value= 0
	Variable/G CCTypeRaw=0; Variable/G CCTypeShuff=0; Variable/G CCTypeNorm=1
	CheckBox	CheckBoxCCTypeRaw	pos={80,Tab2PosOffset+97}, 		size={100,20}, 	proc=ProcToggleCCRaw,			title="Raw",				value=0
	CheckBox	CheckBoxCCTypeShuff	pos={80,Tab2PosOffset+97+15}, 	size={100,20}, 	proc=ProcToggleCCShuff,		title="Shuffled",			value=0
	CheckBox	CheckBoxCCTypeNorm	pos={80,Tab2PosOffset+97+30}, 	size={100,20}, 	proc=ProcToggleCCNorm,		title="Normalized",		value=1
	MainWindowTab2Proc("MainWindowTab2",0)

	// Loading the detection parameters from waves stored on the disc.
	NewPath/o PathToParams "C:Program Files:WaveMetrics:Igor Pro Folder:Igor Procedures:"
	Open/P=PathToParams/Z=2/R RefNum as "SpAcAnParams.ibw"
	if (V_Flag == 0)
		LoadWave/o/P=PathToParams "SpAcAnParams.ibw" ; Wave Params
		BSLpnts1 = Params[0]; BSLms1 = (BSLpnts1-1)*SI1*1e-3
		RTpnts1 = Params[1]; RTms1 = (RTpnts1+1)*SI1*1e-3
		Ppnts1 = Params[2] ; Pms1 = (Ppnts1-1)*SI1*1e-3
		Threshold1 = Params[3]
		BSLpnts2 = Params[4] ; BSLms2 = (BSLpnts2-1)*SI2*1e-3
		RTpnts2 = Params[5] ; RTms2 = (RTpnts2+1)*SI2*1e-3
		Ppnts2 = Params[6] ; Pms2 = (Ppnts2-1)*SI2*1e-3
		Threshold2 = Params[7]
		CorrTimeWindow = Params[8]
		CorrBin = Params[9]
		FvsTBin = Params[10]
		AvsTBin = Params[11]
		AmpHistoBin = Params[12]
		TWPD1 = Params[13]
		TWPD2 = Params[14]
	else
		// Default params if no params if no SpAcAnParams.ibw file
		Make/N=13/o Params; Params=NaN
		BSLpnts1 = NaN; BSLms1 = NaN; RTpnts1 = NaN; RTms1 = NaN; Ppnts1 = NaN; Pms1 = NaN; Threshold1 = NaN
		BSLpnts2 = NaN; BSLms2 = NaN; RTpnts2 = NaN; RTms2 = NaN; Ppnts2 = NaN; Pms2 = NaN; Threshold2 = NaN
		CorrTimeWindow = 0.5; Execute "SetVariable SetCorrTimeWindow value=" + num2str(CorrTimeWindow)
		CorrBin = 0.005; Execute "SetVariable SetCorrBin value=" + num2str(CorrBin)
		FvsTBin = 5; Execute "SetVariable SetFvsTBin value=" + num2str(FvsTBin)
		AvsTBin = 5; Execute "SetVariable SetAvsTBin value=" + num2str(AvsTBin)
		AmpHistoBin = 1; Execute "SetVariable	SetAmpHistoBin value=" + num2str(AmpHistoBin)
		TWPD1 = 7; TWPD2 = 7
	endif
	Close RefNum
	
	MainWindowTabProc("", 0) //Necessary since the TabControl tb, value=0 doesn't seem to work
END

// FUNCTION THAT SYNCHRONIZES ONE CHANNEL ON TO THE OTHER (BUTTON "SYNC" IN THE MAIN WINDOW)
FUNCTION ButtonSyncProc(ctrlName) : ButtonControl
	String ctrlName
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		SetAxis/W=MainWindow#Channel2 bottom V_min,V_max
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		SetAxis/W=MainWindow#Channel1 bottom V_min,V_max
	endif
END

// FUNCTION CALLED BY THE "FLIP" BUTTON IN THE MAIN WINDOW
FUNCTION ButtonFlip(ctrlName) : ButtonControl
	String ctrlName
	getwindow kwTopWin activeSW
	Svar Filename1, Filename2
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		execute Filename1 + "= -" + Filename1
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		execute Filename2 + "= -" + Filename2
	endif
END

// FUNCTION CALLED BY THE BUTTON GRIDS IN THE MAIN WINDOW
FUNCTION ButtonGridsMW(ctrlName) : ButtonControl
	String ctrlName
	Svar Filename1, Filename2
	Nvar GridState1, GridState2
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GridState1 = 1 - GridState1
		ModifyGraph grid=GridState1,gridRGB=(0,0,0)
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GridState2 = 1 - GridState2
		ModifyGraph grid=GridState2,gridRGB=(0,0,0)
	endif
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "RefCh1Box" and "RefCh2Box" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleRefBox(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar RefCh1BoxVar, RefCh2BoxVar
	RefCh1BoxVar = 1 - RefCh1BoxVar
	RefCh2BoxVar = 1 - RefCh1BoxVar
	Execute "CheckBox RefCh1Box value=RefCh1BoxVar"
	Execute "CheckBox RefCh2Box value=RefCh2BoxVar"
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "TestCh1Box" and "TestCh2Box" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleTestBox(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar TestCh1BoxVar, TestCh2BoxVar
	TestCh1BoxVar = 1 - TestCh1BoxVar
	TestCh2BoxVar = 1 - TestCh1BoxVar
	Execute "CheckBox TestCh1Box value=TestCh1BoxVar"
	Execute "CheckBox TestCh2Box value=TestCh2BoxVar"
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "CheckBoxUsePeakRef" and "CheckBoxUseHRRef" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleRef(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar RefUsePeak, RefUseHR
	RefUsePeak = 1 - RefUsePeak
	RefUseHR = 1 - RefUsePeak
	Execute "CheckBox CheckBoxUsePeakRef value=RefUsePeak"
	Execute "CheckBox CheckBoxUseHRRef value=RefUseHR"
END

// FUNCTION TRIGGERED BY THE CHECKBOXES "CheckBoxUsePeakTest" and "CheckBoxUseHRTest" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleTest(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar TestUsePeak, TestUseHR
	TestUsePeak = 1 - TestUsePeak
	TestUseHR = 1 - TestUsePeak
	Execute "CheckBox CheckBoxUsePeakTest value=TestUsePeak"
	Execute "CheckBox CheckBoxUseHRTest value=TestUseHR"
END

// FUNCTION TRIGGERED BY THE CHECKBOXE "CheckBoxCCTypeRaw" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleCCRaw(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CCTypeRaw, CCTypeShuff, CCTypeNorm
	CCTypeRaw = 1 - CCTypeRaw
	if (CCTypeRaw == 1)
		CCTypeShuff = 0
		CCTypeNorm = 0
	endif
	Execute "CheckBox CheckBoxCCTypeRaw value=CCTypeRaw"
	Execute "CheckBox CheckBoxCCTypeShuff value=CCTypeShuff"
	Execute "CheckBox CheckBoxCCTypeNorm value=CCTypeNorm"
END

// FUNCTION TRIGGERED BY THE CHECKBOXE "CheckBoxCCTypeShuff" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleCCShuff(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CCTypeRaw, CCTypeShuff, CCTypeNorm
	CCTypeShuff = 1 - CCTypeShuff
	if (CCTypeShuff == 1)
		CCTypeRaw = 0
		CCTypeNorm = 0
	endif
	Execute "CheckBox CheckBoxCCTypeRaw value=CCTypeRaw"
	Execute "CheckBox CheckBoxCCTypeShuff value=CCTypeShuff"
	Execute "CheckBox CheckBoxCCTypeNorm value=CCTypeNorm"
END

// FUNCTION TRIGGERED BY THE CHECKBOXE "CheckBoxCCTypeNorm" IN THE CORRELOGRAMS SECTION OF THE MAIN WINDOW
FUNCTION ProcToggleCCNorm(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CCTypeRaw, CCTypeShuff, CCTypeNorm
	CCTypeNorm = 1 - CCTypeNorm
	if (CCTypeNorm == 1)
		CCTypeRaw = 0
		CCTypeShuff = 0
	endif
	Execute "CheckBox CheckBoxCCTypeRaw value=CCTypeRaw"
	Execute "CheckBox CheckBoxCCTypeShuff value=CCTypeShuff"
	Execute "CheckBox CheckBoxCCTypeNorm value=CCTypeNorm"
END

// FUNCTION CALLED BY THE CHECKBOX "USE TAGS" IN THE SECTION "Common plots" OF THE MAIN WINDOW
FUNCTION ProcCheckBoxUseTags(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar UseTags
	UseTags = checked
END

// FUNCTION CALLED BY THE BUTTON "Plot" IN THE SECTION "Common plots" OF THE MAIN WINDOW
FUNCTION ProcButtonPlot (ctrlName) : ButtonControl
	String ctrlName
	Nvar FvsTBin, AvsTBin, AmpHistoBin
	Wave Params
	Params[10] = FvsTBin
	Params[11] = AvsTBin
	Params[12] = AmpHistoBin
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)	// If MainWindow#Channel1 is on top
		PlotFAvsT_ch1()
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)	// If MainWindow#Channel2 is on top
		PlotFAvsT_ch2()
	endif
END

// FUNCTION CALLED BY THE ABOVE FUNCTION (ProcButtonPlot)
FUNCTION PlotFAvsT_ch1()
	Wave Occurrence1, realamp1, Tags1
	Nvar Threshold1, UseTags
	if (UseTags == 0)
		FAvsT(Occurrence1,Realamp1,Threshold1)
	endif
	if (UseTags == 1)
		if (sum(Tags1,0,numpnts(Tags1)-1) == 0)
			Abort "No tagged events. Uncheck the box \"Use tags\" or tag events."
		endif
		Duplicate/o Occurrence1, Occurrence1_BIS
		Duplicate/o Realamp1, Realamp1_BIS
		Occurrence1_BIS = Occurrence1_BIS * Tags1
		Realamp1_BIS = Realamp1_BIS * Tags1
		FAvsT(Occurrence1_BIS,Realamp1_BIS,Threshold1)
		KillWaves Occurrence1_BIS,Realamp1_BIS
	endif
END

// FUNCTION CALLED BY THE ABOVE FUNCTION (ProcButtonPlot)
FUNCTION PlotFAvsT_ch2()
	Nvar UseTags
	Wave Occurrence2, realamp2, Tags2
	Nvar Threshold2
	if (UseTags == 0)
		FAvsT(Occurrence2,Realamp2,Threshold2)
	endif
	if (UseTags == 1)
		Duplicate/o Occurrence2, Occurrence2_BIS
		Duplicate/o Realamp2, Realamp2_BIS
		Occurrence2_BIS = Occurrence2_BIS * Tags2
		Realamp2_BIS = Realamp2_BIS * Tags2
		FAvsT(Occurrence2_BIS,Realamp2_BIS,Threshold2)
		KillWaves Occurrence2_BIS,Realamp2_BIS
	endif
END

// FUNCTION THAT DISPLAYS THE BEGINNING OF THE TWO WAVES IN MAIN WINDOW
FUNCTION ButtonStart(ctrlName) : ButtonControl
	String ctrlName
	Nvar LoadedChannels
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = V_max-V_min; SetAxis/W=MainWindow#Channel1 bottom 0, Mi
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel2 bottom 0, Mi
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = V_max-V_min; SetAxis/W=MainWindow#Channel2 bottom 0, Mi
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel1 bottom 0, Mi
		endif
	endif
END

// FUNCTION THAT DISPLAYS THE END OF THE TWO WAVES IN MAIN WINDOW
FUNCTION ButtonEnd(ctrlName) : ButtonControl
	String ctrlName
	Nvar Length, LoadedChannels
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = V_max-V_min; SetAxis/W=MainWindow#Channel1 bottom Length - Mi, Length
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel2 bottom Length - Mi, Length
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = V_max-V_min; SetAxis/W=MainWindow#Channel2 bottom Length - Mi, Length
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel1 bottom Length - Mi, Length
		endif
	endif
END

// SCROLLING BACKWARD IN MAIN WINDOW
FUNCTION ButtonLeft(ctrlName) : ButtonControl
	String ctrlName
	Nvar LoadedChannels
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel1 bottom V_min-Mi,V_max-Mi
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel2 bottom V_min-Mi,V_max-Mi
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel2 bottom V_min-Mi,V_max-Mi
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel1 bottom V_min-Mi,V_max-Mi
		endif
	endif
END

// SCROLLING FORWARD IN MAIN WINDOW
FUNCTION ButtonRight(ctrlName) : ButtonControl
	String ctrlName
	Nvar LoadedChannels
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel1 bottom V_min+Mi,V_max+Mi
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel2 bottom V_min+Mi, V_max+Mi
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel2 bottom V_min+Mi,V_max+Mi
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel1 bottom V_min+Mi, V_max+Mi
		endif
	endif
END

// FUNCTION THAT EXPANDS THE TWO TRACES IN MAIN WINDOW
FUNCTION ButtonHexpand(ctrlName) : ButtonControl
	String ctrlName
	Nvar LoadedChannels
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = (V_max-V_min)/5; SetAxis/W=MainWindow#Channel1 bottom V_min,V_max-3*Mi
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel2 bottom V_min,V_max-3*Mi
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = (V_max-V_min)/5; SetAxis/W=MainWindow#Channel2 bottom V_min,V_max-3*Mi
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel1 bottom V_min,V_max-3*Mi
		endif
	endif
END

// FUNCTION THAT SHRINKS THE TWO TRACES IN MAIN WINDOW
FUNCTION ButtonHshrink(ctrlName) : ButtonControl
	String ctrlName
	Nvar LoadedChannels
	Variable Mi
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=MainWindow#Channel1 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel1 bottom V_min,V_max+3*Mi
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel2 bottom V_min,V_max+3*Mi
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=MainWindow#Channel2 bottom
		Mi = (V_max-V_min)/2; SetAxis/W=MainWindow#Channel2 bottom V_min,V_max+3*Mi
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel1 bottom V_min,V_max+3*Mi
		endif
	endif
END

// FULL SCALE IN MAIN WINDOW
FUNCTION ButtonFullScale(ctrlName) : ButtonControl
	String ctrlName
	Nvar LoadedChannels
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		SetAxis/W=MainWindow#Channel1/A
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel2/A
		endif
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		SetAxis/W=MainWindow#Channel2/A
		if (LoadedChannels == 2)
			SetAxis/W=MainWindow#Channel1/A
		endif
	endif
END

// LOAD AND DISPLAY TRACE 1 (CHANNEL 1) IN MAIN WINDOW
FUNCTION ButtonLoadWave1 (ctrlName) : ButtonControl
	String ctrlName
	Nvar RefNum ; open/Z=2/R/T="IGBW"/M="Choose first wave" RefNum
	// Flag /Z removes error message if user presses cancel in the dialog.
	// Can not be used with /D flag
	if (V_flag == -1)
		Abort
	endif
	LoadWave/o S_Filename ; String/G FileName1 = S_WaveNames[0,strlen(S_WaveNames)-2]
	Nvar FDC1, SI1
	SI1 = DeltaX($FileName1)*1e6
	SetScale/P x 0,DeltaX($FileName1),"s",$FileName1	// This is important to reset the beginning of the wave to zero in case it is not
	//Execute "SetVariable SetVarSI1 value=SI1"
	Nvar BSLpnts1, BSLms1, SI1; BSLms1 = (BSLpnts1-1)*SI1*1e-3
	Nvar RTpnts1, RTms1, SI1; RTms1 = (RTpnts1+1)*SI1*1e-3
	Nvar Ppnts1, Pms1, SI1; Pms1 = (Ppnts1-1)*SI1*1e-3
	if (FDC1 == 1)
		KillWindow MainWindow#Channel1; Display/HOST=MainWindow/W=(0.27,0,1,0.5)/N=Channel1 $FileName1; ModifyGraph rgb=(0,12800,52224)
		ModifyGraph margin(left)=0; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28; ModifyGraph StandOff=0
	else
		FDC1 = 1 ; Display/HOST=MainWindow/W=(0.27,0,1,0.5)/N=Channel1 $FileName1; ModifyGraph rgb=(0,12800,52224)
		ModifyGraph margin(left)=0; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28; ModifyGraph StandOff=0
	endif
	Variable/G Length1; Length1 = (numpnts($FileName1)-1)*DeltaX($FileName1)
	Nvar Length; Length = Length1
	Nvar LoadedChannels
	if (LoadedChannels == 0 || LoadedChannels == 1)
		LoadedChannels += 1
	endif
	Button DW1 Disable=0, win=MainWindow#MainPanel
	Button FileStart Disable=0, win=MainWindow#MainPanel
	Button Leftward Disable=0, win=MainWindow#MainPanel
	Button Rightward Disable=0, win=MainWindow#MainPanel
	Button FileEnd Disable=0, win=MainWindow#MainPanel
	Button FullScale Disable=0, win=MainWindow#MainPanel
	Button Hexpand Disable=0, win=MainWindow#MainPanel
	Button Hshrink Disable=0, win=MainWindow#MainPanel
	Button FlipWave Disable=0, win=MainWindow#MainPanel
	Button GridsMW Disable=0, win=MainWindow#MainPanel
	if (LoadedChannels == 2)
		Button ButtonSync Disable=0, win=MainWindow#MainPanel
	endif
	Nvar C1E; C1E = 1	// Channel 1 exists
	// Charly this aims at solving a known bug of the MainWindow tabs that are set to the value of the last loaded channel
	Nvar Ch2Detected
	If (Ch2Detected != 1) // If channel 2 has not already been detected
		MainWindowTabProc("", 0) // Set the tab to channel 1 at the end of this function i.e. when you load channel 1
	Endif
	// End
END

// LOAD AND DISPLAY TRACE 2 (CHANNEL 2) IN MAIN WINDOW
FUNCTION ButtonLoadWave2 (ctrlName) : ButtonControl
	String ctrlName
	Nvar RefNum ; open/Z=2/R/T="IGBW"/M="Choose first wave" RefNum
	// Flag /Z removes error message if user presses cancel in the dialog.
	// Can not be used with /D flag
	if (V_flag == -1)
		Abort
	endif
	LoadWave/o S_Filename ; String/G FileName2 = S_WaveNames[0,strlen(S_WaveNames)-2]
	Nvar FDC2, SI2
	SI2 = DeltaX($FileName2)*1e6
	SetScale/P x 0,DeltaX($FileName2),"s",$FileName2	// This is important to reset the beginning of the wave to zero in case it is not
	//Execute "SetVariable SetVarSI2 value=SI2"
	Nvar BSLpnts2, BSLms2, SI2; BSLms2 = (BSLpnts2-1)*SI2*1e-3
	Nvar RTpnts2, RTms2, SI2; RTms2 = (RTpnts2+1)*SI2*1e-3
	Nvar Ppnts2, Pms2, SI2; Pms2 = (Ppnts2-1)*SI2*1e-3
	if (FDC2 == 1)
		KillWindow MainWindow#Channel2 ; Display/HOST=MainWindow/W=(0.27,0.5,1,1)/N=Channel2 $FileName2
		ModifyGraph margin(left)=0; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28; ModifyGraph StandOff=0
	else
		FDC2 = 1 ; Display/HOST=MainWindow/W=(0.27,0.5,1,1)/N=Channel2 $FileName2
		ModifyGraph margin(left)=0; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28; ModifyGraph StandOff=0
	endif
	Variable/G Length2; Length2 = (numpnts($FileName2)-1)*DeltaX($FileName2)
	Nvar Length; Length = Length2
	Nvar LoadedChannels
	if (LoadedChannels == 0 || LoadedChannels == 1)
		LoadedChannels += 1
	endif
	Button DW2 Disable=0, win=MainWindow#MainPanel
	Button FileStart Disable=0, win=MainWindow#MainPanel
	Button Leftward Disable=0, win=MainWindow#MainPanel
	Button Rightward Disable=0, win=MainWindow#MainPanel
	Button FileEnd Disable=0, win=MainWindow#MainPanel
	Button FullScale Disable=0, win=MainWindow#MainPanel
	Button Hexpand Disable=0, win=MainWindow#MainPanel
	Button Hshrink Disable=0, win=MainWindow#MainPanel
	Button FlipWave Disable=0, win=MainWindow#MainPanel
	Button GridsMW Disable=0, win=MainWindow#MainPanel
	if (LoadedChannels == 2)
		Button ButtonSync Disable=0, win=MainWindow#MainPanel
	endif
	Nvar C2E; C2E = 1	// Channel 2 exists
	// Charly this aims at solving a known bug of the MainWindow tabs that are set to the value of the last loaded channel
	Nvar Ch2Detected
	If (Ch2Detected != 1) // If channel 2 has not already been detected
		MainWindowTabProc("", 0) // Set the tab to channel 1 at the end of this function i.e. when you load channel 2 also
	Endif
	// End
END

// FUNCTION CALLED BY BUTTON "DoCorrelogram" IN MAIN WINDOW
FUNCTION ButtonDoCorrelogram (ctrlName) : ButtonControl
	String ctrlName
	//Nvar CorrCh1, CorrCh2, CorrMethod
	Nvar RefCh1BoxVar, RefCh2BoxVar, TestCh1BoxVar, TestCh2BoxVar
	Nvar RefUsePeak, RefUseHR, TestUsePeak, TestUseHR
	Nvar RealPeakPosition1, RealPeakPosition2, Occurrence1, Occurrence2, Tags1, Tags2, Length1, Length2
	Nvar CorrTimeWindow, CorrBin
	Wave Params ; Params[8] = CorrTimeWindow ; Params[9] = CorrBin
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	// First case: ref = Ch1 and test = Ch1
	if (RefCh1BoxVar == 1 && RefCh2BoxVar == 0 && TestCh1BoxVar == 1 && TestCh2BoxVar == 0)
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(RealPeakPosition1, Tags1, RealPeakPosition1, Tags1, Length1, Length1)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(Occurrence1, Tags1, Occurrence1, Tags1, Length1, Length1)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(Occurrence1, Tags1, RealPeakPosition1, Tags1, Length1, Length1)
		endif
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(RealPeakPosition1, Tags1, Occurrence1, Tags1, Length1, Length1)
		endif
	endif
	// Second case: ref = Ch2 and test = Ch2
	if (RefCh1BoxVar == 0 && RefCh2BoxVar == 1 && TestCh1BoxVar == 0 && TestCh2BoxVar == 1)
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(RealPeakPosition2, Tags2, RealPeakPosition2, Tags2, Length2, Length2)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(Occurrence2, Tags2, Occurrence2, Tags2, Length2, Length2)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(Occurrence2, Tags2, RealPeakPosition2, Tags2, Length2, Length2)
		endif
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(RealPeakPosition2, Tags2, Occurrence2, Tags2, Length2, Length2)
		endif
	endif
	// Third case: ref = Ch2 and test = Ch1
	if (RefCh1BoxVar == 0 && RefCh2BoxVar == 1 && TestCh1BoxVar == 1 && TestCh2BoxVar == 0)
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(RealPeakPosition2, Tags2, RealPeakPosition1, Tags1, Length2, Length1)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(Occurrence2, Tags2, Occurrence1, Tags1, Length2, Length1)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(Occurrence2, Tags2, RealPeakPosition1, Tags1, Length2, Length1)
		endif
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(RealPeakPosition2, Tags2, Occurrence1, Tags1, Length2, Length1)
		endif
	endif
	// Fourth case: ref = Ch1 and test = Ch2
	if (RefCh1BoxVar == 1 && RefCh2BoxVar == 0 && TestCh1BoxVar == 0 && TestCh2BoxVar == 1)
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(RealPeakPosition1, Tags1, RealPeakPosition2, Tags2, Length1, Length2)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(Occurrence1, Tags1, Occurrence2, Tags2, Length1, Length2)
		endif
		if (RefUsePeak == 0 && RefUseHR == 1 && TestUsePeak == 1 && TestUseHR == 0)
			Correlogram(Occurrence1, Tags1, RealPeakPosition2, Tags2, Length1, Length2)
		endif
		if (RefUsePeak == 1 && RefUseHR == 0 && TestUsePeak == 0 && TestUseHR == 1)
			Correlogram(RealPeakPosition1, Tags1, Occurrence2, Tags2, Length1, Length2)
		endif
	endif
END

// The value of the baseline is set by the user in points; this procedure calculates the corresponding value in ms
// This function is triggered when the user enters a value for the variable BSLpnts1(2)
FUNCTION SetBSLms (ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName ; Variable varNum ; String varStr ; String varName
	Nvar NumChannels
	if (NumChannels == 2)
		Nvar BSLpnts1, BSLms1, SI1; BSLms1 = (BSLpnts1-1)*SI1*1e-3
		Nvar BSLpnts2, BSLms2, SI2; BSLms2 = (BSLpnts2-1)*SI2*1e-3
	endif
END

// The value of the rise time is set by the user in points; this procedure calculates the corresponding value in ms
// This function is triggered when the user enters a value for the variable RTpnts1(2)
FUNCTION SetRTms (ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName ; Variable varNum ; String varStr ; String varName
	Nvar NumChannels
	if (NumChannels == 2)
		Nvar RTpnts1, RTms1, SI1; RTms1 = (RTpnts1+1)*SI1*1e-3
		Nvar RTpnts2, RTms2, SI2; RTms2 = (RTpnts2+1)*SI2*1e-3
	endif
END

// The value of the peak is set by the user in points; this procedure calculates the corresponding value in ms
// This function is triggered when the user enters a value for the variable Ppnts1(2)
FUNCTION SetPms (ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName ; Variable varNum ; String varStr ; String varName
	Nvar NumChannels
	if (NumChannels == 2)
		Nvar Ppnts1, Pms1, SI1; Pms1 = (Ppnts1-1)*SI1*1e-3
		Nvar Ppnts2, Pms2, SI2; Pms2 = (Ppnts2-1)*SI2*1e-3
	endif
END

// FUNCTION TRIGGERED BY BUTTON "ButtonDW1" (DIFFERENTIATE WAVE 1)
FUNCTION ButtonDW1 (ctrlName) : ButtonControl
	String ctrlName
	Nvar BSLpnts1, RTpnts1, Ppnts1, Threshold1, SI1
	Svar FileName1
	// Before launching the function "Build", writes the detection parameters on the disc in a wave called "Params1"
	Wave Params ; Params[0] = BSLpnts1 ; Params[1] = RTpnts1 ; Params[2] = Ppnts1
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	BUILD(BSLpnts1, RTpnts1, Ppnts1, $FileName1, SI1, FileName1)
	Duplicate/o DIFF, DIFF1; KillWaves DIFF, BASELINE; Duplicate/o PEAK, PEAK1; KillWaves PEAK
	Variable/G Diff1Clicked=1 // Tells that the diff has been performed at least once
	Button Detect1 Disable=0, win=MainWindow#MainPanel
END

// FUNCTION TRIGGERED BY BUTTON "ButtonDW2" (DIFFERENTIATE WAVE 2)
FUNCTION ButtonDW2 (ctrlName) : ButtonControl
	String ctrlName
	Nvar BSLpnts2, RTpnts2, Ppnts2, Threshold2, SI2
	Svar FileName2
	// Before launching the function "Build", writes the detection parameters on the disc in a wave called "Params2"
	Wave Params ; Params[4] = BSLpnts2 ; Params[5] = RTpnts2 ; Params[6] = Ppnts2
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	BUILD(BSLpnts2, RTpnts2, Ppnts2, $FileName2, SI2, FileName2)
	Duplicate/o DIFF, DIFF2; KillWaves DIFF, BASELINE; Duplicate/o PEAK, PEAK2; KillWaves PEAK
	Variable/G Diff2Clicked=1 // Tells that the diff has been performed at least once
	Button Detect2 Disable=0, win=MainWindow#MainPanel
END

// FUNCTION TRIGGERED BY BUTTON "ButtonD1" (DETECT 1)
FUNCTION ButtonD1(ctrlName) : ButtonControl
	String ctrlName
	Nvar BSLms1, RTms1, Pms1, BSLpnts1, RTpnts1, Ppnts1, Threshold1, SI1
	Nvar Ch1Detected, Ch2Detected, Threshold1; Ch1Detected = 1
	Nvar NumberofEventsch1, FreqEvents1, Length, TWPD1
	Svar FileName1
	Wave DIFF1, PEAK1, Params
	Variable Channel ; Channel = 1
	Params[3] = Threshold1
	Params[13] = TWPD1
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	DETECT($FileName1, DIFF1, PEAK1, BSLms1, RTms1, Pms1, BSLpnts1, RTpnts1, Ppnts1, Threshold1, Channel, SI1,TWPD1)
	NumberofEventsch1 = numpnts(DetectedLevels); //Execute "ValDisplay NoEch1 value=root:spacan:NumberofEventsch1"
	FreqEvents1 = NumberofEventsch1 / Length; //Execute "ValDisplay FreqEch1 value=root:spacan:FreqEvents1"
	Duplicate/o DetectedLevels, DetectedLevels1; 		KillWaves DetectedLevels
	Duplicate/o PeakY, PeakY1;								KillWaves PeakY
	Duplicate/o PeakX, PeakX1;								KillWaves PeakX
	Duplicate/o BaselineY, BaselineY1;						KillWaves BaselineY
	Duplicate/o BaselineX, BaselineX1;						KillWaves BaselineX
	Duplicate/o Occurrence, Occurrence1;					KillWaves Occurrence
	Duplicate/o RealAmp, RealAmp1;						KillWaves RealAmp
	Duplicate/o RealPeakPosition, RealPeakPosition1;	KillWaves RealPeakPosition
	Duplicate/o RiseTime2080, RiseTime20801;			KillWaves RiseTime2080
	Duplicate/o RT20, RT201;								KillWaves RT20
	Duplicate/o RT80, RT801;								KillWaves RT80
	// Jan 10, 2010 Guillaume
	Duplicate/o EventFootTime EventFootTime1;			KillWaves EventFootTime
	// End of Jan 10, 2010 Guillaume
	Variable/G EventNumber1 ; EventNumber1 = 0
	K16 = 20 ; K17 = 40 ; Variable/G RT20801, AMP1, EventOccurrence1, EventOccurrenceMin1, EventOccurrenceSec1
	Variable/G KbdSpeed1 = 3
	SHOW1(1,0)
END

// FUNCTION TRIGGERED BY BUTTON "ButtonD2" (DETECT 2)
FUNCTION ButtonD2(ctrlName) : ButtonControl
	String ctrlName
	Nvar BSLms2, RTms2, Pms2, BSLpnts2, RTpnts2, Ppnts2, Threshold2, SI2
	Svar FileName2
	Wave DIFF2, PEAK2, Params
	Variable Channel ; Channel = 1
	Nvar Ch1Detected, Ch2Detected, Threshold2; Ch2Detected = 1
	Nvar NumberofEventsch2, FreqEvents2, Length, TWPD2
	Params[7] = Threshold2
	Params[14] = TWPD2
	Save/o/P=PathToParams Params as "SpAcAnParams.ibw"
	DETECT($FileName2, DIFF2, PEAK2, BSLms2, RTms2, Pms2, BSLpnts2, RTpnts2, Ppnts2, Threshold2, Channel, SI2, TWPD2)
	NumberofEventsch2 = numpnts(DetectedLevels); //Execute "ValDisplay NoEch2 value=root:spacan:NumberofEventsch2"
	FreqEvents2 = NumberofEventsch2 / Length; //Execute "ValDisplay FreqEch2 value=root:spacan:FreqEvents2"
	Duplicate/o DetectedLevels, DetectedLevels2; 		KillWaves DetectedLevels
	Duplicate/o PeakY, PeakY2;								KillWaves PeakY
	Duplicate/o PeakX, PeakX2;								KillWaves PeakX
	Duplicate/o BaselineY, BaselineY2;						KillWaves BaselineY
	Duplicate/o BaselineX, BaselineX2;						KillWaves BaselineX
	Duplicate/o Occurrence, Occurrence2;					KillWaves Occurrence
	Duplicate/o RealAmp, RealAmp2;						KillWaves RealAmp
	Duplicate/o RealPeakPosition, RealPeakPosition2;	KillWaves RealPeakPosition
	Duplicate/o RiseTime2080, RiseTime20802;			KillWaves RiseTime2080
	Duplicate/o RT20, RT202;								KillWaves RT20
	Duplicate/o RT80, RT802;								KillWaves RT80
	// Jan 10, 2010 Guillaume
	Duplicate/o EventFootTime EventFootTime2;			KillWaves EventFootTime
	// End of Jan 10, 2010 Guillaume
	Variable/G EventNumber2 ; EventNumber2 = 0
	K18 = 20 ; K19 = 40 ; Variable/G RT20802, AMP2, EventOccurrence2, EventOccurrenceMin2, EventOccurrenceSec2
	Variable/G KbdSpeed2 = 3
	SHOW2(1,0)
END

// FUNCTION THAT CALCULATES THE TIME DERIVATIVE OF THE ORIGINAL TRACE
FUNCTION BUILD (Baseline_pnts, RiseTime_pnts, Peak_pnts, DataWave, SampleInterval, CurrentFileName)
	Variable Baseline_pnts, RiseTime_pnts, Peak_pnts, Sampleinterval
	String CurrentFileName
	Wave DataWave
	// Baseline_pnts, RiseTime_pnts & Peak_pnts must be in points
	// Baseline_pnts & Peak_pnts must be odd numbers
	// The function BUILD uses the user's parameters to build a differentiate wave from the original wave (DataWave).
	// The use of the right number of points accounting for the rise time is critical.	
	Wave DIFF ; KillWaves DIFF
	// ---------- FIRST STEP: DEFINE NUMBER OF POINTS FOR BASELINE, RISE TIME & PEAK ----------
	Variable Baseline_ms ; Baseline_ms = (Baseline_pnts-1)*SampleInterval*1e-3
	Variable RiseTime_ms ; RiseTime_ms = (RiseTime_pnts+1)*SampleInterval*1e-3
	Variable Peak_ms ; Peak_ms = (Peak_pnts-1)*SampleInterval*1e-3
	// --------- SECOND STEP: BUILD THE WAVE THAT WILL BE USED FOR DETECTION ----------
	Duplicate/o DataWave BASELINE
	Smooth/B Baseline_pnts, BASELINE
	Duplicate/o DataWave PEAK
	Smooth/B Peak_pnts, PEAK
	Duplicate/o BASELINE DIFF
	DIFF = BASELINE[p] - PEAK[p+floor(Baseline_pnts/2)+RiseTime_pnts+floor(Peak_pnts/2)+1]
END	

// FUNCTION THAT USES THE TIME DERIVATIVE OF THE SIGNAL TO PERFORM THE THRESHOLD DETECTION
FUNCTION DETECT(DataWave, DifferentiateWave, WavePeak, Baseline_ms, RiseTime_ms, Peak_ms, Baseline_pnts, RiseTime_pnts, Peak_pnts, Thrshld, ChannelNumber, SampleInterval, TWPD)
	Wave DataWave, DifferentiateWave, WavePeak
	Variable Baseline_ms, RiseTime_ms, Peak_ms, Baseline_pnts, RiseTime_pnts, Peak_pnts, Thrshld, ChannelNumber, SampleInterval, TWPD
	String CurrentFileName
	// --------- FIRST STEP: DETECT EVENTS IN DIFFERENTIATE WAVE ----------
	// Charly **************************Here I just use my own findLevels function to be able to load more heavy waves (slower but less memory use)
	My_find_levels(DifferentiateWave, "DetectedLevels", Thrshld)
	Wave DetectedLevels
	//***************************Original method*************
//	FindLevels/Q DifferentiateWave, Thrshld
//	Wave W_FindLevels ; Duplicate/o W_FindLevels DetectedLevels; KillWaves W_FindLevels
	// Charly End of my changes*******************************************
	
	//Nvar NumberOfEvents ; NumberOfEvents = numpnts(DetectedLevels)
	Variable i
	For ( i = (NumPnts(DetectedLevels)-1) ; i >= 1 ; i-=1 )
		if ( DifferentiateWave[ floor(DetectedLevels[i] / (SampleInterval*1e-6)) + 1 ] - DifferentiateWave[ floor(DetectedLevels[i] / (SampleInterval*1e-6)) ] < 0 || DetectedLevels[i] - DetectedLevels[i-1] <= .001)
			DeletePoints i, 1, DetectedLevels
		endif
	endfor
	if ( DifferentiateWave[ floor(DetectedLevels[0] / (SampleInterval*1e-6)) + 1 ] - DifferentiateWave[ floor(DetectedLevels[0] / (SampleInterval*1e-6)) ] < 0 )
		DeletePoints 0, 1, DetectedLevels
	endif
	// ---------- SECOND STEP: SAVE AMPLITUDE OF DETECTED EVENTS ----------
	
	// Charly********************************Here I use duplicate to avoid putting values in the loop
//	Make/o/N=(numpnts(DetectedLevels)) BaselineX, BaselineY, PeakX, PeakY
	Make/o/N=(numpnts(DetectedLevels)) BaselineY, PeakX, PeakY
	Duplicate/O DetectedLevels, BaselineX
	// Charly End of my changes*******************************************
	// Make/o/N=(numpnts(DetectedLevels)) Amplitudes
	For ( i = 0; i < numpnts(DetectedLevels); i += 1)
		// Two ways of choosing the baseline: either we trust the detection settings, especially the rise time and so we calculate the baseline corresponding
		// to the local maximum of the differentiate wave (DIFF), as follow:
		// Wavestats/Q/R=(W_FindLevels[i], W_FindLevels[i] + .006) DifferentiateWave
		//Amplitudes[i] = -V_max
		//BaselineX[i] = V_maxLoc
		// Second way: we consider that the baseline corresponding to the point crossing the threshold will be correct, and so:
		// Charly**************************** Now useless
//		BaselineX[i] = DetectedLevels[i]
		// Charly End of my changes*******************************************
		//Wave DataWave
		BaselineY[i] = mean(DataWave, BaselineX[i] - Baseline_ms*.001/2, BaselineX[i] + Baseline_ms*.001/2)
		//PeakX[i] = BaselineX[i] + Baseline_ms*.001/2 + RiseTime_ms*.001 + Peak_ms*.001/2
		//PeakY[i] = Amplitudes[i] + BaselineY[i]
	Endfor
	// ---------- THIRD STEP: CALCULATING THE TIME OF OCCURRENCE OF DETECTED EVENTS ----------
	
	// Charly *******************************Here I use duplicate to be sure wave have same number of points and are double precision (usefull for long recordings)
	Duplicate/O DetectedLevels, Occurrence, RealPeakPosition, RiseTime2080, RT20, RT80
//	Make/o/N=(numpnts(DetectedLevels)) Occurrence
	//****************This one normaly doesn't need to be double precision	
	Make/o/N=(numpnts(DetectedLevels)) RealAmp
//	Make/o/N=(numpnts(DetectedLevels)) RealPeakPosition
//	Make/o/N=(numpnts(DetectedLevels)) RiseTime2080
//	Make/o/N=(numpnts(DetectedLevels)) RT20
//	Make/o/N=(numpnts(DetectedLevels)) RT80
	// Charly End of my changes*******************************************

	// Guillaume Jan 10, 2010 - adding a wave to store a different time of occurrence:
	// the intersection between the baseline and the 20-80% rise time line (the "foot" of the event).
	Duplicate/o DetectedLevels EventFootTime
	// End of Guillaume Jan 10, 2010

	For (i = 0; i < numpnts(DetectedLevels); i += 1)
		// Measure the value of the peak and the peak location
		Wavestats/Q/R=(BaselineX[i], BaselineX[i] + Baseline_ms*.001/2 + RiseTime_ms*.001 + Peak_ms*.001/2 + TWPD*0.001) WavePeak
		RealAmp[i] = V_min - BaselineY[i]
		RealPeakPosition[i] = V_minloc
		// Measure the position of the half-rise
		FindLevel/Q/R=(BaselineX[i], RealPeakPosition[i]) WavePeak (BaselineY[i] + RealAmp[i]/2)
		Occurrence[i] = V_LevelX
		// Measure the position of the 80% rise time
		FindLevel/Q/R=(Occurrence[i], Occurrence[i] + (RiseTime_pnts+floor(Peak_pnts/2)+20)*SampleInterval*1e-6) DataWave (BaselineY[i] + RealAmp[i]*0.8)
		RT80[i] = V_LevelX
		// Measure the position of the 20% rise time
		Duplicate/o/R=(Occurrence[i]-.01, Occurrence[i]) DataWave TEMPWAVE
		TEMPWAVE = TEMPWAVE[numpnts(TEMPWAVE)-1-p]
		SetScale/P x 0,SampleInterval*1e-6, "s", TEMPWAVE
		FindLevel/Q TEMPWAVE (BaselineY[i] + RealAmp[i]*0.2)
		RT20[i] = Occurrence[i] - V_LevelX
		// 20-80% rise time
		RiseTime2080[i] = RT80[i] - RT20[i]
		// Guillaume Jan 10, 2010
		EventFootTime[i] = RT20[i] - (1/3)*(RT80[i] - RT20[i])
		// end of Guillaume Jan 10, 2010
	Endfor
	KillWaves TEMPWAVE
	// ---------- END OF THIRD STEP ----------
END	

Function Show1TabProc(CtrlName, TabNum) : TabControl
	String CtrlName
	Variable TabNum
	
	Variable isTab0= tabNum==0
	Variable isTab1= tabNum==1
	Variable isTab2= tabNum==2
	Variable isTab3= tabNum==3
	Variable isTab4=tabNum==4
	
	// Template if you want to add a new tab
	//Variable isTab5= tabNum==5
	
	// note: disable=0 means "show", disable=1 means "hide"
	
	
	// This line may be in use later 
	SetActiveSubWindow DetectedEvents1#DetectedEvents1Panel
	
	// *************************************For Average Tab***************************************
	ModifyControl CheckRiseTime1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetRTinf1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetRTsup1 disable= !isTab0	// hide if not Tab 0
	ModifyControl CheckBaselineSD1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBslnSDinf1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBslnSDsup1 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonShowRT2080histo1 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonShowBslnSDHisto1 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonAverage1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBaseline1 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetDecay1 disable= !isTab0	// hide if not Tab 0
	ModifyControl NumEventsAveraged1 disable= !isTab0	// hide if not Tab 0
	ModifyControl FiltersDisplayText1 disable= !isTab0
	ModifyControl ButtonSupEvents1 disable= !isTab0
	ModifyControl CheckBoxIncludeAv1 disable= !isTab0
	ModifyControl TitleBoxAlign1 disable= !isTab0
	ModifyControl CheckBoxAlign1Foot disable= !isTab0
	ModifyControl CheckBoxAlign1Occ disable= !isTab0
	ModifyControl CheckBoxAlign1Peak disable= !isTab0
	
	//****************************************for Kinetics Tab*************************************
	
	ModifyControl CFPE1 disable= !isTab1	// hide if not Tab 1
	ModifyControl CFAE1 disable= !isTab1	// hide if not Tab 1
	ModifyControl FreeOffset1 disable= !isTab1	// hide if not Tab 1
	ModifyControl FitSingleExp1 disable= !isTab1	// hide if not Tab 1
	ModifyControl FitdbExp1 disable= !isTab1	// hide if not Tab 1
	ModifyControl BFitsigdbexp1 disable= !isTab1	// hide if not Tab 1
	ModifyControl BFitsigmoid1 disable= !isTab1	// hide if not Tab 1
	ModifyControl BFitAllEventsdbExp1 disable= !isTab1	// hide if not Tab 1
	ModifyControl Start_Fit1 disable= !isTab1	// hide if not Tab 1
	ModifyControl End_Fit1 disable= !isTab1	// hide if not Tab 1
	ModifyControl TauFast1 disable= !isTab1	// hide if not Tab 1
	ModifyControl TauSlow1 disable= !isTab1	// hide if not Tab 1
	ModifyControl FastPercentage1 disable= !isTab1	// hide if not Tab 1
	ModifyControl SlowPercentage1 disable= !isTab1	// hide if not Tab 1
	ModifyControl ValOffset1 disable= !isTab1	// hide if not Tab 1
	ModifyControl KineticIndex1 disable= !isTab1	// hide if not Tab 1
	
/// Find out howto fix
	ModifyControl CheckBFitSelectedChan11 disable= !isTab1
	ModifyControl CheckBFitSelectedChan12 disable= !isTab1
	//ModifyControl FitDisplayText1 disable= !isTab1



	// *********************************For Pairs Tab ***************************************

/// Find out howto fix	
	ModifyControl ButtonSync1 disable= !isTab2		// hide if not Tab 2
	ModifyControl ButtonMAS1 disable= !isTab2	// hide if not Tab 2
	
	ModifyControl SetSyncWinStart1 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetSyncWinEnd1 disable= !isTab2		// hide if not Tab 2
	
	ModifyControl SetMASPeakWinWidth1 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASBslWidth1 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASAPRiseTime1 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASLagToPeak1 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASTrueHistbinSize1 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASNoiseHistBinSize1 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASHistMin1 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASHistMax1 disable= !isTab2		// hide if not Tab 2	

	//*************************************** TAB "TAGS" ***********************************
	ModifyControl SetIsolateEventsWinStart1 disable= !isTab3		// hide if not Tab 3
	ModifyControl SetIsolateEventsWinEnd1 disable= !isTab3
	ModifyControl ButtonisolateEvents1 disable= !isTab3
	ModifyControl ButtonLowPass1 disable= !isTab3
	ModifyControl SetLowPassThrshld1 disable= !isTab3
	ModifyControl ButtonTagRangeOfEvents1 disable= !isTab3
	ModifyControl ButtonUntagRangeOfEvents1 disable= !isTab3
	ModifyControl SetBegin1 disable= !isTab3
	ModifyControl SetEnd1	 disable= !isTab3
	ModifyControl ButtonTagRangeOfEvents1_pnts disable= !isTab3
	ModifyControl ButtonUntagRangeOfEvents1_pnts disable= !isTab3
	ModifyControl SetBegin1_pnts disable= !isTab3
	ModifyControl SetEnd1_pnts disable= !isTab3
	ModifyControl BtnSaveTags1 disable= !isTab3
	ModifyControl BtnDeleteNaN1 disable= !isTab3
	ModifyControl PopupOriginOfTags1 disable= !isTab3
	
	//******************************************TAB "Integrals"*******************************
	ModifyControl CheckBIntSelectedChan11 disable = !isTab4
	ModifyControl CheckBIntSelectedChan12 disable = !isTab4
	ModifyControl CheckBIntEvent1 disable = !isTab4
	ModifyControl CheckBIntAvg1 disable = !isTab4
	ModifyControl SetIntegralStart1 disable = !isTab4
	ModifyControl SetIntegralEnd1 disable = !isTab4
	ModifyControl SetIntAvgWidth1 disable = !isTab4
	ModifyControl ChkboxIntegralOffset1 disable = !isTab4
	ModifyControl Integrate1 disable = !isTab4
	ModifyControl DisplayArea1 disable = !isTab4 
//	
	// ***********************************Add your tab here***********************************
	
	//************************************************************************************
	Nvar CurrentTab1
	CurrentTab1 = TabNum
	return 0

End

// FUNCTION THAT BUILDS THE WINDOW SHOWING DETECTED EVENNTS (CHANNEL 1)
FUNCTION SHOW1(Number, Event_Number)
	Variable Number,Event_Number
	Nvar KbdSpeed1
	Wave DetectedLevels1, Tags1, Occurrence1, BaselineY1, BaselineX1, PeakY1, PeakX1, RealAmp1, RealPeakPosition1, RiseTime20801, RT201, RT801
	Wave EventFootTime1
	Nvar BSLms1, Pms1, SI1, SI2, Length, NumberofEventsch1
	Svar FileName1, FileName2
	// Number says if it is the first time that the function SHOW is used (Number = 1) or not (Number = 2)
	// ----------------- SAVING THE EVENT TO BE DISPLAYED IN A WAVE -----------------
	Duplicate/o/R=(Occurrence1[Event_Number] - K16*.001, Occurrence1[Event_Number] + K17*.001) $FileName1 TEMPEVENT1
	TEMPEVENT1 = TEMPEVENT1 - BaselineY1[Event_Number]
	SetScale/P x -K16*.001, (SI1*1e-6), "s", TEMPEVENT1
	// ----------------- CURSORS -----------------
	Make/o/N=2 YcsrA1, XcsrA1, YcsrB1, XcsrB1, YcsrC1, XcsrC1	// Line cursors
	Make/o/N=1 YcsrD1, XcsrD1, YcsrE1, XcsrE1, YcsrF1, XcsrF1	// Point cursors
	// Cursor A represents the baseline
	YcsrA1 = 0
	XcsrA1[0] = BaselineX1[Event_Number] - BSLms1*.001/2 - Occurrence1[Event_Number]
	XcsrA1[1] = BaselineX1[Event_Number] + BSLms1*.001/2 - Occurrence1[Event_Number]
	// Cursor C represents the real event amplitude
	YcsrC1 = RealAmp1[Event_Number]
	XcsrC1[0] = RealPeakPosition1[Event_Number] - Pms1*.001/2 - Occurrence1[Event_Number]
	XcsrC1[1] = RealPeakPosition1[Event_Number] + Pms1*.001/2 - Occurrence1[Event_Number]
	// Cursor D marks RT20%
	YcsrD1 = RealAmp1[Event_Number] * 0.2
	XcsrD1 = RT201[Event_Number] - Occurrence1[Event_Number]
	// Cursor E marks RT80%
	YcsrE1 = RealAmp1[Event_Number] * 0.8
	XcsrE1 = RT801[Event_Number] - Occurrence1[Event_Number]
	Nvar RT20801, AMP1, EventOccurrence1, EventOccurrenceMin1, EventOccurrenceSec1
	RT20801 = 1000*RiseTime20801[Event_Number]
	AMP1 = RealAmp1[Event_Number]
	// Cursor F marks the foot of the event
	YcsrF1 = 0
	XcsrF1 = EventFootTime1[Event_Number] - Occurrence1[Event_Number] 
	
	If (Number == 1) // IF FUNCTION RAN FOR THE FIRST TIME
		Make/o/N=(numpnts(DetectedLevels1)) Tags1 ; Tags1 = 0	// INITIALIZE TAGS
		DoWindow/K DetectedEvents1; Display/W=(150,0,845,450)/N=DetectedEvents1; DoWindow/C DetectedEvents1	// INITIALIZE WINDOW
		
		// BUILDING LEFT PANEL
		NewPanel/K=1/N=DetectedEvents1Panel/HOST=DetectedEvents1/FG=(FL, FT, gRight, FB )/W=(0,0,275,450) ; ModifyPanel cbRGB=(65500,65500,65500)
		
		// The following line creates the tab menu
		Variable/G CurrentTab1 = 0
		TabControl Show1Tab,pos={8,225},size={350,180}, fstyle=1,proc=Show1TabProc, win=DetectedEvents1#DetectedEvents1Panel, value=CurrentTab1
		// The following three lines create the different tabs of the tab menu
		TabControl Show1Tab,tabLabel(0)="Average",value= 0
		TabControl Show1Tab,tabLabel(1)="Kinetics",value= 0
		TabControl Show1Tab,tabLabel(2)="Pairs",value= 0
		TabControl Show1Tab,tabLabel(3)="Tags",value= 0
		TabControl Show1Tab,tabLabel(4)="Integrals",value=0
		
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1,fsize=16 ; DrawText 135,25,"Channel 1"
		
		// FIRST GROUP: controls
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,47,"Controls"
		Variable/G Ch1ControlsPosOffset = 45		
		Button close_window1 					pos={310,8},									size={40,20},fstyle=1,	proc=ButtonCloseDetectedEvents1,title="Close"
		SetVariable Set_Event_Number1 		pos={10,Ch1ControlsPosOffset+6},		size={120,20},			proc=SetEvent1,size={133,20}, title="Event #:",limits={0,(numpnts(DetectedLevels1)-1),1} ; Execute "SetVariable Set_Event_Number1 value= EventNumber1"
		Button Kbd1 								pos={137,Ch1ControlsPosOffset+5},	size={83,20},			proc=ButtonKbd1,title="use keyboard"
		SetVariable SetKbdSpeed1				pos={225,Ch1ControlsPosOffset+6},	size={85,20},						title="Speed:", value=KbdSpeed1, limits={1,inf,1}
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 0; DrawText 10,90,"Show:"
		SetVariable msBeforeEvent1 			pos={50,Ch1ControlsPosOffset+28},	size={80,20}, 			proc=SetEvent1, title="left:",limits={-Inf,Inf,1}, value= K16
		SetVariable msAfterEvent1 				pos={137,Ch1ControlsPosOffset+28},	size={113,20}, 			proc=SetEvent1, title="right (ms):",limits={-Inf,Inf,1}, value= K17
		CheckBox Grids1 							pos={305,Ch1ControlsPosOffset+29},	size={45,17},			proc=CheckBoxGrids1, title="Grid", value=0
		Variable /G ShowCh2 ; ShowCh2 = 0
		CheckBox CheckShowChannel2			pos={256,Ch1ControlsPosOffset+29},	size={54,20},			proc=ShowChannel2, title="Ch.2", value=0
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 0 ;  DrawText 10,111,"Browsing: "
		CheckBox BTDE1							pos={77,Ch1ControlsPosOffset+50}, 	size={100,20}, 		title="tagged", proc=ProcBTDE1
		CheckBox BUDE1							pos={155,Ch1ControlsPosOffset+50}, 	size={100,20}, 		title="untagged events only.", proc=ProcBUDE1
		SetActiveSubwindow DetectedEvents1#DetectedEvents1Panel //Needed for drawEnv after displaying tags
		Variable/G ToOHR_1, ToOPeak_1; ToOHR_1 = 1; ToOPeak_1 = 0
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 0 ;  DrawText 10,131,"Tag/untag:"
		Variable/G TaggedEvents1 ; TaggedEvents1 = 0 ; Variable/G CheckedTagAll1 ; CheckedTagAll1 = 0
		CheckBox CheckBoxTag1 				pos={77,Ch1ControlsPosOffset+71},					proc=ProcCheckBoxTag1,size={45,20}, title="this event", value=0
		ValDisplay DisplayTaggedEvents1 		pos={225,Ch1ControlsPosOffset+70},size={110,20},title="# tagged:"
		Execute "ValDisplay DisplayTaggedEvents1 value=" + num2str(TaggedEvents1)
		CheckBox CheckBoxTagAll1 			pos={155,Ch1ControlsPosOffset+71}, 						proc=ProcCheckBoxTagAll1, size={45,20}, title="all/none"
	
		Variable/G BTDE_1, BUDE_1; BTDE_1=0; BUDE_1=0
		// DISPLAY TAGS IN A GRAPH
		display/HOST=DetectedEvents1#DetectedEvents1Panel/W=(10,Ch1ControlsPosOffset+90,340,Ch1ControlsPosOffset+169) tags1
		ModifyGraph margin=0
		ModifyGraph gbRGB=(65500,65500,65500); ModifyGraph wbRGB=(65500,65500,65500)
		ModifyGraph standoff(left)=0;SetAxis left 0,1; ModifyGraph manTick(left)={0,1,0,0},manMinor(left)={0,0}; ModifyGraph margin=20; ModifyGraph margin(right)=0; ModifyGraph margin(top)=0; ModifyGraph margin(bottom)=28
		Make/o/N=1 CursorTags1_X, CursorTags1_Y; CursorTags1_X = 0; CursorTags1_Y = 0
		Appendtograph CursorTags1_Y vs CursorTags1_X
		ModifyGraph mode(CursorTags1_Y)=3,marker(CursorTags1_Y)=10;ModifyGraph rgb(CursorTags1_Y)=(0,0,0)
		ModifyGraph msize(CursorTags1_Y)=5,mrkThick(CursorTags1_Y)=5
		
		// USE THIS VARIABLE TO MOVE BLOCKS OF CONTROLS
		Variable/G Ch1TabPosOffset=-125
		
		// THIRD GROUP: Average
		// Here we have to use a title box instead of simple text to be able to hide it with the tab procedure.
		TitleBox FiltersDisplayText1  frame=0,	pos={15,Ch1TabPosOffset+380}, size={15, 20},Title="Filters (enter range):"
		Variable /G CheckRT20801 ; CheckRT20801 = 0 ; Variable /G RTinf1 ; RTinf1 = 0 ; Variable /G RTsup1 ; RTsup1 = 1
		CheckBox CheckRiseTime1 				pos={15,Ch1TabPosOffset+400},	size={100,20},	title="20-80% rise time (ms)", value=0, 	proc=ProcCheckRiseTime1
		SetVariable SetRTinf1 					pos={160,Ch1TabPosOffset+400},	size={60,20},	title=" ",limits={0,inf,0.1}
		Execute "SetVariable SetRTinf1 value=RTinf1"
		SetVariable SetRTsup1 					pos={225,Ch1TabPosOffset+400},	size={70,20},	title="-",limits={0,inf,0.1}
		Execute "SetVariable SetRTsup1 value=RTsup1"
		Variable /G CheckBslnSD1 ; CheckBslnSD1 = 0 ; Variable /G BslnSDinf1 ; BslnSDinf1 = 0 ; Variable /G BslnSDsup1 ; BslnSDsup1 = 3
		CheckBox CheckBaselineSD1	 		pos={15,Ch1TabPosOffset+420},	size={100,20},	title="Baseline SD (xx)", value=0, 		proc=ProcCheckBaselineSD1
		SetVariable SetBslnSDinf1 				pos={160,Ch1TabPosOffset+420},	size={60,20},	title=" ",limits={0,inf,0.1}
		Execute "SetVariable SetBslnSDinf1 value=BslnSDinf1"
		SetVariable SetBslnSDsup1 				pos={225,Ch1TabPosOffset+420},	size={70,20},	title="-",limits={0,inf,0.1}
		Execute "SetVariable SetBslnSDsup1 value=BslnSDsup1"
		Button ButtonShowRT2080histo1 		pos={300,Ch1TabPosOffset+400},	size={52,20},	title="Histo", 									proc=ProcButtonShowRT2080histo1
		Button ButtonShowBslnSDHisto1 		pos={300,Ch1TabPosOffset+420},	size={52,20},	title="Histo", 									proc=ProcButtonShowBslnSDHisto1
		Button ButtonAverage1 					pos={15,Ch1TabPosOffset+440},																		proc=ProcButtonAverage1, size={52,20},title="Average"
		Variable /G Baseline1 ; Baseline1 = 20 ; Variable /G Decay1 ; Decay1 = 40
		SetVariable SetBaseline1 				pos={75,Ch1TabPosOffset+440},	size={75,20},	title="Left:",limits={0,inf,1} ; Execute "SetVariable SetBaseline1 value=Baseline1"
		SetVariable SetDecay1 					pos={155,Ch1TabPosOffset+440},	size={113,20},	title="Right (ms):",limits={0,inf,1} ; Execute "SetVariable SetDecay1 value=Decay1"
		Variable/G NumberOfAveragedEvents1 = 0
		Valdisplay NumEventsAveraged1		pos={275,Ch1TabPosOffset+440},	size={80,20},	title="#:"; Execute "Valdisplay NumEventsAveraged1 value=" + num2str(NumberOfAveragedEvents1)
		// Here I'm adding an option to superimposed all tagged events
		Button ButtonSupEvents1			pos={15,Ch1TabPosOffset+465},size={145,20},title="Superimpose tagged events", 			proc=ProcButtonSupEvents1
		Variable/G CheckIncludeAv1 = 0
		CheckBox CheckBoxIncludeAv1		pos={170,Ch1TabPosOffset+468},size={145,20},title="Include average", 					proc=ProcCBIncludeAv1, value=0
		TitleBox 		TitleBoxAlign1  	frame=0, 		pos={15,Ch1TabPosOffset+490}, 		size={15, 20},											title="Align on:"
		Variable/G CheckAlign1Foot=0; Variable/G CheckAlign1Occ=1; Variable/G CheckAlign1Peak=0
		CheckBox CheckBoxAlign1Foot		pos={65,Ch1TabPosOffset+490},size={40,20},title="Foot", proc=procToggleAlign1Foot
		CheckBox CheckBoxAlign1Occ		pos={110,Ch1TabPosOffset+490},size={40,20},title="Half-rise", proc=procToggleAlign1Occ
		CheckBox CheckBoxAlign1Peak		pos={173,Ch1TabPosOffset+490},size={40,20},title="Peak", proc=procToggleAlign1Peak
		Execute "CheckBox CheckBoxAlign1Occ value=1"
		
		// FOURTH GROUP: FITS
		Variable/G CheckFit1 = 0 // When CheckFit1 = 0, the check box CFPE1 is on; when CheckFit1 = 1, the check box CFAE1 is on.
		Variable/G CheckOffset1 = 0
		// Same as previous for title box
		//TitleBox FitDisplayText1  frame=0, 		pos={15,Ch1TabPosOffset+380}, size={15, 20},Title="Fit"		
		CheckBox CheckBFitSelectedChan11 	pos={15,Ch1TabPosOffset+380},size={100,20},title="Channel1", 	proc=fit_chan_select1, 	value = 1
		CheckBox CheckBFitSelectedChan12	pos={15,Ch1TabPosOffset+400},size={100,20},title="Channel2", 	proc=fit_chan_select1, 	value = 0	//, disable=2 // CFAE : Check Fit Average Event
		Variable/G Fit_chan_select_nb1 = 1 // Seems needed to avoid bug on first run
		CheckBox CFPE1						pos={140,Ch1TabPosOffset+380},size={100,20},title="event", 		proc=PCFPE1, 	value = 1 - CheckFit1 // CFPE : Check Fit Present Event
		CheckBox CFAE1						pos={140,Ch1TabPosOffset+400},size={100,20},title="average", 	proc=PCFAE1, 	value = CheckFit1, disable=2 // CFAE : Check Fit Average Event
		CheckBox FreeOffset1					pos={245,Ch1TabPosOffset+380},size={100,20},title="free offset", proc=CheckBoxFreeOffset1, value = CheckOffset1
		Variable/G StartFit1 ; StartFit1 = 10;
		Variable/G EndFit1; EndFit1=Decay1
		Button FitSingleExp1					pos={15, Ch1TabPosOffset+420},size={80,20},							proc=FitSingleExp_select1,title="Single exp."
		Button Fitdbexp1						pos={15,Ch1TabPosOffset+440},size={80,20},							proc=Fitdbexp_select1,title="double exp."
		Button BFitsigdbexp1 				pos={15,Ch1TabPosOffset+460}, size={80,20},  						proc=FitSigDbExp_select1, title="sigm.+dbexp"
		Button BFitsigmoid1 				pos={15,Ch1TabPosOffset+480}, size={80,20},  						proc=FitSigmoid_select1, title="sigmoid"
		Button BFitAllEventsdbexp1			pos={15,Ch1TabPosOffset+500}, size={80,20}, 							proc=FitAllEventsdbexp1, title="Fit all events"
		Variable/G OffsetFitdbexp1 ; OffsetFitdbexp1=0
		SetVariable Start_Fit1 			pos={100,Ch1TabPosOffset+420},size={115,20}, title="Start (%):",limits={0,100,1}, value= StartFit1
		SetVariable End_Fit1 			pos={227,Ch1TabPosOffset+420}, size={115,20}, title="End (ms):", limits={0,Decay1,1}, value=EndFit1
		ValDisplay TauFast1 					pos={100,Ch1TabPosOffset+440},size={115,20},title="Tau(fast):",value=0
		ValDisplay TauSlow1					pos={227,Ch1TabPosOffset+440},size={115,20},title="Tau(slow):",value=0
		ValDisplay FastPercentage1 		pos={100,Ch1TabPosOffset+460}, size={115,20}, title="% fast:", value=0
		ValDisplay SlowPercentage1 		pos={227,Ch1TabPosOffset+460}, size={115,20}, title="% slow:", value=0
		ValDisplay ValOffset1 				pos={100,Ch1TabPosOffset+480}, size={115,20}, title="Offset =",value=0
		Valdisplay KineticIndex1 				pos={227,Ch1TabPosOffset+480}, size={115,20}, value=0, title="Kinetic Index :"
		
		// FIFTH GROUP : Paired Recordings
		
		// Charly*******************************************My new buttons for channel one************************************
		
		Button ButtonSync1			pos={15,Ch1TabPosOffset+380},size={160,20},						proc=SyncChan1From2,title="Synchronise ch1 from ch2"		
		Button ButtonMAS1			pos={185,Ch1TabPosOffset+380},size={160,20},						proc=btn_MAS,title="Amplitude distribution"
		
		// variables for Sync
		Variable/G SyncWinStart1, SyncWinEnd1
		SetVariable SetSyncWinStart1 				pos={15,Ch1TabPosOffset+405},size={203,20}, title="Sync window start (ms):",limits={-100,100,1}, value= SyncWinStart1
		SetVariable SetSyncWinEnd1 				pos={223,Ch1TabPosOffset+405},size={122,20}, title="end (ms):",limits={-100,100,1}, value= SyncWinEnd1
		
		// variables for MAS
		Variable/G MASPeakWinWidth1=1, MASBslWidth1=3, MASAPRiseTime1, MASLagToPeak1, MASTrueHistbinSize1=10, MASNoiseHistBinSize1=10
		Variable/G MASHistMin1 = -500, MASHistMax1 = 300
		SetVariable SetMASPeakWinWidth1 		pos={15,Ch1TabPosOffset+425},size={150,20}, title="Peak window (ms):",limits={0,100,1}, value= MASPeakWinWidth1
		SetVariable SetMASBslWidth1 				pos={175,Ch1TabPosOffset+425},size={170,20}, title="Baseline window (ms):",limits={0,100,1}, value= MASBslWidth1
		SetVariable SetMASAPRiseTime1	 		pos={15,Ch1TabPosOffset+445},size={150,20}, title="AP Rise Time (ms) :",limits={0,100,1}, value= MASAPRiseTime1
		SetVariable SetMASLagToPeak1 			pos={175,Ch1TabPosOffset+445},size={170,20}, title="Lag to event peak(ms):",limits={0,100,1}, value= MASLagToPeak1
		SetVariable SetMASTrueHistBinSize1 		pos={15,Ch1TabPosOffset+465},size={180,20}, title="Data histo bin size (pA):",limits={0,100,1}, value= MASTrueHistBinSize1
		SetVariable SetMASNoiseHistBinSize1		pos={15,Ch1TabPosOffset+465},size={180,20}, title="Noise histo bin size (pA):",limits={0,100,1}, value= MASNoiseHistBinSize1
		SetVariable SetMASHistMin1				pos={15,Ch1TabPosOffset+485},size={170,20}, title="Histogram min (pA):",limits={-1000,0,1}, value= MASHistMin1
		SetVariable SetMASHistMax1				pos={195,Ch1TabPosOffset+485},size={110,20}, title="max (pA):",limits={0,1000,1}, value= MASHistMax1			
		
		// Charly End of my changes*******************************************************************
		
		// SIXTH GROUP : MANAGE TAGS
		Variable/G IsolateEventsWinStart1 = 40, IsolateEventsWinEnd1 = 40, LowPassThrshld1 = 200
		SetVariable SetIsolateEventsWinStart1		pos={15,Ch1TabPosOffset+420},size={185,20}, title="No events before (ms):",limits={0,1000,1}, value= IsolateEventsWinStart1
		SetVariable SetIsolateEventsWinEnd1		pos={205,Ch1TabPosOffset+420},size={86,20}, title="after:",limits={0,1000,1}, value= IsolateEventsWinEnd1
		Button ButtonIsolateEvents1				pos={300,Ch1TabPosOffset+419},size={40,20},title="apply",	proc=ProcIsolateEvents
		SetVariable SetLowPassThrshld1			pos={15,Ch1TabPosOffset+440},size={185,20}, title="No events below (pA):",limits={0,2000,1}, value= LowPassThrshld1
		Button ButtonLowPass1					pos={300,Ch1TabPosOffset+439},size={40,20}, title="apply", 	proc=ProcLowPass
		Variable /G LimitBegin1 ; LimitBegin1 = 0 ; Variable /G LimitEnd1 ; LimitEnd1 = 0
		Button ButtonTagRangeOfEvents1			pos={15,Ch1TabPosOffset+380},size={40,20},title="Tag", 	proc=ProcTagRange1
		Button ButtonUntagRangeOfEvents1			pos={55,Ch1TabPosOffset+380},size={40,20},title="Untag", 	proc=ProcUntagRange1
		SetVariable SetBegin1						pos={100,Ch1TabPosOffset+379},size={165,20},title="events between (s)", value = LimitBegin1, limits={0,Length,1}
		SetVariable SetEnd1						pos={270,Ch1TabPosOffset+379},size={73,20},title="&", value = LimitEnd1, limits={LimitBegin1,Length,1}
		Variable /G LimitBegin1_pnts ; LimitBegin1_pnts = 0 ; Variable /G LimitEnd1_pnts ; LimitEnd1_pnts = 0
		Button ButtonTagRangeOfEvents1_pnts		pos={15,Ch1TabPosOffset+400},size={40,20},title="Tag", 	proc=ProcTagRange1_pnts
		Button ButtonUntagRangeOfEvents1_pnts		pos={55,Ch1TabPosOffset+400},size={40,20},title="Untag", 	proc=ProcUntagRange1_pnts
		SetVariable SetBegin1_pnts				pos={100,Ch1TabPosOffset+399},size={165,20},title="events between #", value = LimitBegin1_pnts, limits={0,NumberofEventsch1-1,1}
		SetVariable SetEnd1_pnts					pos={270,Ch1TabPosOffset+399},size={73,20},title="&", value = LimitEnd1_pnts, limits={LimitBegin1_pnts,NumberofEventsch1-1,1}
		Button BtnSaveTags1						pos={90,Ch1TabPosOffset+460},size={80,20},				proc=ProcSaveTags1,title="Save Tags"
		Button BtnDeleteNaN1						pos={15,Ch1TabPosOffset+460}, size={70,20},				proc=ProcDeleteNaN,title="Untag NaNs"
		Variable/G Tags1_origin
		PopupMenu	PopupOriginOfTags1		pos={10,Ch1TabPosOffset+485}, title="Select origin of Tags1 : ",size={20,20},proc=OriginOfTags1, 	value="Tags1 (true);Tags 1 synchronised",	Disable=0
	

		// SEVENTH GROUP : INTeGRALS
		Variable/G IntegralStart1 = (-Baseline1), IntegralEnd1 = Decay1, CheckIntegralOffset1 = 1
		Variable/G IntAvgWidth1 = 3, IntegrateChanSelectNb1 = 1, Area1=0 // IntegrateChanSelectNb1 tels channel to integrate
		CheckBox CheckBIntSelectedChan11 	pos={15,Ch1TabPosOffset+380},size={100,20},title="Channel1", 	proc=IntegrateChanSelect1, 	value = 1
		CheckBox CheckBIntSelectedChan12		pos={15,Ch1TabPosOffset+400},size={100,20},title="Channel2", 	proc=IntegrateChanSelect1, 	value = 0
		Variable/G CheckIntegrate1 = 1 // Variable that tels to integrate current evnt or avg
		CheckBox CheckBIntEvent1				pos={140,Ch1TabPosOffset+380},size={100,20},title="event", 		proc=IntegrateEvent1, 	value = CheckIntegrate1
		CheckBox CheckBIntAvg1				pos={140,Ch1TabPosOffset+400},size={100,20},title="average", 	proc=IntegrateAvg1, 	value = (1 - CheckIntegrate1)
		SetVariable SetIntegralStart1			pos={15,Ch1TabPosOffset+420},size={165,20},title="Integrate from : ", value = IntegralStart1, limits={(-Baseline1),0,1}
		SetVariable SetIntegralEnd1			pos={185,Ch1TabPosOffset+420},size={73,20},title=" to : ", value = IntegralEnd1, limits={0,decay1,1}
		SetVariable SetIntAvgWidth1			pos={15,Ch1TabPosOffset+450},size={200,20},title="Average window width : ", value = IntAvgWidth1, limits={0,(Baseline1 + Decay1),1}
		CheckBox ChkboxIntegralOffset1			pos={245,Ch1TabPosOffset+380},size={100,20},title="Offset trace", proc=SetCheckIntegralOffset, value = CheckIntegralOffset1
		Button Integrate1						pos={15,Ch1TabPosOffset+480},size={80,20},title="Integrate !", 	proc=SpAcAnIntegrate
		Valdisplay DisplayArea1 				pos={227,Ch1TabPosOffset+480}, size={115,20}, value=0, title="Area :"
		// *******************************************************Right panels**********************************
		// DISPLAY EVENT
		Display/HOST=DetectedEvents1/N=EventChannel1/W=(0.4,0.01,0.69,0.49) TEMPEVENT1
		AppendToGraph YcsrA1 vs XcsrA1 ; ModifyGraph lsize(YcsrA1)=5 ; ModifyGraph rgb(YcsrA1)=(65280,0,0)
		AppendToGraph YcsrC1 vs XcsrC1 ; ModifyGraph lsize(YcsrC1)=5 ; ModifyGraph rgb(YcsrC1)=(65280,0,0)
		AppendToGraph YcsrD1 vs XcsrD1 ; ModifyGraph mode(YcsrD1)=3,mrkThick(YcsrD1)=2; ModifyGraph rgb(YcsrD1)=(0,52224,0)
		AppendToGraph YcsrE1 vs XcsrE1 ; ModifyGraph mode(YcsrE1)=3,mrkThick(YcsrE1)=2; ModifyGraph rgb(YcsrE1)=(0,52224,0)
		AppendToGraph YcsrF1 vs XcsrF1 ; ModifyGraph mode(YcsrF1)=2,lsize(YcsrF1)=5; ModifyGraph rgb(YcsrF1)=(52224,0,41728)
		Cursor/F/H=1/A=1/W=MainWindow#Channel1 A $FileName1 Occurrence1[Event_Number], BaselineY1[Event_Number]
		ModifyGraph margin(left)=40; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28
		ModifyGraph rgb(TEMPEVENT1)=(0,0,65280); ModifyGraph StandOff = 0
		SetDrawEnv ycoord = left, linefgc = (47872,47872,47872), dash = 1
		DrawLine 0, 0, 1, 0
		
		Variable/G ACFT1 = 0; 	// says if DetectedEvents1#Average1 exists
		Variable/G ACFT1ch2 = 0		// says is DetectedEvents1#Average12 exists.
		
		// Prepare the waves which will be used for the cursors of the fit
		Make/o/N=2 Curseur_debut_fit_X1, Curseur_debut_fit_X1_Av
		Make/o/N=2 Curseur_debut_fit_Y1, Curseur_debut_fit_Y1_Av
		Make/o/N=2 Curseur_fin_fit_X1, Curseur_fin_fit_X1_Av
		Make/o/N=2 Curseur_fin_fit_Y1, Curseur_fin_fit_Y1_Av
		
		Variable/G PreviousEvent1	// important to keep memory of previous event, used in "browsing through tagged events only"
		PreviousEvent1 = 0
		Execute "TabControl Show1Tab value=CurrentTab1"
		Execute "Show1TabProc(\"Show1Tab\", CurrentTab1)"
	Else
		CheckBox CheckBoxTag1 value=Tags1[Event_Number]
		Cursor/F/H=1/A=1/W=MainWindow#Channel1 A $FileName1 Occurrence1[Event_Number], BaselineY1[Event_Number]
		CursorTags1_X = Event_Number; CursorTags1_Y = Tags1[Event_Number]
		//if (ToOHR_1 == 1)
		//	Execute "ValDisplay DisplayOccurrence1 value=root:spacan:Occurrence1[" + num2str(Event_Number) + "]"
		//else
		//	Execute "ValDisplay DisplayOccurrence1 value=root:spacan:realpeakposition1[" + num2str(Event_Number) + "]"
		//endif
		//Execute "ValDisplay DisplayAmp1 value=root:spacan:realamp1[" + num2str(Event_Number) + "]"
		//Execute "ValDisplay Display2080RiseTime1 value=root:spacan:RiseTime20801[" + num2str(Event_Number) + "]"
		Execute "TabControl Show1Tab value=CurrentTab1"
		Execute "Show1TabProc(\"Show1Tab\", CurrentTab1)"
	Endif
		SetAxis/W=DetectedEvents1#EventChannel1 left 1.2*YcsrC1[0], -.4*YcsrC1[0]
	Nvar ShowCh2
	if (ShowCh2 == 1)
		Duplicate/o/R=( Occurrence1[Event_Number] - K16*.001 , Occurrence1[Event_Number] + K17*.001 ) $FileName2 TEMPEVENT_CH2
		SetScale/P x -K16*.001, (SI2*1e-6), "s", TEMPEVENT_CH2
		
		//Charly ****************************** Here I substract baseline of the event in channel 2 in front of events in 1 :
		
			NVAR Baseline1
			Variable substracter = mean(TEMPEVENT_CH2, -(7 * 0.001), -(4 * 0.001))
			TEMPEVENT_CH2 -= substracter
		
		// Charly End of my changes*******************************************************************
		
		Wavestats/Q TEMPEVENT_CH2
		SetAxis/W=DetectedEvents1#EventChannel12 left V_min - (V_max - V_min)*0.2, V_max + (V_max - V_min)*0.2
		Cursor/F/H=1/A=1/W=MainWindow#Channel2 A $FileName2 Occurrence1[Event_Number], BaselineY1[Event_Number]
	endif
END

// FUNCTION TRIGGERED BY THE CHECKBOX "CheckBoxAlign1Foot" IN THE DETECTED EVENTS WINDOW (CHANNEL 1)
FUNCTION procToggleAlign1Foot(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckAlign1Foot, CheckAlign1Occ, CheckAlign1Peak
	CheckAlign1Foot = 1 - CheckAlign1Foot
	if (CheckAlign1Foot == 1)
		CheckAlign1Occ = 0
		CheckAlign1Peak = 0
	endif
	Execute "CheckBox CheckBoxAlign1Foot value=CheckAlign1Foot"
	Execute "CheckBox CheckBoxAlign1Occ value=CheckAlign1Occ"
	Execute "CheckBox CheckBoxAlign1Peak value=CheckAlign1Peak"
END

// FUNCTION TRIGGERED BY THE CHECKBOX "CheckBoxAlign1Occ" IN THE DETECTED EVENTS WINDOW (CHANNEL 1)
FUNCTION procToggleAlign1Occ(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckAlign1Foot, CheckAlign1Occ, CheckAlign1Peak
	CheckAlign1Occ = 1 - CheckAlign1Occ
	if (CheckAlign1Occ == 1)
		CheckAlign1Foot = 0
		CheckAlign1Peak = 0
	endif
	Execute "CheckBox CheckBoxAlign1Foot value=CheckAlign1Foot"
	Execute "CheckBox CheckBoxAlign1Occ value=CheckAlign1Occ"
	Execute "CheckBox CheckBoxAlign1Peak value=CheckAlign1Peak"
END

// FUNCTION TRIGGERED BY THE CHECKBOX "CheckBoxAlign1Peak" IN THE DETECTED EVENTS WINDOW (CHANNEL 1)
FUNCTION procToggleAlign1Peak(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckAlign1Foot, CheckAlign1Occ, CheckAlign1Peak
	CheckAlign1Peak = 1 - CheckAlign1Peak
	if (CheckAlign1Peak == 1)
		CheckAlign1Foot = 0
		CheckAlign1Occ = 0
	endif
	Execute "CheckBox CheckBoxAlign1Foot value=CheckAlign1Foot"
	Execute "CheckBox CheckBoxAlign1Occ value=CheckAlign1Occ"
	Execute "CheckBox CheckBoxAlign1Peak value=CheckAlign1Peak"
END

Function Show2TabProc(CtrlName, TabNum) : TabControl
	String CtrlName
	Variable TabNum
	
	Variable isTab0= tabNum==0
	Variable isTab1= tabNum==1
	Variable isTab2= tabNum==2
	Variable isTab3= tabNum==3
	Variable isTab4= tabNum==4
	
	// Template if you want to add a new tab
	//Variable isTab5= tabNum==5
	
	// note: disable=0 means "show", disable=1 means "hide"
	
	// This line may be in use later 
	SetActiveSubWindow DetectedEvents2#DetectedEvents2Panel
	
	// *************************************For Average Tab***************************************
	ModifyControl CheckRiseTime2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetRTinf2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetRTsup2 disable= !isTab0	// hide if not Tab 0
	ModifyControl CheckBaselineSD2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBslnSDinf2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBslnSDsup2 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonShowRT2080histo2 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonShowBslnSDHisto2 disable= !isTab0	// hide if not Tab 0
	ModifyControl ButtonAverage2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetBaseline2 disable= !isTab0	// hide if not Tab 0
	ModifyControl SetDecay2 disable= !isTab0	// hide if not Tab 0
	ModifyControl NumEventsAveraged2 disable= !isTab0	// hide if not Tab 0
	ModifyControl FiltersDisplayText2 disable= !isTab0
	ModifyControl ButtonSupEvents2 disable= !isTab0
	ModifyControl CheckBoxIncludeAv2 disable= !isTab0
	ModifyControl TitleBoxAlign2 disable= !isTab0
	ModifyControl CheckBoxAlign2Foot disable= !isTab0
	ModifyControl CheckBoxAlign2Occ disable= !isTab0
	ModifyControl CheckBoxAlign2Peak disable= !isTab0		
		
	//****************************************for Kinetics Tab*************************************
	ModifyControl CFPE2 disable= !isTab1	// hide if not Tab 1
	ModifyControl CFAE2 disable= !isTab1	// hide if not Tab 1
	ModifyControl FreeOffset2 disable= !isTab1	// hide if not Tab 1
	ModifyControl FitSingleExp2 disable= !isTab1	// hide if not Tab 1
	ModifyControl FitdbExp2 disable= !isTab1	// hide if not Tab 1
	ModifyControl BFitsigdbexp2 disable= !isTab1	// hide if not Tab 1
	ModifyControl BFitsigmoid2 disable= !isTab1	// hide if not Tab 1
	ModifyControl BFitAllEventsdbExp2 disable= !isTab1	// hide if not Tab 1
	ModifyControl Start_Fit2 disable= !isTab1	// hide if not Tab 1
	ModifyControl End_Fit2 disable= !isTab1	// hide if not Tab 1
	ModifyControl TauFast2 disable= !isTab1	// hide if not Tab 1
	ModifyControl TauSlow2 disable= !isTab1	// hide if not Tab 1
	ModifyControl FastPercentage2 disable= !isTab1	// hide if not Tab 1
	ModifyControl SlowPercentage2 disable= !isTab1	// hide if not Tab 1
	ModifyControl ValOffset2 disable= !isTab1	// hide if not Tab 1
	ModifyControl KineticIndex2 disable= !isTab1	// hide if not Tab 1

/// Find out  howto modify here

	ModifyControl CheckBFitSelectedChan21 disable= !isTab1
	ModifyControl CheckBFitSelectedChan22 disable= !isTab1
	//ModifyControl FitDisplayText2 disable= !isTab1

	// *********************************For Pairs Tab ***************************************
	
	ModifyControl ButtonSync2 disable= !isTab2		// hide if not Tab 2
	ModifyControl ButtonMAS2 disable= !isTab2	// hide if not Tab 2
	
	ModifyControl SetSyncWinStart2 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetSyncWinEnd2 disable= !isTab2		// hide if not Tab 2
	
	ModifyControl SetMASPeakWinWidth2 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASBslWidth2 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASAPRiseTime2 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASLagToPeak2 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASTrueHistbinSize2 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASNoiseHistBinSize2 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASHistMin2 disable= !isTab2		// hide if not Tab 2
	ModifyControl SetMASHistMax2 disable= !isTab2		// hide if not Tab 2
	
	//*************************************** TAB "MANAGE TAGS" ***********************************
	ModifyControl SetIsolateEventsWinStart2 disable= !isTab3		// hide if not Tab 3
	ModifyControl SetIsolateEventsWinEnd2 disable= !isTab3
	ModifyControl ButtonisolateEvents2 disable= !isTab3
	ModifyControl ButtonLowPass2 disable= !isTab3
	ModifyControl SetLowPassThrshld2 disable= !isTab3
	ModifyControl ButtonTagRangeOfEvents2 disable= !isTab3
	ModifyControl ButtonUntagRangeOfEvents2 disable= !isTab3
	ModifyControl SetBegin2 disable= !isTab3
	ModifyControl SetEnd2	 disable= !isTab3
	ModifyControl ButtonTagRangeOfEvents2_pnts disable= !isTab3
	ModifyControl ButtonUntagRangeOfEvents2_pnts disable= !isTab3
	ModifyControl SetBegin2_pnts disable= !isTab3
	ModifyControl SetEnd2_pnts disable= !isTab3
	ModifyControl BtnSaveTags2 disable= !isTab3
	ModifyControl BtnDeleteNaN2 disable= !isTab3
	ModifyControl PopupOriginOfTags2 disable= !isTab3
	
	//******************************************TAB "Integrals"*******************************
	ModifyControl CheckBIntSelectedChan21 disable = !isTab4
	ModifyControl CheckBIntSelectedChan22 disable = !isTab4
	ModifyControl CheckBIntEvent2 disable = !isTab4
	ModifyControl CheckBIntAvg2 disable = !isTab4
	ModifyControl SetIntegralStart2 disable = !isTab4
	ModifyControl SetIntegralEnd2 disable = !isTab4
	ModifyControl SetIntAvgWidth2 disable = !isTab4
	ModifyControl ChkboxIntegralOffset2 disable = !isTab4
	ModifyControl Integrate2 disable = !isTab4
	ModifyControl DisplayArea2 disable = !isTab4 
	// ***********************************Add your tab here***********************************
	
	//************************************************************************************
	Nvar CurrentTab2
	CurrentTab2 = TabNum
	return 0

End


// The function that builds the window showing detected events and average events from channel 2
FUNCTION SHOW2(Number, Event_Number)
	Variable Number,Event_Number
	Nvar KbdSpeed2
	Wave DetectedLevels2, Tags2, Occurrence2, BaselineY2, BaselineX2, PeakY2, PeakX2, RealAmp2, RealPeakPosition2, RiseTime20802, RT202, RT802
	Wave EventFootTime2
	Nvar BSLms2, Pms2, SI1, SI2, Length, NumberofEventsch2
	Svar FileName1, FileName2
	// Number says if it is the first time that the function SHOW is used (Number = 1) or not (Number = 2)
	// ----------------- SAVING THE EVENT TO BE DISPLAYED IN A WAVE -----------------
	Duplicate/o/R=(Occurrence2[Event_Number] - K18*.001, Occurrence2[Event_Number] + K19*.001 ) $FileName2 TEMPEVENT2
	TEMPEVENT2 = TEMPEVENT2 - BaselineY2[Event_Number]
	SetScale/P x -K18*.001, (SI2*1e-6), "s", TEMPEVENT2
	// ----------------- DEFINING THE CURSORS -----------------
	Make/o/N=2 YcsrA2, XcsrA2, YcsrB2, XcsrB2, YcsrC2, XcsrC2
	Make/o/N=1 YcsrD2, XcsrD2, YcsrE2, XcsrE2, YcsrF2, XcsrF2
	// Cursor A represents the baseline
	YcsrA2 = 0
	XcsrA2[0] = BaselineX2[Event_Number] - BSLms2*.001/2 - Occurrence2[Event_Number]
	XcsrA2[1] = BaselineX2[Event_Number] + BSLms2*.001/2 - Occurrence2[Event_Number]
	// Cursor C represents the real event amplitude
	YcsrC2 = RealAmp2[Event_Number]
	XcsrC2[0] = RealPeakPosition2[Event_Number] - Pms2*.001/2 - Occurrence2[Event_Number]
	XcsrC2[1] = RealPeakPosition2[Event_Number] + Pms2*.001/2 - Occurrence2[Event_Number]
	// Cursor D marks RT20%
	YcsrD2 = RealAmp2[Event_Number] * 0.2
	XcsrD2 = RT202[Event_Number] - Occurrence2[Event_Number]
	// Cursor D marks RT80%
	YcsrE2 = RealAmp2[Event_Number] * 0.8
	XcsrE2 = RT802[Event_Number] - Occurrence2[Event_Number]
	Nvar RT20802, AMP2, EventOccurrence2, EventOccurrenceMin2, EventOccurrenceSec2
	RT20802 = 1000*RiseTime20802[Event_Number]
	AMP2 = RealAmp2[Event_Number]
	// Cursor F marks the foot of the event
	YcsrF2 = 0
	XcsrF2 = EventFootTime2[Event_Number] - Occurrence2[Event_Number] 

	If (Number == 1) // If the function is ran for the first time 
		Make/o/N=(numpnts(DetectedLevels2)) Tags2 ; Tags2 = 0 // INITIALIZE TAGS
		DoWindow/K DetectedEvents2; Display/W=(150,0,845,450)/N=DetectedEvents2; DoWindow/C DetectedEvents2	// INITIALIZE WINDOW
		
		// BUILDING LEFT PANEL
		NewPanel/K=1/N=DetectedEvents2Panel/HOST=DetectedEvents2/FG=(FL, FT, gRight, FB )/W=(0,0,275,450) ; ModifyPanel cbRGB=(65500,65500,65500)
		
		// The following line creates the tab menu
		Variable/G CurrentTab2 = 0
		TabControl Show2Tab,pos={8,225},size={350,180}, fstyle=1,proc=Show2TabProc, win=DetectedEvents2#DetectedEvents2Panel, value= CurrentTab2
		// The following three lines create the different tabs of the tab menu
		TabControl Show2Tab,tabLabel(0)="Average",value= 0
		TabControl Show2Tab,tabLabel(1)="Kinetics",value= 0
		TabControl Show2Tab,tabLabel(2)="Pairs",value= 0
		TabControl Show2Tab,tabLabel(3)="Tags",value= 0
		TabControl Show2Tab,tabLabel(4)="Integrals",value=0
		
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1, fsize=16 ; DrawText 135,25,"Channel 2"
		
		// FIRST GROUP: controls
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,47,"Controls"
		Variable/G Ch2ControlsPosOffset = 45
		Button close_window2 				pos={310,8}, size={40,20}, fstyle=1,						proc=ButtonCloseDetectedEvents2,title="Close"
		SetVariable Set_Event_Number2 	pos={10,Ch2ControlsPosOffset+6},	size={120,20},		proc=SetEvent2, size={133,20}, title="Event # :",limits={0,(numpnts(DetectedLevels2)-1),1} ; Execute "SetVariable Set_Event_Number2 value= EventNumber2"
		Button Kbd2 							pos={137,Ch2ControlsPosOffset+5}, size={83,20},		proc=ButtonKbd2,title="use keyboard"
		SetVariable SetKbdSpeed2			pos={225,Ch2ControlsPosOffset+6}, size={85,20},		title="Speed:", value=KbdSpeed2, limits={1,inf,1}
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 0; DrawText 10,90,"Show:"
		SetVariable msBeforeEvent2 		pos={50,Ch2ControlsPosOffset+28}, size={80,20}, 		proc=SetEvent2, title="Left:",limits={-Inf,Inf,1}, value= K18
		SetVariable msAfterEvent2 			pos={137,Ch2ControlsPosOffset+28}, size={113,20}, 	proc=SetEvent2, title="right (ms):",limits={-Inf,Inf,1}, value= K19
		CheckBox Grids2 						pos={305,Ch2ControlsPosOffset+29}, size={45,17},		proc=CheckBoxGrids2, title="Grids", value=0
		
		Variable /G ShowCh1 ; ShowCh1 = 0
		CheckBox CheckShowChannel1		pos={256,Ch2ControlsPosOffset+29}, size={54,20},		proc=ShowChannel1, title="ch. 1", value=0
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 0 ;  DrawText 10,111,"Browsing: "
		CheckBox BTDE2						pos={77,Ch2ControlsPosOffset+50}, size={100,20}, 		title="tagged", proc=ProcBTDE2
		CheckBox BUDE2						pos={155,Ch2ControlsPosOffset+50}, size={100,20}, 		title="untagged events only.", proc=ProcBUDE2
		SetActiveSubwindow DetectedEvents2#DetectedEvents2Panel //Needed for drawEnv after displaying tags
		Variable/G ToOHR_2, ToOPeak_2; ToOHR_2 = 1; ToOPeak_2 = 0
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 0 ;  DrawText 10,131,"Tag/untag:"
		Variable/G TaggedEvents2 ; TaggedEvents2 = 0 ; Variable/G CheckedTagAll2 ; CheckedTagAll2 = 0
		CheckBox CheckBoxTag2 				pos={77,Ch2ControlsPosOffset+71},							proc=ProcCheckBoxTag2,size={45,20}, title="this event", value=0
		ValDisplay DisplayTaggedEvents2 		pos={225,Ch2ControlsPosOffset+70},size={110,20},title="# tagged:"
		Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
		CheckBox CheckBoxTagAll2 			pos={155,Ch2ControlsPosOffset+71}, 						proc=ProcCheckBoxTagAll2, size={45,20}, title="all/none"
	
		Variable/G BTDE_2, BUDE_2; BTDE_2=0; BUDE_2=0
		// DISPLAY TAGS IN A GRAPH
		display/HOST=DetectedEvents2#DetectedEvents2Panel/W=(10,Ch2ControlsPosOffset+90,340,Ch2ControlsPosOffset+169) tags2
		ModifyGraph margin=0
		ModifyGraph gbRGB=(65500,65500,65500); ModifyGraph wbRGB=(65500,65500,65500)
		ModifyGraph standoff(left)=0; SetAxis left 0,1; ModifyGraph manTick(left)={0,1,0,0},manMinor(left)={0,0}; ModifyGraph margin=20; ModifyGraph margin(right)=0; ModifyGraph margin(top)=0; ModifyGraph margin(bottom)=28
		Make/o/N=1 CursorTags2_X, CursorTags2_Y; CursorTags2_X = 0; CursorTags2_Y = 0
		Appendtograph CursorTags2_Y vs CursorTags2_X
		ModifyGraph mode(CursorTags2_Y)=3,marker(CursorTags2_Y)=10;ModifyGraph rgb(CursorTags2_Y)=(0,0,0)
		ModifyGraph msize(CursorTags2_Y)=5,mrkThick(CursorTags2_Y)=5
		
		// USE THIS VARIABLE TO MOVE BLOCKS OF CONTROLS
		Variable/G Ch2TabPosOffset=-125
		
		// THIRD GROUP: Average
		// Here we have to use a title box instead of simple text to be able to hide it with the tab procedure.
		TitleBox FiltersDisplayText2  frame=0, pos={15,Ch2TabPosOffset+380}, size={15, 20},Title="Filters (enter range):"
		Variable /G CheckRT20802 ; CheckRT20802 = 0 ; Variable /G RTinf2 ; RTinf2 = 0 ; Variable /G RTsup2 ; RTsup2 = 1
		CheckBox CheckRiseTime2 			pos={15,Ch2TabPosOffset+400},size={100,20},title="20-80% rise time (ms)", value=0, proc=ProcCheckRiseTime2
		SetVariable SetRTinf2 				pos={160,Ch2TabPosOffset+400},size={60,20},title=" ",limits={0,inf,0.1} ; Execute "SetVariable SetRTinf2 value=RTinf2"
		SetVariable SetRTsup2 				pos={225,Ch2TabPosOffset+400},size={70,20},title="-",limits={0,inf,0.1} ; Execute "SetVariable SetRTsup2 value=RTsup2"
		Variable /G CheckBslnSD2 ; CheckBslnSD2 = 0 ; Variable /G BslnSDinf2 ; BslnSDinf2 = 0 ; Variable /G BslnSDsup2 ; BslnSDsup2 = 3
		CheckBox CheckBaselineSD2	 		pos={15,Ch2TabPosOffset+420},size={100,20},title="Baseline SD (xx)", value=0, proc=ProcCheckBaselineSD2
		SetVariable SetBslnSDinf2 				pos={160,Ch2TabPosOffset+420},size={60,20},title=" ",limits={0,inf,0.1} ; Execute "SetVariable SetBslnSDinf2 value=BslnSDinf2"
		SetVariable SetBslnSDsup2 			pos={225,Ch2TabPosOffset+420},size={70,20},title="-",limits={0,inf,0.1} ; Execute "SetVariable SetBslnSDsup2 value=BslnSDsup2"
		Button ButtonShowRT2080histo2 		pos={300,Ch2TabPosOffset+400},size={52,20},title="Histo", 		proc=ProcButtonShowRT2080histo2
		Button ButtonShowBslnSDHisto2 		pos={300,Ch2TabPosOffset+420},size={52,20},title="Histo", 		proc=ProcButtonShowBslnSDHisto2
		Button ButtonAverage2 					pos={15,Ch2TabPosOffset+440},								proc=ProcButtonAverage2, size={52,20},title="Average"
		Variable /G Baseline2 ; Baseline2 = 20 ; Variable /G Decay2 ; Decay2 = 40
		SetVariable SetBaseline2 				pos={75,Ch2TabPosOffset+440},size={75,20},title="Left:",limits={0,inf,1} ; Execute "SetVariable SetBaseline2 value=Baseline2"
		SetVariable SetDecay2 				pos={155,Ch2TabPosOffset+440},size={113,20},title="Right (ms):",limits={0,inf,1} ; Execute "SetVariable SetDecay2 value=Decay2"
		Variable/G NumberOfAveragedEvents2 = 0
		Valdisplay NumEventsAveraged2		pos={275,Ch2TabPosOffset+440},size={80,20},title="#:"; Execute "Valdisplay NumEventsAveraged2 value=" + num2str(NumberOfAveragedEvents2)
		// Here I'm adding an option to superimposed all tagged events
		Button ButtonSupEvents2			pos={15,Ch2TabPosOffset+465},size={145,20},title="Superimpose tagged events", 			proc=ProcButtonSupEvents2
		Variable/G CheckIncludeAv2 = 0
		CheckBox CheckBoxIncludeAv2		pos={170,Ch2TabPosOffset+468},size={145,20},title="Include average", 						proc=ProcCBIncludeAv2, value=0
		TitleBox 		TitleBoxAlign2  	frame=0, 		pos={15,Ch2TabPosOffset+490}, 		size={15, 20},											title="Align on:"
		Variable/G CheckAlign2Foot=0; Variable/G CheckAlign2Occ=1; Variable/G CheckAlign2Peak=0
		CheckBox CheckBoxAlign2Foot		pos={65,Ch2TabPosOffset+490},size={40,20},title="Foot", proc=procToggleAlign2Foot
		CheckBox CheckBoxAlign2Occ		pos={110,Ch2TabPosOffset+490},size={40,20},title="Half-rise", proc=procToggleAlign2Occ
		CheckBox CheckBoxAlign2Peak		pos={173,Ch2TabPosOffset+490},size={40,20},title="Peak", proc=procToggleAlign2Peak
		Execute "CheckBox CheckBoxAlign2Occ value=1"
			
		// FOURTH GROUP: FITS
		Variable/G CheckFit2 = 0 // When CheckFit2 = 0, the check box CFPE2 is on; when CheckFit2 = 1, the check box CFAE2 is on.
		Variable/G CheckOffset2 = 0
		// Same as previous for title box
		Variable/G Fit_chan_select_nb2 = 2
		CheckBox CheckBFitSelectedChan22 	pos={15,Ch2TabPosOffset+380},size={100,20},title="Channel2", 	proc=fit_chan_select2, 	value = 1
		CheckBox CheckBFitSelectedChan21	pos={15,Ch2TabPosOffset+400},size={100,20},title="Channel1", 	proc=fit_chan_select2, 	value = 0
		CheckBox CFPE2						pos={140,Ch2TabPosOffset+380},size={100,20},title="event", 		proc=PCFPE2, 	value = 1 - CheckFit2 // CFPE : Check Fit Present Event
		CheckBox CFAE2						pos={140,Ch2TabPosOffset+400},size={100,20},title="average", 	proc=PCFAE2, 	value = CheckFit2, disable=2 // CFAE : Check Fit Average Event
		CheckBox FreeOffset2					pos={245,Ch2TabPosOffset+380},size={100,20},title="free offset", proc=CheckBoxFreeOffset2, value = CheckOffset2
		Variable/G StartFit2 ; StartFit2 = 10;
		Variable/G EndFit2; EndFit2=Decay2
		Button FitSingleExp2					pos={15, Ch2TabPosOffset+420},size={80,20},							proc=FitSingleExp_select2,title="Single exp."
		Button Fitdbexp2						pos={15,Ch2TabPosOffset+440},size={80,20},							proc=Fitdbexp_select2,title="double exp."
		Button BFitsigdbexp2 				pos={15,Ch2TabPosOffset+460}, size={80,20},  						proc=FitSigDbExp_select2, title="sigm.+dbexp"
		Button BFitsigmoid2 				pos={15,Ch2TabPosOffset+480}, size={80,20},  						proc=FitSigmoid_select2, title="sigmoid"
		Button BFitAllEventsdbexp2			pos={15,Ch2TabPosOffset+500}, size={80,20}, 							proc=FitAllEventsdbexp2, title="Fit all events"
		Variable/G OffsetFitdbexp2 ; OffsetFitdbexp2=0
		SetVariable Start_Fit2 				pos={100,Ch2TabPosOffset+420},size={115,20}, title="Start (%):",limits={0,100,1}, value= StartFit2
		SetVariable End_Fit2 				pos={227,Ch2TabPosOffset+420}, size={115,20}, title="End (ms):", limits={0,Decay2,1}, value=EndFit2
		ValDisplay TauFast2 					pos={100,Ch2TabPosOffset+440},size={115,20},title="Tau(fast):",value=0
		ValDisplay TauSlow2					pos={227,Ch2TabPosOffset+440},size={115,20},title="Tau(slow):",value=0
		ValDisplay FastPercentage2 		pos={100,Ch2TabPosOffset+460}, size={115,20}, title="% fast:", value=0
		ValDisplay SlowPercentage2 		pos={227,Ch2TabPosOffset+460}, size={115,20}, title="% slow:", value=0
		ValDisplay ValOffset2 				pos={100,Ch2TabPosOffset+480}, size={115,20}, title="Offset =",value=0
		Valdisplay KineticIndex2 				pos={227,Ch2TabPosOffset+480}, size={115,20}, value=0, title="Kinetic Index :"
		
		// FIFTH GROUP : Paired Recordings
		
		// Charly*******************************************My new buttons for channel one************************************
		
		Button ButtonSync2			pos={15,Ch2TabPosOffset+380},size={160,20},						proc=SyncChan2From1,title="Synchronise ch2 from ch1"		
		Button ButtonMAS2			pos={185,Ch2TabPosOffset+380},size={160,20},						proc=btn_MAS,title="Amplitude distribution"
		
		// variables for Sync
		Variable/G SyncWinStart2, SyncWinEnd2
		SetVariable SetSyncWinStart2 				pos={15,Ch2TabPosOffset+405},size={203,20}, title="Sync window start (ms):",limits={-100,100,1}, value= SyncWinStart2
		SetVariable SetSyncWinEnd2 				pos={223,Ch2TabPosOffset+405},size={122,20}, title="end (ms):",limits={-100,100,1}, value= SyncWinEnd2
		
		// variables for MAS
		Variable/G MASPeakWinWidth2=1, MASBslWidth2=3, MASAPRiseTime2, MASLagToPeak2, MASTrueHistbinSize2=10, MASNoiseHistBinSize2=10
		Variable/G MASHistMin2 = -500, MASHistMax2 = 300
		SetVariable SetMASPeakWinWidth2 		pos={15,Ch2TabPosOffset+425},size={150,20}, title="Peak window (ms):",limits={0,100,1}, value= MASPeakWinWidth2
		SetVariable SetMASBslWidth2 				pos={175,Ch2TabPosOffset+425},size={170,20}, title="Baseline window (ms):",limits={0,100,1}, value= MASBslWidth2
		SetVariable SetMASAPRiseTime2	 		pos={15,Ch2TabPosOffset+445},size={150,20}, title="AP Rise Time (ms) :",limits={0,100,1}, value= MASAPRiseTime2
		SetVariable SetMASLagToPeak2			pos={175,Ch2TabPosOffset+445},size={170,20}, title="Lag to event peak(ms):",limits={0,100,1}, value= MASLagToPeak2
		SetVariable SetMASTrueHistBinSize2 		pos={15,Ch2TabPosOffset+465},size={180,20}, title="Data histo bin size (pA):",limits={0,100,1}, value= MASTrueHistBinSize2
		SetVariable SetMASNoiseHistBinSize2		pos={15,Ch2TabPosOffset+465},size={180,20}, title="Noise histo bin size (pA):",limits={0,100,1}, value= MASNoiseHistBinSize2
		SetVariable SetMASHistMin2				pos={15,Ch2TabPosOffset+485},size={170,20}, title="Histogram min (pA):",limits={-1000,0,1}, value= MASHistMin2
		SetVariable SetMASHistMax2				pos={195,Ch2TabPosOffset+485},size={110,20}, title="max (pA):",limits={0,1000,1}, value= MASHistMax2			
		
		// Charly End of my changes*******************************************************************
		
		// SIXTH GROUP : MANAGE TAGS
		Variable/G IsolateEventsWinStart2 = 40, IsolateEventsWinEnd2 = 40, LowPassThrshld2 = 200
		SetVariable SetIsolateEventsWinStart2		pos={15,Ch2TabPosOffset+420},size={185,20}, title="No events before (ms):",limits={0,1000,1}, value= IsolateEventsWinStart2
		SetVariable SetIsolateEventsWinEnd2		pos={205,Ch2TabPosOffset+420},size={86,20}, title="after:",limits={0,1000,1}, value= IsolateEventsWinEnd2
		Button ButtonIsolateEvents2					pos={300,Ch2TabPosOffset+419},size={40,20},title="apply",	proc=ProcIsolateEvents
		SetVariable SetLowPassThrshld2			pos={15,Ch2TabPosOffset+440},size={185,20}, title="No events below (pA):",limits={0,2000,1}, value= LowPassThrshld2
		Button ButtonLowPass2						pos={300,Ch2TabPosOffset+439},size={40,20}, title="apply", 	proc=ProcLowPass
		Variable /G LimitBegin2 ; LimitBegin2 = 0 ; Variable /G LimitEnd2 ; LimitEnd2 = 0
		Button ButtonTagRangeOfEvents2			pos={15,Ch2TabPosOffset+380},size={40,20},title="Tag", 	proc=ProcTagRange2
		Button ButtonUntagRangeOfEvents2		pos={55,Ch2TabPosOffset+380},size={40,20},title="Untag", 	proc=ProcUntagRange2
		SetVariable SetBegin2						pos={100,Ch2TabPosOffset+379},size={165,20},title="events between (s)", value = LimitBegin2, limits={0,Length,1}
		SetVariable SetEnd2							pos={270,Ch2TabPosOffset+379},size={73,20},title="&", value = LimitEnd2, limits={LimitBegin2,Length,1}
		Variable /G LimitBegin2_pnts ; LimitBegin2_pnts = 0 ; Variable /G LimitEnd2_pnts ; LimitEnd2_pnts = 0
		Button ButtonTagRangeOfEvents2_pnts		pos={15,Ch2TabPosOffset+400},size={40,20},title="Tag", 	proc=ProcTagRange2_pnts
		Button ButtonUntagRangeOfEvents2_pnts	pos={55,Ch2TabPosOffset+400},size={40,20},title="Untag", 	proc=ProcUntagRange2_pnts
		SetVariable SetBegin2_pnts					pos={100,Ch2TabPosOffset+399},size={165,20},title="events between #", value = LimitBegin2_pnts, limits={0,NumberofEventsch2-1,1}
		SetVariable SetEnd2_pnts					pos={270,Ch2TabPosOffset+399},size={73,20},title="&", value = LimitEnd2_pnts, limits={LimitBegin2_pnts,NumberofEventsch2-1,1}
		Button BtnSaveTags2						pos={90,Ch2TabPosOffset+460},size={80,20},				proc=ProcSaveTags2,title="Save Tags"
		Button BtnDeleteNaN2						pos={15,Ch2TabPosOffset+460}, size={70,20},				proc=ProcDeleteNaN,title="Untag NaNs"
		Variable/G Tags2_origin
		PopupMenu	PopupOriginOfTags2		pos={10,Ch2TabPosOffset+485}, title="Select origin of Tags2 : ",size={20,20},proc=OriginOfTags2, 	value="Tags2 (true);Tags 2 synchronised",	Disable=0
		
		
		// SEVENTH GROUP : INTeGRALS
		Variable/G IntegralStart2 = (-Baseline2), IntegralEnd2 = Decay2, CheckIntegralOffset2 = 1
		Variable/G IntAvgWidth2 = 3, IntegrateChanSelectNb2 = 2, Area2=0 // IntegrateChanSelectNb2 tels channel to integrate
		CheckBox CheckBIntSelectedChan22	pos={15,Ch2TabPosOffset+380},size={100,20},title="Channel2", 	proc=IntegrateChanSelect2, 	value = 1
		CheckBox CheckBIntSelectedChan21 	pos={15,Ch2TabPosOffset+400},size={100,20},title="Channel1", 	proc=IntegrateChanSelect2, 	value = 0
		Variable/G CheckIntegrate2 = 1 // Variable that tels to integrate current evnt or avg
		CheckBox CheckBIntEvent2				pos={140,Ch2TabPosOffset+380},size={100,20},title="event", 		proc=IntegrateEvent2, 	value = CheckIntegrate2
		CheckBox CheckBIntAvg2				pos={140,Ch2TabPosOffset+400},size={100,20},title="average", 	proc=IntegrateAvg2, 	value = (1 - CheckIntegrate2)
		SetVariable SetIntegralStart2			pos={15,Ch2TabPosOffset+420},size={165,20},title="Integrate from : ", value = IntegralStart2, limits={(-Baseline2),0,1}
		SetVariable SetIntegralEnd2			pos={185,Ch2TabPosOffset+420},size={73,20},title=" to : ", value = IntegralEnd2, limits={0,decay2,1}
		SetVariable SetIntAvgWidth2			pos={15,Ch2TabPosOffset+450},size={200,20},title="Average window width : ", value = IntAvgWidth2, limits={0,(Baseline2 + Decay2),1}
		CheckBox ChkboxIntegralOffset2			pos={245,Ch2TabPosOffset+380},size={100,20},title="Offset trace", proc=SetCheckIntegralOffset, value = CheckIntegralOffset2
		Button Integrate2						pos={15,Ch2TabPosOffset+480},size={80,20},title="Integrate !", 	proc=SpAcAnIntegrate
		Valdisplay DisplayArea2 				pos={227,Ch2TabPosOffset+480}, size={115,20}, value=0, title="Area :"
		// *******************************************************Right panels**********************************
		// DISPLAY EVENT
		Display/HOST=DetectedEvents2/N=EventChannel2/W=(0.4,0.01,0.69,0.49) TEMPEVENT2
		AppendToGraph YcsrA2 vs XcsrA2 ; ModifyGraph lsize(YcsrA2)=5 ; ModifyGraph rgb(YcsrA2)=(0,0,65280)
		AppendToGraph YcsrC2 vs XcsrC2 ; ModifyGraph lsize(YcsrC2)=5 ; ModifyGraph rgb(YcsrC2)=(0,0,65280)
		AppendToGraph YcsrD2 vs XcsrD2 ; ModifyGraph mode(YcsrD2)=3,mrkThick(YcsrD2)=2; ModifyGraph rgb(YcsrD2)=(0,52224,0)
		AppendToGraph YcsrE2 vs XcsrE2 ; ModifyGraph mode(YcsrE2)=3,mrkThick(YcsrE2)=2; ModifyGraph rgb(YcsrE2)=(0,52224,0)
		AppendToGraph YcsrF2 vs XcsrF2 ; ModifyGraph mode(YcsrF2)=2,lsize(YcsrF2)=5; ModifyGraph rgb(YcsrF2)=(52224,0,41728)
		
		Cursor/F/H=1/A=1/W=MainWindow#Channel2 A $FileName2 Occurrence2[Event_Number], BaselineY2[Event_Number]
		ModifyGraph margin(left)=40; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28
		ModifyGraph rgb(TEMPEVENT2)=(65280,0,0); ModifyGraph StandOff = 0
		SetDrawEnv ycoord = left, linefgc = (47872,47872,47872), dash = 1
		DrawLine 0, 0, 1, 0
		
		Variable/G ACFT2 = 0; 	// says if DetectedEvents2#Average2 exists
		Variable/G ACFT2ch1 = 0		// says is DetectedEvents2#Average21 exists.
		
		// Prepare the waves which will be used for the cursors of the fit
		Make/o/N=2 Curseur_debut_fit_X2, Curseur_debut_fit_X2_Av
		Make/o/N=2 Curseur_debut_fit_Y2, Curseur_debut_fit_Y2_Av
		Make/o/N=2 Curseur_fin_fit_X2, Curseur_fin_fit_X2_Av
		Make/o/N=2 Curseur_fin_fit_Y2, Curseur_fin_fit_Y2_Av
		
		Variable/G PreviousEvent2	// important to keep memory of previous event, used in "browsing through tagged events only"
		PreviousEvent2 = 0
		Execute "TabControl Show2Tab value=CurrentTab2"
		Execute "Show2TabProc(\"Show2Tab\", CurrentTab2)"
	Else
		CheckBox CheckBoxTag2 value=Tags2[Event_Number]
		Cursor/F/H=1/A=1/W=MainWindow#Channel2 A $FileName2 Occurrence2[Event_Number], BaselineY2[Event_Number]
		CursorTags2_X = Event_Number; CursorTags2_Y = Tags2[Event_Number]
		//if (ToOHR_1 == 1)
		//	Execute "ValDisplay DisplayOccurrence1 value=root:spacan:Occurrence1[" + num2str(Event_Number) + "]"
		//else
		//	Execute "ValDisplay DisplayOccurrence1 value=root:spacan:realpeakposition1[" + num2str(Event_Number) + "]"
		//endif
		//Execute "ValDisplay DisplayAmp1 value=root:spacan:realamp1[" + num2str(Event_Number) + "]"
		//Execute "ValDisplay Display2080RiseTime1 value=root:spacan:RiseTime20801[" + num2str(Event_Number) + "]"
		Execute "TabControl Show2Tab value=CurrentTab2"
		Execute "Show2TabProc(\"Show2Tab\", CurrentTab2)"
	Endif
		SetAxis/W=DetectedEvents2#EventChannel2 left 1.2*YcsrC2[0], -.4*YcsrC2[0]
	Nvar ShowCh1
	if (ShowCh1 == 1)
		Duplicate/o/R=( Occurrence2[Event_Number] - K18*.001 , Occurrence2[Event_Number] + K19*.001 ) $FileName1 TEMPEVENT_CH1
		SetScale/P x -K18*.001, (SI1*1e-6), "s", TEMPEVENT_CH1
		
		//Charly ****************************** Here I substract baseline of the event in channel 1 in front of events in 2 :
		
			NVAR Baseline2
			Variable substracter = mean(TEMPEVENT_CH1, -(7 * 0.001), -(4 * 0.001))
			TEMPEVENT_CH1 -= substracter
		
		// Charly End of my changes*******************************************************************
		
		Wavestats/Q TEMPEVENT_CH1
		SetAxis/W=DetectedEvents2#EventChannel21 left V_min - (V_max - V_min)*0.2, V_max + (V_max - V_min)*0.2
		Cursor/F/H=1/A=1/W=MainWindow#Channel1 A $FileName1 Occurrence2[Event_Number], BaselineY2[Event_Number]
	endif
		
END

// FUNCTION TRIGGERED BY THE CHECKBOX "CheckBoxAlign2Foot" IN THE DETECTED EVENTS WINDOW (CHANNEL 2)
FUNCTION procToggleAlign2Foot(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckAlign2Foot, CheckAlign2Occ, CheckAlign2Peak
	CheckAlign2Foot = 1 - CheckAlign2Foot
	if (CheckAlign2Foot == 1)
		CheckAlign2Occ = 0
		CheckAlign2Peak = 0
	endif
	Execute "CheckBox CheckBoxAlign2Foot value=CheckAlign2Foot"
	Execute "CheckBox CheckBoxAlign2Occ value=CheckAlign2Occ"
	Execute "CheckBox CheckBoxAlign2Peak value=CheckAlign2Peak"
END

// FUNCTION TRIGGERED BY THE CHECKBOX "CheckBoxAlign2Occ" IN THE DETECTED EVENTS WINDOW (CHANNEL 2)
FUNCTION procToggleAlign2Occ(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckAlign2Foot, CheckAlign2Occ, CheckAlign2Peak
	CheckAlign2Occ = 1 - CheckAlign2Occ
	if (CheckAlign2Occ == 1)
		CheckAlign2Foot = 0
		CheckAlign2Peak = 0
	endif
	Execute "CheckBox CheckBoxAlign2Foot value=CheckAlign2Foot"
	Execute "CheckBox CheckBoxAlign2Occ value=CheckAlign2Occ"
	Execute "CheckBox CheckBoxAlign2Peak value=CheckAlign2Peak"
END

// FUNCTION TRIGGERED BY THE CHECKBOX "CheckBoxAlign2Peak" IN THE DETECTED EVENTS WINDOW (CHANNEL 2)
FUNCTION procToggleAlign2Peak(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckAlign2Foot, CheckAlign2Occ, CheckAlign2Peak
	CheckAlign2Peak = 1 - CheckAlign2Peak
	if (CheckAlign2Peak == 1)
		CheckAlign2Foot = 0
		CheckAlign2Occ = 0
	endif
	Execute "CheckBox CheckBoxAlign2Foot value=CheckAlign2Foot"
	Execute "CheckBox CheckBoxAlign2Occ value=CheckAlign2Occ"
	Execute "CheckBox CheckBoxAlign2Peak value=CheckAlign2Peak"
END

Function ProcOffsetFitdbexp1(ctrlName,checked) : CheckBoxControl
		String ctrlName
		Variable checked
		Nvar OffsetFPEdbexp1
		OffsetFPEdbexp1 = 1 - OffsetFPEdbexp1
End

// Charly now useless
//Function ProcOffsetFitdbexp2(ctrlName,checked) : CheckBoxControl
//		String ctrlName
//		Variable checked
//		Nvar OffsetFPEdbexp2
//		OffsetFPEdbexp2 = 1 - OffsetFPEdbexp2
//End
// Charly end

Function PCFPE1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckFit1
	CheckFit1 = 1 - CheckFit1
	CheckBox CFAE1 value = CheckFit1
End

Function PCFAE1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckFit1
	CheckFit1 = 1 - CheckFit1
	CheckBox CFPE1 value = 1 - CheckFit1
End

Function PCFPE2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckFit2
	CheckFit2 = 1 - CheckFit2
	CheckBox CFAE2 value = CheckFit2
End

Function PCFAE2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckFit2
	CheckFit2 = 1 - CheckFit2
	CheckBox CFPE2 value = 1 - CheckFit2
End

// THE FUNCTION TRIGGERED BY THE "Set_Event_Number1" CONTROL
Function SetEvent1(ctrlName,varNum,varStr,varName) : SetVariableControl
		String ctrlName
		Variable varNum
		String varStr
		String varName
		Wave DetectedLevels1, Tags1, Occurrence1, BaselineY1, BaselineX1, PeakY1, PeakX1, RealAmp1, RealPeakPosition1, RiseTime20801, RT201, RT801
		Nvar EventNumber1, Baselinems1, Peakms1, BTDE_1, BUDE_1, NumberofEventsch1, PreviousEvent1
		GetWindow DetectedEvents1#EventChannel1, WaveList
		Wave/T W_WaveList = W_WaveList
		Variable i; String TEMPSTR
		for (i=1; i < numpnts(W_WaveList)/3; I+=1)
			TEMPSTR = W_WaveList[i][0]
			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
				RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
			endif
			if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
				RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
			endif
			if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
				RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
			endif
		endfor
		if (BTDE_1 == 1)
			if (tags1[EventNumber1] == 0)
				if (PreviousEvent1 < EventNumber1)	// if we go right
					FindLevel/P/Q/R=[EventNumber1,NumberofEventsch1-1] Tags1, 1	// Find the next tagged event
					if (V_flag == 0)
						EventNumber1 = V_LevelX
						PreviousEvent1 = EventNumber1
					else
						EventNumber1 = PreviousEvent1
					endif
				endif
				if (PreviousEvent1 > EventNumber1)	// if we go left
					FindLevel/P/Q/R=[EventNumber1,0] Tags1, 1	// Find the previous tagged event
					if (V_flag == 0)
						EventNumber1 = V_LevelX
						PreviousEvent1 = EventNumber1
					else
						EventNumber1 = PreviousEvent1
					endif
				endif
			endif
		endif
		if (BUDE_1 == 1)
			if (tags1[EventNumber1] == 1)
				if (PreviousEvent1 < EventNumber1)	// if we go right
					FindLevel/P/Q/R=[EventNumber1,NumberofEventsch1-1] Tags1, 0	// Find the next untagged event
					if (V_flag == 0)
						EventNumber1 = V_LevelX
						PreviousEvent1 = EventNumber1
					else
						EventNumber1 = PreviousEvent1
					endif
				endif
				if (PreviousEvent1 > EventNumber1)	// if we go left
					FindLevel/P/Q/R=[EventNumber1,0] Tags1, 0	// Find the previous tagged event
					if (V_flag == 0)
						EventNumber1 = V_LevelX
						PreviousEvent1 = EventNumber1
					else
						EventNumber1 = PreviousEvent1
					endif
				endif
			endif
		endif
		if (BTDE_1 == 0 && BUDE_1 == 0)
			PreviousEvent1 = EventNumber1
		endif
		SHOW1(2,EventNumber1)
End

// THE FUNCTION TRIGGERED BY THE "Set_Event_Number2" CONTROL
Function SetEvent2(ctrlName,varNum,varStr,varName) : SetVariableControl
		String ctrlName
		Variable varNum
		String varStr
		String varName
		Wave DetectedLevels2, Tags2, Occurrence2, BaselineY2, BaselineX2, PeakY2, PeakX2, RealAmp2, RealPeakPosition2, RiseTime20802, RT202, RT802
		Nvar EventNumber2, Baselinems2, Peakms2, BTDE_2, BUDE_2, NumberofEventsch2, PreviousEvent2
		GetWindow DetectedEvents2#EventChannel2, WaveList
		Wave/T W_WaveList = W_WaveList
		Variable i; String TEMPSTR
		for (i=1; i < numpnts(W_WaveList)/3; I+=1)
			TEMPSTR = W_WaveList[i][0]
			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
				RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
			endif
			if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
				RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
			endif
			if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
				RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
			endif
		endfor
		if (BTDE_2 == 1)
			if (tags2[EventNumber2] == 0)
				if (PreviousEvent2 < EventNumber2)	// if we go right
					FindLevel/P/Q/R=[EventNumber2,NumberofEventsch2-1] Tags2, 1	// Find the next tagged event
					if (V_flag == 0)
						EventNumber2 = V_LevelX
						PreviousEvent2 = EventNumber2
					else
						EventNumber2 = PreviousEvent2
					endif
				endif
				if (PreviousEvent2 > EventNumber2)	// if we go left
					FindLevel/P/Q/R=[EventNumber2,0] Tags2, 1	// Find the previous tagged event
					if (V_flag == 0)
						EventNumber2 = V_LevelX
						PreviousEvent2 = EventNumber2
					else
						EventNumber2 = PreviousEvent2
					endif
				endif
			endif
		endif
		if (BUDE_2 == 1)
			if (tags2[EventNumber2] == 1)
				if (PreviousEvent2 < EventNumber2)	// if we go right
					FindLevel/P/Q/R=[EventNumber2,NumberofEventsch2-1] Tags2, 0	// Find the next untagged event
					if (V_flag == 0)
						EventNumber2 = V_LevelX
						PreviousEvent2 = EventNumber2
					else
						EventNumber2 = PreviousEvent2
					endif
				endif
				if (PreviousEvent2 > EventNumber2)	// if we go left
					FindLevel/P/Q/R=[EventNumber2,0] Tags2, 0	// Find the previous tagged event
					if (V_flag == 0)
						EventNumber2 = V_LevelX
						PreviousEvent2 = EventNumber2
					else
						EventNumber2 = PreviousEvent2
					endif
				endif
			endif
		endif
		if (BTDE_2 == 0 && BUDE_2 == 0)
			PreviousEvent2 = EventNumber2
		endif
		SHOW2(2,EventNumber2)
End

Function ButtonCloseDetectedEvents1(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K DetectedEvents1
End

Function ButtonCloseDetectedEvents2(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K DetectedEvents2
End

Function CheckBoxGrids1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ShowCh2, ACFT1, ACFT1ch2
	if (checked==1)
		ModifyGraph/W=DetectedEvents1#EventChannel1 grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=DetectedEvents1#EventChannel1 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		if (ShowCh2 == 1)
			ModifyGraph/W=DetectedEvents1#EventChannel12 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=DetectedEvents1#EventChannel12 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
		if (ACFT1 == 1)
			ModifyGraph/W=DetectedEvents1#Average1 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=DetectedEvents1#Average1 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
		if (ACFT1ch2 == 1)
			ModifyGraph/W=DetectedEvents1#Average12 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=DetectedEvents1#Average12 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
	else
		ModifyGraph/W=DetectedEvents1#EventChannel1 grid(left)=0
		ModifyGraph/W=DetectedEvents1#EventChannel1 grid(bottom)=0
		if (ShowCh2 == 1)
			ModifyGraph/W=DetectedEvents1#EventChannel12 grid(left)=0
			ModifyGraph/W=DetectedEvents1#EventChannel12 grid(bottom)=0
		endif
		if (ACFT1 == 1)
			ModifyGraph/W=DetectedEvents1#Average1 grid(left)=0
			ModifyGraph/W=DetectedEvents1#Average1  grid(bottom)=0
		endif
		if (ACFT1ch2 == 1)
			ModifyGraph/W=DetectedEvents1#Average12 grid(left)=0
			ModifyGraph/W=DetectedEvents1#Average12 grid(bottom)=0
		endif
	endif
End

Function CheckBoxGrids2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ShowCh1, ACFT2, ACFT2ch1
	if (checked==1)
		ModifyGraph/W=DetectedEvents2#EventChannel2 grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=DetectedEvents2#EventChannel2 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		if (ShowCh1 == 1)
			ModifyGraph/W=DetectedEvents2#EventChannel21 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=DetectedEvents2#EventChannel21 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
		if (ACFT2 == 1)
			ModifyGraph/W=DetectedEvents2#Average2 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=DetectedEvents2#Average2 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
		if (ACFT2ch1 == 1)
			ModifyGraph/W=DetectedEvents2#Average21 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=DetectedEvents2#Average21 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
	else
		ModifyGraph/W=DetectedEvents2#EventChannel2 grid(left)=0
		ModifyGraph/W=DetectedEvents2#EventChannel2 grid(bottom)=0
		if (ShowCh1 == 1)
			ModifyGraph/W=DetectedEvents2#EventChannel21 grid(left)=0
			ModifyGraph/W=DetectedEvents2#EventChannel21 grid(bottom)=0
		endif
		if (ACFT2 == 1)
			ModifyGraph/W=DetectedEvents2#Average2 grid(left)=0
			ModifyGraph/W=DetectedEvents2#Average2 grid(bottom)=0
		endif
		if (ACFT2ch1 == 1)
			ModifyGraph/W=DetectedEvents2#Average21 grid(left)=0
			ModifyGraph/W=DetectedEvents2#Average21 grid(bottom)=0
		endif
	endif
End

Function CheckBoxFreeOffset1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckOffset1
	CheckOffset1 = 1-CheckOffset1
End

Function CheckBoxFreeOffset2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckOffset2
	CheckOffset2 = 1-CheckOffset2
End

// Charly
Function KbdCtrlWinCloser(CtrlName) : ButtonControl
	String CtrlName
	DoWindow/K ControlsPanel // Self killing
End
// End

Function ButtonKbd1(ctrlName) : ButtonControl
	String ctrlName
	
	// Charly
	Nvar KbdBrowseChecked
	If (! Nvar_exists(KbdBrowseChecked)) // If it has not already been used
		NewPanel/W=(200, 200, 400, 400)/N=ControlsPanel/k=1 as "Controls"
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,20,"Previous event : Left arrow"
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,50,"Next event : Right arrow"
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,80,"Tag/Untag event : top arrow"
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,110,"Stop : \"C\""
		Button ClosingButton, pos ={50, 140}, size={100, 20}, proc=KbdCtrlWinCloser, Title="Close"
		PauseForUser ControlsPanel
		Variable/G KbdBrowseChecked
	EndIf
	// End
	
	GetWindow DetectedEvents1#EventChannel1, WaveList
	Wave/T W_WaveList = W_WaveList
	Variable i; String TEMPSTR
	for (i=1; i < numpnts(W_WaveList)/3; I+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif
	endfor
	DoUpdate
	WaitForKey1()
End

Function ButtonKbd2(ctrlName) : ButtonControl
	String ctrlName
	
		// Charly
	Nvar KbdBrowseChecked
	If (! Nvar_exists(KbdBrowseChecked)) // If it has not already been used
		NewPanel/W=(200, 200, 400, 400)/N=ControlsPanel/k=1 as "Controls"
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,20,"Previous event : Left arrow"
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,50,"Next event : Right arrow"
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,80,"Tag/Untag event : top arrow"
		SetDrawLayer UserBack ; SetDrawEnv fstyle= 1 ; DrawText 10,110,"Stop : \"C\""
		Button ClosingButton, pos ={50, 140}, size={100, 20}, proc=KbdCtrlWinCloser, Title="Close"
		PauseForUser ControlsPanel
		Variable/G KbdBrowseChecked
	EndIf
	// End

	
	GetWindow DetectedEvents2#EventChannel2, WaveList
	Wave/T W_WaveList = W_WaveList
	Variable i; String TEMPSTR
	for (i=1; i < numpnts(W_WaveList)/3; I+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
			RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
			RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
			RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
		endif
	endfor
	DoUpdate
	WaitForKey2()
End

// CHECKBOX THAT TAGS/UNTAGS SINGLE EVENTS (Channel 1)
Function ProcCheckBoxTag1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave Tags1, CursorTags1_Y
	Nvar EventNumber1, TaggedEvents1, NumberofEventsch1, BTDE_1, BUDE_1, CheckedTagAll1
	Tags1[EventNumber1]=checked
	CursorTags1_Y = checked
	TaggedEvents1 = sum(Tags1,0,numpnts(Tags1)-1)
	Execute "ValDisplay DisplayTaggedEvents1 value=" + num2str(TaggedEvents1)
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==0)
		BTDE_1 = 0; Checkbox BTDE1 value = 0
	endif
	if (CheckedTagAll1 == 1)
		If ( (sum(Tags1,0,numpnts(Tags1)-1)) < NumberofEventsch1)
			CheckedTagAll1 = 0; CheckBox CheckBoxTagAll1 value=0
		endif
	endif
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==NumberofEventsch1)
		BUDE_1 = 0; Checkbox BUDE1 value = 0
		CheckedTagAll1 = 1; CheckBox CheckBoxTagAll1 value=1
	endif
End

// CHECKBOX THAT TAGS/UNTAGS SINGLE EVENTS (Channel 2)
Function ProcCheckBoxTag2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave Tags2, CursorTags2_Y
	Nvar EventNumber2, TaggedEvents2, NumberofEventsch2, BTDE_2, BUDE_2, CheckedTagAll2
	Tags2[EventNumber2]=checked
	CursorTags2_Y = checked
	TaggedEvents2 = sum(Tags2,0,numpnts(Tags2)-1)
	Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==0)
		BTDE_2 = 0; Checkbox BTDE2 value = 0
	endif
	if (CheckedTagAll2 == 1)
		If ( (sum(Tags2,0,numpnts(Tags2)-1)) < NumberofEventsch2)
			CheckedTagAll2 = 0; CheckBox CheckBoxTagAll2 value=0
		endif
	endif
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==NumberofEventsch2)
		BUDE_2 = 0; Checkbox BUDE2 value = 0
		CheckedTagAll2 = 1; CheckBox CheckBoxTagAll2 value=1
	endif
End

// CHECKBOX THAT TAGS/UNTAGS ALL DETECTED EVENTS (Channel 1)
Function ProcCheckBoxTagAll1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave Tags1, CursorTags1_Y
	Nvar TaggedEvents1, CheckedTagAll1, TaggedEvents1, BTDE_1, BUDE_1, NumberofEventsch1
	if (CheckedTagAll1 == 0)
		Tags1=1
		TaggedEvents1 = numpnts(Tags1)
		Execute "ValDisplay DisplayTaggedEvents1 value=" + num2str(TaggedEvents1)
		CheckBox CheckBoxTag1 value=1
		CursorTags1_Y = 1
	else
		Tags1=0
		TaggedEvents1=0
		Execute "ValDisplay DisplayTaggedEvents1 value=" + num2str(TaggedEvents1)
		CheckBox CheckBoxTag1 value=0
		CursorTags1_Y = 0
	endif
		CheckedTagAll1 = 1 - CheckedTagAll1
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==0)
		BTDE_1 = 0; Checkbox BTDE1 value = 0
	endif
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==NumberofEventsch1)
		BUDE_1 = 0; Checkbox BUDE1 value = 0
	endif
End

// CHECKBOX THAT TAGS/UNTAGS ALL DETECTED EVENTS (Channel 2)
Function ProcCheckBoxTagAll2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave Tags2, CursorTags2_Y
	Nvar TaggedEvents2, CheckedTagAll2, TaggedEvents2, BTDE_2, BUDE_2, NumberofEventsch2
	if (CheckedTagAll2 == 0)
		Tags2=1
		TaggedEvents2 = numpnts(Tags2)
		Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
		CheckBox CheckBoxTag2 value=1
		CursorTags2_Y = 1
	else
		Tags2=0
		TaggedEvents2=0
		Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
		CheckBox CheckBoxTag2 value=0
		CursorTags2_Y = 0
	endif
		CheckedTagAll2 = 1 - CheckedTagAll2
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==0)
		BTDE_2 = 0; Checkbox BTDE2 value = 0
	endif
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==NumberofEventsch2)
		BUDE_2 = 0; Checkbox BUDE2 value = 0
	endif
End

// Tag or untag all events within temporal limits
Function TagOrUntagRange(ChannelNb, TagOrUntag, TagsW, OccurrenceW, CursorW, LimitBegin, LimitEnd, EvntNb, TaggedEvnt, CheckedTagAll, NbOfEvtsCh, BUDE, BTDE) // TagOrUntag is a variable that takes value 1 for tag or 0 for untag
	Variable ChannelNb, TagOrUntag
	Wave TagsW, OccurrenceW, CursorW
	Variable LimitBegin, LimitEnd, &EvntNb, &TaggedEvnt, &CheckedTagAll, &NbOfEvtsCh, &BUDE, &BTDE
	
	//Locals
	Variable FirstEvent, LastEvent

	FirstEvent = X_finder(OccurrenceW, LimitBegin, SupBorder=1) // Take event with occurring time closest to LimitBegin1 and superior or equal to LimitBegin1
	LastEvent=X_finder(OccurrenceW, LimitEnd, InfBorder=1)
	
	TagsW[FirstEvent,LastEvent]=TagOrUntag // If TagOr Untag is 1 events will be tagged If 0 they'll be untagged
	
	Execute "CheckBox CheckBoxTag" + num2Str(ChannelNb)+ " value=Tags" + num2str(ChannelNb) + "[" + num2str(EvntNb) + "]"
	Execute "CheckBox CheckBoxTagAll" + num2str(ChannelNb) + " value = 0"
	CheckedTagAll = 0
	TaggedEvnt = sum(TagsW,0,numpnts(TagsW)-1)
	CursorW = TagsW[EvntNb]
	Execute "ValDisplay DisplayTaggedEvents" + num2str(ChannelNb) + " value=" + num2str(TaggedEvnt)
	
	
	If ((TagOrUntag == 1) && (BUDE == 1)) // If for untagged events
		BUDE = 0
		Execute "Checkbox BUDE" + num2str(ChannelNb) +" value = 0"
	ElseIf ((TagOrUntag == 0) && (BTDE == 1))
		BTDE = 0
		Execute "Checkbox BTDE" + num2str(ChannelNb) +" value = 0"
	endif
		
	If ( (TaggedEvnt==NbOfEvtsCh) || (TaggedEvnt==0)) // if all events are tagged or untagged
		CheckedTagAll = TagOrUntag // The CehckBox CheckedTagAll will take 0 if we were untagging or 1 if we were tagging (Everything tagged must correspond to we were tagging) and conversly
		Execute "CheckBox CheckBoxTagAll" + num2str(Channelnb) + " value=" + num2str(TagOrUntag)
	endif

End



Function ProcTagOrUntagRange(ChannelNb, TagOrUntag) 
	Variable ChannelNb, TagOrUntag
	
	// For ch1
	Wave Tags1, Occurrence1, CursorTags1_Y
	Nvar LimitBegin1, LimitEnd1, EventNumber1, TaggedEvents1, CheckedTagAll1, NumberofEventsch1, BUDE_1, BTDE_1
	// Create locals to be called by reference (cannot call a global by reference)
	Variable EvntNb1 = EventNumber1, TaggedEvnt1=TaggedEvents1, CheckedTagAllLoc1=CheckedTagAll1, NbOfEvtsCh1=NumberOfEventsch1, BUDE1loc=BUDE_1, BTDE1Loc=BTDE_1
	
	// For Ch2
	Wave Tags2, Occurrence2, CursorTags2_Y
	Nvar LimitBegin2, LimitEnd2, EventNumber2, TaggedEvents2, CheckedTagAll2, NumberofEventsch2, BUDE_2, BTDE_2
	// Create locals to be called by reference (cannot call a global by reference)
	Variable EvntNb2 = EventNumber2, TaggedEvnt2=TaggedEvents2, CheckedTagAllLoc2=CheckedTagAll2, NbOfEvtsCh2=NumberOfEventsch2, BUDE2loc=BUDE_2, BTDE2Loc=BTDE_2
	
	
	If (ChannelNb == 1)
		TagOrUntagRange(1, TagOrUntag, Tags1, Occurrence1, CursorTags1_Y, LimitBegin1, LimitEnd1, EvntNb1, TaggedEvnt1, CheckedTagAllLoc1, nbOfEvtsCh1, BUDE1Loc, BTDE1Loc)
		// Put values back into the globals
		TaggedEvents1 = TaggedEvnt1
		CheckedTagAll1 = CheckedTagAllLoc1
		BUDE_1 = BUDE1Loc
		BTDE_1 = BTDE1Loc
	ElseIf (ChannelNb == 2)
		TagOrUntagRange(2, TagOrUntag, Tags2, Occurrence2, CursorTags2_Y, LimitBegin2, LimitEnd2, EvntNb2, TaggedEvnt2, CheckedTagAllLoc2, nbOfEvtsCh2, BUDE2Loc, BTDE2Loc)
		// Put values back into the globals
		TaggedEvents2 = TaggedEvnt2
		CheckedTagAll2 = CheckedTagAllLoc2
		BUDE_2 = BUDE2Loc
		BTDE_2 = BTDE2Loc
	EndIf	
End

Function ProcTagRange1(CtrlName) : ButtonControl
	String CtrlName
	
	ProcTagOrUntagRange(1, 1)
End

Function ProcTagRange2(CtrlName) : ButtonControl
	String CtrlName
	
	ProcTagOrUntagRange(2, 1)
End

Function ProcUntagRange1(CtrlName) : ButtonControl
	String CtrlName
	
	ProcTagOrUntagRange(1, 0)
End

Function ProcUntagRange2(CtrlName) : ButtonControl
	String CtrlName
	
	ProcTagOrUntagRange(2, 0)
End

//----------------------------------- tagging untagging w/ Indexes---------------------------------------------
// UNTAG ALL EVENTS WITHIN A RANGE OF NUMBERS (Channel 1)
Function ProcUntagRange1_pnts(ctrlName) : ButtonControl
	String ctrlName
	Wave Tags1, Occurrence1, CursorTags1_Y
	Nvar LimitBegin1_pnts, LimitEnd1_pnts, EventNumber1, TaggedEvents1, CheckedTagAll1, NumberofEventsch1, BTDE_1
	Tags1[LimitBegin1_pnts,LimitEnd1_pnts] = 0
	CursorTags1_Y = Tags1[EventNumber1]
	CheckBox CheckBoxTag1 value=Tags1[EventNumber1]
	TaggedEvents1 = sum(Tags1,0,numpnts(Tags1)-1)
	Execute "ValDisplay DisplayTaggedEvents1 value=" + num2str(TaggedEvents1)
	If (BTDE_1 == 1)
		BTDE_1 = 0; Checkbox BTDE1 value = 0
	endif
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==0)
		CheckedTagAll1 = 0; CheckBox CheckBoxTagAll1 value=0
	endif
End

// UNTAG ALL EVENTS WITHIN A RANGE OF NUMBERS (Channel 2)
Function ProcUntagRange2_pnts(ctrlName) : ButtonControl
	String ctrlName
	Wave Tags2, Occurrence2, CursorTags2_Y
	Nvar LimitBegin2_pnts, LimitEnd2_pnts, EventNumber2, TaggedEvents2, CheckedTagAll2, NumberofEventsch2, BTDE_2
	Tags2[LimitBegin2_pnts,LimitEnd2_pnts] = 0
	CursorTags2_Y = Tags2[EventNumber2]
	CheckBox CheckBoxTag2 value=Tags2[EventNumber2]
	TaggedEvents2 = sum(Tags2,0,numpnts(Tags2)-1)
	Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
	If (BTDE_2 == 1)
		BTDE_2 = 0; Checkbox BTDE2 value = 0
	endif
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==0)
		CheckedTagAll2 = 0; CheckBox CheckBoxTagAll2 value=0
	endif
End

// SWITCH ON/OFF OF THE FILTER FOR RISE TIME - FOR CALCULATION OF THE AVERAGE (Channel 1)
Function ProcCheckRiseTime1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckRT20801
	CheckRT20801 = checked
End

// SWITCH ON/OFF OF THE FILTER FOR RISE TIME - FOR CALCULATION OF THE AVERAGE (Channel 2)
Function ProcCheckRiseTime2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckRT20802
	CheckRT20802 = checked
End


// SWITCH ON/OFF OF THE FILTER FOR BASELINE SD - FOR CALCULATION OF THE AVERAGE (Channel 1)
Function ProcCheckBaselineSD1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckBslnSD1
	CheckBslnSD1 = checked
End

// SWITCH ON/OFF OF THE FILTER FOR BASELINE SD - FOR CALCULATION OF THE AVERAGE (Channel 2)
Function ProcCheckBaselineSD2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckBslnSD2
	CheckBslnSD2 = checked
End

// DRAWS THE RISE TIME HISTOGRAM (TAGGED EVENTS ONLY - Channel 1)
Function ProcButtonShowRT2080histo1(ctrlName) : ButtonControl
	String ctrlName
	Wave RiseTime20801, Tags1
	if (sum(Tags1, 0, numpnts(tags1)-1) == 0 )
		Abort "Please tag events first. This function only displays the rise time histogram of tagged events."
	endif
	FindLevels/P/Q Tags1 1
	Wave W_FindLevels
	Make/o/N=0 RTvalues1
	Variable i
	for (i=0 ; i < numpnts(W_FindLevels) ; i+=1)
		InsertPoints i, 1, RTvalues1
		RTvalues1[i] = RiseTime20801[W_FindLevels[i]]
	endfor
	Duplicate/o RTvalues1 RiseTime2080Histo1
	Histogram/B=1 RTvalues1,RiseTime2080Histo1
	Killwaves RTvalues1
	DoWindow/K Window_RT_Histo1
	Display RiseTime2080Histo1 as "20-80% Rise Time Histogram"; ModifyGraph mode=5
	DoWindow/C Window_RT_Histo1; ModifyGraph standoff=0; ModifyGraph rgb=(0,12800,52224)
	ControlBar/W=Window_RT_Histo1 35
	Button CloseWindow_RT1 pos={480,10},size={40,20},title="close", proc=procCloseWindow_RT1
	GetAxis/Q bottom ; SetAxis bottom V_min - ((V_min+V_max)/2 - V_min), V_max + ((V_min+V_max)/2 - V_min)
End

// DRAWS THE RISE TIME HISTOGRAM (TAGGED EVENTS ONLY - Channel 2)
Function ProcButtonShowRT2080histo2(ctrlName) : ButtonControl
	String ctrlName
	Wave RiseTime20802, Tags2
	if (sum(Tags2, 0, numpnts(tags2)-1) == 0 )
		Abort "Please tag events first. This function only displays the rise time histogram of tagged events."
	endif
	FindLevels/P/Q Tags2 1
	Wave W_FindLevels
	Make/o/N=0 RTvalues2
	Variable i
	for (i=0 ; i < numpnts(W_FindLevels) ; i+=1)
		InsertPoints i, 1, RTvalues2
		RTvalues2[i] = RiseTime20802[W_FindLevels[i]]
	endfor
	Duplicate/o RTvalues2 RiseTime2080Histo2
	Histogram/B=1 RTvalues2,RiseTime2080Histo2
	Killwaves RTvalues2
	DoWindow/K Window_RT_Histo2
	Display RiseTime2080Histo2 as "20-80% Rise Time Histogram"; ModifyGraph mode=5
	DoWindow/C Window_RT_Histo2; ModifyGraph standoff=0; ModifyGraph rgb=(0,12800,52224)
	ControlBar/W=Window_RT_Histo2 35
	Button CloseWindow_RT2 pos={480,10},size={40,20},title="close", proc=procCloseWindow_RT2
	GetAxis/Q bottom ; SetAxis bottom V_min - ((V_min+V_max)/2 - V_min), V_max + ((V_min+V_max)/2 - V_min)
End

// DRAWS THE BASELINE SD HISTOGRAM (TAGGED EVENTS ONLY - Channel 1)
Function ProcButtonShowBslnSDHisto1(ctrlName) : ButtonControl
	String ctrlName
	Wave Tags1, DetectedLevels1
	if (sum(Tags1, 0, numpnts(tags1)-1) == 0 )
		Abort "Please tag events first. This function only displays the baseline SD histogram of tagged events."
	endif
	FindLevels/Q Tags1 1
	Wave W_FindLevels
	Nvar Baseline1
	Svar FileName1
	Variable i
	Make/o/N=0 BaselineSD1
	For (i=0 ; i < numpnts(W_FindLevels) ; i+=1)
		WaveStats/Q/R=(DetectedLevels1[W_FindLevels[i]] - Baseline1*.001, DetectedLevels1[W_FindLevels[i]]) $FileName1
		InsertPoints i, 1, BaselineSD1
		BaselineSD1[i] = V_sdev
	Endfor
	Duplicate/o BaselineSD1, BaselineSDHisto1 ; Histogram/B=1 BaselineSD1,BaselineSDHisto1
	DoWindow/K Window_BslnSD_histo1
	Display BaselineSDHisto1 as "Baseline SD Histogram"; ModifyGraph mode=5
	DoWindow/C Window_BslnSD_histo1; ModifyGraph standoff=0; ModifyGraph rgb=(0,12800,52224)
	ControlBar/W=Window_BslnSD_histo1 35
	Button CloseWindow_BslnSD1 pos={480,10},size={40,20},title="close", proc=procCloseWindow_BslnSD1
	GetAxis/Q bottom ; SetAxis bottom V_min - ((V_min+V_max)/2 - V_min), V_max + ((V_min+V_max)/2 - V_min)
	ShowInfo
End

// DRAWS THE BASELINE SD HISTOGRAM (TAGGED EVENTS ONLY - Channel 2)
Function ProcButtonShowBslnSDHisto2(ctrlName) : ButtonControl
	String ctrlName
	Wave Tags2, DetectedLevels2
	if (sum(Tags2, 0, numpnts(tags2)-1) == 0 )
		Abort "Please tag events first. This function only displays the baseline SD histogram of tagged events."
	endif
	FindLevels/Q Tags2 1
	Wave W_FindLevels
	Nvar Baseline2
	Svar FileName2
	Variable i
	Make/o/N=0 BaselineSD2
	For (i=0 ; i < numpnts(W_FindLevels) ; i+=1)
		WaveStats/Q/R=(DetectedLevels2[W_FindLevels[i]] - Baseline2*.001, DetectedLevels2[W_FindLevels[i]]) $FileName2
		InsertPoints i, 1, BaselineSD2
		BaselineSD2[i] = V_sdev
	Endfor
	Duplicate/o BaselineSD2, BaselineSDHisto2 ; Histogram/B=1 BaselineSD2,BaselineSDHisto2
	DoWindow/K Window_BslnSD_histo2
	Display BaselineSDHisto2 as "Baseline SD Histogram"; ModifyGraph mode=5
	DoWindow/C Window_BslnSD_histo2; ModifyGraph standoff=0; ModifyGraph rgb=(0,12800,52224)
	ControlBar/W=Window_BslnSD_histo2 35
	Button CloseWindow_BslnSD2 pos={480,10},size={40,20},title="close", proc=procCloseWindow_BslnSD2
	GetAxis/Q bottom ; SetAxis bottom V_min - ((V_min+V_max)/2 - V_min), V_max + ((V_min+V_max)/2 - V_min)
	ShowInfo
End

Function procCloseWindow_RT1(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K Window_RT_Histo1
end

Function procCloseWindow_RT2(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K Window_RT_Histo2
end

Function procCloseWindow_BslnSD1(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K Window_BslnSD_histo1
end

Function procCloseWindow_BslnSD2(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K Window_BslnSD_histo2
end

// FUNCTION CALLED BY BUTTON "ButtonSupEvents" IN DETECTED EVENTS 1
// Superimpose all tagged events
// Filters (baseline SD and rise time) also apply here on top of the tags
Function ProcButtonSupEvents1(ctrlName) : ButtonControl
	String ctrlName
	// Same objects declared as in the average function below
	Nvar CheckRT20801, CheckBslnSD1
	Nvar TaggedEvents1, CheckIncludeAv1
	Nvar CheckAlign1Foot, CheckAlign1Occ, CheckAlign1Peak
	Nvar RTinf1, RTSup1
	Nvar BslnSDinf1, BslnSDsup1, ShowCh2
	Nvar Baseline1, Decay1, SI1, SI2, ACFT1, ACFT1ch2
	Svar FileName1, FileName2
	Wave Tags1, RiseTime20801, DetectedLevels1, Occurrence1
	Wave BaselineY1
	Wave EventFootTime1, RealPeakPosition1
	Newdatafolder/O root:spacan:events	// where the waves of individual events will be stored
	Variable i
	if (sum(Tags1,0,numpnts(Tags1)-1) == 0)
		Abort "No tagged events. Only tagged events will be displayed."
	endif
	
	FindLevels/Q Tags1 1 ; Wave W_FindLevels
	Duplicate/o Tags1 Tags_modified1
	For (i = numpnts(W_FindLevels)-1 ; i > -1 ; i-=1)
		if (Numtype(Occurrence1[W_FindLevels[i]]) == 2)		// Removing bad events (NaN)
			Tags_modified1[W_FindLevels[i]] = 0
		endif
		if (Occurrence1[W_FindLevels[i]] < Baseline1*0.001 || Occurrence1[W_FindLevels[i]] > pnt2x($Filename1,numpnts($Filename1)-1) - Decay1*0.001)
			Tags_modified1[W_FindLevels[i]] = 0
		endif
		if (CheckRT20801 == 1)
			if (RiseTime20801[W_FindLevels[i]] < RTinf1*.001 || RiseTime20801[W_FindLevels[i]] > RTsup1*.001)
				Tags_modified1[W_FindLevels[i]] = 0		// Filter for rise time
			endif
		endif
		if (CheckBslnSD1 == 1)
			WaveStats/Q/R=(DetectedLevels1[W_FindLevels[i]] - Baseline1*0.001, DetectedLevels1[W_FindLevels[i]]) $FileName1
			if (V_sdev < BslnSDinf1 || V_sdev > BslnSDsup1)		// Filter for baseline
				Tags_modified1[W_FindLevels[i]] = 0
			endif
		endif
	Endfor
	FindLevels/Q Tags_modified1 1
	if (V_flag == 2)
		Abort "No event compatible with filter(s) range(s)."
	endif
	Wave W_FindLevels
	// So far same as function average
	// Here the game is to store every event in a wave and display it on top of the others
	// Will have to create an empty graph here.
	
	String TmpWaveName1, TmpWaveName2
	DoWindow/K SupEvents1
	display/W=(10,20,400,400) as "Superimposed tagged events - channel 1"
	DoWindow/C SupEvents1
	if (ShowCh2 == 1)
		Display/HOST=SupEvents1/N=SupEvents11/W=(0.01,0.01,0.99,0.49)
		Display/HOST=SupEvents1/N=SupEvents12/W=(0.01,0.51,0.99,0.99)
	endif

	// Control Bar
	ControlBar 50
	ValDisplay DispNumTaggedEvents1		pos={10,5},size={200,20},title="Number of tagged events (ch1): "
	Execute "ValDisplay DispNumTaggedEvents1 value=" + num2str(TaggedEvents1)
	ValDisplay DispNumAvEvents1		pos={10,25},size={200,20},title="Number of events superimposed: "
	Execute "ValDisplay DispNumAvEvents1 value=" + num2str(sum(Tags_modified1,0,numpnts(Tags_modified1)))
	
	For (i = 0 ; i < Numpnts(W_FindLevels) ; i+=1)
		TmpWaveName1 = "root:spacan:events:Ch1Event" + num2str(W_FindLevels[i])
		if (CheckAlign1Occ == 1)	// means align on half-rise
			Duplicate/o/R=(Occurrence1[W_FindLevels[i]] - Baseline1*.001, Occurrence1[W_FindLevels[i]] + Decay1*.001) $FileName1 TEMPEVENT
		endif
		if (CheckAlign1Foot == 1)	// means align on foot
			Duplicate/o/R=(EventFootTime1[W_FindLevels[i]] - Baseline1*.001, EventFootTime1[W_FindLevels[i]] + Decay1*.001) $FileName1 TEMPEVENT
		endif
		if (CheckAlign1Peak == 1)	// means align on peak time
			Duplicate/o/R=(RealPeakPosition1[W_FindLevels[i]] - Baseline1*.001, RealPeakPosition1[W_FindLevels[i]] + Decay1*.001) $FileName1 TEMPEVENT
		endif
		SetScale/P x -Baseline1*.001, SI1*1e-6, "s", TEMPEVENT
		TEMPEVENT = TEMPEVENT - BaselineY1[W_FindLevels[i]]
		Duplicate/o TEMPEVENT $TmpWaveName1
		if (ShowCh2 == 1)
			Appendtograph/W=SupEvents1#SupEvents11 $TmpWaveName1
			TmpWaveName2 = "root:spacan:events:Ch2Event" + num2str(W_FindLevels[i])
			if (CheckAlign1Occ == 1)	// means align on half-rise
				Duplicate/o/R=(Occurrence1[W_FindLevels[i]] - Baseline1*.001, Occurrence1[W_FindLevels[i]] + Decay1*.001) $FileName2 TEMPEVENT
			endif
			if (CheckAlign1Foot == 1)	// means align on foot
				Duplicate/o/R=(EventFootTime1[W_FindLevels[i]] - Baseline1*.001, EventFootTime1[W_FindLevels[i]] + Decay1*.001) $FileName2 TEMPEVENT
			endif
			if (CheckAlign1Peak == 1)	// means align on peak time
				Duplicate/o/R=(RealPeakPosition1[W_FindLevels[i]] - Baseline1*.001, RealPeakPosition1[W_FindLevels[i]] + Decay1*.001) $FileName2 TEMPEVENT
			endif
			SetScale/P x -Baseline1*.001, SI1*1e-6, "s", TEMPEVENT
			Duplicate/o TEMPEVENT $TmpWaveName2
			appendtograph/W=SupEvents1#SupEvents12 $TmpWaveName2
		else
			Appendtograph/W=SupEvents1 $TmpWaveName1
		endif
	Endfor
	
	if (ShowCh2 == 1)
		ModifyGraph/W=SupEvents1#SupEvents11 rgb=(0,0,52224)
	else
		ModifyGraph/W=SupEvents1 rgb=(0,0,52224)
	endif
	Killwaves TEMPEVENT

	// Here add the average if CheckIncludeAv1 = 1
	if (CheckIncludeAv1 == 1)
		Nvar NumberOfAveragedEvents1
		NumberOfAveragedEvents1 = sum(Tags_modified1,0,numpnts(Tags_modified1))
		if (CheckAlign1Occ == 1)	// means align on half-rise
			Duplicate/o/R=(Occurrence1[W_FindLevels[0]] - Baseline1*.001, Occurrence1[W_FindLevels[0]] + Decay1*.001) $FileName1 TEMPWAVE1
		endif
		if (CheckAlign1Foot == 1)	// means align on foot
			Duplicate/o/R=(EventFootTime1[W_FindLevels[0]] - Baseline1*.001, EventFootTime1[W_FindLevels[0]] + Decay1*.001) $FileName1 TEMPWAVE1
		endif
		if (CheckAlign1Peak == 1)	// means align on peak
			Duplicate/o/R=(RealPeakPosition1[W_FindLevels[0]] - Baseline1*.001, RealPeakPosition1[W_FindLevels[0]] + Decay1*.001) $FileName1 TEMPWAVE1
		endif		
		if (ShowCh2 == 1)
			if (CheckAlign1Occ == 1)	// means align on half-rise
				Duplicate/o/R=(Occurrence1[W_FindLevels[0]] - Baseline1*.001, Occurrence1[W_FindLevels[0]] + Decay1*.001) $FileName2 TEMPWAVE1ch2
			endif
			if (CheckAlign1Foot == 1)	// means align on foot
				Duplicate/o/R=(EventFootTime1[W_FindLevels[0]] - Baseline1*.001, EventFootTime1[W_FindLevels[0]] + Decay1*.001) $FileName2 TEMPWAVE1ch2
			endif
			if (CheckAlign1Peak == 1)	// means align on peak
				Duplicate/o/R=(RealPeakPosition1[W_FindLevels[0]] - Baseline1*.001, RealPeakPosition1[W_FindLevels[0]] + Decay1*.001) $FileName2 TEMPWAVE1ch2
			endif
			
		endif
		TEMPWAVE1 = TEMPWAVE1 - BaselineY1[W_FindLevels[0]]
		For (i = 1 ; i < Numpnts(W_FindLevels) ; i+=1)
			if (CheckAlign1Occ == 1)	// means align on half-rise
				Duplicate/o/R=(Occurrence1[W_FindLevels[i]] - Baseline1*.001, Occurrence1[W_FindLevels[i]] + Decay1*.001) $FileName1 TEMPWAVE2
			endif
			if (CheckAlign1Foot == 1)	// means align on foot
				Duplicate/o/R=(EventFootTime1[W_FindLevels[i]] - Baseline1*.001, EventFootTime1[W_FindLevels[i]] + Decay1*.001) $FileName1 TEMPWAVE2
			endif
			if (CheckAlign1Peak == 1)	// means align on peak
				Duplicate/o/R=(RealPeakPosition1[W_FindLevels[i]] - Baseline1*.001, RealPeakPosition1[W_FindLevels[i]] + Decay1*.001) $FileName1 TEMPWAVE2
			endif
			TEMPWAVE2 = TEMPWAVE2 - BaselineY1[W_FindLevels[i]]
			TEMPWAVE1 = TEMPWAVE1 + TEMPWAVE2
			if (ShowCh2 == 1)
				if (CheckAlign1Occ == 1)	// means align on half-rise
					Duplicate/o/R=(Occurrence1[W_FindLevels[i]] - Baseline1*.001, Occurrence1[W_FindLevels[i]] + Decay1*.001) $FileName2 TEMPWAVE2ch2
				endif
				if (CheckAlign1Occ == 1)	// means align on half-rise
					Duplicate/o/R=(EventFootTime1[W_FindLevels[i]] - Baseline1*.001, EventFootTime1[W_FindLevels[i]] + Decay1*.001) $FileName2 TEMPWAVE2ch2
				endif
				if (CheckAlign1Peak == 1)	// means align on peak
					Duplicate/o/R=(RealPeakPosition1[W_FindLevels[i]] - Baseline1*.001, RealPeakPosition1[W_FindLevels[i]] + Decay1*.001) $FileName2 TEMPWAVE2ch2
				endif
				TEMPWAVE1ch2 = TEMPWAVE1ch2 + TEMPWAVE2ch2
			endif
		Endfor
		TEMPWAVE1 = TEMPWAVE1 / NumberOfAveragedEvents1
		Duplicate/o TEMPWAVE1 AvEvent1 ; KillWaves TEMPWAVE1
		SetScale/P x -Baseline1*.001, SI1*1e-6, "s", AvEvent1
		if (ShowCh2 == 1)
			TEMPWAVE1ch2 = TEMPWAVE1ch2 / NumberOfAveragedEvents1
			Duplicate/o TEMPWAVE1ch2 AvEvent1ch2 ; KillWaves TEMPWAVE1ch2
			SetScale/P x -Baseline1*.001, SI2*1e-6, "s", AvEvent1ch2
			Appendtograph/W=SupEvents1#SupEvents11 AvEvent1; Modifygraph/W=SupEvents1#SupEvents11 rgb(AvEvent1)=(0,0,0)
			Appendtograph/W=SupEvents1#SupEvents12 AvEvent1ch2; Modifygraph/W=SupEvents1#SupEvents12 rgb(AvEvent1ch2)=(0,0,0)
		else
			Appendtograph/W=SupEvents1 AvEvent1; Modifygraph/W=SupEvents1 rgb(AvEvent1)=(0,0,0)
		endif
	endif

	// Add here some more buttons
	Button ButtonCloseSupEvents1		pos={465,3},size={40,20},	title="Close", proc=ProcButtonCloseSupEvents1
	if (ShowCh2 == 1)
		Button ButtonSupEvents1Sync			pos={420,3},size={40,20},	title="Sync", 	proc=ProcButtonSupEvents1Sync
	endif
	CheckBox CheckBoxSupEvents1Grid	pos={375,7},size={40,20},	title="Grid", 	proc=ProcSupEvents1Grid
end

Function ProcSupEvents1Grid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ShowCh2
	if (checked == 1)
		if (ShowCh2 == 1)
			ModifyGraph/W=SupEvents1#SupEvents11 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=SupEvents1#SupEvents11 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
			ModifyGraph/W=SupEvents1#SupEvents12 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=SupEvents1#SupEvents12 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		else
			ModifyGraph/W=SupEvents1 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=SupEvents1 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
	else
		if (ShowCh2 == 1)
			ModifyGraph/W=SupEvents1#SupEvents11 grid(left)=0
			ModifyGraph/W=SupEvents1#SupEvents11 grid(bottom)=0
			ModifyGraph/W=SupEvents1#SupEvents12 grid(left)=0
			ModifyGraph/W=SupEvents1#SupEvents12 grid(bottom)=0
		else
			ModifyGraph/W=SupEvents1 grid(left)=0
			ModifyGraph/W=SupEvents1 grid(bottom)=0
		endif
	endif	
End

Function ProcButtonSupEvents1Sync(ctrlName) : ButtonControl
	String ctrlName
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=SupEvents1#SupEvents11 bottom
		SetAxis/W=SupEvents1#SupEvents12 bottom V_min,V_max
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=SupEvents1#SupEvents12 bottom
		SetAxis/W=SupEvents1#SupEvents11 bottom V_min,V_max
	endif
end

Function ProcButtonCloseSupEvents1(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K SupEvents1
end

Function ProcCBIncludeAv1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIncludeAv1
	CheckIncludeAv1 = checked
End

// FUNCTION THAT CALCULATES THE AVERAGE EVENT (Channel 1), CALLED BY THE BUTTON "AVERAGE"
Function ProcButtonAverage1(ctrlName) : ButtonControl
	String ctrlName
	Nvar CheckRT20801, CheckBslnSD1
	Nvar RTinf1, RTSup1
	Nvar BslnSDinf1, BslnSDsup1, ShowCh2
	Nvar Baseline1, Decay1, SI1, SI2, ACFT1, ACFT1ch2
	Svar FileName1, FileName2
	Wave Tags1, RiseTime20801, DetectedLevels1, Occurrence1
	Wave BaselineY1
	Variable i
	if (sum(Tags1,0,numpnts(Tags1)-1) == 0)
		Abort "No tagged events. The average is calculated from tagged events only."
	endif
	FindLevels/Q Tags1 1 ; Wave W_FindLevels
	Duplicate/o Tags1 Tags_modified1
	For (i = numpnts(W_FindLevels)-1 ; i > -1 ; i-=1)
		if (Numtype(Occurrence1[W_FindLevels[i]]) == 2)		// Removing bad events (NaN)
			Tags_modified1[W_FindLevels[i]] = 0
		endif
		if (Occurrence1[W_FindLevels[i]] < Baseline1*0.001 || Occurrence1[W_FindLevels[i]] > pnt2x($Filename1,numpnts($Filename1)-1) - Decay1*0.001)
			Tags_modified1[W_FindLevels[i]] = 0
		endif
		if (CheckRT20801 == 1)
			if (RiseTime20801[W_FindLevels[i]] < RTinf1*.001 || RiseTime20801[W_FindLevels[i]] > RTsup1*.001)
				Tags_modified1[W_FindLevels[i]] = 0		// Filter for rise time
			endif
		endif
		if (CheckBslnSD1 == 1)
			WaveStats/Q/R=(DetectedLevels1[W_FindLevels[i]] - Baseline1*0.001, DetectedLevels1[W_FindLevels[i]]) $FileName1
			if (V_sdev < BslnSDinf1 || V_sdev > BslnSDsup1)		// Filter for baseline
				Tags_modified1[W_FindLevels[i]] = 0
			endif
		endif
	Endfor
	FindLevels/Q Tags_modified1 1
	if (V_flag == 2)
		Abort "No event compatible with filter(s) range(s)."
	endif
	Wave W_FindLevels
	Nvar NumberOfAveragedEvents1
	NumberOfAveragedEvents1 = sum(Tags_modified1,0,numpnts(Tags_modified1))
	Duplicate/o/R=(Occurrence1[W_FindLevels[0]] - Baseline1*.001, Occurrence1[W_FindLevels[0]] + Decay1*.001) $FileName1 TEMPWAVE1
	if (ShowCh2 == 1)
		Duplicate/o/R=(Occurrence1[W_FindLevels[0]] - Baseline1*.001, Occurrence1[W_FindLevels[0]] + Decay1*.001) $FileName2 TEMPWAVE1ch2
	endif
	TEMPWAVE1 = TEMPWAVE1 - BaselineY1[W_FindLevels[0]]
	For (i = 1 ; i < Numpnts(W_FindLevels) ; i+=1)
		Duplicate/o/R=(Occurrence1[W_FindLevels[i]] - Baseline1*.001, Occurrence1[W_FindLevels[i]] + Decay1*.001) $FileName1 TEMPWAVE2
		TEMPWAVE2 = TEMPWAVE2 - BaselineY1[W_FindLevels[i]]
		TEMPWAVE1 = TEMPWAVE1 + TEMPWAVE2
		if (ShowCh2 == 1)
			Duplicate/o/R=(Occurrence1[W_FindLevels[i]] - Baseline1*.001, Occurrence1[W_FindLevels[i]] + Decay1*.001) $FileName2 TEMPWAVE2ch2
			TEMPWAVE1ch2 = TEMPWAVE1ch2 + TEMPWAVE2ch2
		endif
	Endfor
	TEMPWAVE1 = TEMPWAVE1 / NumberOfAveragedEvents1
	Duplicate/o TEMPWAVE1 AverageEvent1 ; KillWaves TEMPWAVE1
	SetScale/P x -Baseline1*.001, SI1*1e-6, "s", AverageEvent1
	if (ShowCh2 == 1)
		TEMPWAVE1ch2 = TEMPWAVE1ch2 / NumberOfAveragedEvents1
		Duplicate/o TEMPWAVE1ch2 AverageEvent1ch2 ; KillWaves TEMPWAVE1ch2
		SetScale/P x -Baseline1*.001, SI2*1e-6, "s", AverageEvent1ch2
		
		//Charly **************************************************** Here I substract baseline from AverageEvent1ch2
			
			Variable substracter = mean(AverageEvent1ch2, -(7 * 0.001), -(4 * 0.001))
			AverageEvent1ch2 -= substracter
					
		//Charly End of changes***********************************************
		
	endif
	if (ACFT1 == 1)	// if DetectedEvents1#Average1 exists
		KillWindow DetectedEvents1#Average1
	else
		ACFT1 = 1
	endif
	Display/HOST=DetectedEvents1/N=Average1/W=(0.71,0,0.99,0.49) AverageEvent1
	SetScale/P x -Baseline1*.001,SI1*1e-6,"s", AverageEvent1
	ModifyGraph margin(left)=40; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28
	ModifyGraph rgb(AverageEvent1)=(0,0,65280); ModifyGraph StandOff = 0
	Wavestats/Q AverageEvent1
	SetAxis left 1.2*V_min, -.4*V_min
	SetDrawEnv ycoord = left, linefgc = (47872,47872,47872), dash = 1
	DrawLine 0, 0, 1, 0
	ShowInfo
	if (ShowCh2 == 1)
		if (ACFT1ch2 == 1)	// if DetectedEvents1#Average12 exists
			KillWindow DetectedEvents1#Average12
		else
			ACFT1ch2 = 1
		endif
		Display/HOST=DetectedEvents1/N=Average12/W=(0.71,0.51,0.99,0.99) AverageEvent1ch2
		ModifyGraph margin(left)=40; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28
		ModifyGraph rgb(AverageEvent1ch2)=(65280,0,0); ModifyGraph StandOff = 0
		Wavestats/Q AverageEvent1ch2
		SetAxis/W=DetectedEvents1#Average12 left V_min - (V_max - V_min)*0.2, V_max + (V_max - V_min)*0.2
	endif
	//CheckBox CFPE1 disable=0; CheckBox CFAE1 disable=0
	Execute "Valdisplay NumEventsAveraged1 value=" + num2str(NumberOfAveragedEvents1)
	SetVariable End_Fit1 limits={0, Decay1, 1}
End

// FUNCTION CALLED BY BUTTON "ButtonSupEvents" IN DETECTED EVENTS 2
// Superimpose all tagged events
// Filters (baseline SD and rise time) also apply here on top of the tags
Function ProcButtonSupEvents2(ctrlName) : ButtonControl
	String ctrlName
	// Same objects declared as in the average function below
	Nvar CheckRT20802, CheckBslnSD2
	Nvar TaggedEvents2, CheckIncludeAv2
	Nvar CheckAlign2Foot, CheckAlign2Occ, CheckAlign2Peak
	Nvar RTinf2, RTSup2
	Nvar BslnSDinf2, BslnSDsup2, ShowCh1
	Nvar Baseline2, Decay2, SI2, SI1, ACFT2, ACFT1ch1
	Svar FileName1, FileName2
	Wave Tags2, RiseTime20802, DetectedLevels2, Occurrence2
	Wave BaselineY2
	Wave EventFootTime2, RealPeakPosition2
	Newdatafolder/O root:spacan:events	// where the waves of individual events will be stored
	Variable i
	if (sum(Tags2,0,numpnts(Tags2)-1) == 0)
		Abort "No tagged events. Only tagged events will be displayed."
	endif
	
	FindLevels/Q Tags2 1 ; Wave W_FindLevels
	Duplicate/o Tags2 Tags_modified2
	For (i = numpnts(W_FindLevels)-1 ; i > -1 ; i-=1)
		if (Numtype(Occurrence2[W_FindLevels[i]]) == 2)		// Removing bad events (NaN)
			Tags_modified2[W_FindLevels[i]] = 0
		endif
		if (Occurrence2[W_FindLevels[i]] < Baseline2*0.001 || Occurrence2[W_FindLevels[i]] > pnt2x($Filename2,numpnts($Filename2)-1) - Decay2*0.001)
			Tags_modified2[W_FindLevels[i]] = 0
		endif
		if (CheckRT20802 == 1)
			if (RiseTime20802[W_FindLevels[i]] < RTinf2*.001 || RiseTime20802[W_FindLevels[i]] > RTsup2*.001)
				Tags_modified2[W_FindLevels[i]] = 0		// Filter for rise time
			endif
		endif
		if (CheckBslnSD2 == 1)
			WaveStats/Q/R=(DetectedLevels2[W_FindLevels[i]] - Baseline2*0.001, DetectedLevels2[W_FindLevels[i]]) $FileName2
			if (V_sdev < BslnSDinf2 || V_sdev > BslnSDsup2)		// Filter for baseline
				Tags_modified2[W_FindLevels[i]] = 0
			endif
		endif
	Endfor
	FindLevels/Q Tags_modified2 1
	if (V_flag == 2)
		Abort "No event compatible with filter(s) range(s)."
	endif
	Wave W_FindLevels
	// So far same as function average
	// Here the game is to store every event in a wave and display it on top of the others
	// Will have to create an empty graph here.
	
	String TmpWaveName1, TmpWaveName2
	DoWindow/K SupEvents2
	display/W=(10,20,400,400) as "Superimposed tagged events - channel 2"
	DoWindow/C SupEvents2
	if (ShowCh1 == 1)
		Display/HOST=SupEvents2/N=SupEvents22/W=(0.01,0.01,0.99,0.49)
		Display/HOST=SupEvents2/N=SupEvents21/W=(0.01,0.51,0.99,0.99)
	endif

	// Control Bar
	ControlBar 50
	ValDisplay DispNumTaggedEvents2		pos={10,5},size={200,20},title="Number of tagged events (ch2): "
	Execute "ValDisplay DispNumTaggedEvents2 value=" + num2str(TaggedEvents2)
	ValDisplay DispNumAvEvents2		pos={10,25},size={200,20},title="Number of events superimposed: "
	Execute "ValDisplay DispNumAvEvents2 value=" + num2str(sum(Tags_modified2,0,numpnts(Tags_modified2)))	
	
	For (i = 0 ; i < Numpnts(W_FindLevels) ; i+=1)
		TmpWaveName2 = "root:spacan:events:Ch2Event" + num2str(W_FindLevels[i])
		if (CheckAlign2Occ == 1)	// means align on half-rise
			Duplicate/o/R=(Occurrence2[W_FindLevels[i]] - Baseline2*.001, Occurrence2[W_FindLevels[i]] + Decay2*.001) $FileName2 TEMPEVENT
		endif
		if (CheckAlign2Foot == 1)	// means align on foot
			Duplicate/o/R=(EventFootTime2[W_FindLevels[i]] - Baseline2*.001, EventFootTime2[W_FindLevels[i]] + Decay2*.001) $FileName2 TEMPEVENT
		endif
		if (CheckAlign2Peak == 1)	// means align on peak time
			Duplicate/o/R=(RealPeakPosition2[W_FindLevels[i]] - Baseline2*.001, RealPeakPosition2[W_FindLevels[i]] + Decay2*.001) $FileName2 TEMPEVENT
		endif		
		SetScale/P x -Baseline2*.001, SI2*1e-6, "s", TEMPEVENT
		TEMPEVENT = TEMPEVENT - BaselineY2[W_FindLevels[i]]
		Duplicate/o TEMPEVENT $TmpWaveName2
		if (ShowCh1 == 1)
			Appendtograph/W=SupEvents2#SupEvents22 $TmpWaveName2
			TmpWaveName1 = "root:spacan:events:Ch1Event" + num2str(W_FindLevels[i])
			if (CheckAlign2Occ == 1)	// means align on half-rise
				Duplicate/o/R=(Occurrence2[W_FindLevels[i]] - Baseline2*.001, Occurrence2[W_FindLevels[i]] + Decay2*.001) $FileName1 TEMPEVENT
			endif
			if (CheckAlign2Foot == 1)	// means align on foot
				Duplicate/o/R=(EventFootTime2[W_FindLevels[i]] - Baseline2*.001, EventFootTime2[W_FindLevels[i]] + Decay2*.001) $FileName1 TEMPEVENT
			endif
			if (CheckAlign2Peak == 1)	// means align on peak time
				Duplicate/o/R=(RealPeakPosition2[W_FindLevels[i]] - Baseline2*.001, RealPeakPosition2[W_FindLevels[i]] + Decay2*.001) $FileName1 TEMPEVENT
			endif
			SetScale/P x -Baseline2*.001, SI2*1e-6, "s", TEMPEVENT
			Duplicate/o TEMPEVENT $TmpWaveName1
			appendtograph/W=SupEvents2#SupEvents21 $TmpWaveName1
		else
			Appendtograph/W=SupEvents2 $TmpWaveName2
		endif
	Endfor
	
	if (ShowCh1 == 1)
		ModifyGraph/W=SupEvents2#SupEvents21 rgb=(0,0,52224)
	else
		ModifyGraph/W=SupEvents2 rgb=(65280,0,0)
	endif
	Killwaves TEMPEVENT

	// Here add the average if CheckIncludeAv2 = 1
	if (CheckIncludeAv2 == 1)
		Nvar NumberOfAveragedEvents2
		NumberOfAveragedEvents2 = sum(Tags_modified2,0,numpnts(Tags_modified2))
		if (CheckAlign2Occ == 1)	// means align on half-rise
			Duplicate/o/R=(Occurrence2[W_FindLevels[0]] - Baseline2*.001, Occurrence2[W_FindLevels[0]] + Decay2*.001) $FileName2 TEMPWAVE1
		endif
		if (CheckAlign2Foot == 1)	// means align on foot
			Duplicate/o/R=(EventFootTime2[W_FindLevels[0]] - Baseline2*.001, EventFootTime2[W_FindLevels[0]] + Decay2*.001) $FileName2 TEMPWAVE1
		endif
		if (CheckAlign2Peak == 1)	// means align on peak
			Duplicate/o/R=(RealPeakPosition2[W_FindLevels[0]] - Baseline2*.001, RealPeakPosition2[W_FindLevels[0]] + Decay2*.001) $FileName2 TEMPWAVE1
		endif
		if (ShowCh1 == 1)
			if (CheckAlign2Occ == 1)	// means align on half-rise
				Duplicate/o/R=(Occurrence2[W_FindLevels[0]] - Baseline2*.001, Occurrence2[W_FindLevels[0]] + Decay2*.001) $FileName1 TEMPWAVE1ch1
			endif
			if (CheckAlign2Foot == 1)	// means align on foot
				Duplicate/o/R=(EventFootTime2[W_FindLevels[0]] - Baseline2*.001, EventFootTime2[W_FindLevels[0]] + Decay2*.001) $FileName1 TEMPWAVE1ch1
			endif
			if (CheckAlign2Peak == 1)	// means align on peak
				Duplicate/o/R=(RealPeakPosition2[W_FindLevels[0]] - Baseline2*.001, RealPeakPosition2[W_FindLevels[0]] + Decay2*.001) $FileName1 TEMPWAVE1ch1
			endif
		endif
		TEMPWAVE1 = TEMPWAVE1 - BaselineY2[W_FindLevels[0]]
		For (i = 1 ; i < Numpnts(W_FindLevels) ; i+=1)
			if (CheckAlign2Occ == 1)	// means align on half-rise
				Duplicate/o/R=(Occurrence2[W_FindLevels[i]] - Baseline2*.001, Occurrence2[W_FindLevels[i]] + Decay2*.001) $FileName2 TEMPWAVE2
			endif
			if (CheckAlign2Foot == 1)	// means align on foot
				Duplicate/o/R=(EventFootTime2[W_FindLevels[i]] - Baseline2*.001, EventFootTime2[W_FindLevels[i]] + Decay2*.001) $FileName2 TEMPWAVE2
			endif
			if (CheckAlign2Peak == 1)	// means align on peak
				Duplicate/o/R=(RealPeakPosition2[W_FindLevels[i]] - Baseline2*.001, RealPeakPosition2[W_FindLevels[i]] + Decay2*.001) $FileName2 TEMPWAVE2
			endif			
			TEMPWAVE2 = TEMPWAVE2 - BaselineY2[W_FindLevels[i]]
			TEMPWAVE1 = TEMPWAVE1 + TEMPWAVE2
			if (ShowCh1 == 1)
				if (CheckAlign2Occ == 1)	// means align on half-rise
					Duplicate/o/R=(Occurrence2[W_FindLevels[i]] - Baseline2*.001, Occurrence2[W_FindLevels[i]] + Decay2*.001) $FileName1 TEMPWAVE2ch1
				endif
				if (CheckAlign2Foot == 1)	// means align on foot
					Duplicate/o/R=(EventFootTime2[W_FindLevels[i]] - Baseline2*.001, EventFootTime2[W_FindLevels[i]] + Decay2*.001) $FileName1 TEMPWAVE2ch1
				endif
				if (CheckAlign2Peak == 1)	// means align on peak
					Duplicate/o/R=(RealPeakPosition2[W_FindLevels[i]] - Baseline2*.001, RealPeakPosition2[W_FindLevels[i]] + Decay2*.001) $FileName1 TEMPWAVE2ch1
				endif					
				TEMPWAVE1ch1 = TEMPWAVE1ch1 + TEMPWAVE2ch1
			endif
		Endfor
		TEMPWAVE1 = TEMPWAVE1 / NumberOfAveragedEvents2
		Duplicate/o TEMPWAVE1 AvEvent2 ; KillWaves TEMPWAVE1
		SetScale/P x -Baseline2*.001, SI2*1e-6, "s", AvEvent2
		if (ShowCh1 == 1)
			TEMPWAVE1ch1 = TEMPWAVE1ch1 / NumberOfAveragedEvents2
			Duplicate/o TEMPWAVE1ch1 AvEvent2ch1 ; KillWaves TEMPWAVE1ch1
			SetScale/P x -Baseline2*.001, SI1*1e-6, "s", AvEvent2ch1
			Appendtograph/W=SupEvents2#SupEvents22 AvEvent2; Modifygraph/W=SupEvents2#SupEvents22 rgb(AvEvent2)=(0,0,0)
			Appendtograph/W=SupEvents2#SupEvents21 AvEvent2ch1; Modifygraph/W=SupEvents2#SupEvents21 rgb(AvEvent2ch1)=(0,0,0)
		else
			Appendtograph/W=SupEvents2 AvEvent2; Modifygraph/W=SupEvents2 rgb(AvEvent2)=(0,0,0)
		endif
	endif

	// Add here some more buttons
	Button ButtonCloseSupEvents2		pos={465,3},size={40,20},	title="Close", proc=ProcButtonCloseSupEvents2
	if (ShowCh1 == 1)
		Button ButtonSupEvents2Sync			pos={420,3},size={40,20},	title="Sync", 	proc=ProcButtonSupEvents2Sync
	endif
	CheckBox CheckBoxSupEvents2Grid	pos={375,7},size={40,20},	title="Grid", 	proc=ProcSupEvents2Grid
end

Function ProcSupEvents2Grid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ShowCh1
	if (checked == 1)
		if (ShowCh1 == 1)
			ModifyGraph/W=SupEvents2#SupEvents22 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=SupEvents2#SupEvents22 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
			ModifyGraph/W=SupEvents2#SupEvents21 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=SupEvents2#SupEvents21 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		else
			ModifyGraph/W=SupEvents2 grid(left)=1,gridRGB(left)=(0,0,0)
			ModifyGraph/W=SupEvents2 grid(bottom)=1,gridRGB(bottom)=(0,0,0)
		endif
	else
		if (ShowCh1 == 1)
			ModifyGraph/W=SupEvents2#SupEvents22 grid(left)=0
			ModifyGraph/W=SupEvents2#SupEvents22 grid(bottom)=0
			ModifyGraph/W=SupEvents2#SupEvents21 grid(left)=0
			ModifyGraph/W=SupEvents2#SupEvents21 grid(bottom)=0
		else
			ModifyGraph/W=SupEvents2 grid(left)=0
			ModifyGraph/W=SupEvents2 grid(bottom)=0
		endif
	endif	
End

Function ProcButtonSupEvents2Sync(ctrlName) : ButtonControl
	String ctrlName
	getwindow kwTopWin activeSW
	if (str2num(S_Value[strlen(S_Value)-1]) == 2)
		GetAxis/Q/W=SupEvents2#SupEvents22 bottom
		SetAxis/W=SupEvents2#SupEvents21 bottom V_min,V_max
	endif
	if (str2num(S_Value[strlen(S_Value)-1]) == 1)
		GetAxis/Q/W=SupEvents2#SupEvents21 bottom
		SetAxis/W=SupEvents2#SupEvents22 bottom V_min,V_max
	endif
end

Function ProcButtonCloseSupEvents2(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K SupEvents2
end

Function ProcCBIncludeAv2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIncludeAv2
	CheckIncludeAv2 = checked
End

// FUNCTION THAT CALCULATES THE AVERAGE EVENT (Channel 2), CALLED BY THE BUTTON "AVERAGE"
Function ProcButtonAverage2(ctrlName) : ButtonControl
	String ctrlName
	Nvar CheckRT20802, CheckBslnSD2
	Nvar RTinf2, RTSup2
	Nvar BslnSDinf2, BslnSDsup2, ShowCh1
	Nvar Baseline2, Decay2, SI2, SI1, ACFT2, ACFT2ch1
	Svar FileName2, FileName1
	Wave Tags2, RiseTime20802, DetectedLevels2, Occurrence2
	Wave BaselineY2
	Variable i
	if (sum(Tags2,0,numpnts(Tags2)-1) == 0)
		Abort "No tagged events. The average is calculated from tagged events only."
	endif
	FindLevels/Q Tags2 1 ; Wave W_FindLevels
	Duplicate/o Tags2 Tags_modified2
	For (i = numpnts(W_FindLevels)-1 ; i > -1 ; i-=1)
		if (Numtype(Occurrence2[W_FindLevels[i]]) == 2)		// Removing bad events (NaN)
			Tags_modified2[W_FindLevels[i]] = 0
		endif
		if (Occurrence2[W_FindLevels[i]] < Baseline2*0.001 || Occurrence2[W_FindLevels[i]] > pnt2x($Filename2,numpnts($Filename2)-1) - Decay2*0.001)
			Tags_modified2[W_FindLevels[i]] = 0
		endif
		if (CheckRT20802 == 1)
			if (RiseTime20802[W_FindLevels[i]] < RTinf2*.001 || RiseTime20802[W_FindLevels[i]] > RTsup2*.001)
				Tags_modified2[W_FindLevels[i]] = 0		// Filter for rise time
			endif
		endif
		if (CheckBslnSD2 == 1)
			WaveStats/Q/R=(DetectedLevels2[W_FindLevels[i]] - Baseline2*0.001, DetectedLevels2[W_FindLevels[i]]) $FileName2
			if (V_sdev < BslnSDinf2 || V_sdev > BslnSDsup2)		// Filter for baseline
				Tags_modified2[W_FindLevels[i]] = 0
			endif
		endif
	Endfor
	FindLevels/Q Tags_modified2 1
	if (V_flag == 2)
		Abort "No event compatible with filter(s) range(s)."
	endif
	Wave W_FindLevels
	Nvar NumberOfAveragedEvents2
	NumberOfAveragedEvents2 = sum(Tags_modified2,0,numpnts(Tags_modified2))
	Duplicate/o/R=(Occurrence2[W_FindLevels[0]] - Baseline2*.001, Occurrence2[W_FindLevels[0]] + Decay2*.001) $FileName2 TEMPWAVE1
	if (ShowCh1 == 1)
		Duplicate/o/R=(Occurrence2[W_FindLevels[0]] - Baseline2*.001, Occurrence2[W_FindLevels[0]] + Decay2*.001) $FileName1 TEMPWAVE1ch1
	endif
	TEMPWAVE1 = TEMPWAVE1 - BaselineY2[W_FindLevels[0]]
	For (i = 1 ; i < Numpnts(W_FindLevels) ; i+=1)
		Duplicate/o/R=(Occurrence2[W_FindLevels[i]] - Baseline2*.001, Occurrence2[W_FindLevels[i]] + Decay2*.001) $FileName2 TEMPWAVE2
		TEMPWAVE2 = TEMPWAVE2 - BaselineY2[W_FindLevels[i]]
		TEMPWAVE1 = TEMPWAVE1 + TEMPWAVE2
		if (ShowCh1 == 1)
			Duplicate/o/R=(Occurrence2[W_FindLevels[i]] - Baseline2*.001, Occurrence2[W_FindLevels[i]] + Decay2*.001) $FileName1 TEMPWAVE2ch1
			TEMPWAVE1ch1 = TEMPWAVE1ch1 + TEMPWAVE2ch1
		endif
	Endfor
	TEMPWAVE1 = TEMPWAVE1 / NumberOfAveragedEvents2
	Duplicate/o TEMPWAVE1 AverageEvent2 ; KillWaves TEMPWAVE1
	SetScale/P x -Baseline2*.001, SI2*1e-6,"s", AverageEvent2
	
	// Charly **********************Here I substract the baseline from the mean event******************
	
	//Nvar Baseline2
	//Variable Substracter = mean(AverageEvent2, -(7 * 0.001), -(4 * 0.001))
	//AverageEvent2 -= Substracter
	
	// Charly End
	
	if (ShowCh1 == 1)
		TEMPWAVE1ch1 = TEMPWAVE1ch1 / NumberOfAveragedEvents2
		Duplicate/o TEMPWAVE1ch1 AverageEvent2ch1 ; KillWaves TEMPWAVE1ch1
		SetScale/P x -Baseline2*.001, SI1*1e-6, "s", AverageEvent2ch1
	endif
	if (ACFT2 == 1)	// if DetectedEvents2#Average2 exists
		KillWindow DetectedEvents2#Average2
	else
		ACFT2 = 1
	endif
	Display/HOST=DetectedEvents2/N=Average2/W=(0.71,0,0.99,0.49) AverageEvent2
	SetScale/P x -Baseline2*.001,SI2*1e-6,"s", AverageEvent2
	ModifyGraph margin(left)=40; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28
	ModifyGraph rgb(AverageEvent2)=(65280,0,0); ModifyGraph StandOff = 0
	Wavestats/Q AverageEvent2
	SetAxis left 1.2*V_min, -.4*V_min
	SetDrawEnv ycoord = left, linefgc = (47872,47872,47872), dash = 1
	DrawLine 0, 0, 1, 0
	ShowInfo
	if (ShowCh1 == 1)
		if (ACFT2ch1 == 1)	// if DetectedEvents2#Average21 exists
			KillWindow DetectedEvents2#Average21
		else
			ACFT2ch1 = 1
		endif
		Display/HOST=DetectedEvents2/N=Average21/W=(0.71,0.51,0.99,0.99) AverageEvent2ch1
		ModifyGraph margin(left)=40; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28
		ModifyGraph rgb(AverageEvent2ch1)=(0,0,65280); ModifyGraph StandOff = 0
		Wavestats/Q AverageEvent2ch1
		SetAxis/W=DetectedEvents2#Average21 left V_min - (V_max - V_min)*0.2, V_max + (V_max - V_min)*0.2
	endif
	//CheckBox CFPE2 disable=0; CheckBox CFAE2 disable=0
	Execute "Valdisplay NumEventsAveraged2 value=" + num2str(NumberOfAveragedEvents2)
	SetVariable End_Fit2 limits={0, Decay2, 1}
End

// ================== CONTINUE UPDATING CHANNEL 2 FUNCTIONS FROM HERE =======================

Function WaitForKey1()
		// Le boutton "Keyboard" permet de prendre le contrle de la fentre "SingleEvents"  l'aide du clavier
		// Le corps de la fonction WaitForKey est constitu d'une boucle Do... While(1) 
		// Pour sortir de la boucle, taper "C"
		String state
		Variable i
		Wave DetectedLevels1, Tags1, Occurrence1, BaselineY1, BaselineX1, PeakY1, PeakX1, RealAmp1, RealPeakPosition1, RiseTime20801, RT201, RT801
		Nvar EventNumber1, Baselinems1, Peakms1, KbdSpeed1, BTDE_1, BUDE_1, NumberofEventsch1, PreviousEvent1, CheckedTagAll1, TaggedEvents1
		Wave CursorTags1_X, CursorTags1_Y
	Do
		state = KeyboardState("")

		If (char2num(state[9])==67)	// Stopper la function  l'aide de la touche "C"
			Break
		Endif

		If (str2num(state[5])==1)		// Afficher l'vnement prcdent
			if (EventNumber1>0)
				if (BTDE_1 == 1)
					if (tags1[EventNumber1-1] == 0)
						FindLevel/P/Q/R=[EventNumber1-1,0] Tags1, 1	// Find the previous tagged event
						if (V_flag == 0)
							EventNumber1 = V_LevelX
							PreviousEvent1 = EventNumber1
						endif
					else
						EventNumber1 -= 1
					endif
				endif
				if (BUDE_1 == 1)
					if (tags1[EventNumber1-1] == 1)
						FindLevel/P/Q/R=[EventNumber1-1,0] Tags1, 0	// Find the previous untagged event
						if (V_flag == 0)
							EventNumber1 = V_LevelX
							PreviousEvent1 = EventNumber1
						endif
					else
						EventNumber1 -= 1
					endif
				endif
				if (BTDE_1 == 0 && BUDE_1 == 0)
					EventNumber1 -= 1
					PreviousEvent1 = EventNumber1
				endif
				SHOW1(2,EventNumber1)
				DoUpdate
				i=0
				Do
					i+=1
				While (i<400000/(1+exp(KbdSpeed1/3)))
			endif
		Endif
		
		If (str2num(state[6])==1)		// Afficher l'vnement suivant
			if (EventNumber1<(numpnts(DetectedLevels1)-1))
				if (BTDE_1 == 1)
					if (tags1[EventNumber1+1] == 0)
						FindLevel/P/Q/R=[EventNumber1+1,NumberofEventsch1-1] Tags1, 1	// Find the next tagged event
						if (V_flag == 0)
							EventNumber1 = V_LevelX
							PreviousEvent1 = EventNumber1
						endif
					else
						EventNumber1 += 1
					endif
				endif
				if (BUDE_1 == 1)
					if (tags1[EventNumber1+1] == 1)
						FindLevel/P/Q/R=[EventNumber1+1,NumberofEventsch1-1] Tags1, 0	// Find the next untagged event
						if (V_flag == 0)
							EventNumber1 = V_LevelX
							PreviousEvent1 = EventNumber1
						endif
					else
						EventNumber1 += 1
					endif
				endif
				if (BTDE_1 == 0 && BUDE_1 == 0)
					EventNumber1 += 1
					PreviousEvent1 = EventNumber1
				endif
				SHOW1(2,EventNumber1)
				DoUpdate
				i=0
				Do
					i+=1
				While (i<400000/(1+exp(KbdSpeed1/3)))
			endif
		Endif
		
		If (str2num(state[7])==1)		// Bascule Checkbox "Tag"
			Tags1[EventNumber1]=1-Tags1[EventNumber1]
			CheckBox CheckBoxTag1 value=(Tags1[EventNumber1])
			TaggedEvents1 = sum(Tags1,0,numpnts(Tags1)-1)
			CursorTags1_Y = Tags1[EventNumber1]
			Execute "ValDisplay DisplayTaggedEvents1 value=" + num2str(TaggedEvents1)
			If ( (sum(Tags1,0,numpnts(Tags1)-1))==0)
				BTDE_1 = 0; Checkbox BTDE1 value = 0
			endif
			if (CheckedTagAll1 == 1)
				If ( (sum(Tags1,0,numpnts(Tags1)-1)) < NumberofEventsch1)
					CheckedTagAll1 = 0; CheckBox CheckBoxTagAll1 value=0
				endif
			endif
			If ( (sum(Tags1,0,numpnts(Tags1)-1))==NumberofEventsch1)
				BUDE_1 = 0; Checkbox BUDE1 value = 0
				CheckedTagAll1 = 1; CheckBox CheckBoxTagAll1 value=1
			endif		
			DoUpdate
			Do
			 	state = KeyboardState("")
			While (str2num(state[7])==1)
		Endif
		
	While(1)
End

Function WaitForKey2()
		// Le boutton "Keyboard" permet de prendre le contrle de la fentre "SingleEvents"  l'aide du clavier
		// Le corps de la fonction WaitForKey est constitu d'une boucle Do... While(1) 
		// Pour sortir de la boucle, taper "C"
		String state
		Variable i
		Wave DetectedLevels2, Tags2, Occurrence2, BaselineY2, BaselineX2, PeakY2, PeakX2, RealAmp2, RealPeakPosition2, RiseTime20802, RT202, RT802
		Nvar EventNumber2, Baselinems2, Peakms2, KbdSpeed2, BTDE_2, BUDE_2, NumberofEventsch2, PreviousEvent2, CheckedTagAll2, TaggedEvents2
		Wave CursorTags2_X, CursorTags2_Y
	Do
		state = KeyboardState("")

		If (char2num(state[9])==67)	// Stopper la function  l'aide de la touche "C"
			Break
		Endif

		If (str2num(state[5])==1)		// Afficher l'vnement prcdent
			if (EventNumber2>0)
				if (BTDE_2 == 1)
					if (tags2[EventNumber2-1] == 0)
						FindLevel/P/Q/R=[EventNumber2-1,0] Tags2, 1	// Find the previous tagged event
						if (V_flag == 0)
							EventNumber2 = V_LevelX
							PreviousEvent2 = EventNumber2
						endif
					else
						EventNumber2 -= 1
					endif
				endif
				if (BUDE_2 == 1)
					if (tags2[EventNumber2-1] == 1)
						FindLevel/P/Q/R=[EventNumber2-1,0] Tags2, 0	// Find the previous untagged event
						if (V_flag == 0)
							EventNumber2 = V_LevelX
							PreviousEvent2 = EventNumber2
						endif
					else
						EventNumber2 -= 1
					endif
				endif
				if (BTDE_2 == 0 && BUDE_2 == 0)
					EventNumber2 -= 1
					PreviousEvent2 = EventNumber2
				endif
				SHOW2(2,EventNumber2)
				DoUpdate
				i=0
				Do
					i+=1
				While (i<400000/(1+exp(KbdSpeed2/3)))
			endif
		Endif
		
		If (str2num(state[6])==1)		// Afficher l'vnement suivant
			if (EventNumber2<(numpnts(DetectedLevels2)-1))
				if (BTDE_2 == 1)
					if (tags2[EventNumber2+1] == 0)
						FindLevel/P/Q/R=[EventNumber2+1,NumberofEventsch2-1] Tags2, 1	// Find the next tagged event
						if (V_flag == 0)
							EventNumber2 = V_LevelX
							PreviousEvent2 = EventNumber2
						endif
					else
						EventNumber2 += 1
					endif
				endif
				if (BUDE_2 == 1)
					if (tags2[EventNumber2+1] == 1)
						FindLevel/P/Q/R=[EventNumber2+1,NumberofEventsch2-1] Tags2, 0	// Find the next untagged event
						if (V_flag == 0)
							EventNumber2 = V_LevelX
							PreviousEvent2 = EventNumber2
						endif
					else
						EventNumber2 += 1
					endif
				endif
				if (BTDE_2 == 0 && BUDE_2 == 0)
					EventNumber2 += 1
					PreviousEvent2 = EventNumber2
				endif
				SHOW2(2,EventNumber2)
				DoUpdate
				i=0
				Do
					i+=1
				While (i<400000/(1+exp(KbdSpeed2/3)))
			endif
		Endif
		
		If (str2num(state[7])==1)		// Bascule Checkbox "Tag"
			Tags2[EventNumber2]=1-Tags2[EventNumber2]
			CheckBox CheckBoxTag2 value=(Tags2[EventNumber2])
			TaggedEvents2 = sum(Tags2,0,numpnts(Tags2)-1)
			CursorTags2_Y = Tags2[EventNumber2]
			Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
			If ( (sum(Tags2,0,numpnts(Tags2)-1))==0)
				BTDE_2 = 0; Checkbox BTDE2 value = 0
			endif
			if (CheckedTagAll2 == 1)
				If ( (sum(Tags2,0,numpnts(Tags2)-1)) < NumberofEventsch2)
					CheckedTagAll2 = 0; CheckBox CheckBoxTagAll2 value=0
				endif
			endif
			If ( (sum(Tags2,0,numpnts(Tags2)-1))==NumberofEventsch2)
				BUDE_2 = 0; Checkbox BUDE2 value = 0
				CheckedTagAll2 = 1; CheckBox CheckBoxTagAll2 value=1
			endif		
			DoUpdate
			Do
			 	state = KeyboardState("")
			While (str2num(state[7])==1)
		Endif
		
	While(1)
End

//Function WaitForKey2()
//		// Le boutton "Keyboard" permet de prendre le contrle de la fentre "SingleEvents"  l'aide du clavier
//		// Le corps de la fonction WaitForKey est constitu d'une boucle Do... While(1) 
//		// Pour sortir de la boucle, taper "C"
//		String state
//		Variable i
//		Wave DetectedLevels2, Tags2, Occurrence2, BaselineY2, BaselineX2, PeakY2, PeakX2, RealAmp2, RealPeakPosition2, RiseTime20802, RT202, RT802
//		Nvar EventNumber2, Baselinems2, Peakms2, KbdSpeed2
//				
//		Nvar EventNumber2, TaggedEvents2
//	Do
//		state = KeyboardState("")
//
//		If (char2num(state[9])==67)	// Stopper la function  l'aide de la touche "C"
//			Break
//		Endif
//
//		If (str2num(state[5])==1)		// Afficher l'vnement prcdent
//			if (EventNumber2>0)
//				EventNumber2=EventNumber2-1
//				SHOW2(2,EventNumber2)
//				DoUpdate
//				i=0
//				Do
//					i+=1
//				While (i<400000/(1+exp(KbdSpeed2/3)))
//			endif
//		Endif
//		
//		If (str2num(state[6])==1)		// Afficher l'vnement suivant
//			if (EventNumber2<(numpnts(DetectedLevels2)-1))
//				EventNumber2=EventNumber2+1
//				SHOW2(2,EventNumber2)
//				DoUpdate
//				i=0
//				Do
//					i+=1
//				While (i<400000/(1+exp(KbdSpeed2/3)))
//			endif
//		Endif
//		
//		If (str2num(state[7])==1)		// Bascule Checkbox "Tag"
//			Tags2[EventNumber2]=1-Tags2[EventNumber2]
//			CheckBox CheckBoxTag2 value=(Tags2[EventNumber2])
//			TaggedEvents2 = sum(Tags2,0,numpnts(Tags2)-1)
//			Execute "ValDisplay DisplayTaggedEvents2 value=" + num2str(TaggedEvents2)
//			Do
//			 	state = KeyboardState("")
//			While (str2num(state[7])==1)
//		Endif
//		
//	While(1)
//End

// CHECKBOX TO SELECT TIME OF OCCURRENCE AS THE HALF-RISE (Channel 2)
Function ProcToOHR2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ToOHR_2, ToOPeak_2, EventNumber2
	ToOHR_2 = 1-ToOHR_2
	ToOPeak_2 = 1-ToOPeak_2
	CheckBox ToOPeak2 value=ToOPeak_2
	if (ToOHR_2 == 1)
		Execute "ValDisplay DisplayOccurrence2 value=root:spacan:Occurrence2[" + num2str(EventNumber2) + "]"
	else
		Execute "ValDisplay DisplayOccurrence2 value=root:spacan:realpeakposition2[" + num2str(EventNumber2) + "]"
	endif
End

// CHECKBOX TO SELECT TIME OF OCCURRENCE AS THE TIME OF THE PEAK (Channel 2)
Function ProcToOPeak2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ToOHR_2, ToOPeak_2, EventNumber2
	ToOHR_2 = 1-ToOHR_2
	ToOPeak_2 = 1-ToOPeak_2
	CheckBox ToOHR2 value=ToOHR_2
	if (ToOHR_2 == 1)
		Execute "ValDisplay DisplayOccurrence2 value=root:spacan:Occurrence2[" + num2str(EventNumber2) + "]"
	else
		Execute "ValDisplay DisplayOccurrence2 value=root:spacan:realpeakposition2[" + num2str(EventNumber2) + "]"
	endif
End

// CHECKBOX TO CHOSE TO BROWSE THROUGH TAGGED EVENTS ONLY (Channel 1)
Function ProcBTDE1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar BTDE_1, BUDE_1
	Wave Tags1
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==0)
		BTDE_1 = 0; Checkbox BTDE1 value = 0
		Abort "No tagged event."
	else
		BTDE_1 = 1-BTDE_1
		BUDE_1 = 0
		Checkbox BUDE1 value = 0
	endif
End

// CHECKBOX TO CHOSE TO BROWSE THROUGH TAGGED EVENTS ONLY (Channel 2)
Function ProcBTDE2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar BTDE_2, BUDE_2
	Wave Tags2
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==0)
		BTDE_2 = 0; Checkbox BTDE2 value = 0
		Abort "No tagged event."
	else
		BTDE_2 = 1-BTDE_2
		BUDE_2 = 0
		Checkbox BUDE2 value = 0
	endif
End

// CHECKBOX TO CHOSE TO BROWSE THROUGH UNTAGGED EVENTS ONLY (Channel 1)
Function ProcBUDE1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar BTDE_1, BUDE_1, NumberofEventsch1
	Wave Tags1
	If ( (sum(Tags1,0,numpnts(Tags1)-1))==NumberofEventsch1)
		BUDE_1 = 0; Checkbox BUDE1 value = 0
		Abort "No untagged event."
	else
		BUDE_1 = 1-BUDE_1
		BTDE_1 = 0
		Checkbox BTDE1 value = 0
	endif
End

// CHECKBOX TO CHOSE TO BROWSE THROUGH UNTAGGED EVENTS ONLY (Channel 2)
Function ProcBUDE2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar BTDE_2, BUDE_2, NumberofEventsch2
	Wave Tags2
	If ( (sum(Tags2,0,numpnts(Tags2)-1))==NumberofEventsch2)
		BUDE_2 = 0; Checkbox BUDE2 value = 0
		Abort "No untagged event."
	else
		BUDE_2 = 1-BUDE_2
		BTDE_2 = 0
		Checkbox BTDE2 value = 0
	endif
End

// SHOW CHANNEL 1 WHEN BROWSING THROUGH DETECTED EVENTS IN CHANNEL 2
Function ShowChannel1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ShowCh1
	Nvar C1E	// C1E defined in function "Initialize". C1E = 1 if channel 1 exists
	if (C1E == 0)
		//ShowCh1 = 0
		CheckBox CheckShowChannel1 value=0
		Abort "No wave loaded in channel 1 !"
	endif
	// WHEN THE USER DISPLAYS AN EVENT IN CHANNEL 2, THIS FUNCTION SHOWS THE CORRESPONDING TRACE IN CHANNEL 1	
	Wave Occurrence2, DetectedLevels2
	Nvar SI1, EventNumber2, ACFT2, ACFT2ch1
	Svar FileName1
	String /G WindowShowCh1 ; WindowShowCh1 = "Channel 1"
	if (checked == 1)		// THE CHECKBOX IS ACTIVATED -> display channel 1
		Duplicate/o/R=( Occurrence2[EventNumber2] - K18*.001 , Occurrence2[EventNumber2] + K19*.001 ) $FileName1 TEMPEVENT_CH1
		SetScale/P x -K18*.001, (SI1*1e-6), "s", TEMPEVENT_CH1
		Display/HOST=DetectedEvents2/N=EventChannel21/W=(0.4,0.51,0.69,0.99) TEMPEVENT_CH1 as WindowShowCh1
		ModifyGraph margin(left)=40; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28
		ModifyGraph StandOff = 0; ModifyGraph rgb(TEMPEVENT_CH1)=(0,0,65280)
		Wavestats/Q TEMPEVENT_CH1
		SetAxis/W=DetectedEvents2#EventChannel21 left V_min - (V_max - V_min)*0.2, V_max + (V_max - V_min)*0.2
	else
		KillWindow DetectedEvents2#EventChannel21
		if (ACFT2ch1 == 1) // DetectedEvents2#Average21 exists
			KillWindow DetectedEvents2#Average21; ACFT2ch1 = 0
		endif
	endif
	ShowCh1 = 1 - ShowCh1
End

// SHOW CHANNEL 2 WHEN BROWSING THROUGH DETECTED EVENTS IN CHANNEL 1
Function ShowChannel2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar ShowCh2
	Nvar C2E
	if (C2E == 0)
		//ShowCh2 = 0
		CheckBox CheckShowChannel2 value=0
		Abort "No wave loaded in channel 2 !"
	endif
	// WHEN THE USER DISPLAYS AN EVENT IN CHANNEL 1, THIS FUNCTION SHOWS THE CORRESPONDING TRACE IN CHANNEL 2	
	Wave Occurrence1, DetectedLevels1
	Nvar SI2, EventNumber1, ACFT1, ACFT1ch2
	Svar FileName2
	String /G WindowShowCh2 ; WindowShowCh2 = "Channel 2"
	if (checked == 1)		// THE CHECKBOX IS ACTIVATED -> display channel 2
		Duplicate/o/R=( Occurrence1[EventNumber1] - K16*.001 , Occurrence1[EventNumber1] + K17*.001 ) $FileName2 TEMPEVENT_CH2
		SetScale/P x -K16*.001, (SI2*1e-6), "s", TEMPEVENT_CH2
		
		//Charly ************************************** Here I substract baseline of the period in front of tempEvent1
		
			NVAR Baseline1
			Variable substracter = mean(TEMPEVENT_CH2, -(7 * 0.001), -(4 * 0.001))
			TEMPEVENT_CH2 -= substracter
		
		//Charly End of changes************************************************
		
		Display/HOST=DetectedEvents1/N=EventChannel12/W=(0.4,0.51,0.69,0.99) TEMPEVENT_CH2 as WindowShowCh2
		ModifyGraph margin(left)=40; ModifyGraph margin(top)=1; ModifyGraph margin(right)=1; ModifyGraph margin(bottom)=28
		ModifyGraph StandOff = 0; ModifyGraph rgb(TEMPEVENT_CH2)=(65280,0,0)
		Wavestats/Q TEMPEVENT_CH2
		SetAxis/W=DetectedEvents1#EventChannel12 left V_min - (V_max - V_min)*0.2, V_max + (V_max - V_min)*0.2
	else		// THE CHECKBOX IS UNCHECKED -> kill DetectedEvents1#Average12 & DetectedEvents1#EventChannel12
		KillWindow DetectedEvents1#EventChannel12
		if (ACFT1ch2 == 1) // DetectedEvents1#Average12 exists
			KillWindow DetectedEvents1#Average12; ACFT1ch2 = 0
		endif
	endif
	ShowCh2 = 1 - ShowCh2
End

// =================== BELOW: A SERIES OF FIT FUNCTIONS ===================

// THE FIT FUNCTION TO FIT POSTSYNAPTIC CURRENTS: A SIGMOID TO FIT THE RISE AND A DOUBLE EXPONENTIAL TO FIT THE DECAY
Function Sigmoid_Dbexp(w,x) : FitFunc
	Wave w; Variable x
	return ( w[0] / ( 1 + exp( (w[1] - x) / w[2] )) ) * ( w[3] + w[4]*exp(-w[5]*x) + w[6]*exp(-w[7]*x) )
	// Sigmoid - w[0] : max
	// Sigmoid - w[1] : xhalf
	// Sigmoid - w[2] : rate
	// Dbexp - w[3] : offset
	// Dbexp - w[4] : amplitude exp1
	// Dbexp - w[5] : invtau1
	// Dbexp - w[6] : amplitude exp2
	// Dbexp - w[7] : invtau2
End

// A TRIPLE EXPONENTIAL FUNCTION.
// CAN BE REMOVED FROM SPACAN.
Function Trplexp(w,x) : FitFunc
	Wave w; Variable x
	return w[0] + w[1]*exp(-x/w[2]) + w[3]*exp(-x/w[4]) + w[5]*exp(-x/w[6])
End

// A QUADRUPLE EXPONENTIAL FUNCTION.
// CAN BE REMOVED FROM SPACAN.
Function Qdrplexp(w,x) : FitFunc
	Wave w; Variable x
	return w[0] + w[1]*exp(-x/w[2]) + w[3]*exp(-x/w[4]) + w[5]*exp(-x/w[6]) + w[7]*exp(-x/w[8])
End

// ANOTHER CUSTOM-MADE FIT FUNCTION
// CAN BE REMOVED FROM SPACAN.
Function Loglin(w,x) : FitFunc
	Wave w; Variable x
	return log(w[0] * exp(w[1]*x))
End

// ANOTHER CUSTOM-MADE FIT FUNCTION: THE SUM OF TWO GAUSSIAN CURVES.
// CAN BE REMOVED FROM SPACAN.
Function GaussSum(w,x) : FitFunc
	Wave w; Variable x
	return w[0] + w[1] * exp( - (x - w[2])*(x - w[2]) / (w[3]*w[3])) + w[4] * exp( - (x - w[5])*(x - w[5]) / (w[6]*w[6]))
End

// A FUNCTION USED TO FIT THE CENTRAL PEAK OF SPIKE CROSS-CORRELOGRAMS IN GOLGI CELLS PAIRED RECORDINGS
// CAN BE REMOVED FROM SPACAN.
Function Fit_CC(wave_to_fit)
	Wave wave_to_fit
	Nvar CorrBin
	Make/D/N=7/O W_coef
	Variable Var1, Var2, Var3, Var4, Var5, Var6, Var7
	Var1 = -1; Var2 = 12; Var3 = -0.0014; Var4 = 0.0009; Var5 = 12; Var6 = 0.0011; Var7 = 0.0009
	Prompt Var1, "y0 ?"
	Prompt Var2, "A1 ?"
	Prompt Var3, "x1 ?"
	Prompt Var4, "width1 ?"
	Prompt Var5, "A2 ?"
	Prompt Var6, "x2 ?"
	Prompt Var7, "width2 ?"
	Doprompt "fit parameters", Var1, Var2, Var3, Var4, Var5, Var6, Var7
	If (V_flag == 1)
		Abort
	endif
	W_coef[0] = {Var1,Var2,Var3,Var4,Var5,Var6,Var7}
	FuncFit GaussSum W_coef  Corr /D
	Duplicate/o W_coef dummy
	Make/D/N=7/O W_coef
	W_coef[0] = {dummy[0],dummy[1],dummy[2],dummy[3],dummy[4],dummy[5],dummy[6]}
	Wave Fit_Corr
	//Removefromgraph fit_Corr
	FuncFit GaussSum W_coef  Corr /D
	ModifyGraph offset(fit_Corr)={CorrBin/2,0}
	ModifyGraph rgb(fit_Corr)=(0,12800,52224)
	DrawText 0.06,0.15,"y0 = " + num2str(W_coef[0])
	DrawText 0.06,0.25,"A1 = " + num2str(W_coef[1])
	DrawText 0.06,0.35,"x0 = " + num2str(W_coef[2] + (CorrBin/2))
	DrawText 0.06,0.45,"width = " + num2str(W_coef[3])
	DrawText 0.6,0.25,"A2 = " + num2str(W_coef[4])
	DrawText 0.6,0.35,"x0 = " + num2str(W_coef[5] + (CorrBin/2))
	DrawText 0.6,0.45,"width = " + num2str(W_coef[6])
End






// Charly ****************************************************************Here is the main part of added things************************
FUNCTION LoadABF()


	Variable number
	String /G CurrentFile, CurrentFileName
	Variable /G NumChannels, lADCResolution, DataSectionPtr, SampleInterval, refnum, lDataSectionPtr, lActualAcqLength, lNumSamplesPerEpisode
	
	//---------- FIRST STEP: EXAMINING THE FILE'S HEADER ----------
	Variable counter
	open /d/r/t="????" refnum					// does not actually open the file, puts the file name into S_filename
	CurrentFile = S_filename
	counter = -12								// 12: number of characters in the name of a .abf file
	CurrentFileName=""
	Do
		CurrentFileName = CurrentFileName + CurrentFile[strlen(currentfile) + counter]
		counter += 1
	while (counter < 1)
	open /r/t="????" refnum as CurrentFile		// opens the file, allows the fstatus operation
		fstatus refnum						// returns 6 variables, in particular S_filename and V_logEOF (the total number of bytes in the file)
	close refnum
	if (V_logEOF < 2048)
		return(1)								// the current procedure stops executing immediately
	endif
	LOAD_FILE_PARAMETERS(CurrentFile)	// load parameters from header
	wave sweep_params
	execute "SampleInterval = str2num(sweep_params[29][1])"
	execute "NumChannels = str2num(sweep_params[28][1])"
	if (NumChannels == 2)
		SampleInterval = SampleInterval * 2
	endif
	execute "lADCResolution = str2num(sweep_params[54][1])"
	execute "lDataSectionPtr = str2num(sweep_params[12][1])"
	KillWaves sweep_params
	// ---------- END OF FIRST STEP ----------

	// ---------- SECOND STEP: PROMPTING THE GAIN ---------- 	In the future, the gain should be found in the header
	If (NumChannels==1)
		Variable /G Gain
		Variable GainValue ; GainValue = 5		// Local variable used for prompting
		Prompt GainValue, "Enter gain value (mV/pA)"
		Doprompt "Gain", GainValue
		if (V_flag==1)
			Abort
		endif
		Gain = GainValue			// Gain is a global variable
		Variable ScaleFactor
		ScaleFactor = 1 / (GainValue*.001*lADCResolution/10) ; PRINT lADCResolution
	endif
	if (NumChannels==2)
		Variable /G Gain1, Gain2
		Variable GainValue_1 ; GainValue_1 = 10
		Variable GainValue_2 ; GainValue_2 = 10
		Prompt GainValue_1, "Enter gain value for channel 1 (mV/pA)"
		Prompt GainValue_2, "Enter gain value for channel 2 (mV/pA)"
		DoPrompt "Gain", GainValue_1, GainValue_2
		if (V_flag==1)
			Abort
		endif
		Gain1 = GainValue_1 ; Gain2 = GainValue_2
		Variable ScaleFactor1, ScaleFactor2
		ScaleFactor1 = 1 / (GainValue_1*.001*lADCResolution/10)
		ScaleFactor2 = 1 / (GainValue_2*.001*lADCResolution/10)
	endif
	
		// ajout le 17-04-2005 
		Variable fADCRange, lADCResolution2
		Execute "GBLoadWave/O/B/Q/N=header/T={32,4}/S=2/W=1/U=511 (CurrentFile)"
		Wave Header0
		lActualAcqLength = header0[2]
		lNumSamplesPerEpisode = header0[34]
		Execute "GBLoadWave/O/B/Q/N=header/T={32,4}/W=1/U=512 (CurrentFile)"
		Wave Header0
		lADCResolution2 =  header0[63]
		Execute "GBLoadWave/O/B/Q/N=header/T={2,4}/W=1/U=511 (CurrentFile)"
		fADCRange = header0[61]
		ScaleFactor = fADCRange/(lADCResolution*Gain*.001)

	
	// ---------- END OF SECOND STEP ----------
	
	// ---------- THIRD STEP: LOADING THE FILE ----------
	
	if (NumChannels == 2)
		execute "GBLoadWave/Q/B/T={16,4}/S=(lDataSectionPtr*512)/O/N=Data/U=(lActualAcqLength / 2)/W=2/V/Y={0,1} (CurrentFile)"
		Wave Data0, Data1
		Data0 = Data0 * ScaleFactor1
		Data1 = Data1 * ScaleFactor2
		
		SetScale/P x 0,SampleInterval*1e-6, "s", Data0
		SetScale/P x (0+(SampleInterval*1e-6)/2),SampleInterval*1e-6, "s", Data1
		Variable /G FileLength ; FileLength = (numpnts(Data0)-1) * SampleInterval*1e-6
		String TEMP1, TEMP2
		TEMP1 = "Trace_" + CurrentFileName[0,7] + "_channel1"
		TEMP2 = "Trace_" + CurrentFileName[0,7] + "_channel2"
		Rename Data0 $TEMP1
		Rename Data1 $TEMP2
	endif
	if (NumChannels == 1)
		execute "GBLoadWave/Q/B/T={16,4}/S=(lDataSectionPtr*512)/O/N=Data/W=1/U=(lActualAcqLength)/Y={0,1} (CurrentFile)"
		Wave Data0
		Data0 = Data0 * ScaleFactor
		SetScale/P x 0,SampleInterval*1e-6, "s", Data0
		Variable /G FileLength ; FileLength = (numpnts(Data0)-1) * SampleInterval*1e-6	// will be used later...
		String TEMP
		TEMP = "Trace_" + CurrentFileName[0,7]
		Rename Data0 $TEMP
		Save/C $TEMP
	endif
	// ---------- END OF THIRD STEP ----------
	
	// ---------- FOURTH STEP: PRINTING VARIABLES IN COMMAND WINDOW ----------
	if (NumChannels==1)
		print "File name: " + CurrentFileName + " - 1 channel" + " - Sample interval = " + num2str(SampleInterval) + " s/pt - Output gain = " + num2str(Gain) + " mV/pA"
	endif
	if (NumChannels==2)
		print "File name: " + CurrentFileName + " - 2 channels" + " - Sample interval = " + num2str(SampleInterval) + " s/pt - Output gain channel 1 = " + num2str(Gain1) + " mV/pA - Output gain channel 2 = " + num2str(Gain2) + " mV/pA"
	endif
	// ---------- END OF FOURTH STEP ----------
END

////Fit second channel triggered on first channel in detected events 1 window
//Function Fitdbexp_ch2_on_win1()
//	
//	// SI2 for sample interval of trace 2 Ppnts2 for numbre of points in the peak for trace 2,
//	 //to use the checkboxes on screen : checkoffset1, decay1, baseline1, start, End, checkfit1
//	Nvar SI2, CheckOffset1, Decay1, baseline1, StartFitdbexp1, EndFitdbexp1, CheckFit1, Ppnts2
//	//TEMPEVENT_CH2 to select the current event for the channel 2 on the detected events 1 window
//	//AverageEvent1ch2 to select the average event for the channel 2 on the detected events 1 window
//	Wave TEMPEVENT_CH2, AverageEvent1ch2
//	Variable debut, fin, counter
//	Variable V_FitOptions = 4
//	
//	String TEMPSTR
//	
//	Make/o/N=10 FitParametersFitdbexp1
//	
//	//If single event box checked
//	if (CheckFit1 == 0)
//		// The following lines are meant to remove any previous fit wave on the graph. 
//		//Otherwise it gets messy and Igor superimpose many waves with the same name.
//		
//		GetWindow DetectedEvents1#EventChannel12, WaveList //remplact EventChannel1 par EventChannel12
//		Wave/T W_WaveList = W_WaveList
//		for (counter = 0; counter < DimSize(W_WaveList, 0); counter += 1)
//			TEMPSTR = W_WaveList[counter][0]
//			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//				RemoveFromGraph/W=DetectedEvents1#EventChannel12 $W_WaveList[counter][0]
//			endif		
//		endfor
//		// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//		Make/o fit_TEMPEVENT_CH2 = 0
//		Appendtograph/W=DetectedEvents1#EventChannel12 fit_TEMPEVENT_CH2
//		ModifyGraph/W=DetectedEvents1#EventChannel12 rgb(fit_TEMPEVENT_CH2)=(0,15872,65280) //Just to set the color of the fit curve to blue to see it over the original red trace
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Duplicate/O TEMPEVENT_CH2, TEMPWAVE
//		Smooth/B Ppnts2, TEMPWAVE
//		Wavestats/Q/R=(0, Decay1*.001) TEMPWAVE
//		//FindPeak/N/Q/R=(0,Decay1*.001) TEMPWAVE // /N means search for a negative peak, /Q no history printing no error reporting if no peak found, value is X by default /R = range for search
//		//Print "Peak Value in wave is : " + num2str(V_peakVal)
//		FindLevel/Q/R=(0,Decay1*.001) TEMPWAVE, V_min
//		
////		If (V_flag == 0)
////			FindLevel/Q/R=(V_PeakLoc,Decay1*.001) TEMPWAVE V_min*(1-StartFitdbexp1*.01)
////		Else
////			Abort "No peak found"
////		EndIf
//		
//		If (V_flag == 0)
//			Print "minimum location is : " + num2str(V_levelX) 
//			FindLevel/Q/R=(V_levelX,Decay1*.001) TEMPWAVE V_min*(1-StartFitdbexp1*.01)
//		Else
//			Abort "No peak found"
//		EndIf
//		
//		If (V_flag == 0)
//			debut = V_LevelX
//		Else
//			Abort "No level found"
//		EndIf
//		
//		fin = EndFitdbexp1*.001
//		K0 = 0
//		// The fit command
//		if (CheckOffset1==1)
//			CurveFit/Q/H="00000" dblexp TEMPEVENT_CH2 (debut, fin) /D
//		else
//			CurveFit/Q/H="10000" dblexp TEMPEVENT_CH2 (debut, fin) /D
//		endif
//		//AppendToGraph/W=DetectedEvents1#EventChannel12 fit_TEMPEVENT1
//		FitParametersFitdbexp1[0]=(1/K4)*1000
//		FitParametersFitdbexp1[1]=(1/K2)*1000
//		FitParametersFitdbexp1[8]=K0
//		FitParametersFitdbexp1[2] = (K3 / (K1 + K3))*100
//		FitParametersFitdbexp1[3] = (K1 / (K1 + K3))*100
//		execute "ValDisplay TauFastdbexp1 value=" + num2str(FitParametersFitdbexp1[0])
//		execute "ValDisplay TauSlowdbexp1 value=" + num2str(FitParametersFitdbexp1[1])
//		execute "ValDisplay FastPercentagedbexp1 value=" + num2str(FitParametersFitdbexp1[2])
//		execute "ValDisplay SlowPercentagedbexp1 value=" + num2str(FitParametersFitdbexp1[3])
//		execute "ValDisplay ValOffsetdbexp1 value=" + num2str(FitParametersFitdbexp1[8])
//		execute "Valdisplay KineticIndexdbexp1 value=" + num2str((FitParametersFitdbexp1[2]/100)*FitParametersFitdbexp1[0] + (FitParametersFitdbexp1[3]/100)*FitParametersFitdbexp1[1])
//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		
//	//For the average box checked ***************************************************************************************************************************************
//	Elseif (CheckFit1 == 1)
//		// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
//		// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
//		GetWindow DetectedEvents1#Average12, WaveList
//		Wave/T W_WaveList = W_WaveList
//		for (counter = 1; counter < DimSize(W_WaveList, 0); counter += 1)
//			TEMPSTR = W_WaveList[counter][0]
//			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//				RemoveFromGraph/W=DetectedEvents1#Average12 $W_WaveList[counter][0]
//			endif		
//		endfor
//		// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//		Make/o fit_AverageEvent1ch2 = 0
//		Appendtograph/W=DetectedEvents1#Average12 fit_AverageEvent1ch2
//		ModifyGraph/W=DetectedEvents1#Average12 rgb(fit_AverageEvent1ch2)=(0,15872,65280)//Just to set the color of the fit curve to blue to see it over the original red trace
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Wavestats/Q/R=(0, Decay1*.001) AverageEvent1ch2
//		//FindPeak/N/Q/R=(0, (Decay1 * 0.001)) AverageEvent1ch2
//		
//		FindLevel/Q/R=(0, (Decay1 * 0.001)) AverageEvent1ch2, V_min
//		If (V_flag == 0)
//			Print "minimum location is : " + num2str(V_levelX)
//			FindLevel/Q/R=(V_levelX, (Decay1 * 0.001)) AverageEvent1ch2, (V_min * (1 - StartFitdbexp1 * 0.01))
//		Else
//			Abort "level no found"
//		EndIf
//		
//		If (V_flag == 0)
//			debut = V_LevelX
//		Else
//			Abort "Level not found"
//		EndIf
//		fin = (EndFitdbexp1 * 0.001)
//		K0 = 0
//		// The fit command
//		if (CheckOffset1 == 1)
//			CurveFit/Q/H="00000" dblexp AverageEvent1ch2(debut, fin) /D
//		else
//			CurveFit/Q/H="10000" dblexp AverageEvent1ch2(debut, fin) /D
//		endif
//		FitParametersFitdbexp1[0]=(1/K4)*1000
//		FitParametersFitdbexp1[1]=(1/K2)*1000
//		FitParametersFitdbexp1[8]=K0
//		FitParametersFitdbexp1[2] = (K3 / (K1 + K3))*100
//		FitParametersFitdbexp1[3] = (K1 / (K1 + K3))*100
//		execute "ValDisplay TauFastdbexp1 value=" + num2str(FitParametersFitdbexp1[0]) // Tau fast
//		execute "ValDisplay TauSlowdbexp1 value=" + num2str(FitParametersFitdbexp1[1]) // Tau Slow
//		execute "ValDisplay FastPercentagedbexp1 value=" + num2str(FitParametersFitdbexp1[2]) // % Fast
//		execute "ValDisplay SlowPercentagedbexp1 value=" + num2str(FitParametersFitdbexp1[3]) // % Slow
//		execute "ValDisplay ValOffsetdbexp1 value=" + num2str(FitParametersFitdbexp1[8])
//		execute "Valdisplay KineticIndexdbexp1 value=" + num2str((FitParametersFitdbexp1[2]/100)*FitParametersFitdbexp1[0] + (FitParametersFitdbexp1[3]/100)*FitParametersFitdbexp1[1])
//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		
//	endif
//End

//Function ProcFitSingleExp1(CtrlName) : ButtonControl
//	String CtrlName
//	
//	 //to use the checkboxes on screen : checkoffset1, decay1, baseline1, start, End, checkfit1
//	Nvar SI1, CheckOffset1, Decay1, baseline1, StartFit1, EndFit1, CheckFit1, Ppnts1
//	Wave TEMPEVENT1, AverageEvent1, Curseur_debut_fit_X1, Curseur_debut_fit_Y1, Curseur_fin_fit_X1, Curseur_fin_fit_Y1
//	Wave Curseur_debut_fit_X1_Av, Curseur_debut_fit_Y1_Av, Curseur_fin_fit_X1_Av, Curseur_fin_fit_Y1_Av
//	Variable debut, fin, counter
//	Variable V_FitOptions = 4
//	
//	String TEMPSTR
//	
//	Make/o/N=10 FitParametersFitdbexp1
//	
//	//If single event box checked
//	if (CheckFit1 == 0)
//		// The following lines are meant to remove any previous fit wave on the graph. 
//		//Otherwise it gets messy and Igor superimpose many waves with the same name.
//		GetWindow DetectedEvents1#EventChannel1, WaveList
//		Wave/T W_WaveList = W_WaveList
//		for (counter = 0; counter < DimSize(W_WaveList, 0); counter += 1)
//			TEMPSTR = W_WaveList[counter][0]
//			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//				RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[counter][0]
//			endif
//			if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[counter][0]
//			endif
//			if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[counter][0]
//			endif		
//		endfor
//		// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//		Make/o fit_TEMPEVENT1 = 0
//		Appendtograph/W=DetectedEvents1#EventChannel1 fit_TEMPEVENT1
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Duplicate/O TEMPEVENT1, TEMPWAVE
//		Smooth/B Ppnts1, TEMPWAVE
//		Wavestats/Q/R=(0, Decay1*.001) TEMPWAVE
//		//FindPeak/N/Q/R=(0,Decay1*.001) TEMPWAVE // /N means search for a negative peak, /Q no history printing no error reporting if no peak found, value is X by default /R = range for search
//		// Better w/ findlevel to find the peak of the event (more sure than find peak because smallest value
//		FindLevel/Q/R=(0,Decay1*.001) TEMPWAVE, V_min
//		
//		If (V_flag == 0)
//			FindLevel/Q/R=(V_levelX,Decay1*.001) TEMPWAVE V_min*(1-StartFit1*.01)
//		Else
//			Abort "No peak found"
//		EndIf
//		
//		If (V_flag == 0)
//			debut = V_LevelX
//		Else
//			Abort "No level found"
//		EndIf
//		
//		fin = EndFit1*.001
//		K0 = 0
//		// The fit command
//		if (CheckOffset1==1)
//			CurveFit/Q/H="00000" exp TEMPEVENT1 (debut, fin) /D
//		else
//			CurveFit/Q/H="10000" exp TEMPEVENT1 (debut, fin) /D
//		endif
//		
//		FitParametersFitdbexp1[0]= (1/K2)*1000
//		FitParametersFitdbexp1[1]= NaN
//		FitParametersFitdbexp1[8]=K0
//		FitParametersFitdbexp1[2] = 100
//		FitParametersFitdbexp1[3] = NaN
//		execute "ValDisplay TauFast1 value=" + num2str(FitParametersFitdbexp1[0])
//		execute "ValDisplay TauSlow1 value=" + num2str(FitParametersFitdbexp1[1])
//		execute "ValDisplay FastPercentage1 value=" + num2str(FitParametersFitdbexp1[2])
//		execute "ValDisplay SlowPercentage1 value=" + num2str(FitParametersFitdbexp1[3])
//		execute "ValDisplay ValOffset1 value=" + num2str(FitParametersFitdbexp1[8])
//		execute "Valdisplay KineticIndex1 value=" + num2str((1/K2) * 1000) // Insert the time constant converted to ms
//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		getaxis/Q/W=DetectedEvents1#EventChannel1 bottom
//		Curseur_debut_fit_X1[0]= (Debut - (.05*(V_max-V_min)))
//		Curseur_debut_fit_X1[1]= (Debut + (.05*(V_max-V_min)))
//		Curseur_debut_fit_Y1[0]= K0+ K1*exp(-K2*debut)
//		Curseur_debut_fit_Y1[1]= K0+ K1*exp(-K2*debut)
//		Curseur_fin_fit_X1[0] = EndFit1*.001
//		Curseur_fin_fit_X1[1] = EndFit1*.001
//		getaxis/W=DetectedEvents1#EventChannel1/Q left
//		Curseur_fin_fit_Y1[0] = K0+ K1*exp(-K2*fin) - 0.05*(V_max-V_min)
//		Curseur_fin_fit_Y1[1] = K0+ K1*exp(-K2*fin) + 0.05*(V_max-V_min)
//		appendtograph/W=DetectedEvents1#EventChannel1 Curseur_debut_fit_Y1 vs Curseur_debut_fit_X1; ModifyGraph/W=DetectedEvents1#EventChannel1 rgb(Curseur_debut_fit_Y1)=(0,0,0)
//		appendtograph/W=DetectedEvents1#EventChannel1 Curseur_fin_fit_Y1 vs Curseur_fin_fit_X1; ModifyGraph/W=DetectedEvents1#EventChannel1 rgb(Curseur_fin_fit_Y1)=(0,0,0)
//		ModifyGraph/W=DetectedEvents1#EventChannel1 rgb(fit_TEMPEVENT1)=(0,0,0)
//		
//	//For the average box checked ***************************************************************************************************************************************
//	Elseif (CheckFit1 == 1)
//		// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
//		// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
//		GetWindow DetectedEvents1#Average1, WaveList
//		Wave/T W_WaveList = W_WaveList
//		for (counter = 1; counter < DimSize(W_WaveList, 0); counter += 1)
//			TEMPSTR = W_WaveList[counter][0]
//			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//				RemoveFromGraph/W=DetectedEvents1#Average1 $W_WaveList[counter][0]
//			endif
//			if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents1#Average1 $W_WaveList[counter][0]
//			endif
//			if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents1#Average1 $W_WaveList[counter][0]
//			endif		
//		endfor
//		// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//		Make/o fit_AverageEvent1 = 0
//		Appendtograph/W=DetectedEvents1#Average1 fit_AverageEvent1
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Wavestats/Q/R=(0, Decay1*.001) AverageEvent1
//		
//		// Search the peak using the minimum value
//		FindLevel/Q/R=(0, (Decay1 * 0.001)) AverageEvent1, V_min
//		If (V_flag == 0)
//			FindLevel/Q/R=(V_levelX, (Decay1 * 0.001)) AverageEvent1, (V_min * (1 - StartFit1 * 0.01))
//		Else
//			Abort "level not found"
//		EndIf
//		
//		If (V_flag == 0)
//			debut = V_LevelX
//		Else
//			Abort "Level not found"
//		EndIf
//		fin = (EndFit1 * 0.001)
//		K0 = 0
//		// The fit command
//		if (CheckOffset1 == 1)
//			CurveFit/Q/H="00000" exp AverageEvent1(debut, fin) /D
//		else
//			CurveFit/Q/H="10000" exp AverageEvent1(debut, fin) /D
//		endif
//		FitParametersFitdbexp1[0]= (1/K2)*1000
//		FitParametersFitdbexp1[1]= NaN
//		FitParametersFitdbexp1[8]=K0
//		FitParametersFitdbexp1[2] = 100
//		FitParametersFitdbexp1[3] = NaN
//		execute "ValDisplay TauFast1 value=" + num2str(FitParametersFitdbexp1[0])
//		execute "ValDisplay TauSlow1 value=" + num2str(FitParametersFitdbexp1[1])
//		execute "ValDisplay FastPercentage1 value=" + num2str(FitParametersFitdbexp1[2])
//		execute "ValDisplay SlowPercentage1 value=" + num2str(FitParametersFitdbexp1[3])
//		execute "ValDisplay ValOffset1 value=" + num2str(FitParametersFitdbexp1[8])
//		execute "Valdisplay KineticIndex1 value=" + num2str((1/K2) * 1000) // Insert the time constant converted to ms
//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		getaxis/Q/W=DetectedEvents1#EventChannel1 bottom
//		Curseur_debut_fit_X1_Av[0]= (Debut - (.05*(V_max-V_min)))
//		Curseur_debut_fit_X1_Av[1]= (Debut + (.05*(V_max-V_min)))
//		Curseur_debut_fit_Y1_Av[0]= K0+ K1*exp(-K2*debut)
//		Curseur_debut_fit_Y1_Av[1]= K0+ K1*exp(-K2*debut)
//		Curseur_fin_fit_X1_Av[0] = EndFit1*.001
//		Curseur_fin_fit_X1_Av[1] = EndFit1*.001
//		getaxis/W=DetectedEvents1#EventChannel1/Q left
//		Curseur_fin_fit_Y1_Av[0] = K0+ K1*exp(-K2*fin) - 0.05*(V_max-V_min)
//		Curseur_fin_fit_Y1_Av[1] = K0+ K1*exp(-K2*fin) + 0.05*(V_max-V_min)
//		appendtograph/W=DetectedEvents1#Average1 Curseur_debut_fit_Y1_Av vs Curseur_debut_fit_X1_Av; ModifyGraph/W=DetectedEvents1#Average1 rgb(Curseur_debut_fit_Y1_Av)=(0,0,0)
//		appendtograph/W=DetectedEvents1#Average1 Curseur_fin_fit_Y1_Av vs Curseur_fin_fit_X1_Av; ModifyGraph/W=DetectedEvents1#Average1 rgb(Curseur_fin_fit_Y1_Av)=(0,0,0)
//		ModifyGraph/W=DetectedEvents1#Average1 rgb(fit_AverageEvent1)=(0,0,0)
//	endif
//End
//
//
//Function ProcFitSingleExp2(CtrlName) : ButtonControl
//	String CtrlName
//	
//	 //to use the checkboxes on screen : checkoffset1, decay1, baseline1, start, End, checkfit1
//	Nvar SI2, CheckOffset2, Decay2, baseline2, StartFit2, EndFit2, CheckFit2, Ppnts2
//	Wave TEMPEVENT2, AverageEvent2, Curseur_debut_fit_X2, Curseur_debut_fit_Y2, Curseur_fin_fit_X2, Curseur_fin_fit_Y2
//	Wave Curseur_debut_fit_X2_Av, Curseur_debut_fit_Y2_Av, Curseur_fin_fit_X2_Av, Curseur_fin_fit_Y2_Av
//	Variable debut, fin, counter
//	Variable V_FitOptions = 4
//	
//	String TEMPSTR
//	
//	Make/o/N=10 FitParametersFitdbexp2
//	
//	//If single event box checked
//	if (CheckFit2 == 0)
//		// The following lines are meant to remove any previous fit wave on the graph. 
//		//Otherwise it gets messy and Igor superimpose many waves with the same name.
//		
//		GetWindow DetectedEvents2#EventChannel2, WaveList
//		Wave/T W_WaveList = W_WaveList
//		for (counter = 0; counter < DimSize(W_WaveList, 0); counter += 1)
//			TEMPSTR = W_WaveList[counter][0]
//			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[counter][0]
//			endif
//			if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[counter][0]
//			endif
//			if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[counter][0]
//			endif		
//		endfor
//		// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//		Make/o fit_TEMPEVENT2 = 0
//		Appendtograph/W=DetectedEvents2#EventChannel2 fit_TEMPEVENT2
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Duplicate/O TEMPEVENT2, TEMPWAVE
//		Smooth/B Ppnts2, TEMPWAVE
//		Wavestats/Q/R=(0, Decay2*.001) TEMPWAVE
//		//FindPeak/N/Q/R=(0,Decay2*.001) TEMPWAVE // /N means search for a negative peak, /Q no history printing no error reporting if no peak found, value is X by default /R = range for search
//		// Better w/ findlevel to find the peak of the event (more sure than find peak because smallest value
//		FindLevel/Q/R=(0,Decay2*.001) TEMPWAVE, V_min
//		
//		If (V_flag == 0)
//			FindLevel/Q/R=(V_levelX,Decay2*.001) TEMPWAVE V_min*(1-StartFit2*.01)
//		Else
//			Abort "No peak found"
//		EndIf
//		
//		If (V_flag == 0)
//			debut = V_LevelX
//		Else
//			Abort "No level found"
//		EndIf
//		
//		fin = EndFit2*.001
//		K0 = 0
//		// The fit command
//		if (CheckOffset2==1)
//			CurveFit/Q/H="00000" exp TEMPEVENT2 (debut, fin) /D
//		else
//			CurveFit/Q/H="10000" exp TEMPEVENT2 (debut, fin) /D
//		endif
//		
//		FitParametersFitdbexp2[0]= (1/K2)*1000
//		FitParametersFitdbexp2[1]= NaN
//		FitParametersFitdbexp2[8]=K0
//		FitParametersFitdbexp2[2] = 100
//		FitParametersFitdbexp2[3] = NaN
//		execute "ValDisplay TauFast2 value=" + num2str(FitParametersFitdbexp2[0])
//		execute "ValDisplay TauSlow2 value=" + num2str(FitParametersFitdbexp2[1])
//		execute "ValDisplay FastPercentage2 value=" + num2str(FitParametersFitdbexp2[2])
//		execute "ValDisplay SlowPercentage2 value=" + num2str(FitParametersFitdbexp2[3])
//		execute "ValDisplay ValOffset2 value=" + num2str(FitParametersFitdbexp2[8])
//		execute "Valdisplay KineticIndex2 value=" + num2str((1/K2) * 1000) // Insert the time constant converted to ms
//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		getaxis/Q/W=DetectedEvents2#EventChannel2 bottom
//		Curseur_debut_fit_X2[0]= (Debut - (.05*(V_max-V_min)))
//		Curseur_debut_fit_X2[1]= (Debut + (.05*(V_max-V_min)))
//		Curseur_debut_fit_Y2[0]= K0+ K1*exp(-K2*debut)
//		Curseur_debut_fit_Y2[1]= K0+ K1*exp(-K2*debut)
//		Curseur_fin_fit_X2[0] = EndFit2*.001
//		Curseur_fin_fit_X2[1] = EndFit2*.001
//		getaxis/W=DetectedEvents2#EventChannel2/Q left
//		Curseur_fin_fit_Y2[0] = K0+ K1*exp(-K2*fin) - 0.05*(V_max-V_min) - 0.05*(V_max-V_min)
//		Curseur_fin_fit_Y2[1] = K0+ K1*exp(-K2*fin) + 0.05*(V_max-V_min) + 0.05*(V_max-V_min)
//		appendtograph/W=DetectedEvents2#EventChannel2 Curseur_debut_fit_Y2 vs Curseur_debut_fit_X2; ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(Curseur_debut_fit_Y2)=(0,0,0)
//		appendtograph/W=DetectedEvents2#EventChannel2 Curseur_fin_fit_Y2 vs Curseur_fin_fit_X2; ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(Curseur_fin_fit_Y2)=(0,0,0)
//		ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(fit_TEMPEVENT2)=(0,0,0)
//		
//	//For the average box checked ***************************************************************************************************************************************
//	Elseif (CheckFit2 == 1)
//		// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
//		// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
//		GetWindow DetectedEvents2#Average2, WaveList
//		Wave/T W_WaveList = W_WaveList
//		for (counter = 1; counter < DimSize(W_WaveList, 0); counter += 1)
//			TEMPSTR = W_WaveList[counter][0]
//			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#Average2 $W_WaveList[counter][0]
//			endif
//			if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#Average2 $W_WaveList[counter][0]
//			endif
//			if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#Average2 $W_WaveList[counter][0]
//			endif		
//		endfor
//		// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//		Make/o fit_AverageEvent2 = 0
//		Appendtograph/W=DetectedEvents2#Average2 fit_AverageEvent2
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Wavestats/Q/R=(0, Decay2*.001) AverageEvent2
//		
//		// Search the peak using the minimum value
//		FindLevel/Q/R=(0, (Decay2 * 0.001)) AverageEvent2, V_min
//		If (V_flag == 0)
//			FindLevel/Q/R=(V_levelX, (Decay2 * 0.001)) AverageEvent2, (V_min * (1 - StartFit2 * 0.01))
//		Else
//			Abort "level not found"
//		EndIf
//		
//		If (V_flag == 0)
//			debut = V_LevelX
//		Else
//			Abort "Level not found"
//		EndIf
//		fin = (EndFit2 * 0.001)
//		K0 = 0
//		// The fit command
//		if (CheckOffset2 == 1)
//			CurveFit/Q/H="00000" exp AverageEvent2(debut, fin) /D
//		else
//			CurveFit/Q/H="10000" exp AverageEvent2(debut, fin) /D
//		endif
//		FitParametersFitdbexp2[0]= (1/K2)*1000
//		FitParametersFitdbexp2[1]= NaN
//		FitParametersFitdbexp2[8]=K0
//		FitParametersFitdbexp2[2] = 100
//		FitParametersFitdbexp2[3] = NaN
//		execute "ValDisplay TauFast2 value=" + num2str(FitParametersFitdbexp2[0])
//		execute "ValDisplay TauSlow2 value=" + num2str(FitParametersFitdbexp2[1])
//		execute "ValDisplay FastPercentage2 value=" + num2str(FitParametersFitdbexp2[2])
//		execute "ValDisplay SlowPercentage2 value=" + num2str(FitParametersFitdbexp2[3])
//		execute "ValDisplay ValOffset2 value=" + num2str(FitParametersFitdbexp2[8])
//		execute "Valdisplay KineticIndex2 value=" + num2str((1/K2) * 1000) // Insert the time constant converted to ms
//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		getaxis/Q/W=DetectedEvents2#EventChannel2 bottom
//		Curseur_debut_fit_X2_Av[0]= (Debut - (.05*(V_max-V_min)))
//		Curseur_debut_fit_X2_Av[1]= (Debut + (.05*(V_max-V_min)))
//		Curseur_debut_fit_Y2_Av[0]= K0+ K1*exp(-K2*debut)
//		Curseur_debut_fit_Y2_Av[1]= K0+ K1*exp(-K2*debut)
//		Curseur_fin_fit_X2_Av[0] = EndFit2*.001
//		Curseur_fin_fit_X2_Av[1] = EndFit2*.001
//		getaxis/W=DetectedEvents2#EventChannel2/Q left
//		Curseur_fin_fit_Y2_Av[0] = K0+ K1*exp(-K2*fin) - 0.05*(V_max-V_min) - 0.05*(V_max-V_min)
//		Curseur_fin_fit_Y2_Av[1] = K0+ K1*exp(-K2*fin) + 0.05*(V_max-V_min) + 0.05*(V_max-V_min)
//		appendtograph/W=DetectedEvents2#Average2 Curseur_debut_fit_Y2_Av vs Curseur_debut_fit_X2_Av; ModifyGraph/W=DetectedEvents2#Average2 rgb(Curseur_debut_fit_Y2_Av)=(0,0,0)
//		appendtograph/W=DetectedEvents2#Average2 Curseur_fin_fit_Y2_Av vs Curseur_fin_fit_X2_Av; ModifyGraph/W=DetectedEvents2#Average2 rgb(Curseur_fin_fit_Y2_Av)=(0,0,0)
//		ModifyGraph/W=DetectedEvents2#Average2 rgb(fit_AverageEvent2)=(0,0,0)
//	endif
//End


// Charly **********************************************here are some usefull functions for paired recordings

// ***************This function aims at deleting NaNs in the occurrence wave (usefull when the findLevels function didn't work properly to find half rise time)
Function ProcDeleteNaN(CtrlName) : ButtonControl
	String CtrlName
	
	String Tags_waveN, Occurrence_waveN
	Tags_waveN = "Tags" + CtrlName[(strlen(CtrlName) - 1)] // 1 or 2 will be added at the end of the name depending on the button that triggered the procedure
	Occurrence_waveN = "Occurrence" + CtrlName[(strlen(CtrlName) - 1)]
	
		Wave Tags = $Tags_waveN
		Wave Occurrence = $Occurrence_waveN
		
		Variable counter
		For (counter = 0; counter < numpnts(Tags); counter += 1)
			If (numtype(Occurrence[counter]) == 2) // If Point is NaN
				If (Tags[counter] == 1)
					Print "NaN foud at step " + num2str(counter) + " will be removed"
				EndIf
				Tags[counter] = 0 //Untag the event
				//Print "NaN found at step : " + num2str(counter) 
			EndIf
		EndFor
End

// Charly ************************************************ This is the function I use instead of Igot built in FindLevels (Uses more processor so longer but less memory use, usefull for big waves)******
Function My_Find_Levels(src_wave, Dest_waveN, thrshld)
	Wave src_wave
	String Dest_waveN
	Variable thrshld
	
	Duplicate/O src_wave, delta_wave
	Make/O/D/N=0 temp_wave
	Variable TEMPO_delta, sample_interval, Yn_min_1, Yn
	delta_wave -= Thrshld
	
	
	Variable counter, counter2
	counter2 = 0
	
	For(counter = 0; counter < numpnts(delta_wave); counter += 1)
		
		//Insert something
		If (counter == (numpnts(delta_wave) - 1))
			//Insert a behaviour for last point
		ElseIf (((Delta_wave[counter] <= 0) && (Delta_wave[counter + 1] > 0)) || ((Delta_wave[counter] >= 0) && (Delta_wave[counter + 1] < 0)))
			InsertPoints counter2, 1, temp_wave
			sample_interval = deltaX(src_wave)
			Yn_min_1 = (Abs(delta_wave[counter]))
			Yn = (Abs(delta_wave[counter + 1]))
			
			TEMPO_delta = (sample_interval * (Yn_min_1 / (Yn_min_1 + Yn)))
			temp_wave[counter2] = ((pnt2X(delta_wave, counter)) + TEMPO_Delta)
			counter2 += 1
		EndIF
	EndFOr
	Duplicate/O TEmp_wave, $dest_waveN
	KillWaves temp_wave, delta_wave
	
End

 // Charly ****************** this loads another function for paired recordings
Function Btn_MAS(ctrlName) : ButtonControl
	String CtrlName
	If (Cmpstr((CtrlName[StrLen(CtrlName) - 1]), "1") == 0)
		MAS_prompt(1)
	ElseIf (Cmpstr((CtrlName[StrLen(CtrlName) - 1]), "2") == 0)
		MAS_prompt(2)
	EndIf
End

// Charly ***************************** Function to select one reference channel to use only tags from target channel that match w/ reference chan.
Function tags_synchroniser(check_win_start, check_win_end, Occurrence_ch1, Occurrence_ch2, True_Tags_ch1, True_Tags_ch2, PeakPositionW_ch2, Tags_ch2_syncN)
	Variable check_win_start, check_win_end //Defines the borders of the window set by the crosscorelogram
	Wave Occurrence_ch1, Occurrence_ch2, True_Tags_ch1, True_Tags_ch2, PeakPositionW_ch2//Defines what is the reference occurrence wave (ch1)
	String Tags_ch2_syncN
	// invert order of arguments if necessary (Same applies for Tags) True means original wave
	
	If (WaveExists(Tags_ch1_save) == 0)//Saves the wave if this is not already the case
		Duplicate/O True_Tags_ch1, Tags_ch1_save
	EndIf
	
	If (WaveExists(Tags_ch2_save) == 0) //Saves the wave if this is not already the case
		Duplicate/O True_tags_ch2, Tags_ch2_save
	EndIf
	
	Duplicate/O True_tags_ch2, Tags_ch2_sync // The temporary output (the wave after synchronisation)
	
	Variable counter, counter_2 = 0
	
	For (counter = 0; (counter < numpnts(Occurrence_ch1)); counter += 1) // Browse through channel 1
			
		// If the occurrence of the point is NaN go to next point untill the next non NaN point
		For ( ; ((numtype(Occurrence_ch1[counter]) == 2) && (counter < numpnts(Occurrence_ch1))); ) 
			counter += 1
		EndFor
	
			If ((True_Tags_ch1[counter]) == 1) // If event selected on chan 1
			
				// If the occurrence of the point is NaN go to next point untill the next non NaN point
				For ( ; ((numtype(Occurrence_ch2[counter_2]) == 2) && (counter_2 < numpnts(Occurrence_ch2))); ) 
					counter_2 += 1
				EndFor
				
				// As long as event on channel 2 is before the end of the window compared to event on channel 1
				For	(; ((Occurrence_ch2[counter_2] < (Occurrence_ch1[counter] + check_win_end)) && (counter_2 < numpnts(Occurrence_ch2))); ) // We use a For/endFor loop to test condition before running the loop	
					
					// If the occurrence of the point is NaN go to next point untill the next non NaN point
					For ( ; ((numtype(Occurrence_ch2[counter_2]) == 2) && (counter_2 < numpnts(Occurrence_ch2))); ) 
						counter_2 += 1
					EndFor
					
					
			 		If ((True_tags_ch2[counter_2]) == 1) // And if event selected on chan 2
						// Check if event on chan 2 corresponds to event on chan 1
			
						//If the event is before the window
						If((Occurrence_ch2[counter_2]) < ((Occurrence_ch1[counter]) + check_win_start))
							Tags_ch2_sync[counter_2] = 0 // untag it (We use untag instead of tag to take into account the previous state of the tags
						Endif	
					
					EndIf
						
						counter_2 += 1	// Update the counter each time an event on chan 2 is checked
				EndFor
			
			EndIf
		
	EndFor
	// Clear potentially remaining tags
	Tags_ch2_sync[counter_2, (Numpnts(Occurrence_ch2) - 1)] = 0
	Duplicate/O Tags_ch2_sync, $Tags_ch2_syncN
	KillWaves Tags_ch2_sync // Kill the temporary output
	
End

// Charly **************** Selects the origin of the channel 1 tags after synch 
Function OriginOfTags1(CtrlName,popNum,popStr) : PopupMenuControl
	String CtrlName
	Variable popNum // Selected item reference number
	String popStr // content of current popup item as string
	Nvar Tags1_origin
	If (popNum == 1)
		If (WaveExists(Tags_ch1_save))
			Duplicate/O Tags_ch1_save, Tags1
			Print "Tags1 reinitialized"
			Tags1_origin = 1
		EndIf
	Elseif (popNum == 2)
		If (WaveExists(Tags1Sync))
			Duplicate/O Tags1Sync, Tags1
			Tags1_origin = 2
		EndIf
	EndIf
End


// Charly **************** Selects the origin of the channel 2 tags after synch 
Function OriginOfTags2(CtrlName,popNum,popStr) : PopupMenuControl
	String CtrlName
	Variable popNum // Selected item reference number
	String popStr // content of current popup item as string
	Nvar Tags2_origin
	If (popNum == 1)
		If (WaveExists(Tags_ch2_save))
			Duplicate/O Tags_ch2_save, Tags2
			Print "Tags2 reinitialized"
			Tags2_origin = 1
		EndIf
	Elseif (popNum == 2)
		If (WaveExists(Tags2Sync))
			Duplicate/O Tags2Sync, Tags2
			Tags2_origin = 2
		EndIf
	EndIf
End

// Charly ******************************Triggers sync of events on chan 1 corresponding to events on chan 2
Function SyncChan1From2(ctrlName) : ButtonControl
	String CtrlName
	
	DoAlert 1, "Do you want to reinitialise tags1"
	
	If (V_flag == 1)
		// To prevent use of false tags1 as ref
		OriginOfTags1("",1,"")
	EndIf
	
	// Borders of the detection window
	NVar SyncWinStart1, SyncWinEnd1
	Variable check_win_start, check_win_end
	check_win_start = SyncWinStart1 * 0.001 // set to secs
	check_win_end = SyncWinEnd1 * 0.001 // set tio secs
	
	// If all the arguments are present
	If (WaveExists(Occurrence1) && WaveExists(Occurrence2) && WaveExists(Tags1) && WaveExists(Tags2))
		Wave Occurrence1, Occurrence2, Tags1, Tags2, RealPeakPosition1
		tags_synchroniser(check_win_start, check_win_end, Occurrence2, Occurrence1, Tags2, Tags1, RealPeakPosition1, "Tags1Sync")
	Else
		Abort "Be sure to tag events on both channels before"
	EndIf
	DoAlert 1, "Do you want to update and overwrite tags1"
	If (V_flag == 1) // If yes is clicked
		OriginOfTags1("",2,"")
	EndIf
End

// Charly ******************************Triggers sync of events on chan 2 corresponding to events on chan 1
Function SyncChan2From1(ctrlName) : ButtonControl
	String CtrlName
	
	DoAlert 1, "Do you want to reinitialise tags2"
	
	If (V_flag == 1)
		// To prevent use of false tags2 as ref
		OriginOfTags2("",1,"")
	EndIf
	
	// Borders of the detection window
	NVar SyncWinStart2, SyncWinEnd2
	Variable check_win_start, check_win_end
	check_win_start = SyncWinStart2 * 0.001 // set to secs
	check_win_end = SyncWinEnd2 * 0.001 // set tio secs
	
	// If all the arguments are present
	If (WaveExists(Occurrence1) && WaveExists(Occurrence2) && WaveExists(Tags1) && WaveExists(Tags2))
		Wave Occurrence1, Occurrence2, Tags1, Tags2, RealPeakPosition2
		tags_synchroniser(check_win_start, check_win_end, Occurrence1, Occurrence2, Tags1, Tags2, RealPeakPosition2, "Tags2Sync")
	Else
		Abort "Be sure to tag events on both channels before"
	EndIf
	DoAlert 1, "Do you want to update and overwrite tags2"
	If (V_flag == 1) // If yes is clicked
		OriginOfTags2("",2,"")
	EndIf
End

// Charly ************************* Two function to save easyly tags as separate waves
Function ProcSaveTags1(ctrlName) : ButtonControl
	String CtrlName
	If (WaveExists(Tags_ch1_save))
		DoAlert 1, "Caution, you are about to save the tags1 backup if you want to save the actual copy click NO"
		If (V_Flag == 1)
			Save/C Tags_ch1_save
		Else
			Save/C Tags1
		EndIf
	Else
		Save/C Tags1
	EndIf
End

Function ProcSaveTags2(ctrlName) : ButtonControl
	String CtrlName
	If (WaveExists(Tags_ch2_save))
		DoAlert 1, "Caution, you are about to save the tags2 backup if you want to save the actual copy click NO"
		If (V_Flag == 2)
			Save/C Tags_ch2_save
		Else
			Save/C Tags2
		EndIf
	Else
		Save/C Tags2
	EndIf
End

// Charly different Fit functions called by the procedure launched by the buttons
Function FitDbExp1()
	SpAcAnGeneralFit(1, 1, "dblexp")
End

Function FitDbExp2()
	SpAcAnGeneralFit(2, 2, "dblexp")
End

Function Fitdbexp_ch1_on_win2()
	SpAcAnGeneralFit(2, 1, "dblexp")
End

Function Fitdbexp_ch2_on_win1()
	SpAcAnGeneralFit(1, 2, "dblexp")
End

Function FitSingleExp1()
	SpAcAnGeneralFit(1, 1, "exp")
End

Function FitSingleExp2()
	SpAcAnGeneralFit(2, 2, "exp")
End

Function FitSingleExp_ch1_on_win2()
	SpAcAnGeneralFit(2, 1, "exp")
End

Function FitSingleExp_ch2_on_win1()
	SpAcAnGeneralFit(1, 2, "exp")
End

Function FitSigDbExp1()
	SpAcAnGeneralFit(1, 1, "SigDbExp")
End

Function FitSigDbExp_ch2_on_win1()
	SpAcAnGeneralFit(1, 2, "SigDbExp")
End

Function FitSigDbExp2()
	SpAcAnGeneralFit(2, 2, "SigDbExp")
End

Function FitSigDbExp_ch1_on_win2()
	SpAcAnGeneralFit(2, 1, "SigDbExp")
End

Function FitSigmoid1()
	SpAcAnGeneralFit(1, 1, "Sigmoid")
End

Function FitSigmoid_ch2_on_win1()
	SpAcAnGeneralFit(1, 2, "Sigmoid")
End

Function FitSigmoid2()
	SpAcAnGeneralFit(2, 2, "Sigmoid")
End

Function FitSigmoid_ch1_on_win2()
	SpAcAnGeneralFit(2, 1, "Sigmoid")
End

// Charly *****************Function to set the role of the fitdbexp button depending on the value of the selected chan popup menu (either fit channel 1 or 2)
Function Fitdbexp_select1(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb1
	
	If (fit_chan_select_nb1 == 1)
		Fitdbexp1()
	ElseIf (fit_chan_select_nb1 == 2)
		Fitdbexp_ch2_on_win1()
	EndIf
End

Function Fitdbexp_select2(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb2
	
	If (fit_chan_select_nb2 == 2)
		Fitdbexp2()
	ElseIf (fit_chan_select_nb2 == 1)
		Fitdbexp_ch1_on_win2()
	EndIf
End

Function FitSingleExp_select1(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb1
	
	If (fit_chan_select_nb1 == 1)
		FitSingleExp1()
	ElseIf (fit_chan_select_nb1 == 2)
		FitSingleExp_ch2_on_win1()
	EndIf
End

Function FitSingleExp_select2(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb2
	
	If (fit_chan_select_nb2 == 2)
		FitSingleExp2()
	ElseIf (fit_chan_select_nb2 == 1)
		FitSingleExp_ch1_on_win2()
	EndIf
End

Function FitSigDbExp_select1(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb1
	
	If (fit_chan_select_nb1 == 1)
		FitSigDbExp1()
	ElseIf (fit_chan_select_nb1 == 2)
		FitSigDbExp_ch2_on_win1()
	EndIf
End

Function FitSigDbExp_select2(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb2
	
	If (fit_chan_select_nb2 == 2)
		FitSigDbExp2()
	ElseIf (fit_chan_select_nb2 == 1)
		FitSigDbExp_ch1_on_win2()
	EndIf
End

Function FitSigmoid_select1(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb1
	
	If (fit_chan_select_nb1 == 1)
		FitSigmoid1()
	ElseIf (fit_chan_select_nb1 == 2)
		FitSigmoid_ch2_on_win1()
	EndIf
End

Function FitSigmoid_select2(ctrlName) : ButtonControl
	String CtrlName
	
	Nvar fit_chan_select_nb2
	
	If (fit_chan_select_nb2 == 2)
		FitSigmoid2()
	ElseIf (fit_chan_select_nb2 == 1)
		FitSigmoid_ch1_on_win2()
	EndIf
End

// Function to select the role of the checkboxes menu used to select the channel that has to be fitted
Function fit_chan_select1(Name,CtrlVal)
	String Name
	Variable CtrlVal // Selected item reference number
	
	Variable/G fit_chan_select_nb1
	
	If (cmpstr(Name[strlen(Name) - 1], "1") == 0) // If last caracter of name is 1 (for ch1 checkBox)
		If (CtrlVal == 1) // If box selected
			Fit_chan_select_nb1 = 1 // Then select channel 1
			Checkbox CheckBFitSelectedChan12, value= 0 // Deselect box 2
		ElseIf (CtrlVal == 0) // If box Deselected
			Fit_chan_select_nb1 = 2 // Then select channel 2
			Checkbox CheckBFitSelectedChan12, value= 1 // Select box 2
		EndIf
	ElseIf(cmpstr(Name[strlen(Name) - 1], "2") == 0) // If last caracter of name is 2 (for ch2 checkBox)
		If (CtrlVal == 1) // If box selected
			Fit_chan_select_nb1 = 2 // Then select channel 2
			Checkbox CheckBFitSelectedChan11, value= 0 // Deselect box 1
		ElseIf (CtrlVal == 0) // If box Deselected
			Fit_chan_select_nb1 = 1 // Then select channel 1
			Checkbox CheckBFitSelectedChan11, value= 1 // Select box 1
		EndIf
	EndIf
End

Function fit_chan_select2(Name,CtrlVal)
	String Name
	Variable CtrlVal // Selected item reference number
	
	Variable/G fit_chan_select_nb2
	
	If(cmpstr(Name[strlen(Name) - 1], "2") == 0) // If last caracter of name is 2 (for ch2 checkBox)
		If (CtrlVal == 1) // If box selected
			Fit_chan_select_nb2 = 2 // Then select channel 2
			Checkbox CheckBFitSelectedChan21, value= 0 // Deselect box 1
		ElseIf (CtrlVal == 0) // If box Deselected
			Fit_chan_select_nb2 = 1 // Then select channel 1
			Checkbox CheckBFitSelectedChan21, value= 1 // Select box 1
		EndIf
	ElseIf (cmpstr(Name[strlen(Name) - 1], "1") == 0) // If last caracter of name is 1 (for ch1 checkBox)
		If (CtrlVal == 1) // If box selected
			Fit_chan_select_nb2 = 1 // Then select channel 1
			Checkbox CheckBFitSelectedChan22, value= 0 // Deselect box 2
		ElseIf (CtrlVal == 0) // If box Deselected
			Fit_chan_select_nb2 = 2 // Then select channel 2
			Checkbox CheckBFitSelectedChan22, value= 1 // Select box 2
		EndIf
	
	EndIf
End


Function IntegrateChanSelect1(Name,CtrlVal)
	String Name
	Variable CtrlVal // Selected item reference number
	
	Variable/G IntegrateChanSelectNb1
	
	If (cmpstr(Name[strlen(Name) - 1], "1") == 0) // If last caracter of name is 1 (for ch1 checkBox)
		If (CtrlVal == 1) // If box selected
			IntegrateChanSelectNb1 = 1 // Then select channel 1
			Checkbox CheckBIntSelectedChan12, value= 0 // Deselect box 2
		ElseIf (CtrlVal == 0) // If box Deselected
			IntegrateChanSelectNb1 = 2 // Then select channel 2
			Checkbox CheckBIntSelectedChan12, value= 1 // Select box 2
		EndIf
	ElseIf(cmpstr(Name[strlen(Name) - 1], "2") == 0) // If last caracter of name is 2 (for ch2 checkBox)
		If (CtrlVal == 1) // If box selected
			IntegrateChanSelectNb1 = 2 // Then select channel 2
			Checkbox CheckBIntSelectedChan11, value= 0 // Deselect box 1
		ElseIf (CtrlVal == 0) // If box Deselected
			IntegrateChanSelectNb1 = 1 // Then select channel 1
			Checkbox CheckBIntSelectedChan11, value= 1 // Select box 1
		EndIf
	EndIf
End

Function IntegrateChanSelect2(Name,CtrlVal)
	String Name
	Variable CtrlVal // Selected item reference number
	
	Variable/G IntegrateChanSelectNb2
	
	If(cmpstr(Name[strlen(Name) - 1], "2") == 0) // If last caracter of name is 2 (for ch2 checkBox)
		If (CtrlVal == 1) // If box selected
			IntegrateChanSelectNb2 = 2 // Then select channel 2
			Checkbox CheckBIntSelectedChan21, value= 0 // Deselect box 1
		ElseIf (CtrlVal == 0) // If box Deselected
			IntegrateChanSelectNb2 = 1 // Then select channel 1
			Checkbox CheckBIntSelectedChan21, value= 1 // Select box 1
		EndIf
	ElseIf (cmpstr(Name[strlen(Name) - 1], "1") == 0) // If last caracter of name is 1 (for ch1 checkBox)
		If (CtrlVal == 1) // If box selected
			IntegrateChanSelectNb2 = 1 // Then select channel 1
			Checkbox CheckBIntSelectedChan22, value= 0 // Deselect box 2
		ElseIf (CtrlVal == 0) // If box Deselected
			IntegrateChanSelectNb2 = 2 // Then select channel 2
			Checkbox CheckBIntSelectedChan22, value= 1 // Select box 2
		EndIf

	EndIf
End

Function IntegrateEvent1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIntegrate1
	CheckIntegrate1 = 1 - CheckIntegrate1
	CheckBox CheckBIntEvent1 value = CheckIntegrate1
	CheckBox CheckBIntAvg1 value = 1 - CheckIntegrate1
End

Function IntegrateAvg1(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIntegrate1
	CheckIntegrate1 = 1 - CheckIntegrate1
	CheckBox CheckBIntAvg1 value = 1 - CheckIntegrate1
	CheckBox CheckBIntEvent1 value = CheckIntegrate1
End

Function IntegrateEvent2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIntegrate2
	CheckIntegrate2 = 1 - CheckIntegrate2
	CheckBox CheckBIntEvent2 value = CheckIntegrate2
	CheckBox CheckBIntAvg2 value = 1 - CheckIntegrate2
End

Function IntegrateAvg2(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Nvar CheckIntegrate2
	CheckIntegrate2 = 1 - CheckIntegrate2
	CheckBox CheckBIntAvg2 value = 1 - CheckIntegrate2
	CheckBox CheckBIntEvent2 value = CheckIntegrate2
End

Function ProcLowPass(CtrlName) : ButtonControl
	String CtrlName
	String ChannelNb=CtrlName[Strlen(CtrlName) - 1]
	NVar Thrshld = $("LowPassThrshld" + ChannelNb)
	LowPass(thrshld, ChannelNb)
End

Function ProcBandPass(CtrlName) : ButtonControl
	String CtrlName
	String ChannelNb=CtrlName[Strlen(CtrlName) - 1]
	Nvar LowerAmpl = BandPassLow
	Nvar UpperAmpl = BandPassHigh
	AmplitudeRangeFilter(str2num(ChannelNb), LowerAmpl, UpperAmpl)
End

Function ProcIsolateEvents(CtrlName) : ButtonControl
	String CtrlName
	String ChannelNb=CtrlName[Strlen(CtrlName) - 1]
	Wave TagsW=$("Tags" + ChannelNb)
	Wave OccurrenceW=$("Occurrence" + ChannelNb)
	NVar IsolateEventsWinStart = $("IsolateEventsWinStart" + ChannelNb)
	NVar IsolateEventsWinEnd = $("IsolateEventsWinEnd" + ChannelNb)
	RemoveDoubleEvents(TagsW, OccurrenceW, IsolateEventsWinEnd, IsolateEventsWinStart)
End


Function SpAcAnGeneralFit(WinNb, ChannelNb, FitTypeStr) // FitTypeStr shall be one of the following strings : exp, dbexp, sigmoid, sigdbexp
	Variable WinNb, ChannelNb
	String FitTypeStr
	
	print WinNb
	
	//***********************************************Local variables***********************************************
	// Name of the wave that will be fitted and the window where it is displayed
	String WaveToFitN, WindowN, TEMPSTR
	Variable debut, fin, counter
	Variable/G V_FitOptions = 4 // V_fitOptions=4 suppresses the fit window (runs faster) (global because used w/ execute
	// Temporary output wave for the fit parameters
	KillWaves/Z TempFitParamW
	Make/O/D/N=10 TEMPFitParamW
	//*********************************** Bring back the necessary global variables and strings******************************
	// Variabel whose name depend on the channel to fit
	NVar Ppnts_V = $("Ppnts" + num2str(ChannelNb))
	NVar SampleInt_V = $("SI" + num2str(ChannelNb))
	
	// Variables depending on the winDow they are accessed from
	NVar CheckOffset_V = $("CheckOffset" + num2str(WinNb))
	NVar Decay_V = $("Decay" + num2str(WinNb))
	NVar CheckFit_V = $("CheckFit" + num2str(WinNb))
	NVar StartFit_V = $("StartFit" + num2str(WinNb))
	NVar EndFit_V = $("EndFit" + num2str(WinNb))

	// Parameters that are only loaded for sigmadbexp fit	
	If (Cmpstr(FitTypeStr, "sigdbexp") == 0) 
		Wave RealAmp_W = $("RealAmp" + num2str(ChannelNb))
		NVar EventNb_V = $("EventNumber" + num2str(ChannelNb))
		SVar FileNameStr = $("FileName" + num2str(ChannelNb))
	EndIf

	//*********************************************Determining wave to fit*********************************
	If (CheckFit_V == 0) // Means fit current event
		If (ChannelNb == WinNb) // If top graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#EventChannel" + num2str(ChannelNb)
			WaveToFitN = "TEMPEVENT" + num2str(WinNb)
		Else // if bottom graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#EventChannel" + num2str(WinNb) + num2str(ChannelNb)
			WaveToFitN = "TEMPEVENT" + "_ch" + num2str(ChannelNb)
		EndIf	
	ElseIf (CheckFit_V == 1) // Means fit average
		If (ChannelNb == WinNb) // If top graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#Average" + num2str(ChannelNb)
			WaveToFitN = "AverageEvent" + num2str(WinNb)
		Else // If bottom graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#Average" + num2str(WinNb) + num2str(ChannelNb)
			WaveToFitN = "AverageEvent" + num2str(WinNb) + "CH" + num2str(ChannelNb)
		EndIf
	EndIf
	
	
	// **************************************Preliminary for sigmoid fit ***********************************
	If (Cmpstr(FitTypeStr[0, 2], "sig") == 0) // For sig or sigdbexp fit
		
		WaveStats/Q $WaveToFitN
		FindLevel/Q/R=(-V_MinLoc,V_MinLoc) $WaveToFitN, V_min*0.01*StartFit_V
		If (V_Flag != 0)
			Abort "No Level Found"
		Endif
		Debut = V_LevelX
	EndIf
	
	// *********************************Common (Fin is where the fit ends)*************************
	Fin = EndFit_V * 0.001	
	
	// **********************The following lines are meant to remove any previous fit wave on the graph*******************
	GetWindow $(WindowN), WaveList
	Wave/T W_WaveList
	
	//Set as active subwindow the window we are going to work in
	DoWindow/F $("DetectedEvents" + num2str(WinNb))
	Execute/Q/Z "SetActiveSubWindow " + WindoWN // Don't know why but it is the only thing that works ($ doesn't)
	
	If (V_Flag != 0) // If execute didn't work
		Abort "Setting active subwindow failed"
	EndIf
	
	For (counter = 1; counter < DimSize(W_WaveList, 0); counter += 1)
		TEMPSTR = W_WaveList[counter] [0]
		If (cmpstr(TEMPSTR[0, 2], "fit") == 0) // If the begginning of the waveN contains fit remove it from the graph
			RemoveFromGraph $W_WaveList[counter][0] // /W not used since active subwindow is supposed to be selected
		EndIf
	EndFor
	
	// A trick to prevent the procedure to display the fit by itself (If so the it's a mess , trust me man)
	Make/O $("fit_" + WaveToFitN) = 0
	AppendToGraph $("fit_" + WaveToFitN) // /W not used since active subwindow is supposed to be selected
	
	If (ChannelNb == 2)
		ModifyGraph rgb($("fit_" + WaveToFitN))=(0, 16000, (2^16 - 1))// Since the curve is red change the fit colour to blue
	EndIf
	
	// temporary wave to be fitted (for naming conveniences
	Duplicate/O $WaveToFitN, TEMPWAVE
	// For the events only (avg doesn't need smoothing)
	If (CheckFit_V == 0)
		Smooth/B Ppnts_V, TEMPWAVE
	EndIf
	
	K0 = 0
	
	//***********************************Following section only required for sigmoid fit************************************************
	If ((Cmpstr(FitTypeStr, "sigdbexp") == 0)) // If we use sigdbexp
		Make/D/o/N=9 SigExp
		WaveStats/Q/R=(0, Decay_V*.001) $WaveToFitN // To find the minimum, i.e. the peak
		// Then, we fit a sigmoid function to the rise phase of the event
		K0 = 0; SigExp[0]=K0		// Sigmoid : base
		CurveFit/Q/N/H="1000" Sigmoid $WaveToFitN (-V_MinLoc,V_MinLoc) /D // Fit sigmoid to the rise (symetrical around half rise)
		SigExp[1, 3] = {K1, K2, K3} // (max, xhalf, rate)
	EndIf
	
	//********************************************Actual fitting commands**************************************************
	// The exponential (or double exponential) fit command
	String ConstantFitParameters // To hold the offset parameter
	If ((Cmpstr(FitTypeStr[StrLen(FitTypeStr) - 3, StrLen(FitTypeStr)], "exp") == 0) && (Cmpstr(FitTypeStr[0,2], "sig") != 0))	// If we fit exponential (single or double)
		
		// First determine the limits of the fit (debut is the beginning)
		WaveStats/Q/R=(0, Decay_V * 0.001) TEMPWAVE // find the X location of the minimum (i.e. the peak)
		FindLevel/Q/R=(V_MinLoc, Decay_V*0.001) TEMPWave V_min*(1-StartFit_V*0.01) // Find, between Peak location and the end of the window, the 1 - StartFit % of the peak value
	
		If (V_Flag != 0) // if FindLevel didn't work
			Abort "No level found"
		EndIf
		
		Debut=V_LevelX // start of the fit in seconds
		
		if (CheckOffset_V==1)
			ConstantFitParameters = "00000"
		else
			ConstantFitParameters = "10000"
		endif
		
		Execute  "CurveFit/Q/N/H=\"" + ConstantFitParameters + "\" " + FitTypeStr + " " + WaveToFitN + "(" + num2str(debut)  + ", "  + num2str(fin) + ") /D"
		
		TEMPFitParamW[0]=(1/K4)*1000 // Tau fast (ms)
		TEMPFitParamW[1]=(1/K2)*1000 // Tau slow
		TEMPFitParamW[2] = (K3 / (K1 + K3))*100 // % Fast
		TEMPFitParamW[3] = (K1 / (K1 + K3))*100 // % slow
		
		If (CmpStr(FitTypeStr, "exp") == 0)
			TempFitParamW[0] = (1/K2)*1000
			TempFitParamW[1,3] = {0, 100, 0}
		EndIf
		TEMPFitParamW[8]=K0 // Offset
	// The sigmoid double exponential fitting command	
	ElseIf (Cmpstr(FitTypeStr, "sigdbexp") == 0)
		WaveStats/Q/R=(0,Decay_V*.001) $WaveToFitN
		K0 = 0
		if (CheckOffset_V==1)
			ConstantFitParameters = "\"10000\""
		else
			ConstantFitParameters = "\"00000\""
		endif
		
		// Initial guess for the biexp part
		Execute "CurveFit/Q/N/H=" + ConstantFitParameters + " dblexp " + WaveToFitN + "(" + num2str(V_MinLoc + (10*SampleInt_V*1e-6)) + ", " + num2str(fin) + ") /D" // Starts the fit 10 points after the peak
		
		SigExp[4]=K3/(K3+K1)		// Dbexp : amplitude fast
		SigExp[5]=K4			// Dbexp : fast time constant
		SigExp[6]=K1/(K3+K1)		// Dbexp : amplitude slow
		SigExp[7]=K2			// Dbexp : slow time constant
		SigExp[8]=K0			// Dbexp : offset
		
		Make/D/N=8/O W_coef
		W_coef[0] = {SigExp[1],SigExp[2],SigExp[3],0,SigExp[4],SigExp[5],SigExp[6],SigExp[7]}
		// W_coef = {max, xhalf, rate, baseline, amplitude fast, fast time constant, amplitude slow, slow time constant}
	
		if (CheckOffset_V==1)
			Execute "FuncFit/N/H=\"00000000\" Sigmoid_Dbexp W_coef "+ WaveToFitN + "("+ num2str(debut) + ", " + num2str(fin) + ") /D" // Offset is set on zero (The fit returns to baseline)
		else
			Execute "FuncFit/N/H=\"00010000\" Sigmoid_Dbexp W_coef " + WaveToFitN + "(" + num2str(debut) + ", " + num2str(fin) + ") /D" // Offset is set on zero (The fit returns to baseline)
		endif
		// The following lines are for displaying the coefficients in the control bar (nicer than the raw result in the command window, isn't it ?) 
		TEMPFitParamW[0] = (1/W_coef[5])*1000	// taufast
		TEMPFitParamW[1] = (1/W_coef[7])*1000	// tauslow
		TEMPFitParamW[2] = (W_coef[4] / ( W_coef[4] + W_coef[6] ))*100	// % fast
		TEMPFitParamW[3] = (W_coef[6] / ( W_coef[4] + W_coef[6] ))*100	// % slow
		TEMPFitParamW[4] = W_coef[3]	// Offset // Usefull ?
		TEMPFitParamW[8] = W_coef[3]	// Offset
	
	// The simple sigmoid fitting command	
	ElseIf (Cmpstr(FitTypeStr, "sigmoid") == 0)
		Make/D/o/N=9 SigExp
		WaveStats/Q/R=(0, Decay_V*.001) $WaveToFitN // To find the minimum, i.e. the peak
		// Then, we fit a sigmoid function to the rise phase of the event
		K0 = 0; SigExp[0]=K0		// Sigmoid : base
		CurveFit/Q/N/H="1000" Sigmoid $WaveToFitN (Debut,V_MinLoc) /D // Fit sigmoid to the rise (symetrical around half rise)
		SigExp[1, 3] = {K1, K2, K3} // (max, xhalf, rate)
		TEMPFitParamW[0, 2] = SigExp[p+1] // (max, xhalf, rate)
	EndIf
	
	//**********************************************Displaying the cursors***************************
	//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		getaxis/W=DetectedEvents2#EventChannel2 bottom
//		Curseur_debut_fit_X2[0] = debut - .05*(V_max-V_min)
//		Curseur_debut_fit_X2[1] = debut + .05*(V_max-V_min)
//		Curseur_debut_fit_Y2[0] = K0 + K1*exp(-K2*debut) + K3*exp(-K4*debut)
//		Curseur_debut_fit_Y2[1] = K0 + K1*exp(-K2*debut) + K3*exp(-K4*debut)
//		getaxis/W=DetectedEvents2#EventChannel2 left
//		Curseur_fin_fit_X2[0] = EndFitdbexp2*.001
//		Curseur_fin_fit_X2[1] = EndFitdbexp2*.001
//		Curseur_fin_fit_Y2[0] = K0 + K1*exp(-K2*fin) + K3*exp(-K4*fin) - 0.05*(V_max-V_min)
//		Curseur_fin_fit_Y2[1] = K0 + K1*exp(-K2*fin) + K3*exp(-K4*fin) + 0.05*(V_max-V_min)
//		AppendToGraph/W=DetectedEvents2#EventChannel2 Curseur_debut_fit_Y2 vs Curseur_debut_fit_X2; ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(Curseur_debut_fit_Y2)=(0,0,0)
//		AppendToGraph/W=DetectedEvents2#EventChannel2 Curseur_fin_fit_Y2 vs Curseur_fin_fit_X2; ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(Curseur_fin_fit_Y2)=(0,0,0)
//	endif
	
	
	//**********************************************Displaying the values in the detected events windows***************************
	execute "ValDisplay TauFast" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[0])
	execute "ValDisplay TauSlow" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[1])
	execute "ValDisplay FastPercentage" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[2])
	execute "ValDisplay SlowPercentage" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[3])
	execute "ValDisplay ValOffset" + num2str(WinNb) + " value=" + num2str(TEMPFitParamW[8])
	Variable KI = (TEMPFitParamW[2]/100)*TEMPFitParamW[0] + (TEMPFitParamW[3]/100)*TEMPFitParamW[1]
	execute "ValDisplay KineticIndex" + num2str(WinNb) + " value=" + num2str(KI)
	
	Duplicate/O TEMPFitParamW, $("FitParametersFit" + FitTypeStr + num2str(winNb))
	KillWaves TEMPFitParamW
	Killvariables V_FitOptions
	
End

Function SetCheckIntegralOffset(Ctrlname, Checked) : CheckBoxControl
	String CtrlName
	Variable Checked
	NVar CheckIntegralOffset = $("CheckIntegralOffset" + CtrlName[strlen(CtrlName) - 1]) // Append the last caracter (i.e. channel number)
	CheckIntegralOffset = 1 - CheckIntegralOffset // 1 if 0 or 0 if 1
End

Function SpAcAnGeneralIntegration(WinNb, ChannelNb)
	Variable WinNb, ChannelNb
	
	//***********************************************Local variables***********************************************
	// Name of the wave that will be fitted and the window where it is displayed
	String WaveToIntN, WindowN, TEMPSTR
	Variable debut, fin, AvgWidth, TheArea
	
	//*********************************** Bring back the necessary global variables and strings******************************
	// Variables depending on the winDow they are accessed from
	NVar CheckIntegralOffset_V = $("CheckIntegralOffset" + num2str(WinNb))
	NVar EventOrAvg_V = $("CheckIntegrate" + num2str(WinNb))
	NVar IntegralStart_V = $("IntegralStart" + num2str(WinNb))
	NVar IntegralEnd_V = $("IntegralEnd" + num2str(WinNb))
	NVar IntAvgWidth_V = $("IntAvgWidth" + num2str(WinNb))
	// Convert ms to seconds
	debut = IntegralStart_V * 0.001
	fin = IntegralEnd_V * 0.001
	AvgWidth = IntAvgWidth_V * 0.001
	//*********************************************Determining wave to fit*********************************
	If (EventOrAvg_V == 1) // Means integrate current event
		If (ChannelNb == WinNb) // If top graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#EventChannel" + num2str(ChannelNb)
			WaveToIntN = "TEMPEVENT" + num2str(WinNb)
		Else // if bottom graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#EventChannel" + num2str(WinNb) + num2str(ChannelNb)
			WaveToIntN = "TEMPEVENT" + "_ch" + num2str(ChannelNb)
		EndIf	
	ElseIf (EventOrAvg_V == 0) // Means fit average
		If (ChannelNb == WinNb) // If top graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#Average" + num2str(ChannelNb)
			WaveToIntN = "AverageEvent" + num2str(WinNb)
		Else // If bottom graphs
			WindoWN = "DetectedEvents" + num2str(WinNb) + "#Average" + num2str(WinNb) + num2str(ChannelNb)
			WaveToIntN = "AverageEvent" + num2str(WinNb) + "CH" + num2str(ChannelNb)
		EndIf
	EndIf	
	
	// temporary wave for the integral (for convenience)
	Duplicate/O/R=(debut, fin) $WaveToIntN, TEMPWAVE	
	
	
	GetWindow $(WindowN), WaveList
	Wave/T W_WaveList
		
	//Set as active subwindow the window we are going to work in
	DoWindow/F $("DetectedEvents" + num2str(WinNb))
	Execute/Q/Z "SetActiveSubWindow " + WindoWN // Don't know why but it is the only thing that works ($ doesn't)
	If (V_Flag != 0) // If execute didn't work
		Abort "Setting active subwindow failed"
	EndIf
	
	Variable counter
	For (counter = 1; counter < DimSize(W_WaveList, 0); counter += 1)
		TEMPSTR = W_WaveList[counter] [0]
		If (cmpstr(TEMPSTR[0, 5], "IntOff") == 0) // If the begginning of the waveN contains fit remove it from the graph
			RemoveFromGraph $W_WaveList[counter][0] // /W not used since active subwindow is supposed to be selected
		EndIf
	EndFor
	
	If (CheckintegralOffset_V == 0) // If no offset
		Duplicate/O/R=(Debut, fin) TempWave, LineAB
		LineAB = 0
	Else // If we offset the trace
		LineOnCurve($WaveToIntN, debut, fin, AvgWidth)
		Wave LineAB
		TempWave[x2pnt(TempWave, debut), x2pnt(TempWave, fin)] -= LineAB // Substract the line to the curve to offset on the appropriate range
	EndIf
	
	Duplicate/O LineAB, $("IntOff" + WaveToIntN)
	KillWaves LineAB
	
	AppendToGraph $("IntOff" + WaveToIntN) // /W not used since active subwindow is supposed to be selected
	If (ChannelNb == 2)
		ModifyGraph rgb($("IntOff" + WaveToIntN))=(0, 16000, (2^16 - 1))// Since the curve is red change the fit colour to blue
	EndIf
	
	// The actual  integration process
	TheArea = Area(TempWave, debut, fin) // caltulate the integral between debut and fin (trapezoidal)
	KillWaves TempWave
	Return TheArea
End

function SpAcAnIntegrate(CtrlName) : buttonControl
	String CtrlName
	
	NVar TheArea = $("Area" + CtrlName[strlen(CtrlName) - 1])
	NVar Channelnb = $("IntegrateChanSelectNb" + CtrlName[strlen(CtrlName) - 1])
	
	Variable WinNb = Str2num(CtrlName[strlen(CtrlName) - 1]) // The last caracter of controlName is the window number
	TheArea = SpAcAnGeneralIntegration(WinNb, ChannelNb)
	
	Execute/Q "ValDisplay DisplayArea" + num2str(WinNb) + " value=" + num2str(TheArea)
End

Function LineOnCurve(TheWave, LineStart, LineEnd, AvgWidth) // Draws the line between two points of a curve
	Wave TheWave
	Variable LineStart, LineEnd, AvgWidth
	
	//Locals
	Variable v_APreMean, v_BPostMean, v_Xpoints, V_Slope
		
	v_APreMean = Mean(TheWave, (LineStart -(AvgWidth/2)), (LineStart +(AvgWidth/2)))   // Variable equal to mean of 5 seconds before burst	
	v_BPostMean = Mean(TheWave, (LineEnd -(AvgWidth/2)), (LineEnd -(AvgWidth/2)))  // Variable equal to to 5 sec after burst
 
	Duplicate/O/R=(LineStart, LineEnd), TheWave, LineAB  ///makes destination waves with correct scaling
	
	v_Xpoints = numpnts(LineAB)									/// Number of points in X wave for calc slope
	v_Slope = (v_BPostMean-v_APreMean)  /  (v_Xpoints) 				/// Cqlc slope Rise/run
	//print v_Slope, "V_Bpostmean" = num2str(v_BpostMean)
	
	LineAB = ((v_APreMean)  + (v_Slope * p))    // p= point value on x wave, multiplied by slope

	//Print "v_Bpostmean" + num2str(v_BpostMean), lineAB[inf]  /// confirms wave arrives at 2nd mean with correct slope
	// Function returns the lineAB wave
END

// Charly End of Changes************************************************************************************************


////Function Fitdbexp2(ctrlName) : ButtonControl
////	String ctrlName
//Function Fitdbexp2()
//	// Charly
//	//Nvar SI2, OffsetFitdbexp2, Decay2, baseline2, StartFitdbexp2, EndFitdbexp2, CheckFit2, Ppnts2
//	Nvar SI2, CheckOffset2, Decay2, baseline2, StartFitdbexp2, EndFitdbexp2, CheckFit2, Ppnts2
//	// Charly End
//	Wave Curseur_debut_fit_X2, Curseur_debut_fit_Y2, Curseur_fin_fit_X2, Curseur_fin_fit_Y2, TEMPEVENT2, AverageEvent2
//	Wave Curseur_debut_fit_X2_Av, Curseur_debut_fit_Y2_Av, Curseur_fin_fit_X2_Av, Curseur_fin_fit_Y2_Av
//	Variable debut, fin, S
//	Make/o/N=10 FitParametersFitdbexp2
//	Variable i; String TEMPSTR
//	if (CheckFit2 == 0)
//		// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
//		// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
//		GetWindow DetectedEvents2#EventChannel2, WaveList
//		Wave/T W_WaveList = W_WaveList
//		for (i=0; i < numpnts(W_WaveList)/3; i+=1)
//			TEMPSTR = W_WaveList[i][0]
//			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
//			endif
//			if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
//			endif
//			if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
//			endif
//		endfor
//		// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//		Make/o fit_TEMPEVENT2 = 0
//		Appendtograph/W=DetectedEvents2#EventChannel2 fit_TEMPEVENT2
//		ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(fit_TEMPEVENT2)=(0,0,0)
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Duplicate/O TEMPEVENT2, TEMPWAVE
//		Smooth/B Ppnts2, TEMPWAVE
//		Wavestats/Q/R=(0,Decay2*.001) TEMPWAVE
//		//FindPeak/N/Q/R=(0,Decay2*.001) TEMPWAVE
//		FindLevel/Q/R=(V_MinLoc,Decay2*.001) TEMPWAVE V_min*(1-StartFitdbexp2*.01)
//		debut = V_LevelX
//		fin = EndFitdbexp2*.001
//		K0 = 0
//		// The fit command
//		// Charly replaced OffsetFitdbExp2 by CheckOffset2
//		if (CheckOffset2==1)
//			CurveFit/Q/H="00000" dblexp TEMPEVENT2(debut, fin) /D
//			else
//			CurveFit/Q/H="10000" dblexp TEMPEVENT2(debut, fin) /D
//		endif
//		// Charly End
//		//AppendToGraph/W=DetectedEvents2#EventChannel2 fit_TEMPEVENT2
//		FitParametersFitdbexp2[0]=(1/K4)*1000
//		FitParametersFitdbexp2[1]=(1/K2)*1000
//		FitParametersFitdbexp2[8]=K0
//		FitParametersFitdbexp2[2] = (K3 / (K1 + K3))*100
//		FitParametersFitdbexp2[3] = (K1 / (K1 + K3))*100
//		execute "ValDisplay TauFastdbexp2 value=FitParametersFitdbexp2[0]"
//		execute "ValDisplay TauSlowdbexp2 value=FitParametersFitdbexp2[1]"
//		execute "ValDisplay FastPercentagedbexp2 value=FitParametersFitdbexp2[2]"
//		execute "ValDisplay SlowPercentagedbexp2 value=FitParametersFitdbexp2[3]"
//		execute "ValDisplay ValOffsetdbexp2 value=FitParametersFitdbexp2[8]"
//		execute "Valdisplay KineticIndexdbexp2 value=(FitParametersFitdbexp2[2]/100)*FitParametersFitdbexp2[0] + (FitParametersFitdbexp2[3]/100)*FitParametersFitdbexp2[1]"
//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		getaxis/W=DetectedEvents2#EventChannel2 bottom
//		Curseur_debut_fit_X2[0] = debut - .05*(V_max-V_min)
//		Curseur_debut_fit_X2[1] = debut + .05*(V_max-V_min)
//		Curseur_debut_fit_Y2[0] = K0 + K1*exp(-K2*debut) + K3*exp(-K4*debut)
//		Curseur_debut_fit_Y2[1] = K0 + K1*exp(-K2*debut) + K3*exp(-K4*debut)
//		getaxis/W=DetectedEvents2#EventChannel2 left
//		Curseur_fin_fit_X2[0] = EndFitdbexp2*.001
//		Curseur_fin_fit_X2[1] = EndFitdbexp2*.001
//		Curseur_fin_fit_Y2[0] = K0 + K1*exp(-K2*fin) + K3*exp(-K4*fin) - 0.05*(V_max-V_min)
//		Curseur_fin_fit_Y2[1] = K0 + K1*exp(-K2*fin) + K3*exp(-K4*fin) + 0.05*(V_max-V_min)
//		AppendToGraph/W=DetectedEvents2#EventChannel2 Curseur_debut_fit_Y2 vs Curseur_debut_fit_X2; ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(Curseur_debut_fit_Y2)=(0,0,0)
//		AppendToGraph/W=DetectedEvents2#EventChannel2 Curseur_fin_fit_Y2 vs Curseur_fin_fit_X2; ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(Curseur_fin_fit_Y2)=(0,0,0)
//	endif
//	if (CheckFit2 == 1)
//		// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
//		// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
//		GetWindow DetectedEvents2#Average2, WaveList
//		Wave/T W_WaveList = W_WaveList
//		for (i=1; i < numpnts(W_WaveList)/3; I+=1)
//			TEMPSTR = W_WaveList[i][0]
//			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#Average2 $W_WaveList[i][0]
//			endif
//			if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#Average2 $W_WaveList[i][0]
//			endif
//			if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#Average2 $W_WaveList[i][0]
//			endif		
//		endfor
//		// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//		Make/o Fit_AverageEvent2 = 0
//		Appendtograph/W=DetectedEvents2#Average2 Fit_AverageEvent2
//		ModifyGraph/W=DetectedEvents2#Average2 rgb(Fit_AverageEvent2)=(0,0,0)
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Wavestats/Q/R=(0,Decay2*.001) AverageEvent2
//		//FindPeak/N/Q/R=(0,Decay2*.001) AverageEvent2
//		FindLevel/Q/R=(V_MinLoc,Decay2*.001) AverageEvent2 V_min*(1-StartFitdbexp2*.01)
//		debut = V_LevelX
//		fin = EndFitdbexp2*.001
//		K0 = 0
//		// The fit command
//		// Charly Replaced OffsetFitdbExp2 By checkOffset
//		if (CheckOffset2==1)
//			CurveFit/Q/H="00000" dblexp AverageEvent2(debut, fin) /D
//			else
//			CurveFit/Q/H="10000" dblexp AverageEvent2(debut, fin) /D
//		endif
//		// Charly End
//		FitParametersFitdbexp2[0]=(1/K4)*1000
//		FitParametersFitdbexp2[1]=(1/K2)*1000
//		FitParametersFitdbexp2[8]=K0
//		FitParametersFitdbexp2[2] = (K3 / (K1 + K3))*100
//		FitParametersFitdbexp2[3] = (K1 / (K1 + K3))*100
//		execute "ValDisplay TauFastdbexp2 value=FitParametersFitdbexp2[0]"
//		execute "ValDisplay TauSlowdbexp2 value=FitParametersFitdbexp2[1]"
//		execute "ValDisplay FastPercentagedbexp2 value=FitParametersFitdbexp2[2]"
//		execute "ValDisplay SlowPercentagedbexp2 value=FitParametersFitdbexp2[3]"
//		execute "ValDisplay ValOffsetdbexp2 value=FitParametersFitdbexp2[8]"
//		execute "Valdisplay KineticIndexdbexp2 value=(FitParametersFitdbexp2[2]/100)*FitParametersFitdbexp2[0] + (FitParametersFitdbexp2[3]/100)*FitParametersFitdbexp2[1]"
//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		getaxis/W=DetectedEvents2#Average2 bottom
//		Curseur_debut_fit_X2_Av[0] = debut - .05*(V_max-V_min)
//		Curseur_debut_fit_X2_Av[1] = debut + .05*(V_max-V_min)
//		Curseur_debut_fit_Y2_Av[0] = K0 + K1*exp(-K2*debut) + K3*exp(-K4*debut)
//		Curseur_debut_fit_Y2_Av[1] = K0 + K1*exp(-K2*debut) + K3*exp(-K4*debut)
//		getaxis/W=DetectedEvents2#Average2 left
//		Curseur_fin_fit_X2_Av[0] = EndFitdbexp2*.001
//		Curseur_fin_fit_X2_Av[1] = EndFitdbexp2*.001
//		Curseur_fin_fit_Y2_Av[0] = K0 + K1*exp(-K2*fin) + K3*exp(-K4*fin) - 0.05*(V_max-V_min)
//		Curseur_fin_fit_Y2_Av[1] = K0 + K1*exp(-K2*fin) + K3*exp(-K4*fin) + 0.05*(V_max-V_min)
//		AppendToGraph/W=DetectedEvents2#Average2 Curseur_debut_fit_Y2_Av vs Curseur_debut_fit_X2_Av; ModifyGraph/W=DetectedEvents2#Average2 rgb(Curseur_debut_fit_Y2_Av)=(0,0,0)
//		AppendToGraph/W=DetectedEvents2#Average2 Curseur_fin_fit_Y2_Av vs Curseur_fin_fit_X2_Av; ModifyGraph/W=DetectedEvents2#Average2 rgb(Curseur_fin_fit_Y2_Av)=(0,0,0)
//	endif
//End
//
////Fit second channel triggered on first channel in detected events 1 window
//Function Fitdbexp_ch1_on_win2()
//	
//	// SI1 for sample interval of trace 1 Ppnts1 for number of points in the peak for trace 1,
//	 //to use the checkboxes on screen : checkoffset2, decay2, baseline2, start, End, checkfit2
//	Nvar SI1, CheckOffset2, Decay2, baseline2, StartFitdbexp2, EndFitdbexp2, CheckFit2, Ppnts1
//	//TEMPEVENT_CH1 to select the current event for the channel 1 on the detected events 2 window
//	//AverageEvent2ch1 to select the average event for the channel 1 on the detected events 2 window
//	Wave TEMPEVENT_CH1, AverageEvent2ch1
//	Variable debut, fin, counter
//	Variable V_FitOptions = 4
//	
//	String TEMPSTR
//	
//	Make/o/N=10 FitParametersFitdbexp2
//	
//	//If single event box checked
//	if (CheckFit2 == 0)
//		// The following lines are meant to remove any previous fit wave on the graph. 
//		//Otherwise it gets messy and Igor superimpose many waves with the same name.
//		
//		GetWindow DetectedEvents2#EventChannel21, WaveList //remplact EventChannel2 par EventChannel21
//		Wave/T W_WaveList = W_WaveList
//		for (counter = 0; counter < DimSize(W_WaveList, 0); counter += 1)
//			TEMPSTR = W_WaveList[counter][0]
//			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#EventChannel21 $W_WaveList[counter][0]
//			endif		
//		endfor
//		// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//		Make/o fit_TEMPEVENT_CH1 = 0
//		Appendtograph/W=DetectedEvents2#EventChannel21 fit_TEMPEVENT_CH1
//		ModifyGraph/W=DetectedEvents2#EventChannel21 rgb(fit_TEMPEVENT_CH1)=(0,15872,65280) //Just to set the color of the fit curve to blue to see it over the original red trace
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Duplicate/O TEMPEVENT_CH1, TEMPWAVE
//		Smooth/B Ppnts1, TEMPWAVE
//		Wavestats/Q/R=(0, Decay2*.001) TEMPWAVE
//		//FindPeak/N/Q/R=(0,Decay2*.001) TEMPWAVE // /N means search for a negative peak, /Q no history printing no error reporting if no peak found, value is X by default /R = range for search
//		//Print "Peak Value in wave is : " + num2str(V_peakVal)
//		FindLevel/Q/R=(0,Decay2*.001) TEMPWAVE, V_min
//		
////		If (V_flag == 0)
////			FindLevel/Q/R=(V_PeakLoc,Decay2*.001) TEMPWAVE V_min*(1-StartFitdbexp2*.01)
////		Else
////			Abort "No peak found"
////		EndIf
//		
//		If (V_flag == 0)
//			Print "minimum location is : " + num2str(V_levelX) 
//			FindLevel/Q/R=(V_levelX,Decay2*.001) TEMPWAVE V_min*(1-StartFitdbexp2*.01)
//		Else
//			Abort "No peak found"
//		EndIf
//		
//		If (V_flag == 0)
//			debut = V_LevelX
//		Else
//			Abort "No level found"
//		EndIf
//		
//		fin = EndFitdbexp2*.001
//		K0 = 0
//		// The fit command
//		if (CheckOffset2==1)
//			CurveFit/Q/H="00000" dblexp TEMPEVENT_CH1 (debut, fin) /D
//		else
//			CurveFit/Q/H="10000" dblexp TEMPEVENT_CH1 (debut, fin) /D
//		endif
//		//AppendToGraph/W=DetectedEvents2#EventChannel21 fit_TEMPEVENT2
//		FitParametersFitdbexp2[0]=(1/K4)*1000
//		FitParametersFitdbexp2[1]=(1/K2)*1000
//		FitParametersFitdbexp2[8]=K0
//		FitParametersFitdbexp2[2] = (K3 / (K1 + K3))*100
//		FitParametersFitdbexp2[3] = (K1 / (K1 + K3))*100
//		execute "ValDisplay TauFastdbexp1 value=" + num2str(FitParametersFitdbexp2[0])
//		execute "ValDisplay TauSlowdbexp1 value=" + num2str(FitParametersFitdbexp2[1])
//		execute "ValDisplay FastPercentagedbexp1 value=" + num2str(FitParametersFitdbexp2[2])
//		execute "ValDisplay SlowPercentagedbexp1 value=" + num2str(FitParametersFitdbexp2[3])
//		execute "ValDisplay ValOffsetdbexp1 value=" + num2str(FitParametersFitdbexp2[8])
//		execute "Valdisplay KineticIndexdbexp1 value=" + num2str((FitParametersFitdbexp2[2]/100)*FitParametersFitdbexp2[0] + (FitParametersFitdbexp2[3]/100)*FitParametersFitdbexp2[1])
//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		
//	//For the average box checked ***************************************************************************************************************************************
//	Elseif (CheckFit2 == 1)
//		// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
//		// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
//		GetWindow DetectedEvents2#Average21, WaveList
//		Wave/T W_WaveList = W_WaveList
//		for (counter = 1; counter < DimSize(W_WaveList, 0); counter += 1)
//			TEMPSTR = W_WaveList[counter][0]
//			if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//				RemoveFromGraph/W=DetectedEvents2#Average21 $W_WaveList[counter][0]
//			endif		
//		endfor
//		// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//		Make/o fit_AverageEvent2ch1 = 0
//		Appendtograph/W=DetectedEvents2#Average21 fit_AverageEvent2ch1
//		ModifyGraph/W=DetectedEvents2#Average21 rgb(fit_AverageEvent2ch1)=(0,15872,65280)//Just to set the color of the fit curve to blue to see it over the original red trace
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Wavestats/Q AverageEvent2ch1
//		//FindPeak/N/Q/R=(0, (Decay2 * 0.001)) AverageEvent2ch1
//		
//		FindLevel/Q/R=(0, (Decay2 * 0.001)) AverageEvent2ch1, V_min
//		If (V_flag == 0)
//			Print "minimum location is : " + num2str(V_levelX)
//			FindLevel/Q/R=(V_levelX, (Decay2 * 0.001)) AverageEvent2ch1, (V_min * (1 - StartFitdbexp2 * 0.01))
//		Else
//			Abort "level no found"
//		EndIf
//		
//		If (V_flag == 0)
//			debut = V_LevelX
//		Else
//			Abort "Level not found"
//		EndIf
//		fin = (EndFitdbexp2 * 0.001)
//		K0 = 0
//		// The fit command
//		if (CheckOffset2 == 1)
//			CurveFit/Q/H="00000" dblexp AverageEvent2ch1(debut, fin) /D
//		else
//			CurveFit/Q/H="10000" dblexp AverageEvent2ch1(debut, fin) /D
//		endif
//		FitParametersFitdbexp2[0]=(1/K4)*1000
//		FitParametersFitdbexp2[1]=(1/K2)*1000
//		FitParametersFitdbexp2[8]=K0
//		FitParametersFitdbexp2[2] = (K3 / (K1 + K3))*100
//		FitParametersFitdbexp2[3] = (K1 / (K1 + K3))*100
//		execute "ValDisplay TauFastdbexp1 value=" + num2str(FitParametersFitdbexp2[0]) // Tau fast
//		execute "ValDisplay TauSlowdbexp1 value=" + num2str(FitParametersFitdbexp2[1]) // Tau Slow
//		execute "ValDisplay FastPercentagedbexp1 value=" + num2str(FitParametersFitdbexp2[2]) // % Fast
//		execute "ValDisplay SlowPercentagedbexp1 value=" + num2str(FitParametersFitdbexp2[3]) // % Slow
//		execute "ValDisplay ValOffsetdbexp1 value=" + num2str(FitParametersFitdbexp2[8])
//		execute "Valdisplay KineticIndexdbexp1 value=" + num2str((FitParametersFitdbexp2[2]/100)*FitParametersFitdbexp2[0] + (FitParametersFitdbexp2[3]/100)*FitParametersFitdbexp2[1])
//		// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//		
//	endif
//End

//Function Fitsigdbexp1(ctrlName) : ButtonControl
//		String ctrlName
//		Nvar SI1, Baseline1, Decay1, StartFit1, EndFit1, EventNumber1, CheckFit1, CheckOffset1
//		Wave TEMPEVENT1, Occurrence1, BaselineY1, Curseur_debut_fit_X1, Curseur_debut_fit_Y1, RealAmp1, Curseur_fin_fit_X1, Curseur_fin_fit_Y1, AverageEvent1
//		Wave Curseur_debut_fit_X1_Av, Curseur_debut_fit_Y1_Av, Curseur_fin_fit_X1_Av, Curseur_fin_fit_Y1_Av
//		Variable debut, fin, S
//		Variable V_FitOptions=4
//		Svar FileName1
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Variable i; String TEMPSTR
//		if (CheckFit1 == 0) // CheckFit1 = 0 means that the function has to fit single detected events.
//			// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
//			// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
//			GetWindow DetectedEvents1#EventChannel1, WaveList
//			Wave/T W_WaveList = W_WaveList
//			for (i=1; i < numpnts(W_WaveList)/3; I+=1)
//				TEMPSTR = W_WaveList[i][0]
//				if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//					RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
//				endif
//				if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
//					RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
//				endif
//				if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
//					RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
//				endif
//			endfor
//			// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//			Make/o fit_TEMPEVENT1 = 0
//			Appendtograph/W=DetectedEvents1#EventChannel1 fit_TEMPEVENT1
//			// First, determine the limits of the fit
//			// Debut is where the fit starts
//			if (StartFit1 < 50)	
//				Duplicate/o/R=(-.01, 0) TEMPEVENT1 TEMPWAVE
//				TEMPWAVE = TEMPWAVE[numpnts(TEMPWAVE)-1-p]
//				SetScale/P x 0,SI1*1e-6, "s", TEMPWAVE
//				FindLevel/Q TEMPWAVE RealAmp1[EventNumber1]*0.01*StartFit1
//				Debut = - V_LevelX
//			endif
//			if (StartFit1 == 50)
//				Debut = 0
//			endif
//			if (StartFit1 > 50)
//				FindLevel/Q/R=(0,0.01) TEMPEVENT1 RealAmp1[EventNumber1]*0.01*StartFit1
//				Debut = V_LevelX
//			endif
//			// Fin is where the fit ends
//			fin = EndFit1*.001
//			// The real work starts here.
//			// First, I fit a biexponential function to the decay of the average event to give an initial guess of the decay time constants
//			//FindPeak/N/Q/R=(0,Decay1*.001) TEMPEVENT1
//			WaveStats/Q/R=(0,Decay1*.001) TEMPEVENT1
//			K0 = 0
//			if (CheckOffset1==1)
//				CurveFit/Q/H="00000" dblexp TEMPEVENT1(V_MinLoc + (10*SI1*1e-6), fin) /D // Starts the fit 10 points after the peak
//			else
//				CurveFit/Q/H="10000" dblexp TEMPEVENT1(V_MinLoc + (10*SI1*1e-6), fin) /D // Starts the fit 10 points after the peak
//			endif
//			Make/o/N=9 SigExp
//			SigExp[4]=K3/(K3+K1)	// Dbexp : amplitude fast
//			SigExp[5]=K4			// Dbexp : fast time constant
//			SigExp[6]=K1/(K3+K1)	// Dbexp : amplitude slow
//			SigExp[7]=K2			// Dbexp : slow time constant
//			SigExp[8]=K0			// Dbexp : offset
//			// Then, we fit a sigmoid function to the rise phase of the average event
//			K0 = 0; SigExp[0]=K0		// Sigmoid : base
//			CurveFit/Q/H="1000" Sigmoid TEMPEVENT1(-V_MinLoc,V_MinLoc) /D
//			SigExp[1]=K1			// Sigmoid : max
//			SigExp[2]=K2			// Sigmoid : xhalf
//			SigExp[3]=K3			// Sigmoid : rate
//			// The real fitting procedure
//			Make/D/N=8/O W_coef
//			W_coef[0] = {SigExp[1],SigExp[2],SigExp[3],0,SigExp[4],SigExp[5],SigExp[6],SigExp[7]}
//			// W_coef = {max, xhalf, rate, baseline, amplitude fast, fast time constant, amplitude slow, slow time constant}
//			// W_coef[0] = max
//			// W_coef[1] = xhalf
//			// W_coef[2] = rate
//			// W_coef[3] = baseline
//			// W_coef[4] = amplitude fast
//			// W_coef[5] = fast time constant
//			// W_coef[6] = amplitude slow
//			// W_coef[7] = slow time constant
//			if (CheckOffset1==1)
//				FuncFit/H="00000000" Sigmoid_Dbexp W_coef TEMPEVENT1(debut,fin) /D // Offset is set on zero (The fit returns to baseline)
//			else
//				FuncFit/H="00010000" Sigmoid_Dbexp W_coef TEMPEVENT1(debut,fin) /D // Offset is set on zero (The fit returns to baseline)
//			endif
//			// The following lines are for displaying the coefficients in the control bar (nicer than the raw result in the command window, isn't it ?) 
//			Make/o/N=5 FitParametersFitsigdbexp1
//			FitParametersFitsigdbexp1[0] = (1/W_coef[5])*1000	// taufast
//			FitParametersFitsigdbexp1[1] = (1/W_coef[7])*1000	// tauslow
//			FitParametersFitsigdbexp1[2] = (W_coef[4] / ( W_coef[4] + W_coef[6] ))*100	// % fast
//			FitParametersFitsigdbexp1[3] = (W_coef[6] / ( W_coef[4] + W_coef[6] ))*100	// % slow
//			FitParametersFitsigdbexp1[4] = W_coef[3]	// Offset
//			execute "ValDisplay TauFast1 value=" + num2str(FitParametersFitsigdbexp1[0])
//			execute "ValDisplay TauSlow1 value=" + num2str(FitParametersFitsigdbexp1[1])
//			execute "ValDisplay FastPercentage1 value=" + num2str(FitParametersFitsigdbexp1[2])
//			execute "ValDisplay SlowPercentage1 value=" + num2str(FitParametersFitsigdbexp1[3])
//			execute "Valdisplay KineticIndex1 value=" + num2str((FitParametersFitsigdbexp1[0]/100)*FitParametersFitsigdbexp1[2] + (FitParametersFitsigdbexp1[1]/100)*FitParametersFitsigdbexp1[3])
//			execute "ValDisplay ValOffset1 value=" + num2str(FitParametersFitsigdbexp1[4])
//			// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//			getaxis/Q/W=DetectedEvents1#EventChannel1 bottom
//			Curseur_debut_fit_X1[0]= (Debut - (.05*(V_max-V_min)))
//			Curseur_debut_fit_X1[1]= (Debut + (.05*(V_max-V_min)))
//			Curseur_debut_fit_Y1[0]=( W_coef[0] / ( 1 + exp( (W_coef[1] - debut) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*debut) + W_coef[6]*exp(-W_coef[7]*debut) )
//			Curseur_debut_fit_Y1[1]=( W_coef[0] / ( 1 + exp( (W_coef[1] - debut) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*debut) + W_coef[6]*exp(-W_coef[7]*debut) )
//			Curseur_fin_fit_X1[0] = EndFit1*.001
//			Curseur_fin_fit_X1[1] = EndFit1*.001
//			getaxis/W=DetectedEvents1#EventChannel1/Q left
//			Curseur_fin_fit_Y1[0] = ( W_coef[0] / ( 1 + exp( (W_coef[1] - fin) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*fin) + W_coef[6]*exp(-W_coef[7]*fin) ) - 0.05*(V_max-V_min)
//			Curseur_fin_fit_Y1[1] = ( W_coef[0] / ( 1 + exp( (W_coef[1] - fin) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*fin) + W_coef[6]*exp(-W_coef[7]*fin) ) + 0.05*(V_max-V_min)
//			appendtograph/W=DetectedEvents1#EventChannel1 Curseur_debut_fit_Y1 vs Curseur_debut_fit_X1; ModifyGraph/W=DetectedEvents1#EventChannel1 rgb(Curseur_debut_fit_Y1)=(0,0,0)
//			appendtograph/W=DetectedEvents1#EventChannel1 Curseur_fin_fit_Y1 vs Curseur_fin_fit_X1; ModifyGraph/W=DetectedEvents1#EventChannel1 rgb(Curseur_fin_fit_Y1)=(0,0,0)
//			ModifyGraph/W=DetectedEvents1#EventChannel1 rgb(fit_TEMPEVENT1)=(0,0,0)
//		endif
//		if (CheckFit1 == 1) // CheckFit1 = 1 means that the function has to fit the average event.
//			// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
//			// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
//			GetWindow DetectedEvents1#Average1, WaveList
//			Wave/T W_WaveList = W_WaveList
//			for (i=1; i < numpnts(W_WaveList)/3; I+=1)
//				TEMPSTR = W_WaveList[i][0]
//				if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//					RemoveFromGraph/W=DetectedEvents1#Average1 $W_WaveList[i][0]
//				endif
//				if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
//					RemoveFromGraph/W=DetectedEvents1#Average1 $W_WaveList[i][0]
//				endif
//				if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
//					RemoveFromGraph/W=DetectedEvents1#Average1 $W_WaveList[i][0]
//				endif
//			endfor
//			// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//			Make/o Fit_AverageEvent1 = 0
//			Appendtograph/W=DetectedEvents1#Average1 Fit_AverageEvent1
//			// First, determine the limits of the fit
//			// Debut is where the fit starts
//			if (StartFit1 < 50)	
//				Duplicate/o/R=(-.01, 0) AverageEvent1 TEMPWAVE
//				TEMPWAVE = TEMPWAVE[numpnts(TEMPWAVE)-1-p]
//				SetScale/P x 0,SI1*1e-6, "s", TEMPWAVE
//				FindLevel/Q TEMPWAVE RealAmp1[EventNumber1]*0.01*StartFit1
//				Debut = - V_LevelX
//			endif
//			if (StartFit1 == 50)
//				Debut = 0
//			endif
//			if (StartFit1 > 50)
//				FindLevel/Q/R=(0,0.01) AverageEvent1 RealAmp1[EventNumber1]*0.01*StartFit1
//				Debut = V_LevelX
//			endif
//			// Fin is where the fit ends
//			fin = EndFit1*.001
//			// The real work starts here.
//			// First, I fit a biexponential function to the decay of the average event to give an initial guess of the decay time constants
//			//FindPeak/N/Q/R=(0,Decay1*.001) AverageEvent1
//			WaveStats/Q/R=(0,Decay1*.001) AverageEvent1
//			K0 = 0
//			if (CheckOffset1==1)
//				CurveFit/Q/H="00000" dblexp AverageEvent1(V_MinLoc + (10*SI1*1e-6), fin) /D // Starts the fit 10 points after the peak
//			else
//				CurveFit/Q/H="10000" dblexp AverageEvent1(V_MinLoc + (10*SI1*1e-6), fin) /D // Starts the fit 10 points after the peak
//			endif
//			Make/o/N=9 SigExp
//			SigExp[4]=K3/(K3+K1)	// Dbexp : amplitude fast
//			SigExp[5]=K4			// Dbexp : fast time constant
//			SigExp[6]=K1/(K3+K1)	// Dbexp : amplitude slow
//			SigExp[7]=K2			// Dbexp : slow time constant
//			SigExp[8]=K0			// Dbexp : offset
//			// Then, we fit a sigmoid function to the rise phase of the average event
//			K0 = 0; SigExp[0]=K0		// Sigmoid : base
//			CurveFit/Q/H="1000" Sigmoid AverageEvent1(-V_MinLoc,V_MinLoc) /D
//			SigExp[1]=K1			// Sigmoid : max
//			SigExp[2]=K2			// Sigmoid : xhalf
//			SigExp[3]=K3			// Sigmoid : rate
//			// The real fitting procedure
//			Make/D/N=8/O W_coef
//			W_coef[0] = {SigExp[1],SigExp[2],SigExp[3],0,SigExp[4],SigExp[5],SigExp[6],SigExp[7]}
//			// W_coef = {max, xhalf, rate, baseline, amplitude fast, fast time constant, amplitude slow, slow time constant}
//			// W_coef[0] = max
//			// W_coef[1] = xhalf
//			// W_coef[2] = rate
//			// W_coef[3] = baseline
//			// W_coef[4] = amplitude fast
//			// W_coef[5] = fast time constant
//			// W_coef[6] = amplitude slow
//			// W_coef[7] = slow time constant
//			if (CheckOffset1==1)
//				FuncFit/H="00000000" Sigmoid_Dbexp W_coef AverageEvent1(debut,fin) /D
//			else
//				FuncFit/H="00010000" Sigmoid_Dbexp W_coef AverageEvent1(debut,fin) /D
//			endif
//			// The following lines are for displaying the coefficients in the control bar (nicer than the raw result in the command window, isn't it ?) 
//			Make/o/N=5 FitParametersFitsigdbexp1
//			FitParametersFitsigdbexp1[0]=(1/W_coef[5])*1000	// taufast
//			FitParametersFitsigdbexp1[1]=(1/W_coef[7])*1000	// tauslow
//			FitParametersFitsigdbexp1[2] = (W_coef[4] / ( W_coef[4] + W_coef[6] ))*100	// % fast
//			FitParametersFitsigdbexp1[3] = (W_coef[6] / ( W_coef[4] + W_coef[6] ))*100	// % slow
//			FitParametersFitsigdbexp1[4] = W_coef[3]
//			execute "ValDisplay TauFast1 value=" + num2str(FitParametersFitsigdbexp1[0])
//			execute "ValDisplay TauSlow1 value=" + num2str(FitParametersFitsigdbexp1[1])
//			execute "ValDisplay FastPercentage1 value=" + num2str(FitParametersFitsigdbexp1[2])
//			execute "ValDisplay SlowPercentage1 value=" + num2str(FitParametersFitsigdbexp1[3])
//			execute "Valdisplay KineticIndex1 value=" + num2str((FitParametersFitsigdbexp1[0]/100)*FitParametersFitsigdbexp1[2] + (FitParametersFitsigdbexp1[1]/100)*FitParametersFitsigdbexp1[3])
//			execute "ValDisplay ValOffset1 value=" + num2str(FitParametersFitsigdbexp1[4])
//			// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//			getaxis/Q/W=DetectedEvents1#Average1 bottom
//			Curseur_debut_fit_X1_Av[0]= (Debut - (.05*(V_max-V_min)))
//			Curseur_debut_fit_X1_Av[1]= (Debut + (.05*(V_max-V_min)))
//			Curseur_debut_fit_Y1_Av[0]=( W_coef[0] / ( 1 + exp( (W_coef[1] - debut) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*debut) + W_coef[6]*exp(-W_coef[7]*debut) )
//			Curseur_debut_fit_Y1_Av[1]=( W_coef[0] / ( 1 + exp( (W_coef[1] - debut) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*debut) + W_coef[6]*exp(-W_coef[7]*debut) )
//			Curseur_fin_fit_X1_Av[0] = EndFit1*.001
//			Curseur_fin_fit_X1_Av[1] = EndFit1*.001
//			getaxis/W=DetectedEvents1#Average1/Q left
//			Curseur_fin_fit_Y1_Av[0] = ( W_coef[0] / ( 1 + exp( (W_coef[1] - fin) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*fin) + W_coef[6]*exp(-W_coef[7]*fin) ) - 0.05*(V_max-V_min)
//			Curseur_fin_fit_Y1_Av[1] = ( W_coef[0] / ( 1 + exp( (W_coef[1] - fin) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*fin) + W_coef[6]*exp(-W_coef[7]*fin) ) + 0.05*(V_max-V_min)
//			appendtograph/W=DetectedEvents1#Average1 Curseur_debut_fit_Y1_Av vs Curseur_debut_fit_X1_Av; ModifyGraph/W=DetectedEvents1#Average1 rgb(Curseur_debut_fit_Y1_Av)=(0,0,0)
//			appendtograph/W=DetectedEvents1#Average1 Curseur_fin_fit_Y1_Av vs Curseur_fin_fit_X1_Av; ModifyGraph/W=DetectedEvents1#Average1 rgb(Curseur_fin_fit_Y1_Av)=(0,0,0)
//			ModifyGraph/W=DetectedEvents1#Average1 rgb(fit_AverageEvent1)=(0,0,0)
//		endif
//End
//
//Function Fitsigdbexp2(ctrlName) : ButtonControl
//		String ctrlName
//		Nvar SI2, Baseline2, Decay2, StartFit2, EndFit2, EventNumber2, CheckOffset2, CheckFit2
//		Wave TEMPEVENT2, Occurrence2, BaselineY2, Curseur_debut_fit_X2, Curseur_debut_fit_Y2, RealAmp2, Curseur_fin_fit_X2, Curseur_fin_fit_Y2, AverageEvent2
//		Wave Curseur_debut_fit_X2_Av, Curseur_debut_fit_Y2_Av, Curseur_fin_fit_X2_Av, Curseur_fin_fit_Y2_Av
//		Variable debut, fin, S
//		Variable V_FitOptions=4
//		Svar FileName2
//		// First, determine the limits of the fit
//		// Debut is where the fit starts
//		Variable i; String TEMPSTR
//		if (CheckFit2 == 0) // CheckFit2 = 0 means that the function has to fit single detected events.
//			// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
//			// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
//			GetWindow DetectedEvents2#EventChannel2, WaveList
//			Wave/T W_WaveList = W_WaveList
//			for (i=1; i < numpnts(W_WaveList)/3; I+=1)
//				TEMPSTR = W_WaveList[i][0]
//				if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//					RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
//				endif
//				if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
//					RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
//				endif
//				if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
//					RemoveFromGraph/W=DetectedEvents2#EventChannel2 $W_WaveList[i][0]
//				endif
//			endfor
//			// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//			Make/o fit_TEMPEVENT2 = 0
//			Appendtograph/W=DetectedEvents2#EventChannel2 fit_TEMPEVENT2
//			ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(fit_TEMPEVENT2)=(0,0,0)
//			// First, determine the limits of the fit
//			// Debut is where the fit starts
//			if (StartFit2 < 50)	
//				Duplicate/o/R=(-.01, 0) TEMPEVENT2 TEMPWAVE
//				TEMPWAVE = TEMPWAVE[numpnts(TEMPWAVE)-1-p]
//				SetScale/P x 0,SI2*1e-6, "s", TEMPWAVE
//				FindLevel/Q TEMPWAVE RealAmp2[EventNumber2]*0.01*StartFit2
//				Debut = - V_LevelX
//			endif
//			if (StartFit2 == 50)
//				Debut = 0
//			endif
//			if (StartFit2 > 50)
//				FindLevel/Q/R=(0,0.01) TEMPEVENT2 RealAmp2[EventNumber2]*0.01*StartFit2
//				Debut = V_LevelX
//			endif
//			// Fin is where the fit ends
//			fin = EndFit2*.001
//			// The real work starts here.
//			// First, I fit a biexponential function to the decay of the average event to give an initial guess of the decay time constants
//			//FindPeak/N/Q/R=(0,Decay2*.001) TEMPEVENT2
//			WaveStats/Q/R=(0,Decay2*.001) TEMPEVENT2
//			K0 = 0
//			if (CheckOffset2==1)
//				CurveFit/Q/H="00000" dblexp TEMPEVENT2(V_MinLoc + (10*SI2*1e-6), fin) /D // Starts the fit 10 points after the peak
//			else
//				CurveFit/Q/H="10000" dblexp TEMPEVENT2(V_MinLoc + (10*SI2*1e-6), fin) /D // Starts the fit 10 points after the peak
//			endif
//			Make/o/N=9 SigExp
//			SigExp[4]=K3/(K3+K1)	// Dbexp : amplitude fast
//			SigExp[5]=K4			// Dbexp : fast time constant
//			SigExp[6]=K1/(K3+K1)	// Dbexp : amplitude slow
//			SigExp[7]=K2			// Dbexp : slow time constant
//			SigExp[8]=K0			// Dbexp : offset
//			// Then, we fit a sigmoid function to the rise phase of the average event
//			K0 = 0; SigExp[0]=K0		// Sigmoid : base
//			CurveFit/Q/H="1000" Sigmoid TEMPEVENT2(-V_MinLoc,V_MinLoc) /D
//			SigExp[1]=K1			// Sigmoid : max
//			SigExp[2]=K2			// Sigmoid : xhalf
//			SigExp[3]=K3			// Sigmoid : rate
//			// The real fitting procedure
//			Make/D/N=8/O W_coef
//			W_coef[0] = {SigExp[1],SigExp[2],SigExp[3],0,SigExp[4],SigExp[5],SigExp[6],SigExp[7]}
//			// W_coef = {max, xhalf, rate, baseline, amplitude fast, fast time constant, amplitude slow, slow time constant}
//			// W_coef[0] = max
//			// W_coef[1] = xhalf
//			// W_coef[2] = rate
//			// W_coef[3] = baseline
//			// W_coef[4] = amplitude fast
//			// W_coef[5] = fast time constant
//			// W_coef[6] = amplitude slow
//			// W_coef[7] = slow time constant
//			if (CheckOffset2==1)
//				FuncFit/H="00000000" Sigmoid_Dbexp W_coef TEMPEVENT2(debut,fin) /D
//			else
//				FuncFit/H="00010000" Sigmoid_Dbexp W_coef TEMPEVENT2(debut,fin) /D // Offset is set on zero (The fit returns to baseline)
//			endif
//			// The following lines are for displaying the coefficients in the control bar (nicer than the raw result in the command window, isn't it ?) 
//			Make/o/N=5 FitParametersFitsigdbexp2
//			FitParametersFitsigdbexp2[0] = (1/W_coef[5])*1000	// taufast
//			FitParametersFitsigdbexp2[1] = (1/W_coef[7])*1000	// tauslow
//			FitParametersFitsigdbexp2[2] = (W_coef[4] / ( W_coef[4] + W_coef[6] ))*100	// % fast
//			FitParametersFitsigdbexp2[3] = (W_coef[6] / ( W_coef[4] + W_coef[6] ))*100	// % slow
//			FitParametersFitsigdbexp2[4] = W_coef[3]
//			execute "ValDisplay TauFast2 value=" + num2str(FitParametersFitsigdbexp2[0])
//			execute "ValDisplay TauSlow2 value=" + num2str(FitParametersFitsigdbexp2[1])
//			execute "ValDisplay FastPercentage2 value=" + num2str(FitParametersFitsigdbexp2[2])
//			execute "ValDisplay SlowPercentage2 value=" + num2str(FitParametersFitsigdbexp2[3])
//			execute "Valdisplay KineticIndex2 value=" + num2str((FitParametersFitsigdbexp2[0]/100)*FitParametersFitsigdbexp2[2] + (FitParametersFitsigdbexp2[1]/100)*FitParametersFitsigdbexp2[3])
//			execute "ValDisplay ValOffset2 value=" + num2str(FitParametersFitsigdbexp2[4])
//			// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//			getaxis/Q/W=DetectedEvents2#EventChannel2 bottom
//			Curseur_debut_fit_X2[0]= (Debut - (.05*(V_max-V_min)))
//			Curseur_debut_fit_X2[1]= (Debut + (.05*(V_max-V_min)))
//			Curseur_debut_fit_Y2[0]=( W_coef[0] / ( 1 + exp( (W_coef[1] - debut) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*debut) + W_coef[6]*exp(-W_coef[7]*debut) )
//			Curseur_debut_fit_Y2[1]=( W_coef[0] / ( 1 + exp( (W_coef[1] - debut) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*debut) + W_coef[6]*exp(-W_coef[7]*debut) )
//			Curseur_fin_fit_X2[0] = EndFit2*.001
//			Curseur_fin_fit_X2[1] = EndFit2*.001
//			getaxis/W=DetectedEvents2#EventChannel2/Q left
//			Curseur_fin_fit_Y2[0] = ( W_coef[0] / ( 1 + exp( (W_coef[1] - fin) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*fin) + W_coef[6]*exp(-W_coef[7]*fin) ) - 0.05*(V_max-V_min)
//			Curseur_fin_fit_Y2[1] = ( W_coef[0] / ( 1 + exp( (W_coef[1] - fin) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*fin) + W_coef[6]*exp(-W_coef[7]*fin) ) + 0.05*(V_max-V_min)
//			appendtograph/W=DetectedEvents2#EventChannel2 Curseur_debut_fit_Y2 vs Curseur_debut_fit_X2; ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(Curseur_debut_fit_Y2)=(0,0,0)
//			appendtograph/W=DetectedEvents2#EventChannel2 Curseur_fin_fit_Y2 vs Curseur_fin_fit_X2; ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(Curseur_fin_fit_Y2)=(0,0,0)
//			ModifyGraph/W=DetectedEvents2#EventChannel2 rgb(fit_TEMPEVENT2)=(0,0,0)
//		endif
//		if (CheckFit2 == 1) // CheckFit2 = 1 means that the function has to fit the average event.
//			// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
//			// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
//			GetWindow DetectedEvents2#Average2, WaveList
//			Wave/T W_WaveList = W_WaveList
//			for (i=1; i < numpnts(W_WaveList)/3; I+=1)
//				TEMPSTR = W_WaveList[i][0]
//				if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
//					RemoveFromGraph/W=DetectedEvents2#Average2 $W_WaveList[i][0]
//				endif
//				if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
//					RemoveFromGraph/W=DetectedEvents2#Average2 $W_WaveList[i][0]
//				endif
//				if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
//					RemoveFromGraph/W=DetectedEvents2#Average2 $W_WaveList[i][0]
//				endif
//			endfor
//			// a trick to prevent the procedure to display the fit by itself (if so, then it's a mess, trust me man).
//			Make/o Fit_AverageEvent2 = 0
//			Appendtograph/W=DetectedEvents2#Average2 Fit_AverageEvent2
//			ModifyGraph/W=DetectedEvents2#Average2 rgb(Fit_AverageEvent2)=(0,0,0)
//			// First, determine the limits of the fit
//			// Debut is where the fit starts
//			if (StartFit2 < 50)	
//				Duplicate/o/R=(-.01, 0) AverageEvent2 TEMPWAVE
//				TEMPWAVE = TEMPWAVE[numpnts(TEMPWAVE)-1-p]
//				SetScale/P x 0,SI2*1e-6, "s", TEMPWAVE
//				FindLevel/Q TEMPWAVE RealAmp2[EventNumber2]*0.01*StartFit2
//				Debut = - V_LevelX
//			endif
//			if (StartFit2 == 50)
//				Debut = 0
//			endif
//			if (StartFit2 > 50)
//				FindLevel/Q/R=(0,0.01) AverageEvent2 RealAmp2[EventNumber2]*0.01*StartFit2
//				Debut = V_LevelX
//			endif
//			// Fin is where the fit ends
//			fin = EndFit2*.001
//			// The real work starts here.
//			// First, I fit a biexponential function to the decay of the average event to give an initial guess of the decay time constants
//			//FindPeak/N/Q/R=(0,Decay2*.001) AverageEvent2
//			WaveStats/Q/R=(0,Decay2*.001) AverageEvent2
//			K0 = 0
//			if (CheckOffset2==1)
//				CurveFit/Q/H="00000" dblexp AverageEvent2(V_MinLoc + (10*SI2*1e-6), fin) /D // Starts the fit 10 points after the peak
//			else
//				CurveFit/Q/H="10000" dblexp AverageEvent2(V_MinLoc + (10*SI2*1e-6), fin) /D // Starts the fit 10 points after the peak
//			endif
//			Make/o/N=9 SigExp
//			SigExp[4]=K3/(K3+K1)	// Dbexp : amplitude fast
//			SigExp[5]=K4			// Dbexp : fast time constant
//			SigExp[6]=K1/(K3+K1)	// Dbexp : amplitude slow
//			SigExp[7]=K2			// Dbexp : slow time constant
//			SigExp[8]=K0			// Dbexp : offset
//			// Then, we fit a sigmoid function to the rise phase of the average event
//			K0 = 0; SigExp[0]=K0		// Sigmoid : base
//			CurveFit/Q/H="1000" Sigmoid AverageEvent2(-V_MinLoc,V_MinLoc) /D
//			SigExp[1]=K1			// Sigmoid : max
//			SigExp[2]=K2			// Sigmoid : xhalf
//			SigExp[3]=K3			// Sigmoid : rate
//			// The real fitting procedure
//			Make/D/N=8/O W_coef
//			W_coef[0] = {SigExp[1],SigExp[2],SigExp[3],0,SigExp[4],SigExp[5],SigExp[6],SigExp[7]}
//			// W_coef = {max, xhalf, rate, baseline, amplitude fast, fast time constant, amplitude slow, slow time constant}
//			// W_coef[0] = max
//			// W_coef[1] = xhalf
//			// W_coef[2] = rate
//			// W_coef[3] = baseline
//			// W_coef[4] = amplitude fast
//			// W_coef[5] = fast time constant
//			// W_coef[6] = amplitude slow
//			// W_coef[7] = slow time constant
//			if (CheckOffset2==1)
//				FuncFit/H="00000000" Sigmoid_Dbexp W_coef AverageEvent2(debut,fin) /D
//			else
//				FuncFit/H="00010000" Sigmoid_Dbexp W_coef AverageEvent2(debut,fin) /D
//			endif
//			// The following lines are for displaying the coefficients in the control bar (nicer than the raw result in the command window, isn't it ?) 
//			Make/o/N=5 FitParametersFitsigdbexp2
//			FitParametersFitsigdbexp2[0]=(1/W_coef[5])*1000	// taufast
//			FitParametersFitsigdbexp2[1]=(1/W_coef[7])*1000	// tauslow
//			FitParametersFitsigdbexp2[2] = (W_coef[4] / ( W_coef[4] + W_coef[6] ))*100	// % fast
//			FitParametersFitsigdbexp2[3] = (W_coef[6] / ( W_coef[4] + W_coef[6] ))*100	// % slow
//			FitParametersFitsigdbexp2[4] = W_coef[3]
//			execute "ValDisplay TauFast2 value=" + num2str(FitParametersFitsigdbexp2[0])
//			execute "ValDisplay TauSlow2 value=" + num2str(FitParametersFitsigdbexp2[1])
//			execute "ValDisplay FastPercentage2 value=" + num2str(FitParametersFitsigdbexp2[2])
//			execute "ValDisplay SlowPercentage2 value=" + num2str(FitParametersFitsigdbexp2[3])
//			execute "Valdisplay KineticIndex2 value=" + num2str((FitParametersFitsigdbexp2[0]/100)*FitParametersFitsigdbexp2[2] + (FitParametersFitsigdbexp2[1]/100)*FitParametersFitsigdbexp2[3])
//			execute "ValDisplay ValOffset2 value=" + num2str(FitParametersFitsigdbexp2[4])
//			// The following lines are for building nice cursors which will mark the limits if the fit on the graph (completely useless)
//			getaxis/Q/W=DetectedEvents2#Average2 bottom
//			Curseur_debut_fit_X2_Av[0]= (Debut - (.05*(V_max-V_min)))
//			Curseur_debut_fit_X2_Av[1]= (Debut + (.05*(V_max-V_min)))
//			Curseur_debut_fit_Y2_Av[0]=( W_coef[0] / ( 1 + exp( (W_coef[1] - debut) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*debut) + W_coef[6]*exp(-W_coef[7]*debut) )
//			Curseur_debut_fit_Y2_Av[1]=( W_coef[0] / ( 1 + exp( (W_coef[1] - debut) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*debut) + W_coef[6]*exp(-W_coef[7]*debut) )
//			Curseur_fin_fit_X2_Av[0] = EndFit2*.001
//			Curseur_fin_fit_X2_Av[1] = EndFit2*.001
//			getaxis/W=DetectedEvents2#Average2/Q left
//			Curseur_fin_fit_Y2_Av[0] = ( W_coef[0] / ( 1 + exp( (W_coef[1] - fin) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*fin) + W_coef[6]*exp(-W_coef[7]*fin) ) - 0.05*(V_max-V_min)
//			Curseur_fin_fit_Y2_Av[1] = ( W_coef[0] / ( 1 + exp( (W_coef[1] - fin) / W_coef[2] )) ) * ( W_coef[3] + W_coef[4]*exp(-W_coef[5]*fin) + W_coef[6]*exp(-W_coef[7]*fin) ) + 0.05*(V_max-V_min)
//			appendtograph/W=DetectedEvents2#Average2 Curseur_debut_fit_Y2_Av vs Curseur_debut_fit_X2_Av; ModifyGraph/W=DetectedEvents2#Average2 rgb(Curseur_debut_fit_Y2_Av)=(0,0,0)
//			appendtograph/W=DetectedEvents2#Average2 Curseur_fin_fit_Y2_Av vs Curseur_fin_fit_X2_Av; ModifyGraph/W=DetectedEvents2#Average2 rgb(Curseur_fin_fit_Y2_Av)=(0,0,0)
//			ModifyGraph/W=DetectedEvents2#Average2 rgb(fit_AverageEvent2)=(0,0,0)
//		endif
//End

Function FitAllEventsdbexp1(ctrlName) : ButtonControl
	String ctrlName
	
	DoAlert 1, "This function will try to fit a biexponential function on the decay of ALL tagged events. This may take a while. Continue ?"
	if (V_flag == 2)
		Abort
	endif
	Wave Tags1
	if (sum(Tags1, 0, numpnts(Tags1)-1) == 0)
		Abort "No tagged events."
	endif
	Variable i
	Variable V_FitOptions=4
	Svar FileName1
	String TEMPSTR
	Variable Debut, Fin
	Wave DetectedLevels1, RiseTime20801, Occurrence1, BaselineY1, Peak1
	Nvar CheckBslnSD1, Baseline1, Decay1, BslnSDinf1, BslnSDsup1, RTinf1, RTsup1, CheckRT20801, StartFitdbexp1, EndFitdbexp1, CheckOffset1, SI1, Ppnts1
	Make/N=(numpnts(DetectedLevels1))/o FitDbexpTauFast
	Make/N=(numpnts(DetectedLevels1))/o FitDbexpTauSlow
	Make/N=(numpnts(DetectedLevels1))/o FitDbexpAmpFast
	Make/N=(numpnts(DetectedLevels1))/o FitDbexpAmpSlow
	Make/N=(numpnts(DetectedLevels1))/o FitDbexpOffset
	Make/N=(numpnts(DetectedLevels1))/o KI1_dbexp
	// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
	// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
	GetWindow DetectedEvents1#EventChannel1, WaveList
	Wave/T W_WaveList = W_WaveList
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif		
	endfor
	Duplicate/o Tags1, Tags1_modified
	For (i = numpnts(DetectedLevels)-1 ; i > -1 ; i-=1)
		if (Numtype(Occurrence1[i]) == 2)		// Removing bad events (NaN)
			Tags1_modified[i] = 0
		endif
		if (Occurrence1[i] < Baseline1*0.001 || Occurrence1[i] > pnt2x($Filename1,numpnts($Filename1)-1) - Decay1*0.001)
			Tags1_modified[i] = 0
		endif
		if (CheckRT20801 == 1)
			if (RiseTime20801[i] < RTinf1*.001 || RiseTime20801[i] > RTsup1*.001)
				Tags1_modified[i] = 0		// Filter for rise time
			endif
		endif
		if (CheckBslnSD1 == 1)
			WaveStats/Q/R=(DetectedLevels1[i] - Baseline1*0.001, DetectedLevels1[i]) $FileName1
			if (V_sdev < BslnSDinf1 || V_sdev > BslnSDsup1)		// Filter for baseline
				Tags1_modified[i] = 0
			endif
		endif
		if (Occurrence1[i+1] < Occurrence1[i] + Decay1*0.001)	// Filter for events with next event in decay 
			Tags1_modified[i] = 0
		endif
	Endfor
	
	// Add filter for double events
	
	for (i = 0; i < numpnts(DetectedLevels1); i += 1)
		if (Tags1_modified[i] == 0)
			FitDbexpTauFast[i] = NaN; FitDbexpTauSlow[i] = NaN; FitDbexpAmpFast[i] = NaN; FitDbexpAmpSlow[i] = NaN; FitDbexpOffset[i] = NaN
		else
			// ----------------- SAVING THE EVENT TO BE FITTED IN A WAVE -----------------
			Duplicate/o/R=(Occurrence1[i] - Baseline1*.001, Occurrence1[i] + Decay1*.001) $FileName1 TEMPWAVE
			TEMPWAVE = TEMPWAVE - BaselineY1[i]
			SetScale/P x -Baseline1*.001, (SI1*1e-6), "s", TEMPWAVE
			Duplicate/O TEMPWAVE, TEMPWAVE2
			Smooth/B Ppnts1, TEMPWAVE2
			Wavestats/Q TEMPWAVE2
			//FindPeak/N/Q/R=(0, Decay1*.001) TEMPWAVE2
			FindLevel/Q/R=(V_MinLoc, Decay1*.001) TEMPWAVE2 V_min*(1-StartFitdbexp1*.01)
			debut = V_LevelX; fin = EndFitdbexp1*.001
			K0 = 0
			// The fit command
			if (CheckOffset1==1)
				CurveFit/Q/H="00000" dblexp TEMPWAVE(debut, fin) /D
			else
				CurveFit/Q/H="10000" dblexp TEMPWAVE(debut, fin) /D
			endif
			FitDbexpTauFast[i]=(1/K4)*1000
			FitDbexpTauSlow[i]=(1/K2)*1000
			FitDbexpOffset[i]=K0
			FitDbexpAmpFast[i] = (K3 / (K1 + K3))*100
			FitDbexpAmpSlow[i] = (K1 / (K1 + K3))*100
		endif
	endfor
	DisplayFitResults(FitDbexpTauFast, FitDbexpTauSlow, FitDbexpAmpFast, FitDbexpAmpSlow, FitDbexpOffset, 1)
End

Function DisplayFitResults(TauFast, TauSlow, AmpFast, AmpSlow, FitOffset, channel_number)
	Wave TauFast, TauSlow, AmpFast, AmpSlow, FitOffset
	Variable channel_number
	//
	Duplicate/o TauFast, W_HIST_TauFast
	Duplicate/o TauSlow, W_HIST_TauSlow
	Duplicate/o FitOffset, W_HIST_FitOffset
	Duplicate/o AmpFast, W_HIST_AmpFast
	Duplicate/o AmpSlow, W_HIST_AmpSlow
	Duplicate/o TauFast, KI
	KI = 0.01*AmpFast*TauFast + 0.01*AmpSlow*TauSlow
	Duplicate/o KI, W_HIST_KI
	//
	Wavestats/Q TauFast; Histogram/B={0,1,V_max} TauFast,W_HIST_TauFast 
	Wavestats/Q TauSlow; Histogram/B={0,1,V_max} TauSlow,W_HIST_TauSlow
	Wavestats/Q FitOffset; Histogram/B=1 FitOffset,W_HIST_FitOffset
	Wavestats/Q AmpFast; Histogram/B={0,5,V_max/5} AmpFast,W_HIST_AmpFast
	Wavestats/Q AmpSlow; Histogram/B={0,5,V_max/5} AmpSlow,W_HIST_AmpSlow
	Wavestats/Q KI; Histogram/B={0,1,V_max} KI,W_HIST_KI
	//
	DoWindow/K Window_Fits
	Display/W=(50,30,500,500)/L=Left1/B=Bottom1 W_HIST_TauFast as "Fit parameters"; DoWindow/C Window_Fits
	Appendtograph/L=Left1/B=Bottom1 W_HIST_TauSlow
	Appendtograph/L=Left2/B=Bottom2 W_HIST_AmpFast; Appendtograph/L=Left2/B=Bottom2 W_HIST_AmpSlow
	Appendtograph/L=Left3/B=Bottom3 W_HIST_KI
	Appendtograph/L=Left4/B=Bottom4 W_HIST_FitOffset
	//
	ModifyGraph axisEnab(Left1)={0.60,0.99}; 			ModifyGraph axisEnab(Left2)={0.60,0.99}
	ModifyGraph axisEnab(Left3)={0.01,0.45}; 			ModifyGraph axisEnab(Left4)={0.01,0.45}
	ModifyGraph axisEnab(Bottom1)={0.05,0.45}; 		ModifyGraph axisEnab(Bottom2)={0.55,0.95}
	ModifyGraph axisEnab(Bottom3)={0.05,0.45}; 		ModifyGraph axisEnab(Bottom4)={0.55,0.95}
	ModifyGraph freePos={0,kwFraction}; 				ModifyGraph margin(bottom)=43
	ModifyGraph freePos(Left1)={0.03,kwFraction}; 		ModifyGraph freePos(Left2)={0.53,kwFraction}
	ModifyGraph freePos(Left3)={0.03,kwFraction}; 		ModifyGraph freePos(Left4)={0.53,kwFraction}
	ModifyGraph freePos(Bottom1)={0.58,kwFraction}; 	ModifyGraph freePos(Bottom3)={0.0,kwFraction}
	ModifyGraph freePos(Bottom2)={0.58,kwFraction}; 	ModifyGraph freePos(Bottom4)={0.0,kwFraction}
	//
	ModifyGraph wbRGB=(56576,56576,56576); ModifyGraph gbRGB=(56576,56576,56576)
	ModifyGraph rgb(W_HIST_TauFast)=(0,39168,0); ModifyGraph rgb(W_HIST_TauSlow)=(65280,0,52224)
	ModifyGraph rgb(W_HIST_AmpFast)=(0,39168,0); ModifyGraph rgb(W_HIST_AmpSlow)=(65280,0,52224)
	ModifyGraph lsize=1.5
	ModifyGraph rgb(W_HIST_KI)=(0,0,0)
	SetAxis Bottom2 0, 100
	Label Bottom1 "Time constant (ms)";  ModifyGraph lblPosMode=0,lblPos(Bottom1)=43,lblLatPos=0
	Label Bottom2 "Amplitude (%)";  ModifyGraph lblPosMode=0,lblPos(Bottom2)=43,lblLatPos=0
	Label Bottom3 "Kinetic Index (ms)"; ModifyGraph lblPosMode(Bottom3)=1,lblMargin(Bottom3)=1
	Label Bottom4 "Offset (xx)"; ModifyGraph lblPosMode(Bottom4)=1,lblMargin(Bottom3)=1
	SetDrawEnv fstyle= 1, textrgb= (0,39168,0); DrawText 0.3,0.050,"Fast"
	SetDrawEnv fstyle= 1, textrgb= (65280,0,52224); DrawText 0.3,0.080,"Slow"
	ModifyGraph margin(top)=30
	FindLevels/Q Tags1_modified 1
	SetDrawEnv fstyle= 1; DrawText 0.030,-0.03,"Histograms of fit parameters. " + num2str(V_LevelsFound) + " events fitted."
End

Function FitAllEventssigdbexp1(ctrlName) : ButtonControl
	String ctrlName
	
	DoAlert 1, "This function will try to fit the product of a sigmoid and a biexponential function on ALL tagged events. This may take a while. Continue ?"
	if (V_flag == 2)
		Abort
	endif
	Wave Tags1
	if (sum(Tags1, 0, numpnts(Tags1)-1) == 0)
		Abort "No tagged events."
	endif
	Variable i
	Variable V_FitOptions=4
	Svar FileName1
	String TEMPSTR
	Variable Debut, Fin
	Wave DetectedLevels1, RiseTime20801, Occurrence1, BaselineY1, Peak1, RealAmp1
	Nvar CheckBslnSD1, Baseline1, Decay1, BslnSDinf1, BslnSDsup1, RTinf1, RTsup1, CheckRT20801, StartFitsigdbexp1, EndFitsigdbexp1, SI1, Ppnts1, CheckOffset1
	Make/N=(numpnts(DetectedLevels1))/o FitSigdbexpTauFast
	Make/N=(numpnts(DetectedLevels1))/o FitSigdbexpTauSlow
	Make/N=(numpnts(DetectedLevels1))/o FitSigdbexpAmpFast
	Make/N=(numpnts(DetectedLevels1))/o FitSigdbexpAmpSlow
	Make/N=(numpnts(DetectedLevels1))/o KI1_sigdbexp
	Make/N=(numpnts(DetectedLevels1))/o FitSigdbexpOffset
	// The following lines are meant to remove any previous fit wave on the graph. Otherwise it gets messy and Igor superimpose many waves with the same name.
	// As we don't know if there is actually a fit wave on the graph, let's just have a look on the graph with the GetWindow command.
	GetWindow DetectedEvents1#EventChannel1, WaveList
	Wave/T W_WaveList = W_WaveList
	for (i=0; i < numpnts(W_WaveList)/3; i+=1)
		TEMPSTR = W_WaveList[i][0]
		if (cmpstr(TEMPSTR[0,2], "fit" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,18], "Curseur_debut_fit_Y" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif
		if (cmpstr(TEMPSTR[0,16], "Curseur_fin_fit_Y" ) == 0)
			RemoveFromGraph/W=DetectedEvents1#EventChannel1 $W_WaveList[i][0]
		endif		
	endfor
	Duplicate/o Tags1, Tags1_modified
	For (i = numpnts(DetectedLevels)-1 ; i > -1 ; i-=1)
		if (Numtype(Occurrence1[i]) == 2)		// Removing bad events (NaN)
			Tags1_modified[i] = 0
		endif
		if (Occurrence1[i] < Baseline1*0.001 || Occurrence1[i] > pnt2x($Filename1,numpnts($Filename1)-1) - Decay1*0.001)
			Tags1_modified[i] = 0
		endif
		if (CheckRT20801 == 1)
			if (RiseTime20801[i] < RTinf1*.001 || RiseTime20801[i] > RTsup1*.001)
				Tags1_modified[i] = 0		// Filter for rise time
			endif
		endif
		if (CheckBslnSD1 == 1)
			WaveStats/Q/R=(DetectedLevels1[i] - Baseline1*0.001, DetectedLevels1[i]) $FileName1
			if (V_sdev < BslnSDinf1 || V_sdev > BslnSDsup1)		// Filter for baseline
				Tags1_modified[i] = 0
			endif
		endif
		if (Occurrence1[i+1] < Occurrence1[i] + Decay1*0.001)	// Filter for events with next event in decay 
			Tags1_modified[i] = 0
		endif
	Endfor
	Duplicate/o Tags1, Tags1_modified
	for (i = 0; i < numpnts(DetectedLevels1); i += 1)
		if (Tags1_modified[i] == 0)
			FitSigdbexpTauFast[i] = NaN; FitSigdbexpTauSlow[i] = NaN; FitSigdbexpAmpFast[i] = NaN; FitSigdbexpAmpSlow[i] = NaN
		else
			// ----------------- SAVING THE EVENT TO BE FITTED IN A WAVE -----------------
			Duplicate/o/R=(Occurrence1[i] - Baseline1*.001, Occurrence1[i] + Decay1*.001) $FileName1 DUMMY
			DUMMY = DUMMY - BaselineY1[i]
			SetScale/P x -Baseline1*.001, (SI1*1e-6), "s", DUMMY
			// First, determine the limits of the fit
			// Debut is where the fit starts
			if (StartFitsigdbexp1 < 50)	
				Duplicate/o/R=(-.01, 0) DUMMY TEMPWAVE
				TEMPWAVE = TEMPWAVE[numpnts(TEMPWAVE)-1-p]
				SetScale/P x 0,SI1*1e-6, "s", TEMPWAVE
				FindLevel/Q TEMPWAVE RealAmp1[i]*0.01*StartFitsigdbexp1
				Debut = - V_LevelX
			endif
			if (StartFitsigdbexp1 == 50)
				Debut = 0
			endif
			if (StartFitsigdbexp1 > 50)
				FindLevel/Q/R=(0,0.01) DUMMY RealAmp1[i]*0.01*StartFitsigdbexp1
				Debut = V_LevelX
			endif
			// Fin is where the fit ends
			fin = EndFitsigdbexp1*.001
			// The real work starts here.
			// First, I fit a biexponential function to the decay of the average event to give an initial guess of the decay time constants
			//FindPeak/N/Q/R=(0,Decay1*.001) DUMMY
			WaveStats/Q/R=(0,Decay1*.001) DUMMY
			K0 = 0
			if (CheckOffset1==1)
				CurveFit/Q/H="00000" dblexp DUMMY(V_MinLoc + (10*SI1*1e-6), fin) /D // Starts the fit 10 points after the peak
			else
				CurveFit/Q/H="10000" dblexp DUMMY(V_MinLoc + (10*SI1*1e-6), fin) /D // Starts the fit 10 points after the peak
			endif
			Make/o/N=9 SigExp
			SigExp[4]=K3/(K3+K1)	// Dbexp : amplitude fast
			SigExp[5]=K4			// Dbexp : fast time constant
			SigExp[6]=K1/(K3+K1)	// Dbexp : amplitude slow
			SigExp[7]=K2			// Dbexp : slow time constant
			SigExp[8]=K0			// Dbexp : offset
			// Then, we fit a sigmoid function to the rise phase of the average event
			K0 = 0; SigExp[0]=K0		// Sigmoid : base
			CurveFit/Q/H="1000" Sigmoid DUMMY(-V_MinLoc,V_MinLoc) /D
			SigExp[1]=K1			// Sigmoid : max
			SigExp[2]=K2			// Sigmoid : xhalf
			SigExp[3]=K3			// Sigmoid : rate
			// The real fitting procedure
			Make/D/N=8/O W_coef
			W_coef[0] = {SigExp[1],SigExp[2],SigExp[3],0,SigExp[4],SigExp[5],SigExp[6],SigExp[7]}
			// W_coef = {max, xhalf, rate, baseline, amplitude fast, fast time constant, amplitude slow, slow time constant}
			// W_coef[0] = max
			// W_coef[1] = xhalf
			// W_coef[2] = rate
			// W_coef[3] = baseline
			// W_coef[4] = amplitude fast
			// W_coef[5] = fast time constant
			// W_coef[6] = amplitude slow
			// W_coef[7] = slow time constant
			if (CheckOffset1==1)
				FuncFit/H="00000000" Sigmoid_Dbexp W_coef DUMMY(debut,fin) /D	// Free offset
			else
				FuncFit/H="00010000" Sigmoid_Dbexp W_coef DUMMY(debut,fin) /D	// Null offset
			endif
			// The following lines are for displaying the coefficients in the control bar (nicer than the raw result in the command window, isn't it ?) 
			FitSigdbexpTauFast[i] = (1/W_coef[5])*1000	// taufast
			FitSigdbexpTauSlow[i] = (1/W_coef[7])*1000	// tauslow
			FitSigdbexpAmpFast[i] = (W_coef[4] / ( W_coef[4] + W_coef[6] ))*100	// % fast
			FitSigdbexpAmpSlow[i] = (W_coef[6] / ( W_coef[4] + W_coef[6] ))*100	// % slow
			FitSigdbexpOffset[i] = W_coef[3]
		endif
	endfor
	DisplayFitResults(FitSigdbexpTauFast, FitSigdbexpTauSlow, FitSigdbexpAmpFast, FitSigdbexpAmpSlow, FitSigdbexpOffset, 1)
	
	//KI1_sigdbexp = 0.01*FitSigdbexpAmpFast*FitSigdbexpTauFast + 0.01*FitSigdbexpAmpSlow*FitSigdbexpTauSlow
	//Duplicate/o KI1_sigdbexp, W_HIST_KI1_sigdbexp
	///Histogram/B={0,1,50} KI1_sigdbexp,W_HIST_KI1_sigdbexp
	//DoWindow/K Window_KI1_sigdbexp; display W_HIST_KI1_sigdbexp; DoWindow/C Window_KI1_sigdbexp
	//Wavestats/Q KI1_sigdbexp
	//SetAxis bottom -V_sdev, V_avg + 2*V_sdev
	//ModifyGraph standoff=0
	//FindLevels/Q Tags1_modified, 1
	//DrawText 0.04,0.08,"Number of fits: " + num2str(V_LevelsFound)
End

Function FAvsT(WaveOccurrence,WaveAmplitude,VarThrshld)
	Wave WaveOccurrence, WaveAmplitude
	Variable VarThrshld
	String TITLE ; TITLE="Amplitude"
	Nvar Length, FvsTBin, AvsTBin, AmpHistoBin
	Variable BinF, BinA, BinA2, i, X_start
	String TITLE1, TITLE2, TITLE3, TITLE4

	BinF = FvsTBin ; BinA = AvsTBin ; BinA2 = AmpHistoBin
	
	// Here, I could put some error messages in case the bin values are not good
	
	TITLE1 = "Frequency vs Time, bin = " + num2str(BinF) + " s"
	TITLE2 = "Amplitude vs Time"
	TITLE3 = "Amplitude vs Time, bin = " + num2str(BinA) + " s"
	TITLE4 = "Amplitude Histogram"
	
	Duplicate/o WaveOccurrence WaveOccurrence2
	Duplicate/o WaveAmplitude, WaveAmplitude2
	for (i = (numpnts(WaveOccurrence2)-1) ; i > -1 ; i-=1)
		if (numtype(WaveOccurrence2[i]) == 2 || WaveOccurrence2[i] == 0)
			DeletePoints i, 1, WaveOccurrence2
			DeletePoints i, 1, WaveAmplitude2
		endif
	endfor
	WaveAmplitude2 = - WaveAmplitude2
	
	// FIRST, calculate Freq vs Time
	Make/N=0/o TEMP
	Variable X1, X2
	FindLevel/Q WaveOccurrence2, BinF
	if (V_flag == 1)
		if (BinF < WaveOccurrence2[0])
			X2 = 0
		endif
		if (BinF > WaveOccurrence2[numpnts(WaveOccurrence2)-1])
			X2 = numpnts(WaveOccurrence2)
		endif
	else
		X2 = floor(V_LevelX) + 1
	endif
	InsertPoints 0,1,TEMP
	TEMP[0] = X2
	i=1
	For (X_Start = BinF; X_Start < (WaveOccurrence2[numpnts(WaveOccurrence2)-1] - BinF); X_Start += BinF) 
		if (X_start < WaveOccurrence2[0])
			X1 = 0
		else
			FindLevel/Q WaveOccurrence2, X_start
			// permet de trouver le # de l'vnement qui tombe juste avant X_Start
			// Cet vnement est de dernier en dehors de la zone d'intrt (largeur d'un bin)
			X1 = floor(V_LevelX)
		endif
		if ((X_start+BinF) < WaveOccurrence2[0])
			X2 = 0
		else
			Findlevel/Q WaveOccurrence2 (X_Start + BinF)
			//permet de trouver le # de l'vnement qui tombe juste avant la fin du bin
			//Cet vnement est le dernier contenu dans la zone d'intrt
			X2 = floor(V_LevelX)
		endif
		InsertPoints i, 1, TEMP
		TEMP[i] = X2 - X1	// nombre d'vnements  l'intrieur du bin
		i+=1
	Endfor
	
	SetScale/P x, 0,BinF,"s",TEMP
	TEMP = TEMP / BinF
	Duplicate/o TEMP FreqVsTime
	KillWaves TEMP
	
	// => result is wave FreqVsTime
	
	// Plot amplitude (binned) vs. time

	Make/N=0/o TEMP; Make/N=0/o TEMP2
	X1 = NaN
	FindLevel/Q/P WaveOccurrence2, BinA
	// Premier Bin
	if (V_flag == 0)
		X1 = 0
		X2 = floor(V_LevelX)
		InsertPoints 0,1,TEMP
		InsertPoints 0,1,TEMP2
		Wavestats/Q/R=[X1,X2] WaveAmplitude2
		TEMP[i] = V_avg; TEMP2[i] = V_sdev
	else
		X2 = NaN
		InsertPoints 0,1,TEMP
		InsertPoints 0,1,TEMP2
		TEMP[i] = NaN
		TEMP2[i] = NaN
	endif
	// Bins suivants
	i=1
	For (X_Start = BinA; X_Start < (WaveOccurrence2[numpnts(WaveOccurrence2)-1] - BinA); X_Start += BinA)
		FindLevel/Q WaveOccurrence2, X_start
		// permet de trouver le # de l'vnement qui tombe juste avant X_Start
		// Cet vnement est de dernier en dehors de la zone d'intrt (largeur d'un bin)
		if (V_flag == 0)
			if ( WaveOccurrence2[floor(V_LevelX) + 1] < (X_start + BinA) )
				X1 = floor(V_LevelX) + 1
			else
				X1 = NaN
			endif
		else
			FindLevel/Q WaveOccurrence2, (X_start+BinA)
			if (V_flag == 0)
				X1 = 0
			else
				X1 = NaN
			endif
		endif
		FindLevel/Q WaveOccurrence2, (X_start+BinA)
		//permet de trouver le # de l'vnement qui tombe juste avant la fin du bin
		//Cet vnement est le dernier contenu dans la zone d'intrt
		if (V_flag == 0)
			if ( WaveOccurrence2[floor(V_LevelX)] > X_start )
				X2 = floor(V_LevelX)
			else
				X2 = NaN
			endif
		else
			X2 = NaN
		endif
		InsertPoints i, 1, TEMP
		InsertPoints i, 1, TEMP2
		Wavestats/Q/R=[X1,X2] WaveAmplitude2
		TEMP[i] = V_avg; TEMP2[i] = V_sdev
		i+=1
	Endfor
	SetScale/P x, BinA/2,BinA,"s",TEMP
	Duplicate/o TEMP AmpVsTime ; KillWaves TEMP
	Duplicate/o TEMP2 AmpVsTime_sdev1; KillWaves TEMP2
	Duplicate/o AmpVsTime_sdev1, AmpVsTime_sdev2
	//AmpVsTime_sdev2 = -AmpVsTime_sdev1

	// Cumulative amplitude graphs
	
	Duplicate/o WaveAmplitude CumAmp
	CumAmp = - CumAmp
	Sort/R CumAmp CumAmp
	for (i = numpnts(CumAmp)-1; i>-1; i -= 1)
		if (CumAmp[i]==0)
			Deletepoints i,1, CumAmp
		endif
	endfor
	sort CumAmp CumAmp
	Duplicate/o CumAmp WaveXCumAmp
	WaveXCumAmp = p
	WaveXCumAmp = WaveXCumAmp / ((numpnts(WaveXCumAmp)-1))
	
	// Plot Freq vs Time
	DoWindow/K Window_FreqvsTime
	Display/W=(50,50,250,250) FreqVsTime as "Frequency vs Time"
	DoWindow/C Window_FreqvsTime; ModifyGraph standoff=0; ModifyGraph mode=0
	Label left "Frequency (Hz)"; Label bottom "Time (s)"
	Wavestats/Q FreqVsTime ; SetAxis left 0,V_max*1.2 ; SetAxis bottom 0,Length
	ModifyGraph mode=6; ModifyGraph rgb=(0,0,0)
	ControlBar 30
	Button ButtonFvsTClose	pos={200,5},size={40,20},title="Close", proc=ProcButtonFvsTClose
	CheckBox CheckBoxFvsTGrid	pos={150,7},size={40,20},title="Grid", proc=ProcCheckBoxFvsTGrid
	
	// Plot Amp vs Occ & Amp vs Time
	DoWindow/K Window_AmpvsTime
	Display/W=(255,50,455,250) WaveAmplitude2 vs WaveOccurrence2 as "Amplitude vs Time"
	DoWindow/C Window_AmpvsTime
	Appendtograph/W=Window_AmpvsTime AmpVsTime
	ModifyGraph mode(WaveAmplitude2)=3,marker=19,msize=1 ; ModifyGraph rgb=(0,0,0)
	SetAxis bottom 0, Length ; ModifyGraph standoff=0
	Wavestats/Q WaveAmplitude2 ; SetAxis Left 0, 1.2*V_max; SetAxis bottom 0,Length	
	Make/o/N=2 WaveXAvsT, WaveYAvsT
	WaveXAvsT[0] = 0 ; WaveXAvsT[1] = Length ; WaveYAvsT = VarThrshld
	AppendToGraph WaveYAvsT vs WaveXAvsT ; ModifyGraph lstyle(WaveYAvsT)=3
	Label left "Amplitude (xx)" ; Label bottom "Time (s)"
	ModifyGraph mode(AmpVsTime)=3
	ModifyGraph msize(AmpVsTime)=3,opaque(AmpVsTime)=1,rgb(AmpVsTime)=(0,43520,65280)
	ModifyGraph useMrkStrokeRGB(AmpVsTime)=1
	ErrorBars AmpVsTime Y,wave=(AmpVsTime_sdev1,AmpVsTime_sdev2)
	ControlBar 30
	Button ButtonAvsTClose	pos={200,5},size={40,20},title="Close", proc=ProcButtonAvsTClose
	CheckBox CheckBoxAvsTGrid	pos={150,7},size={40,20},title="Grid", proc=ProcCheckBoxAvsTGrid
	
	// Plot Amplitude histogram
	Duplicate/o WaveAmplitude2 AmplitudeHistogram; Wavestats/Q WaveAmplitude2
	Histogram/B={0,BinA2,(V_max/binA2)*1.3} WaveAmplitude2 AmplitudeHistogram
	DoWindow/K Window_AmpHist
	Display/W=(50,270,250,470) AmplitudeHistogram as "Amplitude Histogram"
	DoWindow/C Window_AmpHist
	Label left "N", Label bottom "Amplitude (xx)"
	ModifyGraph rgb(AmplitudeHistogram)=(0,0,0)
	ControlBar 30
	Button ButtonAmpHistClose	pos={200,5},size={40,20},title="Close", proc=ProcButtonAmpHistClose
	CheckBox CheckBoxAmpHistGrid	pos={150,7},size={40,20},title="Grid", proc=ProcCheckBoxAmpHistGrid
	
	// Plot cumulative amplitude
	DoWindow/K Window_CumAmp
	Display/W=(255,270,455,470) WaveXCumAmp vs CumAmp as "Cumulative Amplitude Histogram"
	DoWindow/C Window_CumAmp
	Label left "Cumulative probability", Label bottom "Amplitude (--)"
	ControlBar 30
	Button ButtonCumAmpClose	pos={200,5},size={40,20},title="Close", proc=ProcButtonCumAmpClose
	CheckBox CheckBoxCumAmpGrid	pos={150,7},size={40,20},title="Grid", proc=ProcCheckBoxCumAmpGrid
End

// ============ FUNCTIONS RELATED TO FAvsT(): buttons, checkboxes etc ============
FUNCTION ProcButtonFvsTClose(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_FreqvsTime
END

FUNCTION ProcCheckBoxFvsTGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=Window_FreqvsTime grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=Window_FreqvsTime grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=Window_FreqvsTime grid(left)=0
		ModifyGraph/W=Window_FreqvsTime grid(bottom)=0
	endif
END

FUNCTION ProcButtonAvsTClose(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_AmpvsTime
END

FUNCTION ProcCheckBoxAvsTGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=Window_AmpvsTime grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=Window_AmpvsTime grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=Window_AmpvsTime grid(left)=0
		ModifyGraph/W=Window_AmpvsTime grid(bottom)=0
	endif
END

FUNCTION ProcButtonAmpHistClose(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_AmpHist
END

FUNCTION ProcCheckBoxAmpHistGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=Window_AmpHist grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=Window_AmpHist grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=Window_AmpHist grid(left)=0
		ModifyGraph/W=Window_AmpHist grid(bottom)=0
	endif
END

FUNCTION ProcButtonCumAmpClose(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K Window_CumAmp
END

FUNCTION ProcCheckBoxCumAmpGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=Window_CumAmp grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=Window_CumAmp grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=Window_CumAmp grid(left)=0
		ModifyGraph/W=Window_CumAmp grid(bottom)=0
	endif
END
// ============ END OF FUNCTIONS RELATED TO FAvsT() ============

Function Horiz_Expand_KeepCursor() : GraphMarquee
		string format
		Nvar checkedCsr=checkedCsr,XA=XA,YA=YA
	GetMarquee/K left,bottom
	format = "flag: %g; left: %g; top: %g; right: %g; bottom: %g\r"
	if (checkedCsr==1)
		XA=hcsr(A)
		YA=vcsr(A)
		SetAxis/W=MainWindow bottom V_left,V_right
		cursor/F/A=1/W=MainWindow/A=1/H=1 A Smoothed_Trace XA,YA
	else
		SetAxis/W=MainWindow bottom V_left,V_right
	endif
End


//*****************************************************************************************************************************************************************
// New function for discrete correlation - with the right normalization : shuffling the ISIs
Function Correlogram(Wave_1_bis,Wave_1_tags,Wave_2_bis,Wave_2_tags, Length_Wave_1, Length_Wave_2)
	Wave Wave_1_bis, Wave_2_bis, Wave_1_tags,Wave_2_tags
	Variable Length_Wave_1, Length_Wave_2
	Nvar CCTypeRaw, CCTypeShuff, CCTypeNorm
	// Length of recording. Should be identical
	if (Length_Wave_1 != Length_Wave_2)
		Abort "Length1 and length2 not identical !!"
	endif
	// input waves will be modified (avoids overwritting the original)
	Duplicate/o Wave_1_bis, Wave_1; Duplicate/o Wave_2_bis, Wave_2
	// Variables
	Variable i,j , k, LEFT, RIGHT, jSTART, jMEM, VeryEnd, TimeWindow, Nlong, Bin, GraphWidth
	String CCGraph_Name
	Svar currentfilename
	Make/o/N=0 DT_Wave
	// Parameters set by the user in the main panel.
	Nvar CorrTimeWindow, CorrBin
	TimeWindow = CorrTimeWindow; Bin = CorrBin
	CCGraph_Name = "Correlogram, Bin = " + num2str(Bin) + " seconds"
	// Cleaning the input waves
	for (i=0; i < numpnts(Wave_1); i+=1)
	if ( numtype(Wave_1[i]) == 2)	// ... if Wave_1[i] is NaN
		Wave_1[i] = 0			// ... replaced by 0
	endif
	endfor
	for (i=0; i < numpnts(Wave_2); i+=1)
	if ( numtype(Wave_2[i]) == 2)   // ... if Wave_2[i] is NaN
		Wave_2[i] = 0			// ... replaced by 0
	endif
	endfor
	// Initialize variables
	i = 0 ; k = 0; jSTART = 0; jMEM = 0; VeryEnd = Length_Wave_1; Nlong=numpnts(Wave_2)
	// untagged occurrences will become 0
	Wave_1 = Wave_1 * Wave_1_tags
	Wave_2 = Wave_2 * Wave_2_tags
	
	Variable/G NUMBER_WAVE_1	// number of valid events in Wave_1
	NUMBER_WAVE_1 = sum(Wave_1_tags,0,numpnts(Wave_1_tags)-1)
	FindLevel/P Wave_1_tags, 1	// Find first valid event.
	Variable FirstTaggedEventWave1; FirstTaggedEventWave1 = V_LevelX
	Duplicate/o Wave_1_tags, TEMPWAVE2
	TEMPWAVE2 = Wave_1_tags(numpnts(Wave_1_tags)-1-p)	// Flip the wave
	FindLevel/P TEMPWAVE2, 1	// Find first (last) valid event.
	Variable LastTaggedEventWave1; LastTaggedEventWave1 = numpnts(Wave_1_tags) - (V_LevelX + 1)
	
	// TEMPWAVE2 is re-used to determine the valid events in channel 2
	Duplicate/o Wave_2 TEMPWAVE2
	For (i = 0 ; TEMPWAVE2[i] < (Wave_1[FirstTaggedEventWave1] - (TimeWindow/2)) ; i += 1)
		TEMPWAVE2[i] = 0
	Endfor
	For (i = numpnts(TEMPWAVE2)-1 ; TEMPWAVE2[i] > (Wave_1[LastTaggedEventWave1] + (TimeWindow/2)) ; i -= 1)
		TEMPWAVE2[i] = 0
	Endfor
	TEMPWAVE2 = TEMPWAVE2 / TEMPWAVE2
	for (i=0; i < numpnts(TEMPWAVE2); i+=1)
		if ( numtype(TEMPWAVE2[i]) == 2)	// ... if NaN
			TEMPWAVE2[i] = 0			// ... replaced by 0
		endif
	endfor
	Variable/G NUMBER_WAVE_2	// number of valid events in Wave_2
	NUMBER_WAVE_2 = sum(TEMPWAVE2,0,numpnts(TEMPWAVE2)-1)
	Variable/G DURATION; DURATION = Wave_1[LastTaggedEventWave1] - Wave_1[FirstTaggedEventWave1]

	// Explanation :
	//                                                  |<------------- TimeWindow -------------->|
	//                                               LEFT               Wave_1[i]               RIGHT                                              
 	// Wave_1                *       *            [     *    *              *             *    *     ]             *                      *       Length_Wave_1 -> VeryEnd
	//
	// Wave_2          *          *               [      *         *           *        *            ]  *                    *            *       Length_Wave_2
	//                                                    jSTART
	//                                                    -> jMEM
	
	// The core code to calculate the CC
	For (i=0;i<numpnts(Wave_1);i+=1)
		if ( (Wave_1[i] != 0) && (Wave_1[i] != NaN) )	// Wave_1[i] has to be different from 0 or NaN
			// Reinitializing LEFT
			if ( (Wave_1[i] - (TimeWindow/2) ) > 0 )
				LEFT = Wave_1[i] - (TimeWindow/2)
				else
				LEFT = 0
			endif
			// Reinitializing RIGHT
			if ( (Wave_1[i] + (TimeWindow/2) ) < VeryEnd )
				RIGHT = LEFT + TimeWindow
				else
				RIGHT = VeryEnd
			endif
			// Reinitializing jSTART
			For ( jSTART = jMEM; Wave_2[jSTART] < LEFT && jSTART < numpnts(Wave_2); jSTART += 1)
				// as long as jSTART is strictly inferior to LEFT, keeps incrementing it.
				// ... a way to find the first event of Wave_2 in the [LEFT,RIGHT] window.
			Endfor
			if (Wave_2[jSTART] >= LEFT && Wave_2[jSTART] <= RIGHT)		// Make sure jSTART is in the TimeWindow.
				jMEM = jSTART		// For next loop (avoids starting back from 0)
				for( j = jSTART ; Wave_2[j] < RIGHT && j < numpnts(Wave_2); j += 1 )
					if (Wave_2[j] != 0)		// Occurrence has to be different from 0
						InsertPoints k, 1, DT_Wave
						DT_Wave[k] = Wave_2[j] - Wave_1[i]
						k+=1
					endif
				endfor
			endif
		endif
	Endfor
	
	Duplicate/o DT_Wave Corr
	Histogram/B={-(TimeWindow/2),Bin,(TimeWindow/Bin)} DT_Wave,Corr
	Duplicate/o Corr Corr_raw
	
	// The following line is for normalization using the expected number of spikes form poissonian firing.
	//Corr = (Corr - (NUMBER_WAVE_1*NUMBER_WAVE_2*Bin/DURATION)) / sqrt(NUMBER_WAVE_1*NUMBER_WAVE_2*Bin/DURATION)	// Normalization by offsetting the histogram

	// Now it's time to do the right normalization: shuffling the ISIs
	// 1. take the original wave containing the occurrences in channel 1: wave_1
	// and remove the zeros. Then calculate the ISIs
	For (i = numpnts(wave_1)-1; i > -1; i -= 1)
		if (wave_1[i] == 0)
			DeletePoints i, 1, wave_1
		endif
	Endfor
	Duplicate/o wave_1, TEMPWAVE1; Duplicate/o wave_1, TEMPWAVE2
	InsertPoints numpnts(TEMPWAVE1), 1, TEMPWAVE1
	InsertPoints 0, 1, TEMPWAVE2
	Duplicate/o TEMPWAVE1, INTERVALS1
	INTERVALS1 = TEMPWAVE1 - TEMPWAVE2
	DeletePoints 0, 1, INTERVALS1
	DeletePoints (numpnts(INTERVALS1)-1), 1, INTERVALS1
	
	// Same for channel 2
	For (i = numpnts(wave_2)-1; i > -1; i -= 1)
		if (wave_2[i] == 0)
			DeletePoints i, 1, wave_2
		endif
	Endfor
	Duplicate/o wave_2, TEMPWAVE1; Duplicate/o wave_2, TEMPWAVE2
	InsertPoints numpnts(TEMPWAVE1), 1, TEMPWAVE1
	InsertPoints 0, 1, TEMPWAVE2
	Duplicate/o TEMPWAVE1, INTERVALS2
	INTERVALS2 = TEMPWAVE1 - TEMPWAVE2
	DeletePoints 0, 1, INTERVALS2
	DeletePoints (numpnts(INTERVALS2)-1), 1, INTERVALS2

	Make/N=10/o horizon, SD_horizon
	Variable counter
	For (counter = 0; counter < 10; counter += 1)
		// Calculate the shuffled ISI waves
		Variable TheOneRandomlyChosen
		// For channel 1
		Duplicate/o INTERVALS1, TEMPWAVE
		Duplicate/o INTERVALS1 INTERVALS1_shuffled
		INTERVALS1_shuffled = 0
		For (i = 0; i < numpnts(INTERVALS1_shuffled); i += 1)
			TheOneRandomlyChosen = trunc(abs(enoise(numpnts(TEMPWAVE))))
			INTERVALS1_shuffled[i] = TEMPWAVE[TheOneRandomlyChosen]
			DeletePoints TheOneRandomlyChosen, 1, TEMPWAVE
		Endfor
		// For channel 2
		Duplicate/o INTERVALS2, TEMPWAVE
		Duplicate/o INTERVALS2 INTERVALS2_shuffled
		INTERVALS2_shuffled = 0
		For (i = 0; i < numpnts(INTERVALS2_shuffled); i += 1)
			TheOneRandomlyChosen = trunc(abs(enoise(numpnts(TEMPWAVE))))
			INTERVALS2_shuffled[i] = TEMPWAVE[TheOneRandomlyChosen]
			DeletePoints TheOneRandomlyChosen, 1, TEMPWAVE
		Endfor
	
		// Now we will reconstruct two lists of times of occurrence form the shuffled ISIs
		Duplicate/o wave_1, wave_1_shuffled
		wave_1_shuffled[0] = mean(INTERVALS1_shuffled, 0, numpnts(INTERVALS1_shuffled)-1)
		For (i = 1; i < numpnts(wave_1); i += 1)
			wave_1_shuffled[i] = wave_1_shuffled[i-1] + INTERVALS1_shuffled[i-1]
		Endfor

		// Now we will reconstruct two lists of times of occurrence form the shuffled ISIs
		Duplicate/o wave_2, wave_2_shuffled
		wave_2_shuffled[0] = mean(INTERVALS2_shuffled, 0, numpnts(INTERVALS2_shuffled)-1)
		For (i = 1; i < numpnts(wave_2); i += 1)
			wave_2_shuffled[i] = wave_2_shuffled[i-1] + INTERVALS2_shuffled[i-1]
		Endfor
			
		
		Make/o/N=0 DT_Wave
		// Initialize variables
		i = 0 ; k = 0; jSTART = 0; jMEM = 0; VeryEnd = Length_Wave_1; Nlong=numpnts(wave_2_shuffled)
		// Now calculate the CC
		// The core code to calculate the CC (again)
		For (i=0;i<numpnts(wave_1_shuffled);i+=1)
			if ( (wave_1_shuffled[i] != 0) && (wave_1_shuffled[i] != NaN) )	// Wave_1[i] has to be different from 0 or NaN
				// Reinitializing LEFT
				if ( (wave_1_shuffled[i] - (TimeWindow/2) ) > 0 )
					LEFT = wave_1_shuffled[i] - (TimeWindow/2)
					else
					LEFT = 0
				endif
				// Reinitializing RIGHT
				if ( (wave_1_shuffled[i] + (TimeWindow/2) ) < VeryEnd )
					RIGHT = LEFT + TimeWindow
					else
					RIGHT = VeryEnd
				endif
				// Reinitializing jSTART
				For ( jSTART = jMEM; wave_2_shuffled[jSTART] < LEFT && jSTART < numpnts(wave_2_shuffled); jSTART += 1)
					// as long as jSTART is strictly inferior to LEFT, keeps incrementing it.
					// ... a way to find the first event of Wave_2 in the [LEFT,RIGHT] window.
				Endfor
				if (wave_2_shuffled[jSTART] >= LEFT && wave_2_shuffled[jSTART] <= RIGHT)		// Make sure jSTART is in the TimeWindow.
					jMEM = jSTART		// For next loop (avoids starting back from 0)
					for( j = jSTART ; wave_2_shuffled[j] < RIGHT && j < numpnts(wave_2_shuffled); j += 1 )
						if (wave_2_shuffled[j] != 0)		// Occurrence has to be different from 0
							InsertPoints k, 1, DT_Wave
							DT_Wave[k] = wave_2_shuffled[j] - wave_1_shuffled[i]
							k+=1
						endif
					endfor
				endif
			endif
		Endfor
		Duplicate/o DT_Wave Corr_shuffled
		Histogram/B={-(TimeWindow/2),Bin,(TimeWindow/Bin)} DT_Wave,Corr_shuffled
		Wavestats/Q Corr_shuffled
		horizon[counter] = V_avg
		SD_horizon[counter] = V_sdev
	Endfor

	// Display
	DoWindow/K CrossCorrelogramGraph
	
	if (CCTypeNorm == 1)
		display/W=(0,0,400,300) Corr as CCGraph_Name
	endif
	if (CCTypeShuff == 1)
		display/W=(0,0,400,300) Corr_shuffled as CCGraph_Name
	endif
	if (CCTypeRaw == 1)
		display/W=(0,0,400,300) Corr_raw as CCGraph_Name
	endif
	
	DoWindow/C CrossCorrelogramGraph
	Controlbar/W=CrossCorrelogramGraph 70
	Valdisplay DispSegLength 		pos={10,10}, size={170,20}, title="Segment length (s) = "
	Execute "Valdisplay DispSegLength value=" + num2str(DURATION)
	Valdisplay DispBinSize 			pos={190,10}, size={100,20}, title="Bin (s) = "
	Execute "Valdisplay DispBinSize value=" + num2str(Bin)
	Valdisplay NEventsCh1 			pos={10,27}, size={170,20}, title="Number of events ch.1 = "
	Execute "Valdisplay NEventsCh1 value=" + num2str(NUMBER_WAVE_1)
	Valdisplay NEventsCh2 			pos={10,44}, size={170,20}, title="Number of events ch.2 = "
	Execute "Valdisplay NEventsCh2 value=" + num2str(NUMBER_WAVE_2)
	ValDisplay FreqEventsCh1 		pos={190,27}, size={120,20}, title="<-> Freq (Hz) = "
	Execute "Valdisplay FreqEventsCh1 value=" + num2str(NUMBER_WAVE_1/DURATION)
	ValDisplay FreqEventsCh2 		pos={190,44}, size={120,20}, title="<-> Freq (Hz) = "
	Execute "Valdisplay FreqEventsCh2 value=" + num2str(NUMBER_WAVE_2/DURATION)
	
	CheckBox CheckBoxCorr	pos={435,12},size={40,20},	title="Grid", 	proc=ProcCorrGrid
	
	Button CloseCorrGraph pos={480,9},size={40,20},title="close", proc=procButtonCloseCorrGraph
	
	ModifyGraph standoff=0,font="Arial"; SetAxis bottom -TimeWindow/2,TimeWindow/2; ModifyGraph rgb=(0,0,0); ModifyGraph mode=0
	
	// THE FOLLOWING NORMALIZATION WAS VALID WHEN ASSUMING POISSONIAN FIRING
	// NOTE : confidence interval corresponds to a z score of 2 (i.e. percentile 97.7%, i.e. 2 times SD)
	//Make/o/N=2 ConfidenceInterval_top_Y, ConfidenceInterval_top_X, ConfidenceInterval_bottom_Y, ConfidenceInterval_bottom_X
	//ConfidenceInterval_top_Y = 2
	//ConfidenceInterval_top_X[0] = -TimeWindow/2; ConfidenceInterval_top_X[1] = TimeWindow/2
	//ConfidenceInterval_bottom_Y = -2
	//ConfidenceInterval_bottom_X[0] = -TimeWindow/2; ConfidenceInterval_bottom_X[1] = TimeWindow/2
	//AppendToGraph ConfidenceInterval_top_Y vs ConfidenceInterval_top_X
	//AppendToGraph ConfidenceInterval_bottom_Y vs ConfidenceInterval_bottom_X
	//ModifyGraph lstyle(ConfidenceInterval_top_Y)=2
	//ModifyGraph lstyle(ConfidenceInterval_bottom_Y)=2

	// This one is for the shuffling method
	if (CCTypeNorm == 1)
		Corr = Corr - mean(horizon, 0, numpnts(horizon)-1)
		Corr = Corr / mean(SD_horizon, 0, numpnts(SD_horizon)-1)
		Make/o/N=2 ConfidenceInterval_top_Y, ConfidenceInterval_top_X, ConfidenceInterval_bottom_Y, ConfidenceInterval_bottom_X
		ConfidenceInterval_top_Y = 2
		ConfidenceInterval_top_X[0] = -TimeWindow/2; ConfidenceInterval_top_X[1] = TimeWindow/2
		ConfidenceInterval_bottom_Y = -2
		ConfidenceInterval_bottom_X[0] = -TimeWindow/2; ConfidenceInterval_bottom_X[1] = TimeWindow/2
		AppendToGraph ConfidenceInterval_top_Y vs ConfidenceInterval_top_X
		AppendToGraph ConfidenceInterval_bottom_Y vs ConfidenceInterval_bottom_X
		ModifyGraph lstyle(ConfidenceInterval_top_Y)=2
		ModifyGraph lstyle(ConfidenceInterval_bottom_Y)=2
	endif
	// Axis
	Wavestats/Q Corr; SetAxis left V_min*1.2,V_max*1.2; ModifyGraph tick=2; ModifyGraph mirror=1
	if (CCTypeNorm == 1)
		Label left "Z-score"
	else
		Label left "Count"
	endif
	Label bottom "Lag"
End

FUNCTION procButtonCloseCorrGraph(ctrlName) : ButtonControl
	string ctrlName
	DoWindow/K CrossCorrelogramGraph
END

Function ProcCorrGrid(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	if (checked == 1)
		ModifyGraph/W=CrossCorrelogramGraph grid(left)=1,gridRGB(left)=(0,0,0)
		ModifyGraph/W=CrossCorrelogramGraph grid(bottom)=1,gridRGB(bottom)=(0,0,0)
	else
		ModifyGraph/W=CrossCorrelogramGraph grid(left)=0
		ModifyGraph/W=CrossCorrelogramGraph grid(bottom)=0
	endif
End

// The function that calculates the synchronization width and strength (width and area of the central peak above z score = 2)
Function Synch_width_and_strength() : GraphMarquee	
	String Format
// First step: save the limits of the graph marquees
Variable Vleft, Vright, Vbottom, Vtop
GetMarquee/K bottom
Format = "flag: %g; left: %g; top: %g; right: %g; bottom: %g\r"
Vleft = V_left; Vright = V_right
GetMarquee/K left
Format = "flag: %g; left: %g; top: %g; right: %g; bottom: %g\r"
Vbottom = V_bottom; Vtop = V_top
Wave Corr, ConfidenceInterval_top_Y
// Crossing in terms of X
FindLevels/Q/R=(Vleft, Vright) Corr ConfidenceInterval_top_Y[0]
Duplicate/o W_FindLevels TEMPWAVE1
// Crossing in terms of points
FindLevels/P/Q/R=(Vleft, Vright) Corr ConfidenceInterval_top_Y[0]
Duplicate/o W_FindLevels TEMPWAVE2
// Calculate synchronization width (peak width at z score = 2)
Variable/G SynchWidth = TEMPWAVE1[numpnts(TEMPWAVE1)-1] - TEMPWAVE1[0]
DrawText 0,-0.09,"Synchronization width = " + num2str(SynchWidth) + " s"
// Calculate synchronization strength (area of peak)
Variable TEMPVAR, i, j
Nvar CorrBin
Make/o/N=0 COUNTER_WAVE
String TEMPSTRING1, TEMPSTRING2, MASKERNAME1, MASKERNAME2
// A big mess is following... there is the calculation of the synchronization strength
// but also some code to color the area of the peak above z score = 2
// --> totally useless... this was just a coding challenge... sorry
For (i = 0; i < numpnts(TEMPWAVE2); i += 2)
	Make/o/N=0 FILL_X; Make/o/N=0 FILL_Y
	InsertPoints 0, 1, FILL_X; InsertPoints 0, 1, FILL_Y
	FILL_X[0] = TEMPWAVE1[i]; FILL_Y[0] = ConfidenceInterval_top_Y[0]
	InsertPoints (i/2), 1, COUNTER_WAVE
	Variable k; k = 1
	For (j = floor(TEMPWAVE2[i])+1; j < floor(TEMPWAVE2[i+1])+1; j += 1)
		COUNTER_WAVE[i/2] = COUNTER_WAVE[i/2] + (Corr[j] - ConfidenceInterval_top_Y[0])*CorrBin
		InsertPoints k, 1, FILL_X; InsertPoints k, 1, FILL_Y
		FILL_X[k] = leftX(Corr) + j*deltaX(Corr); FILL_Y[k] = Corr[j]
		k += 1
	Endfor
	InsertPoints k, 1, FILL_X; InsertPoints k, 1, FILL_Y
	FILL_X[k] = TEMPWAVE1[i+1]; FILL_Y[k] = ConfidenceInterval_top_Y[0]
	InsertPoints k+1, 1, FILL_X; InsertPoints k+1, 1, FILL_Y
	FILL_X[k+1] = FILL_X[0]; FILL_Y[k+1] = ConfidenceInterval_top_Y[0]
	TEMPSTRING1 = "Fill" + num2str(i/2) + "_X"
	TEMPSTRING2 = "Fill" + num2str(i/2) + "_Y"
	Duplicate/o FILL_X $TEMPSTRING1; Duplicate/o FILL_Y $TEMPSTRING2
	Appendtograph $TEMPSTRING2 vs $TEMPSTRING1
	ModifyGraph mode($TEMPSTRING2)=7,hbFill($TEMPSTRING2)=4
	Make/o/N=4 MASKER1, MASKER2
	MASKER1[0] = FILL_X[0]; MASKER1[1] = FILL_X[0]
	MASKER1[2] = FILL_X[k]; MASKER1[3] = FILL_X[k]
	MASKER2[0] = 0; MASKER2[1] = ConfidenceInterval_top_Y[0]
	MASKER2[2] = ConfidenceInterval_top_Y[0]; MASKER2[3] = 0
	MASKERNAME1 = "Masker" + num2str(i/2) + "_X"
	MASKERNAME2 = "Masker" + num2str(i/2) + "_Y"
	Duplicate/o MASKER1 $MASKERNAME1
	Duplicate/o MASKER2 $MASKERNAME2
	Appendtograph $MASKERNAME2 vs $MASKERNAME1
	ModifyGraph mode($MASKERNAME2)=7,hbFill($MASKERNAME2)=2
	ModifyGraph rgb($MASKERNAME2)=(65535,65535,65535)
Endfor
KillWaves FILL_X, FILL_Y
removefromgraph Corr
appendtograph Corr; ModifyGraph rgb(Corr)=(0,0,0)
Variable/G SynchStrength = sum(COUNTER_WAVE, 0, numpnts(COUNTER_WAVE)-1)
DrawText 0,-0.02,"Synchronization strength = " + num2str(SynchStrength) + " s"
End

//*****************************************************************************************************************************************************************
// The following functions are not part of the SpAcAn package; there were added for the analysis of Golgi cells paired recordings.
// But it would be interesting to integrate some of them in future versions of SpAcAn. 
//*****************************************************************************************************************************************************************
Function Sup(number, HistoThrshld)		// "Sup" for superimpose. 
	Variable number
	Variable HistoThrshld
	Variable i, j, k, l
	Wave DetectedLevels1, DetectedLevels2, Occurrence1, Occurrence2, Tags1, Tags2
	Svar Filename1, Filename2
	Nvar SI1, SI2, NumChannels
	String WindowName = "Detected events channel " + num2str(number) + ", superimposed"
	String TEMPSTRING1, TEMPSTRING2
	DoWindow/K SuperimposedEvents
	Display/W=(100,100,600,500) as WindowName; DoWindow/C SuperimposedEvents
	Display/HOST=SuperimposedEvents/W=(0,0,0.5,0.5)/N=Window1
	Display/HOST=SuperimposedEvents/W=(0.5,0,1,0.5)/N=Window2
	Display/HOST=SuperimposedEvents/W=(0,0.5,0.5,1)/N=Window3
	If (number == 1)
		Duplicate/o DetectedLevels1, Dummy; Dummy = NaN; l = 0
		For (i = 0; i < numpnts(DetectedLevels1); i += 1)
			If (Tags1[i] == 1)
				TEMPSTRING1 = "E" + num2str(i) + "_1"
				TEMPSTRING2 = "E" + num2str(i) + "_2"
				Duplicate/o/R=(Occurrence1[i] - K16*.001, Occurrence1[i] + K17*.001) $FileName1 TEMPWAVE1
				If (NumChannels == 2)
					Duplicate/o/R=(Occurrence1[i] - K16*.001, Occurrence1[i] + K17*.001) $FileName2 TEMPWAVE2
				Endif
				SetScale/P x -K16*.001, SI1*1e-6,"s", TEMPWAVE1
				SetScale/P x -K16*.001, SI2*1e-6,"s", TEMPWAVE2
				FindLevels/P/Q/R=(.002,K17*.001) TEMPWAVE1, HistoThrshld
				Wave W_FindLevels
				For (k = 0; k < numpnts(W_FindLevels); k += 1)
					If (TEMPWAVE1[W_FindLevels[k+1]] - TEMPWAVE1[W_FindLevels[k]] > 0)
						Dummy[l] = W_FindLevels[k] * SI1 * 1e-6 - K16*.001
						l += 1
					Endif
				Endfor
				Duplicate/o TEMPWAVE1, $TEMPSTRING1
				Duplicate/o TEMPWAVE2, $TEMPSTRING2
				Appendtograph/W=SuperimposedEvents#Window1 $TEMPSTRING1
				Appendtograph/W=SuperimposedEvents#Window3 $TEMPSTRING2
			Endif
		KillWaves TEMPWAVE1, TEMPWAVE2
		Endfor
		Duplicate/o Dummy, HistoEvents
		Histogram/B={0,.02,(K17*.001/.02)} Dummy, HistoEvents
		Appendtograph/W=SuperimposedEvents#Window2 HistoEvents
		ModifyGraph/W=SuperimposedEvents#Window2 mode=6
	Endif
	If (number == 2)
		Duplicate/o DetectedLevels2, Dummy; Dummy = NaN; l = 0
		For (i = 0; i < numpnts(DetectedLevels2); i += 1)
			If (Tags2[i] == 1)
				TEMPSTRING1 = "E" + num2str(i) + "_2"
				TEMPSTRING2 = "E" + num2str(i) + "_1"
				Duplicate/o/R=(Occurrence2[i] - K18*.001, Occurrence2[i] + K19*.001) $FileName2 TEMPWAVE1
				If (NumChannels == 2)
					Duplicate/o/R=(Occurrence2[i] - K18*.001, Occurrence2[i] + K19*.001) $FileName1 TEMPWAVE2
				Endif
				SetScale/P x -K18*.001, SI2*1e-6,"s", TEMPWAVE1
				SetScale/P x -K18*.001, SI1*1e-6,"s", TEMPWAVE2
				FindLevels/P/Q/R=(.002,K17*.001) TEMPWAVE1, HistoThrshld
				Wave W_FindLevels
				For (k = 0; k < numpnts(W_FindLevels); k += 1)
					If (TEMPWAVE1[W_FindLevels[k+1]] - TEMPWAVE1[W_FindLevels[k]] > 0)
						Dummy[l] = W_FindLevels[k] * SI2 * 1e-6 - K18*.001
						l += 1
					Endif
				Endfor
				Duplicate/o TEMPWAVE1, $TEMPSTRING1
				Duplicate/o TEMPWAVE2, $TEMPSTRING2
				Appendtograph/W=SuperimposedEvents#Window1 $TEMPSTRING1
				Appendtograph/W=SuperimposedEvents#Window3 $TEMPSTRING2
			Endif
		KillWaves TEMPWAVE1, TEMPWAVE2
		Endfor
		Duplicate/o Dummy, HistoEvents
		If (number == 1)
			Histogram/B={0,.02,(K17*.001/.02)} Dummy, HistoEvents
		Endif
		If (number == 2)
			Histogram/B={0,.02,(K19*.001/.02)} Dummy, HistoEvents
		Endif
		Appendtograph/W=SuperimposedEvents#Window2 HistoEvents
		ModifyGraph/W=SuperimposedEvents#Window2 mode=6
	Endif
End

//*****************************************************************************************************************************************************************
Function Boost(xvalue,x1value,x2value,x3value,x4value,x5value)
	Variable xvalue,x1value,x2value,x3value,x4value,x5value
	// works with function "sup"
	// assuming that all events are named "E#_N", # being the number of the event and N the number of the channel
	// THERE MUST NOT BE ANY OTHER WAVE IN THE GRAPH
	// Values :
	// xvalue = value where Vm will be measured. Must be Vm AT THE FOOT OF THE SPIKELET.
	// x1value, x2value = limits of the linear fit to take into account the variation of the baseline for calculating the peak of the spikelet.
	// x3value = value where the peak of the spikelet will be measured. Not used in current version (peak is detected with wavestats).
	// x4value = the time before xvalue where there must NOT be an action potential.
	// x5value = the time after xvalue where there must NOT be an action potential.
	GetWindow KwTopWin, WaveList	// get the list of the waves displayed in the graph
	Wave/T W_WaveList
	String TEMPSTRING
	Variable i, Enumber, NumWaves
	TEMPSTRING = W_WaveList[numpnts(W_WaveList)-1][0]		// Get the name of the last wave
	NumWaves = str2num(TEMPSTRING[1,strlen(TEMPSTRING)-3])	// Get the number (#) of the last event
	Make/o/N=(NumWaves +1) Values_1, Values_2
	For (i = 0; i < numpnts(W_WaveList); i += 1)
		TEMPSTRING = W_WaveList[i][0]
		Enumber = str2num(TEMPSTRING[1,strlen(TEMPSTRING)-3])
		Duplicate/o $TEMPSTRING TEMPWAVE
		FindLevel/Q/R=(xvalue - x4value, xvalue + x5value) TEMPWAVE, 0
		If (V_flag == 1)
			Values_1[Enumber] = TEMPWAVE(xvalue)
			CurveFit/Q line  TEMPWAVE (x1value,x2value) /D
			Wavestats/Q/R=(xvalue,xvalue+0.003) TEMPWAVE 	// SET THE SIZE OF THE WINDOW TO LOOK FOR PEAK OF THE SPIKELET; here = 0.003 s
															// Time to peak is greater for depolarized Vm
															// --> So a Wavestats is better than a fixed value
			Values_2[Enumber] = V_max - (K0 + K1*V_maxloc)
		Else
			Values_1[Enumber] = NaN
			Values_2[Enumber] = NaN
		Endif
	Endfor
	KillWaves TEMPWAVE
	Svar Filename1
	String name1, name2
	name1 = "Vm_" + Filename1[0,12]; Duplicate/o Values_1 $name1
	name2 = "Vspk_" + Filename1[0,12]; Duplicate/o Values_2 $name2
	DoWindow/K WindowBoost
	Display $name2 vs $name1; ModifyGraph mode=3,marker=19,msize=1,rgb=(0,0,0)
	DoWindow/C WindowBoost
	Killwaves Values_1, Values_2
	Duplicate/o/R=[0,*][0,0] W_WaveList, EventNames
	Sort $name2 $name1, $name2, EventNames
	Edit $name1, $name2, EventNames
	Save/C $name1, $name2
End

// BOOST2 --> designed to quantify the subthreshold spikelet amplification in paired recorded Golgi cells
// Re-writting of Boost() -- cleaner, simpler... much better and convenient than the previous one.
// CORRECTIONS TO MAKE : for the box plot, If you're finding the median in an ordered set with an even number of values, you must take the average of the two middle numbers.

FUNCTION Boost12()
	Wave Realpeakposition1, Realpeakposition2
	Svar Filename1, Filename2
	Boost2(Realpeakposition1,Realpeakposition2,Filename1, Filename2)
END

FUNCTION Boost21()
	Wave Realpeakposition1, Realpeakposition2
	Svar Filename1, Filename2
	Boost2(Realpeakposition2,Realpeakposition1,Filename2, Filename1)
END

FUNCTION Boost2(TEMPWAVE_1,TEMPWAVE_2,File_name1, File_name2)
	Wave TEMPWAVE_1, TEMPWAVE_2
	String File_name1, File_name2

	Duplicate/o TEMPWAVE_1, TEMPWAVE1; Duplicate/o TEMPWAVE_2, TEMPWAVE2	
	Variable i, k, j, l
	String TEMPSTRING1, TEMPSTRING2, TEMPSTRING3, TEMPSTRING4
	
	
	// Prompt for curvefitting parameters
	Variable synch_left, synch_right, smooth_factor, baseline_limit_left, baseline_limit_right, peak_search_window, int_window, Bin
	synch_left = -0.01; synch_right = 0.015; smooth_factor = 17; baseline_limit_left = -0.005; baseline_limit_right = -0.0005; peak_search_window = 0.01; int_window = 0.01; Bin = 0.001
	Prompt synch_left, "Left synch. window limit (s)"
	Prompt synch_right, "Right synch. window limit (s)"
	Prompt smooth_factor, "Smooth (points) ?"
	Prompt baseline_limit_left, "Left baseline limit (s) ?"
	Prompt baseline_limit_right, "Right Baseline limit (s) ?"
	Prompt peak_search_window, "Peak window (s)?"
	Prompt int_window, "Integration window (s)?"
	Prompt Bin, "Bin (s)?"
	Doprompt "Parameters..." synch_left, synch_right, smooth_factor, baseline_limit_left, baseline_limit_right, peak_search_window, int_window, Bin
	If (V_flag == 1)
		Abort
	Endif

	
	// First loop... added after coding the rest... so not optimal
	// The goal is to quantify the SD of the variation of the baseline around the fit, the main cause for errors.
	Make/N=0/o Chisq
	k = 0
	For (i = 0; i < numpnts(TEMPWAVE1); i += 1)
//		FindLevel/P/Q TEMPWAVE2, TEMPWAVE1[i]
		// if no synchronous event in channel 2...
//		if ( TEMPWAVE2[floor(V_LevelX)] <  TEMPWAVE1[i] + synch_left && TEMPWAVE2[floor(V_LevelX)+1] > TEMPWAVE1[i] + synch_right )
			Duplicate/o/R=(TEMPWAVE1[i]+synch_left,TEMPWAVE1[i]+synch_right) $File_name2 TEMPWAVE3		// The actual wave on which we will work
			SetScale/P x synch_left,deltaX($File_name2),"s",TEMPWAVE3; Smooth smooth_factor, TEMPWAVE3
			CurveFit/Q line TEMPWAVE3(baseline_limit_left,baseline_limit_right)	// Linear fit of the baseline
			InsertPoints k, 1, Chisq; Chisq[k] = V_chisq
			k += 1
//		endif
	Endfor
	// loop complete, now display the SD as a histogram and prompt for cut-off
	Make/N=100/D/O HistoChisq; Histogram/B={0,1e-8,500} Chisq,HistoChisq
	DoWindow/K Histo_Chisq
	display HistoChisq
	DoWindow/C Histo_Chisq
	Variable HistoChisqCutOff
	Prompt HistoChisqCutOff, "Cut off (V) ?"
	Doprompt "I need one more parameter...", HistoChisqCutOff
	
	
	// Set up the display window
	Make/N=0/o Vpk, Vm, Int	// Vpk : size of spikelet, Vm : membrane potential just before the spikelet
	DoWindow/K WindowBoost1
	display/B=Bottom1/L=Left1/W=(0,0,500,500) Vpk vs Vm; ModifyGraph mode(Vpk)=3,marker(Vpk)=8	// setting up the graph...
	DoWindow/C WindowBoost1
	appendtograph/B=Bottom2/L=Left2 Int vs Vm; ModifyGraph mode(Int)=3,marker(Int)=8
	ModifyGraph axisEnab(Bottom1)={0,0.47}; ModifyGraph axisEnab(Bottom2)={0.53,1}
	ModifyGraph rgb(Vpk)=(48896,65280,48896); ModifyGraph rgb(Int)=(48896,52992,65280)
	ModifyGraph margin=43
	
	
	// will browse through detected events in channel 1 and calculate the size and integral of the spikelet
	// and store individual events into waves
	// will also measure Vpk (peak), Vm (potential at the foot of the spikelet) and Int (integral of the spikelet depolarizing phase) for each event
	k = 0
	For (i = 0; i < numpnts(TEMPWAVE1); i += 1)
//		FindLevel/P/Q TEMPWAVE2, TEMPWAVE1[i]
		// if no synchronous event in channel 2...
//		if ( TEMPWAVE2[floor(V_LevelX)] <  TEMPWAVE1[i] + synch_left && TEMPWAVE2[floor(V_LevelX)+1] > TEMPWAVE1[i] + synch_right )
			Duplicate/o/R=(TEMPWAVE1[i]+synch_left,TEMPWAVE1[i]+synch_right) $File_name2 TEMPWAVE3		// The actual wave on which we will work
			SetScale/P x synch_left,deltaX($File_name2),"s",TEMPWAVE3; Smooth smooth_factor, TEMPWAVE3
			TEMPSTRING1 = "event_" + num2str(k)
			Duplicate/o TEMPWAVE3, $TEMPSTRING1		// To keep a memory of the spikelet
			CurveFit/Q line TEMPWAVE3(baseline_limit_left,baseline_limit_right)	// Linear fit of the baseline
			if (V_chisq < HistoChisqCutOff)
				Duplicate/o TEMPWAVE3, TEMPWAVE4
				TEMPWAVE4 = k0 + k1*(p*deltaX($File_name2)+synch_left)	// The result of the fit
				SetScale/P x synch_left,deltaX($File_name2),"s",TEMPWAVE4
				TEMPSTRING2 = "fit_event_" + num2str(k)
				Duplicate/o TEMPWAVE4 $TEMPSTRING2		// To keep a memory of the fit
				Duplicate/o TEMPWAVE3 TEMPWAVE5
				TEMPWAVE5 = TEMPWAVE3 - TEMPWAVE4
				TEMPSTRING3 = "diff_event_" + num2str(k)
				Duplicate/o TEMPWAVE5, $TEMPSTRING3
				FindLevel/Q/R=(0.0008,synch_right) TEMPWAVE5, 0
				if (V_Flag == 0)	// the trace has to cross baseline within "synch_right"
					Wavestats/Q/R=(-0.0005,peak_search_window) TEMPWAVE5	// Find the peak after the baseline
					InsertPoints k, 1, Vpk; InsertPoints k, 1, Vm; InsertPoints k, 1, Int
					Vpk[k] = V_max
					Vm[k] = k0 + k1*-0.0005
					Integrate TEMPWAVE5/D=W_INT
					Wavestats/Q/R=(baseline_limit_left,baseline_limit_right) W_INT; W_INT = W_INT - V_avg	// Important: offset W_INT from baseline value.
					TEMPSTRING4 = "int_event_" + num2str(k)
					Duplicate/o W_INT, $TEMPSTRING4
					Wavestats/Q/R=(-0.0005,int_window) W_INT
					Int[k] = V_max
					k += 1
				endif
			endif
//		endif
	Endfor
	Variable/G Number_of_spk; Number_of_spk = k	// Keep the number of spikelets selected
	
	
	// The following: statistics on the data; TEMPWAVE1 and TEMPWAVE2 are re-used
	// Goal : calculate the mean, SD, SEM, median, first and third quartiles, etc...
	// Will also calculate the average spikelet per bin (default bin value = 1 mV)
	Duplicate/o Vm TEMPWAVE1; Duplicate/o Vpk TEMPWAVE2; Duplicate/o Int TEMPWAVE3
	Make/o/N=(Number_of_spk) EVENT_INDEX; EVENT_INDEX = p
	// Sort data according to membrane potential
	Sort TEMPWAVE1 TEMPWAVE3,TEMPWAVE2,TEMPWAVE1, EVENT_INDEX
	Variable TEMPVAR1, TEMPVAR2
	Make/N=0/o Vpk_binned, Int_binned, Vpk_binned_SD, Int_binned_SD, Vpk_binned_SEM, Int_binned_SEM
	Make/N=0/o Vpk_first_quartile, Vpk_third_quartile, Vpk_median_binned, Int_first_quartile, Int_third_quartile, Int_median_binned
	Make/N=0/o Vpk_max, Vpk_min, Int_max, Int_min
	Make/N=0/o Vm_binned
	k = 0
	String TEMPSTRING
	
	// For each bin, a loop that calculates the mean, SD, SEM, median, first and third quartile of Vpk, etc...
	// calculated also the average spikelet and saves the number of averaged spikelets per bin.
	For (i = 0; i < floor((TEMPWAVE1[rightX(TEMPWAVE1)] - TEMPWAVE1[leftX(TEMPWAVE1)]) / Bin); i += 1)
		FindLevel/Q/P TEMPWAVE1 (TEMPWAVE1[leftX(TEMPWAVE1)]+Bin*i); TEMPVAR1 = floor(V_LevelX)
		FindLevel/Q/P TEMPWAVE1 (TEMPWAVE1[leftX(TEMPWAVE1)]+Bin*(i+1)); TEMPVAR2 = floor(V_LevelX)
		Make/N=0/o Vpk_TEMP, Int_TEMP, Vm_TEMP	// Waves in which we will store the values of Vpk and Vm for each bin
		Wave event_0
		Duplicate/o event_0 AverageSpikelet_TEMP	// the wave used to calculate the average spikelet (per bin)
		print numpnts(AverageSpikelet_TEMP); AverageSpikelet_TEMP = 0	
		l = 0	// the counter
		// In the same loop, look for Vpk, Int (integral of the spikelet depolarizing phase) and Vm inside a bin
		For (j = TEMPVAR1+1; j < TEMPVAR2; j += 1)
			InsertPoints l, 1, Vpk_TEMP; InsertPoints l, 1, Int_TEMP
			Vpk_TEMP[l] = TEMPWAVE2[j]
			Int_TEMP[l] = TEMPWAVE3[j]
			InsertPoints l, 1, Vm_TEMP; Vm_TEMP[l] = TEMPWAVE1[j]
			TEMPSTRING = "event_" + num2str(EVENT_INDEX[j])
			Duplicate/o $TEMPSTRING TEMPWAVE4
			AverageSpikelet_TEMP = AverageSpikelet_TEMP + TEMPWAVE4
			l += 1
		Endfor
		// The average spikelet for this bin (bin number i)
		AverageSpikelet_TEMP = AverageSpikelet_TEMP / l
		TEMPSTRING = "AverageSpikelet_Bin" + num2str(i)
		Duplicate/o AverageSpikelet_TEMP $TEMPSTRING
		
		// Now calculates the mean of Vpk, the SD, th SEM and the extreme values (min and max)
		// First for Vpk
		Wavestats/Q Vpk_TEMP
		InsertPoints k, 1, Vpk_binned; Vpk_binned[k] = V_avg // the mean
		InsertPoints k, 1, Vpk_binned_SD; Vpk_binned_SD[k] = V_sdev // the SD
		InsertPoints k, 1, Vpk_binned_SEM; Vpk_binned_SEM[k] = V_sdev / sqrt(V_npnts) // the SEM
		InsertPoints k, 1, Vpk_max; Vpk_max[k] = V_max
		InsertPoints k, 1, Vpk_min; Vpk_min[k] = V_min
		// and then for Vm
		Wavestats/Q Vm_TEMP
		InsertPoints k, 1, Vm_binned;  Vm_binned[k] = V_avg // the mean
		
		// Now we calculate the quartiles values for Vpk_TEMP, according to the strict method (if the quartile falls between two points,
		// then take the average of the values of the two points).
		Sort Vpk_TEMP Vpk_TEMP
		// Two possibilities: either the number of spikelets in the bin is even, or odd
		// 1. numpnts(Vpk_TEMP) is an even number
		if (numpnts(Vpk_TEMP)*0.5 - floor(numpnts(Vpk_TEMP)*0.5) == 0)
			InsertPoints k, 1, Vpk_median_binned; Vpk_median_binned[k] = 0.5*(Vpk_TEMP[floor(numpnts(Vpk_TEMP)*0.5) - 1] + Vpk_TEMP[floor(numpnts(Vpk_TEMP)*0.5)])
			InsertPoints k, 1, Vpk_first_quartile
			Vpk_first_quartile[k]  = 0.5*( Vpk_TEMP[ floor(0.5*floor(numpnts(Vpk_TEMP)*0.5)) ] + Vpk_TEMP[ floor(0.5*floor(numpnts(Vpk_TEMP)*0.5)) + 1 ] )
			InsertPoints k, 1, Vpk_third_quartile
			Vpk_third_quartile[k] = 0.5*( Vpk_TEMP[ floor(numpnts(Vpk_TEMP)*0.5) + floor(0.5*floor(numpnts(Vpk_TEMP)*0.5)) -1]  +  Vpk_TEMP[ floor(numpnts(Vpk_TEMP)*0.5) + floor(0.5*floor(numpnts(Vpk_TEMP)*0.5)) ] )
		endif
		// 2. numpnts(Vpk_TEMP) is an odd number
		if (numpnts(Vpk_TEMP)*0.5 - floor(numpnts(Vpk_TEMP)*0.5) != 0)
			InsertPoints k, 1, Vpk_median_binned; Vpk_median_binned[k] = Vpk_TEMP[floor(numpnts(Vpk_TEMP)*0.5)]
			InsertPoints k, 1, Vpk_first_quartile
			Vpk_first_quartile[k]  = 0.5*( Vpk_TEMP[ floor(0.5* (floor(numpnts(Vpk_TEMP)*0.5)+1))-1 ] + Vpk_TEMP[ floor(0.5* (floor(numpnts(Vpk_TEMP)*0.5)+1)) ] )
			InsertPoints k, 1, Vpk_third_quartile
			Vpk_third_quartile[k] = 0.5*(Vpk_TEMP[ floor(numpnts(Vpk_TEMP)*0.5) + floor(0.5* (floor(numpnts(Vpk_TEMP)*0.5)+1)) -1] + Vpk_TEMP[ floor(numpnts(Vpk_TEMP)*0.5) + floor(0.5* (floor(numpnts(Vpk_TEMP)*0.5)+1))] )
		endif
	
		// The mean, SD and SEM of Int (integral of the spikelet depolarizing phase).
		Wavestats/Q Int_TEMP
		InsertPoints k, 1, Int_binned; Int_binned[k] = V_avg
		InsertPoints k, 1, Int_binned_SD; Int_binned_SD[k] = V_sdev
		InsertPoints k, 1, Int_binned_SEM; Int_binned_SEM[k] = V_sdev / sqrt(V_npnts)
		InsertPoints k, 1, Int_max; Int_max[k] = V_max
		InsertPoints k, 1, Int_min; Int_min[k] = V_min
		
		// Now we calculate the quartiles values for Int_TEMP: same as for Vpk
		Sort Int_TEMP Int_TEMP
		if (numpnts(Int_TEMP)*0.5 - floor(numpnts(Int_TEMP)*0.5) == 0) // numpnts(Int_TEMP) is an even number
			InsertPoints k, 1, Int_median_binned; Int_median_binned[k] = 0.5*(Int_TEMP[floor(numpnts(Int_TEMP)*0.5) - 1] + Int_TEMP[floor(numpnts(Int_TEMP)*0.5)])
			InsertPoints k, 1, Int_first_quartile
			Int_first_quartile[k]  = 0.5*( Int_TEMP[ floor(0.5*floor(numpnts(Int_TEMP)*0.5)) ] + Int_TEMP[ floor(0.5*floor(numpnts(Int_TEMP)*0.5)) + 1 ] )
			InsertPoints k, 1, Int_third_quartile
			Int_third_quartile[k] = 0.5*( Int_TEMP[ floor(numpnts(Int_TEMP)*0.5) + floor(0.5*floor(numpnts(Int_TEMP)*0.5)) -1]  +  Int_TEMP[ floor(numpnts(Int_TEMP)*0.5) + floor(0.5*floor(numpnts(Int_TEMP)*0.5)) ] )
		endif
		if (numpnts(Int_TEMP)*0.5 - floor(numpnts(Int_TEMP)*0.5) != 0) // numpnts(Int_TEMP) is an odd number
			InsertPoints k, 1, Int_median_binned; Int_median_binned[k] = Int_TEMP[floor(numpnts(Int_TEMP)*0.5)]
			InsertPoints k, 1, Int_first_quartile
			Int_first_quartile[k]  = 0.5*( Int_TEMP[ floor(0.5* (floor(numpnts(Int_TEMP)*0.5)+1))-1 ] + Int_TEMP[ floor(0.5* (floor(numpnts(Int_TEMP)*0.5)+1)) ] )
			InsertPoints k, 1, Int_third_quartile
			Int_third_quartile[k] = 0.5*(Int_TEMP[ floor(numpnts(Int_TEMP)*0.5) + floor(0.5* (floor(numpnts(Int_TEMP)*0.5)+1)) -1] + Int_TEMP[ floor(numpnts(Int_TEMP)*0.5) + floor(0.5* (floor(numpnts(Int_TEMP)*0.5)+1))] )
		endif
		
		k += 1
	Endfor
	// All right, now we have the statistics for every bin.
	
	// We have to scale the waves containing the statistics correctly
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vpk_binned
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vpk_binned_SD
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vpk_binned_SEM
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Int_binned
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Int_binned_SD
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Int_binned_SEM
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vpk_first_quartile
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vpk_third_quartile
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vpk_median_binned
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Int_first_quartile
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Int_third_quartile
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Int_median_binned
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vpk_max
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vpk_min
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Int_max
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Int_min
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vm_binned
	
	// Calcutale the inter-quartile range (IQR)
	Duplicate/o Vpk_first_quartile Vpk_IQR; Vpk_IQR = Vpk_third_quartile - Vpk_first_quartile
	Duplicate/o Vpk_first_quartile Vpk_bottom_outlier; Vpk_bottom_outlier = Vpk_first_quartile - 1.5*Vpk_IQR
	Duplicate/o Vpk_first_quartile Vpk_top_outlier; Vpk_top_outlier = Vpk_third_quartile + 1.5*Vpk_IQR
	Duplicate/o Int_first_quartile Int_IQR; Int_IQR = Int_third_quartile - Int_first_quartile
	Duplicate/o Int_first_quartile Int_bottom_outlier; Int_bottom_outlier = Int_first_quartile - 1.5*Int_IQR
	Duplicate/o Int_first_quartile Int_top_outlier; Int_top_outlier = Int_third_quartile + 1.5*Int_IQR
	
	// SD limits (top and bottom)
	Duplicate/o Vpk_binned Vpk_binned_SD1
	Vpk_binned_SD1 = Vpk_binned - Vpk_binned_SD
	Duplicate/o Vpk_binned Vpk_binned_SD2
	Vpk_binned_SD2 = Vpk_binned + Vpk_binned_SD
	Duplicate/o Vpk_binned Vpk_binned_SEM1
	Vpk_binned_SEM1 = Vpk_binned - Vpk_binned_SEM
	Duplicate/o Vpk_binned Vpk_binned_SEM2
	Vpk_binned_SEM2 = Vpk_binned + Vpk_binned_SEM
	Duplicate/o Int_binned Int_binned_SD1
	Int_binned_SD1 = Int_binned - Int_binned_SD
	Duplicate/o Int_binned Int_binned_SD2
	Int_binned_SD2 = Int_binned + Int_binned_SD
	Duplicate/o Int_binned Int_binned_SEM1
	Int_binned_SEM1 = Int_binned - Int_binned_SEM
	Duplicate/o Int_binned Int_binned_SEM2
	Int_binned_SEM2 = Int_binned + Int_binned_SEM
	appendtograph/B=Bottom1/L=Left1 Vpk_binned
	ModifyGraph mode(Vpk_binned)=3,rgb(Vpk_binned)=(0,0,0), marker(Vpk_binned)=16
	ModifyGraph msize(Vpk_binned)=2
	
	// Fit of the wave containing the binned values of the mean of Vpk
	Make/D/N=3/O W_coef; W_coef[0] = {0.3e-6,0.5,200}
	FuncFit BoostExp W_coef  Vpk_binned /D
	ModifyGraph rgb(fit_Vpk_binned)=(0,0,0)
	
	// Annotations
	String FitResult1 = "\Z08Fit Vpk = A + B.exp(C.Vm)\rA = " + num2str(w_coef[0]) + " V"
	FitResult1 = FitResult1 + "\rB = " + num2str(w_coef[1])
	FitResult1 = FitResult1 + "\rC = " + num2str(1/w_coef[2]) + " V"
	TextBox/C/N=FitResult1/F=0/A=LT/B=1/X=0.00/Y=35 FitResult1
	
	appendtograph/B=Bottom1/L=Left1 Vpk_binned_SD1, Vpk_binned_SD2
	appendtograph/B=Bottom1/L=Left1 Vpk_binned_SEM1, Vpk_binned_SEM2
	ModifyGraph rgb(Vpk_binned_SEM1)=(0,0,0); ModifyGraph rgb(Vpk_binned_SEM2)=(0,0,0)
	ModifyGraph lstyle(Vpk_binned_SEM1)=2; ModifyGraph lstyle(Vpk_binned_SEM2)=2
	appendtograph/B=Bottom2/L=Left2 Int_binned
	ModifyGraph mode(Int_binned)=3,rgb(Int_binned)=(0,0,0), marker(Int_binned)=16
	ModifyGraph msize(Int_binned)=2

	// Fit of the wave containing the binned values of the mean of Int
	Make/D/N=3/O W_coef; W_coef[0] = {0.3e-6,0.5,200}
	FuncFit BoostExp W_coef  Int_binned /D
	ModifyGraph rgb(fit_Int_binned)=(0,0,0)
	
	// Annotations
	String FitResult2 = "\Z08Fit Int = A + B.exp(C.Vm)\rA = " + num2str(w_coef[0]) + " V"
	FitResult2 = FitResult2 + "\rB = " + num2str(w_coef[1])
	FitResult2 = FitResult2 + "\rC = " + num2str(1/w_coef[2]) + " V"
	TextBox/C/N=FitResult2/F=0/A=LT/B=1/X=55.00/Y=0.00 FitResult2
	
	appendtograph/B=Bottom2/L=Left2 Int_binned_SD1, Int_binned_SD2
	appendtograph/B=Bottom2/L=Left2 Int_binned_SEM1, Int_binned_SEM2
	ModifyGraph rgb(Int_binned_SEM1)=(0,0,0); ModifyGraph rgb(Int_binned_SEM2)=(0,0,0)
	ModifyGraph lstyle(Int_binned_SEM1)=2; ModifyGraph lstyle(Int_binned_SEM2)=2
	Wavestats/Q Vpk; SetAxis Left1 0,V_max*1.2
	
	// Annotations again
	String command1 = "\Z08black dots (line) = mean (fit); dotted = SEM; red = SD"
	command1 = command1 + "\rgreen and blue dots (line) = size of average spikelets"
	command1 = command1 + "\r"
	command1 = command1 + "\rReference = " + File_name1 + "\rSpikelets = " + File_name2
	command1 = command1 + "\rsynch_left = " + num2str(synch_left) + " s\rsynch_right = " + num2str(synch_right) + " s\rsmooth_factor = " + num2str(smooth_factor) + " pnts\rbaseline_limit_left = " + num2str(baseline_limit_left) + " s\rbaseline_limit_right = " + num2str(baseline_limit_right)
	command1 = command1 + " s\rpeak_search_window = " + num2str(peak_search_window) + " s\rint_window = " + num2str(int_window) + " s\rBin = " + num2str(Bin)
	command1 = command1 + "\rFit Chisq cut off = " + num2str(HistoChisqCutOff)
	command1 = command1 + "\rNumber of spike(let)s = " + num2str(Number_of_spk) + "\r"
	TextBox/C/N=command1/F=0/A=LT/B=1/X=0.00/Y=-9.00 command1

	// and another graph with the same data but with the box plot (median, quartiles, extremes and outliers)
	DoWindow/K WindowBoost2
	display/B=Bottom1/L=Left1/W=(100,100,600,600) Vpk vs Vm; ModifyGraph mode(Vpk)=3,marker(Vpk)=8 // setting up the graph...
	DoWindow/C WindowBoost2
	appendtograph/B=Bottom2/L=Left2 Int vs Vm; ModifyGraph mode(Int)=3,marker(Int)=8
	ModifyGraph axisEnab(Bottom1)={0,0.47}; ModifyGraph axisEnab(Bottom2)={0.53,1}
	ModifyGraph rgb(Vpk)=(48896,65280,48896); ModifyGraph rgb(Int)=(48896,52992,65280)
	ModifyGraph margin=43
	
	// boxplot Vpk
	appendtograph/B=Bottom1/L=Left1 Vpk_median_binned
	ModifyGraph mode(Vpk_median_binned)=3,rgb(Vpk_median_binned)=(0,0,0), marker(Vpk_median_binned)=16
	ModifyGraph msize(Vpk_median_binned)=2
	
	// Fit of the wave containing the binned values of the median of Vpk
	Make/D/N=3/O W_coef; W_coef[0] = {0.3e-6,0.5,200}
	FuncFit BoostExp W_coef  Vpk_median_binned /D
	String FitResult3 = "\Z08Fit Vpk median = A + B.exp(C.Vm)\rA = " + num2str(w_coef[0]) + " V"
	FitResult3 = FitResult3 + "\rB = " + num2str(w_coef[1])
	FitResult3 = FitResult3 + "\rC = " + num2str(1/w_coef[2]) + " V"
	TextBox/C/N=FitResult3/F=0/A=LT/B=1/X=0.00/Y=35 FitResult3
	
	appendtograph/B=Bottom1/L=Left1 Vpk_first_quartile; ModifyGraph rgb(Vpk_first_quartile)=(0,0,0)
	appendtograph/B=Bottom1/L=Left1 Vpk_third_quartile; ModifyGraph rgb(Vpk_third_quartile)=(0,0,0)
	appendtograph/B=Bottom1/L=Left1 Vpk_bottom_outlier; ModifyGraph rgb(Vpk_bottom_outlier)=(0,0,0); ModifyGraph lstyle(Vpk_bottom_outlier)=3
	appendtograph/B=Bottom1/L=Left1 Vpk_top_outlier; ModifyGraph rgb(Vpk_top_outlier)=(0,0,0); ModifyGraph lstyle(Vpk_top_outlier)=3
	appendtograph/B=Bottom1/L=Left1 Vpk_max; ModifyGraph rgb(Vpk_max)=(0,0,0); ModifyGraph lstyle(Vpk_max)=2
	appendtograph/B=Bottom1/L=Left1 Vpk_min; ModifyGraph rgb(Vpk_min)=(0,0,0); ModifyGraph lstyle(Vpk_min)=2
	
	// boxplot Int
	appendtograph/B=Bottom2/L=Left2 Int_median_binned
	ModifyGraph mode(Int_median_binned)=3,rgb(Int_median_binned)=(0,0,0), marker(Int_median_binned)=16
	ModifyGraph msize(Int_median_binned)=2
	
	// Fit of the wave containing the binned values of the median of Vpk
	Make/D/N=3/O W_coef; W_coef[0] = {0.3e-6,0.5,200}
	FuncFit BoostExp W_coef  Int_median_binned /D
	String FitResult4 = "\Z08Fit Int median = A + B.exp(C.Vm)\rA = " + num2str(w_coef[0]) + " V"
	FitResult4 = FitResult4 + "\rB = " + num2str(w_coef[1])
	FitResult4 = FitResult4 + "\rC = " + num2str(1/w_coef[2]) + " V"
	TextBox/C/N=FitResult4/F=0/A=LT/B=1/X=55.00/Y=0.00 FitResult4
	
	appendtograph/B=Bottom2/L=Left2 Int_first_quartile; ModifyGraph rgb(Int_first_quartile)=(0,0,0)
	appendtograph/B=Bottom2/L=Left2 Int_third_quartile; ModifyGraph rgb(Int_third_quartile)=(0,0,0)
	appendtograph/B=Bottom2/L=Left2 Int_bottom_outlier; ModifyGraph rgb(Int_bottom_outlier)=(0,0,0); ModifyGraph lstyle(Int_bottom_outlier)=3
	appendtograph/B=Bottom2/L=Left2 Int_top_outlier; ModifyGraph rgb(Int_top_outlier)=(0,0,0); ModifyGraph lstyle(Int_top_outlier)=3
	appendtograph/B=Bottom2/L=Left2 Int_max; ModifyGraph rgb(Int_max)=(0,0,0); ModifyGraph lstyle(Int_max)=2
	appendtograph/B=Bottom2/L=Left2 Int_min; ModifyGraph rgb(Int_min)=(0,0,0); ModifyGraph lstyle(Int_min)=2
	
	// Annotations
	String command2 = "\Z08squares = median; red = fit; black (continuous) = first and third quartiles"
	command2 = command2 + "\rdotted (fine) = extremes; dotted (large) = outliers limits (1.5*IQR)"
	command2 = command2 + "\rgreen and blue dots (line) = size of average spikelets"
	command2 = command2 + "\r"
	command2 = command2 + "\rReference = " + File_name1 + "\rSpikelets = " + File_name2
	command2 = command2 + "\rsynch_left = " + num2str(synch_left) + " s\rsynch_right = " + num2str(synch_right) + " s\rsmooth_factor = " + num2str(smooth_factor) + " pnts\rbaseline_limit_left = " + num2str(baseline_limit_left) + " s\rbaseline_limit_right = " + num2str(baseline_limit_right)
	command2 = command2 + " s\rpeak_search_window = " + num2str(peak_search_window) + " s\rint_window = " + num2str(int_window) + " s\rBin = " + num2str(Bin)
	command2 = command2 + "\rFit Chisq cut off = " + num2str(HistoChisqCutOff)
	command2 = command2 + "\rNumber of spike(let)s = " + num2str(Number_of_spk)
	TextBox/C/N=command2/F=0/A=LT/B=1/X=0.00/Y=-9.00 command2
	
	// Now we would like to display the averaged spikelets (one average per bin)
	// --> will be used to make the paper figure
	// The base name is "AverageSpikelet_Bin#"
	// In the meantime, we can calculate Vpk and Int of the averaged spikelets
	Make/N=0/o Vpk_AverageSpikelets; Make/N=0/o Vm_AverageSpikelets; Make/N=0/o Int_AverageSpikelets
	DoWindow/K AveragedSpikelets; display; DoWindow/C AveragedSpikelets
	For (i = 0; i < numpnts(Vpk_binned); i += 1)
		// Display the event
		TEMPSTRING = "AverageSpikelet_Bin" + num2str(i)
		AppendToGraph $TEMPSTRING
		Duplicate/o $TEMPSTRING TEMPWAVE3
		CurveFit/Q line TEMPWAVE3(-0.003,baseline_limit_right)	// Linear fit of the baseline
		Duplicate/o TEMPWAVE3, TEMPWAVE4
		TEMPWAVE4 = k0 + k1*(p*deltaX($File_name2)+synch_left)	// The result of the fit
		SetScale/P x synch_left,deltaX($File_name2),"s",TEMPWAVE4
		Duplicate/o TEMPWAVE3 TEMPWAVE5
		TEMPWAVE5 = TEMPWAVE3 - TEMPWAVE4
		Wavestats/Q/R=(-0.0005,peak_search_window) TEMPWAVE5	// Find the peak after the baseline
		InsertPoints k, 1, Vpk_AverageSpikelets; InsertPoints k, 1, Vm_AverageSpikelets; InsertPoints k, 1, Int_AverageSpikelets
		Vpk_AverageSpikelets[k] = V_max
		Vm_AverageSpikelets[k] = k0 + k1*-0.0005
		Integrate TEMPWAVE5/D=W_INT
		Wavestats/Q/R=(baseline_limit_left,baseline_limit_right) W_INT; W_INT = W_INT - V_avg	// Important: offset W_INT from baseline value.
		Wavestats/Q/R=(-0.0005,int_window) W_INT
		Int_AverageSpikelets[k] = V_max
	Endfor
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vpk_AverageSpikelets
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Vm_AverageSpikelets
	SetScale/P x (TEMPWAVE1[leftX(TEMPWAVE1)] + Bin/2), Bin,"V", Int_AverageSpikelets
	// Display Vpk_AverageSpikelets in WindowBoost1
	DoWindow/F WindowBoost1
	AppendToGraph/L=Left1/B=Bottom1 Vpk_AverageSpikelets //vs Vm_AverageSpikelets
	ModifyGraph mode(Vpk_AverageSpikelets)=3,rgb(Vpk_AverageSpikelets)=(0,26112,13056)
	ModifyGraph marker(Vpk_AverageSpikelets)=16
	ModifyGraph msize(Vpk_AverageSpikelets)=2
	
	// Fit of Vpk_AverageSpikelets
	Make/D/N=3/O W_coef; W_coef[0] = {0.3e-6,0.5,200}
	FuncFit BoostExp W_coef  Vpk_AverageSpikelets /D
	ModifyGraph rgb(fit_Vpk_AverageSpikelets)=(0,26112,13056)
	
	// Annotations
	String FitResult5 = "\K(0,26112,0)\Z08Average spikelets"
	FitResult5 = FitResult5 + "\rFit Vpk = A + B.exp(C.Vm)\rA = " + num2str(w_coef[0]) + " V"
	FitResult5 = FitResult5 + "\rB = " + num2str(w_coef[1])
	FitResult5 = FitResult5 + "\rC = " + num2str(1/w_coef[2]) + " V"
	TextBox/C/N=FitResult5/F=0/A=LT/B=1/X=0.00/Y=48.00 FitResult5
	
	// Display Int_AverageSpikelets in WindowBoost1
	AppendToGraph/L=Left2/B=Bottom2 Int_AverageSpikelets //vs Vm_AverageSpikelets
	ModifyGraph mode(Int_AverageSpikelets)=3,rgb(Int_AverageSpikelets)=(0,9472,39168)
	ModifyGraph marker(Int_AverageSpikelets)=16
	ModifyGraph msize(Int_AverageSpikelets)=2
	
	// Fit of Vpk_AverageSpikelets
	Make/D/N=3/O W_coef; W_coef[0] = {0.3e-6,0.5,200}
	FuncFit BoostExp W_coef  Int_AverageSpikelets /D
	ModifyGraph rgb(fit_Int_AverageSpikelets)=(0,9472,39168)
	
	// Annotations
	String FitResult6 = "\Z08\K(0,0,65280)Average spikelets"
	FitResult6 = FitResult6 + "\rFit Int = A + B.exp(C.Vm)\rA = " + num2str(w_coef[0]) + " V"
	FitResult6 = FitResult6 + "\rB = " + num2str(w_coef[1])
	FitResult6 = FitResult6 + "\rC = " + num2str(1/w_coef[2]) + " V"
	TextBox/C/N=FitResult6/F=0/A=LT/B=1/X=55.00/Y=13.00 FitResult6
	
	// Display Vpk_AverageSpikelets in WindowBoost2
	DoWindow/F WindowBoost2
	AppendToGraph/L=Left1/B=Bottom1 Vpk_AverageSpikelets vs Vm_AverageSpikelets
	AppendToGraph/L=Left1/B=Bottom1 Fit_Vpk_AverageSpikelets
	ModifyGraph mode(Vpk_AverageSpikelets)=3,rgb(Vpk_AverageSpikelets)=(0,26112,13056)
	ModifyGraph msize(Vpk_AverageSpikelets)=2
	ModifyGraph marker(Vpk_AverageSpikelets)=16
	ModifyGraph rgb(fit_Vpk_AverageSpikelets)=(0,26112,13056)
	AppendToGraph/L=Left2/B=Bottom2 Int_AverageSpikelets vs Vm_AverageSpikelets
	AppendToGraph/L=Left2/B=Bottom2 Fit_Int_AverageSpikelets
	ModifyGraph mode(Int_AverageSpikelets)=3,rgb(Int_AverageSpikelets)=(0,9472,39168)
	ModifyGraph marker(Int_AverageSpikelets)=16
	ModifyGraph msize(Int_AverageSpikelets)=2
	ModifyGraph rgb(fit_Int_AverageSpikelets)=(0,9472,39168)
END

Function BoostExp(w,x) : FitFunc
	Wave w; Variable x

	return w[0] + w[1]*exp(w[2]*x)
End

FUNCTION showevent(number)
	Variable number
	String TEMPSTRING
	Variable/G Spikelet_number = number
	TEMPSTRING = "event_" + num2str(number)
	Duplicate/o $TEMPSTRING spikelet
	display/L=Left1/W=(0,0,320,500) spikelet
	TEMPSTRING = "fit_event_" + num2str(number)
	Duplicate/o $TEMPSTRING bsln_fit
	appendtograph/L=Left1 bsln_fit
	ModifyGraph axisEnab(Left1)={0.67,1}
	TEMPSTRING = "diff_event_" + num2str(number)
	Duplicate/o $TEMPSTRING diff_spk_bsln
	appendtograph/L=Left2 diff_spk_bsln
	ModifyGraph axisEnab(Left2)={0.34,0.66}
	ModifyGraph margin(left)=20
	TEMPSTRING = "int_event_" + num2str(number)
	Duplicate/o $TEMPSTRING int_spk
	appendtograph/L=Left3 int_spk
	ModifyGraph axisEnab(Left3)={0,0.32}
	String yAxis = "Left2"
	Variable pos = 0
	SetDrawEnv ycoord = $yAxis, linefgc = (65535,16385,16385), dash = 1
	DrawLine 0, pos, 1, pos
	String echo
	sprintf echo "PPTDrawZeroLine (\"%s\", %g)", yAxis, pos
	print echo
	//ResumeUpdate
	Doupdate
	WaitForKey_showevent()
END

Function WaitForKey_showevent()
		// Goes with previous function
		String state
		Variable i
		Nvar Spikelet_number
		Wave DetectedLevels1, Tags1, Occurrence1, BaselineY1, BaselineX1, PeakY1, PeakX1, RealAmp1, RealPeakPosition1, RiseTime20801, RT201, RT801
		Nvar EventNumber1, Baselinems1, Peakms1, KbdSpeed1
		Nvar Number_of_spk
		String TEMPSTRING
	Do
		state = KeyboardState("")

		If (char2num(state[9])==67)	// Stopper la function  l'aide de la touche "C"
			Break
		Endif

		If (str2num(state[5])==1)		// Afficher l'vnement prcdent
			if (Spikelet_number>0)
				Spikelet_number=Spikelet_number-1
				TEMPSTRING = "event_" + num2str(Spikelet_number)
				Duplicate/o $TEMPSTRING spikelet
				TEMPSTRING = "fit_event_" + num2str(Spikelet_number)
				Duplicate/o $TEMPSTRING bsln_fit
				TEMPSTRING = "diff_event_" + num2str(Spikelet_number)
				Duplicate/o $TEMPSTRING diff_spk_bsln
				TEMPSTRING = "int_event_" + num2str(Spikelet_number)
				Duplicate/o $TEMPSTRING int_spk
				DoUpdate
				i=0
				Do
					i+=1
				While (i<1000000)
			endif
		Endif
		
		If (str2num(state[6])==1)		// Afficher l'vnement suivant
			if (Spikelet_number<Number_of_spk-1)
				Spikelet_number=Spikelet_number+1
				TEMPSTRING = "event_" + num2str(Spikelet_number)
				Duplicate/o $TEMPSTRING spikelet
				TEMPSTRING = "fit_event_" + num2str(Spikelet_number)
				Duplicate/o $TEMPSTRING bsln_fit
				TEMPSTRING = "diff_event_" + num2str(Spikelet_number)
				Duplicate/o $TEMPSTRING diff_spk_bsln
				TEMPSTRING = "int_event_" + num2str(Spikelet_number)
				Duplicate/o $TEMPSTRING int_spk
				DoUpdate
				i=0
				Do
					i+=1
				While (i<1000000)
			endif
		Endif
		
	While(1)
End

FUNCTION showevent_sorted(number)
	Variable number
	Wave EVENT_INDEX
	String TEMPSTRING
	Variable/G Spikelet_number = number
	TEMPSTRING = "event_" + num2str(EVENT_INDEX[number])
	Duplicate/o $TEMPSTRING spikelet
	display/L=Left1/W=(0,0,320,500) spikelet
	TEMPSTRING = "fit_event_" + num2str(EVENT_INDEX[number])
	Duplicate/o $TEMPSTRING bsln_fit
	appendtograph/L=Left1 bsln_fit
	ModifyGraph axisEnab(Left1)={0.67,1}
	TEMPSTRING = "diff_event_" + num2str(EVENT_INDEX[number])
	Duplicate/o $TEMPSTRING diff_spk_bsln
	appendtograph/L=Left2 diff_spk_bsln
	ModifyGraph axisEnab(Left2)={0.34,0.66}
	ModifyGraph margin(left)=20
	TEMPSTRING = "int_event_" + num2str(EVENT_INDEX[number])
	Duplicate/o $TEMPSTRING int_spk
	appendtograph/L=Left3 int_spk
	ModifyGraph axisEnab(Left3)={0,0.32}
	String yAxis = "Left2"
	Variable pos = 0
	SetDrawEnv ycoord = $yAxis, linefgc = (65535,16385,16385), dash = 1
	DrawLine 0, pos, 1, pos
	String echo
	sprintf echo "PPTDrawZeroLine (\"%s\", %g)", yAxis, pos
	print echo
	//ResumeUpdate
	Doupdate
	WaitForKey_showevent_sorted()
END

Function WaitForKey_showevent_sorted()
		// Goes with previous function
		String state
		Variable i
		Variable/G Spikelet_number
		Wave DetectedLevels1, Tags1, Occurrence1, BaselineY1, BaselineX1, PeakY1, PeakX1, RealAmp1, RealPeakPosition1, RiseTime20801, RT201, RT801
		Nvar EventNumber1, Baselinems1, Peakms1, KbdSpeed1
		Nvar Number_of_spk
		Wave EVENT_INDEX
		String TEMPSTRING
	Do
		state = KeyboardState("")

		If (char2num(state[9])==67)	// Stopper la function  l'aide de la touche "C"
			Break
		Endif

		If (str2num(state[5])==1)		// Afficher l'vnement prcdent
			if (Spikelet_number>0)
				Spikelet_number=Spikelet_number-1
				TEMPSTRING = "event_" + num2str(EVENT_INDEX[Spikelet_number])
				Duplicate/o $TEMPSTRING spikelet
				TEMPSTRING = "fit_event_" + num2str(EVENT_INDEX[Spikelet_number])
				Duplicate/o $TEMPSTRING bsln_fit
				TEMPSTRING = "diff_event_" + num2str(EVENT_INDEX[Spikelet_number])
				Duplicate/o $TEMPSTRING diff_spk_bsln
				TEMPSTRING = "int_event_" + num2str(EVENT_INDEX[Spikelet_number])
				Duplicate/o $TEMPSTRING int_spk
				DoUpdate
				i=0
				Do
					i+=1
				While (i<1000000)
			endif
		Endif
		
		If (str2num(state[6])==1)		// Afficher l'vnement suivant
			if (Spikelet_number<Number_of_spk-1)
				Spikelet_number=Spikelet_number+1
				TEMPSTRING = "event_" + num2str(EVENT_INDEX[Spikelet_number])
				Duplicate/o $TEMPSTRING spikelet
				TEMPSTRING = "fit_event_" + num2str(EVENT_INDEX[Spikelet_number])
				Duplicate/o $TEMPSTRING bsln_fit
				TEMPSTRING = "diff_event_" + num2str(EVENT_INDEX[Spikelet_number])
				Duplicate/o $TEMPSTRING diff_spk_bsln
				TEMPSTRING = "int_event_" + num2str(EVENT_INDEX[Spikelet_number])
				Duplicate/o $TEMPSTRING int_spk
				DoUpdate
				i=0
				Do
					i+=1
				While (i<1000000)
			endif
		Endif
		
	While(1)
End

// Procdure d'analyse pour comparer la distribution des ISI d'une cellule (cellule 1) selon que l'autre cellule
// (cellule 2) a dcharg ou pas.

FUNCTION ISI(Occ1,Occ2)
Wave Occ1, Occ2
Variable i, j, k, l, m
Variable N, int, V_median
int = 0.005	// IMPORTANT PARAMETER -> the degre of synchronization tolerated
Make/o/N=0 ISI_1_a, ISI_1_b, ISI_2_a, ISI_2_b, ISI_1_a_a, ISI_1_a_b, ISI_2_a_a, ISI_2_a_b
Make/o/N=0 ISI_1_a_X, ISI_1_b_X, ISI_2_a_X, ISI_2_b_X, ISI_1_a_a_X, ISI_1_a_b_X, ISI_2_a_a_X, ISI_2_a_b_X
j = 0 // counter for ISI_1_a
k = 0 // counter for ISI_1_b
l = 0 // counter for ISI_1_a_a
m = 0 // counter for ISI_1_a_b

// The following loop looks at intervals in Cell1
// if a spike occurs in Cell2 during this interval, it is classified in "ISI_1_b"
// if a spike DOES NOT occur in Cell2 during this interval, it is classified in "ISI_1_a"
// UPDATE: now I'm looking more in details. If a spike does not occur in cell2, there is two possibilities:
// either there was a synchronized spike in cell 2 at the start of the interval (ISI_1_a_a) or not (ISI_1_a_b)
For (i = 0; i < (numpnts(Occ1)-1); i += 1)
	FindLevel/Q Occ2 (Occ1[i] + int); N = trunc(V_LevelX)
	if (Occ2[N+1] > (Occ1[i + 1] - int))	// if event in channel 2 NOT occurring between event i and i + 1 in channel 1
		if ( Occ1[i] - int < Occ2[N] && Occ2[N] < Occ1[i] + int)
			InsertPoints l, 1, ISI_1_a_a; ISI_1_a_a[l] = Occ1[i+1] - Occ1[i]
			InsertPoints l, 1, ISI_1_a_a_X; ISI_1_a_a_X[l] = (Occ1[i] + Occ1[i+1])/2
			l += 1
		endif
		if ( Occ2[N] < Occ1[i] - int)
			InsertPoints m, 1, ISI_1_a_b; ISI_1_a_b[m] = Occ1[i+1] - Occ1[i]
			InsertPoints m, 1, ISI_1_a_b_X; ISI_1_a_b_X[m] = (Occ1[i] + Occ1[i+1])/2
			m += 1
		endif
		InsertPoints j, 1, ISI_1_a; ISI_1_a[j] = Occ1[i+1] - Occ1[i]
		InsertPoints j, 1, ISI_1_a_X; ISI_1_a_X[j] = (Occ1[i] + Occ1[i+1])/2
		j += 1
	else		// if event in channel 2 occurring between event i and i + 1 in channel 1
		InsertPoints k, 1, ISI_1_b; ISI_1_b[k] = Occ1[i+1] - Occ1[i]
		InsertPoints k, 1, ISI_1_b_X; ISI_1_b_X[k] = (Occ1[i] + Occ1[i+1])/2
		k += 1
	endif
Endfor

j = 0 // counter for ISI_1_a
k = 0 // counter for ISI_1_b
l = 0 // counter for ISI_1_a_a
m = 0 // counter for ISI_1_a_b
// Same for cell 2
For (i = 0; i < (numpnts(Occ2)-1); i += 1)
	FindLevel/Q Occ1 (Occ2[i] + int); N = trunc(V_LevelX)
	if (Occ1[N+1] > (Occ2[i + 1] - int))	// if event in channel 1 NOT occurring between event i and i + 1 in channel 2
		if ( Occ2[i] - int < Occ1[N] && Occ1[N] < Occ2[i] + int)
			InsertPoints l, 1, ISI_2_a_a; ISI_2_a_a[l] = Occ2[i+1] - Occ2[i]
			InsertPoints l, 1, ISI_2_a_a_X; ISI_2_a_a_X[l] = (Occ2[i] + Occ2[i+1])/2
			l += 1
		endif
		if ( Occ1[N] < Occ2[i] - int)
			InsertPoints m, 1, ISI_2_a_b; ISI_2_a_b[m] = Occ2[i+1] - Occ2[i]
			InsertPoints m, 1, ISI_2_a_b_X; ISI_2_a_b_X[m] = (Occ2[i] + Occ2[i+1])/2
			m += 1
		endif
		InsertPoints j, 1, ISI_2_a; ISI_2_a[j] = Occ2[i+1] - Occ2[i]
		InsertPoints j, 1, ISI_2_a_X; ISI_2_a_X[j] = (Occ2[i] + Occ2[i+1])/2
		j += 1
	else		// if event in channel 1 occurring between event i and i + 1 in channel 2
		InsertPoints k, 1, ISI_2_b; ISI_2_b[k] = Occ2[i+1] - Occ2[i]
		InsertPoints k, 1, ISI_2_b_X; ISI_2_b_X[k] = (Occ2[i] + Occ2[i+1])/2
		k += 1
	endif
Endfor

Make/N=100/D/O Hist_ISI_1_a_a;DelayUpdate
Histogram/B={0,0.005,100} ISI_1_a_a,Hist_ISI_1_a_a
Make/N=100/D/O Hist_ISI_1_a_b;DelayUpdate
Histogram/B={0,0.005,100} ISI_1_a_b,Hist_ISI_1_a_b
Make/N=100/D/O Hist_ISI_1_a;DelayUpdate
Histogram/B={0,0.005,100} ISI_1_a,Hist_ISI_1_a
Make/N=100/D/O Hist_ISI_1_b;DelayUpdate
Histogram/B={0,0.005,100} ISI_1_b,Hist_ISI_1_b
Make/N=100/D/O Hist_ISI_2_a_a;DelayUpdate
Histogram/B={0,0.005,100} ISI_2_a_a,Hist_ISI_2_a_a
Make/N=100/D/O Hist_ISI_2_a_b;DelayUpdate
Histogram/B={0,0.005,100} ISI_2_a_b,Hist_ISI_2_a_b
Make/N=100/D/O Hist_ISI_2_a;DelayUpdate
Histogram/B={0,0.005,100} ISI_2_a,Hist_ISI_2_a
Make/N=100/D/O Hist_ISI_2_b;DelayUpdate
Histogram/B={0,0.005,100} ISI_2_b,Hist_ISI_2_b
DoWindow/K ISI_Histograms
display/W=(0,0,500,300) Hist_ISI_1_a_a, Hist_ISI_1_a_b, Hist_ISI_1_b
DoWindow/C ISI_Histograms
AppendToGraph/L=Left2 Hist_ISI_2_a, Hist_ISI_2_b
ModifyGraph rgb(Hist_ISI_1_a)=(0,0,52224)
ModifyGraph rgb(Hist_ISI_2_a)=(0,0,52224)
ModifyGraph axisEnab(left)={0.52,1},axisEnab(Left2)={0,0.48}

wavestats/Q ISI_1_a
SetDrawEnv textrgb= (0,0,65280); SetDrawEnv fsize= 10
DrawText 0.65,0.03,"Cellule 1: ISI sans PA ds cellule 2 :"
SetDrawEnv textrgb= (0,0,65280); SetDrawEnv fsize= 10
DrawText 0.65,0.08,"Moyenne : " + num2str(V_avg) + "  " + num2str(V_sdev) + " s"
// Calculating median
Duplicate/o ISI_1_a DUMMY; sort DUMMY, DUMMY
SetScale/P x 0,1,DUMMY
V_median = DUMMY((numpnts(DUMMY)-1)/2)

SetDrawEnv textrgb= (0,0,65280); SetDrawEnv fsize= 10
DrawText 0.65,0.132,"Mdiane : " + num2str(V_median) + " s"

wavestats/Q ISI_1_b
SetDrawEnv textrgb= (65280,0,0); SetDrawEnv fsize= 10
DrawText 0.65,0.2,"Cellule 1: ISI avec PA ds cellule 2 :"
SetDrawEnv textrgb= (65280,0,0); SetDrawEnv fsize= 10
DrawText 0.65,0.25,"Moyenne : " + num2str(V_avg) + "  " + num2str(V_sdev) + " s"
// Calculating median
Duplicate/o ISI_1_b DUMMY; sort DUMMY, DUMMY
SetScale/P x 0,1,DUMMY
V_median = DUMMY((numpnts(DUMMY)-1)/2)

SetDrawEnv textrgb= (65280,0,0); SetDrawEnv fsize= 10
DrawText 0.65,0.30,"Mdiane : " + num2str(V_median) + " s"

wavestats/Q ISI_2_a
SetDrawEnv textrgb= (0,0,65280); SetDrawEnv fsize= 10
DrawText 0.65,0.6,"Cellule 2: ISI sans PA ds cellule 1 :"
SetDrawEnv textrgb= (0,0,65280); SetDrawEnv fsize= 10
DrawText 0.65,0.65,"Moyenne : " + num2str(V_avg) + "  " + num2str(V_sdev) + " s"
// Calculating median
Duplicate/o ISI_2_a DUMMY; sort DUMMY, DUMMY
SetScale/P x 0,1,DUMMY
V_median = DUMMY((numpnts(DUMMY)-1)/2)

SetDrawEnv textrgb= (0,0,65280); SetDrawEnv fsize= 10
DrawText 0.65,0.70,"Mdiane : " + num2str(V_median) + " s"

wavestats/Q ISI_2_b
SetDrawEnv textrgb= (65280,0,0); SetDrawEnv fsize= 10
DrawText 0.65,0.77,"Cellule 2: ISI avec PA ds cellule 1 :"
SetDrawEnv textrgb= (65280,0,0); SetDrawEnv fsize= 10
DrawText 0.65,0.82,"Moyenne : " + num2str(V_avg) + "  " + num2str(V_sdev) + " s"
// Calculating median
Duplicate/o ISI_2_b DUMMY; sort DUMMY, DUMMY
SetScale/P x 0,1,DUMMY
V_median = DUMMY((numpnts(DUMMY)-1)/2)

SetDrawEnv textrgb= (65280,0,0); SetDrawEnv fsize= 10
DrawText 0.65,0.87,"Mdiane : " + num2str(V_median) + " s"

ModifyGraph margin(top)=43
Label bottom "Time (s)"

DoWindow/K ISI_Window
Display/B=Bottom1 ISI_1_a_a vs ISI_1_a_a_X
ModifyGraph marker(ISI_1_a_a)=19
DoWindow/C ISI_Window
ModifyGraph axisEnab(Bottom1)={0,0.35}
Appendtograph/B=bottom1 ISI_1_a_b vs ISI_1_a_b_X
ModifyGraph marker(ISI_1_a_b)=6
AppendToGraph/B=Bottom1 ISI_1_b vs ISI_1_b_X
ModifyGraph rgb=(0,15872,65280)
ModifyGraph mode=3; ModifyGraph marker(ISI_1_b)=8; ModifyGraph marker(ISI_1_a)=19

// Markers for mean and median

Make/N=1/O ISI_1_a_V_avg, ISI_1_a_V_avg_X, ISI_1_a_SD
ISI_1_a_V_avg_X[0] = 0
Wavestats/Q ISI_1_a; ISI_1_a_V_avg[0] = V_avg; ISI_1_a_SD[0] = V_sdev
Appendtograph/B=Bottom2 ISI_1_a_V_avg vs ISI_1_a_V_avg_X
ModifyGraph mode=3
ModifyGraph axisEnab(Bottom2)={0.36,0.46}; ModifyGraph marker(ISI_1_a_V_avg)=19,rgb(ISI_1_a_V_avg)=(0,15872,65280)
ErrorBars ISI_1_a_V_avg Y,wave=(ISI_1_a_SD,ISI_1_a_SD)

Make/N=1/O ISI_1_b_V_avg, ISI_1_b_V_avg_X, ISI_1_b_SD
ISI_1_b_V_avg_X[0] = 0
Wavestats/Q ISI_1_b; ISI_1_b_V_avg[0] = V_avg; ISI_1_b_SD[0] = V_sdev
Appendtograph/B=Bottom2 ISI_1_b_V_avg vs ISI_1_b_V_avg_X
ModifyGraph mode=3
ModifyGraph marker(ISI_1_b_V_avg)=8,rgb(ISI_1_b_V_avg)=(0,15872,65280)
ModifyGraph noLabel(Bottom2)=2,axThick(Bottom2)=0
ErrorBars ISI_1_b_V_avg Y,wave=(ISI_1_b_SD,ISI_1_b_SD)

AppendToGraph/B=Bottom3 ISI_2_a_a vs ISI_2_a_a_X
ModifyGraph marker(ISI_2_a_a)=19
ModifyGraph axisEnab(Bottom3)={0.47,0.82}
AppendToGraph/B=Bottom3 ISI_2_a_b vs ISI_2_a_b_X
ModifyGraph marker(ISI_2_a_b)=6
AppendToGraph/B=Bottom3 ISI_2_b vs ISI_2_b_X
ModifyGraph mode=3; ModifyGraph marker(ISI_2_b)=8; ModifyGraph marker(ISI_2_a)=19

// Markers for mean and median

Make/N=1/O ISI_2_a_V_avg, ISI_2_a_V_avg_X, ISI_2_a_SD
ISI_2_a_V_avg_X[0] = 0
Wavestats/Q ISI_2_a; ISI_2_a_V_avg[0] = V_avg; ISI_2_a_SD[0] = V_sdev
Appendtograph/B=Bottom4 ISI_2_a_V_avg vs ISI_2_a_V_avg_X
ModifyGraph mode=3
ModifyGraph axisEnab(Bottom4)={0.85,0.93}; ModifyGraph marker(ISI_2_a_V_avg)=19
ErrorBars ISI_2_a_V_avg Y,wave=(ISI_2_a_SD,ISI_2_a_SD)

Make/N=1/O ISI_2_b_V_avg, ISI_2_b_V_avg_X, ISI_2_b_SD
ISI_2_b_V_avg_X[0] = 0
Wavestats/Q ISI_2_b; ISI_2_b_V_avg[0] = V_avg; ISI_2_b_SD[0] = V_sdev
Appendtograph/B=Bottom4 ISI_2_b_V_avg vs ISI_2_b_V_avg_X
ModifyGraph mode=3
ModifyGraph marker(ISI_2_b_V_avg)=8
ModifyGraph noLabel(Bottom4)=2,axThick(Bottom4)=0
ErrorBars ISI_2_b_V_avg Y,wave=(ISI_2_b_SD,ISI_2_b_SD)

END

FUNCTION SUPP_SPIKES(Occ1, Occ2)
	// Function used for analysis for the Golgi cells gap junctions paper
	Wave Occ1, Occ2
	Svar Filename1, Filename2
	string namedummy1, namedummy2
	Nvar SI1, SI2
	DoWindow/K Window_supp_spikes
	display/W=(100,50,800,400) as "Superimposed consecutive spikes"
	DoWindow/C Window_supp_spikes
	Variable i
	Variable int = 0.005
	Variable N
	Variable o,p,q,r,s,t
	o = 0; p = 0; q = 0; r = 0; s = 0; t = 0
	Make/O/N=0 ISI_type1, ISI_type2, ISI_type3, ISI_type4, ISI_type5, ISI_type6, ISI_type1_X, ISI_type2_X, ISI_type3_X, ISI_type4_X, ISI_type5_X, ISI_type6_X 
	Make/o/N=0 Delta_T, ISI_pooled, ISI_no_spike
	Make/o/N=0 Delta_T_type1, Delta_T_type2, Delta_T_type3, Delta_T_type4, Delta_T_type5, Delta_T_type6
	For (i = 0; i < (numpnts(Occ1)-2); i += 1)
		FindLevel/Q Occ2 (Occ1[i] + int); N = trunc(V_LevelX)
		// Occ2[N] < Occ1[i] + int ncessairement
		namedummy1 = "Dummy_probe1_" + num2str(i)
		namedummy2 = "Dummy_probe2_" + num2str(i)
		Duplicate/o/R=(Occ1[i]-0.02,Occ1[i]+ 0.11) $Filename2 $namedummy1
		Duplicate/o/R=(Occ1[i]-0.02,Occ1[i]+ 0.11) $Filename1 $namedummy2
		SetScale/P x -0.02,SI1*1e-6,"s",$namedummy1
		SetScale/P x -0.02,SI1*1e-6,"s",$namedummy2
		if ( Occ2[N] < Occ1[i] - int)
			if ( Occ2[N+1] < Occ1[i + 1] - int)
				appendtograph/L=Left1/B=Bottom1 $namedummy1
				appendtograph/L=Left2/B=Bottom1 $namedummy2
				//ModifyGraph rgb($namedummy2)=(52224,0,41728)
				//ModifyGraph rgb($namedummy1)=(52224,0,41728)
				InsertPoints o, 1, ISI_type1; InsertPoints o, 1, ISI_type1_X
				ISI_type1[o] = Occ1[i+1] - Occ1[i]; ISI_type1_X[o] = (Occ1[i] + Occ1[i+1])/2
				InsertPoints o, 1, Delta_T_type1
				Delta_T_type1[o] = Occ2[N+1] - Occ1[i]
				o += 1
				InsertPoints i, 1, Delta_T, ISI_pooled, ISI_no_spike
				Delta_T[i] = Occ2[N+1] - Occ1[i]; ISI_pooled[i] = Occ1[i+1] - Occ1[i]; ISI_no_spike[i]=NaN
			endif
			if ( Occ2[N+1] > Occ1[i+1] - int && Occ2[N+1] < Occ1[i+1] + int )
				appendtograph/L=Left1/B=Bottom2 $namedummy1
				appendtograph/L=Left2/B=Bottom2 $namedummy2
				//ModifyGraph rgb($namedummy2)=(52224,0,41728)
				//ModifyGraph rgb($namedummy1)=(52224,0,41728)
				InsertPoints p, 1, ISI_type2; InsertPoints p, 1, ISI_type2_X
				ISI_type2[p] = Occ1[i+1] - Occ1[i]; ISI_type2_X[p] = (Occ1[i] + Occ1[i+1])/2
				InsertPoints p, 1, Delta_T_type2
				Delta_T_type2[p] = -0.05
				p += 1
				InsertPoints i, 1, Delta_T, ISI_pooled, ISI_no_spike
				Delta_T[i] = -0.05; ISI_pooled[i] = NaN; ISI_no_spike[i] = Occ1[i+1] - Occ1[i]
			endif
			if ( Occ2[N+1] > Occ1[i+1] + int)
				appendtograph/L=Left1/B=Bottom3 $namedummy1
				appendtograph/L=Left2/B=Bottom3 $namedummy2
				//ModifyGraph rgb($namedummy2)=(52224,0,41728)
				//ModifyGraph rgb($namedummy1)=(52224,0,41728)
				InsertPoints q, 1, ISI_type3; InsertPoints q, 1, ISI_type3_X
				ISI_type3[q] = Occ1[i+1] - Occ1[i]; ISI_type3_X[q] = (Occ1[i] + Occ1[i+1])/2
				InsertPoints q, 1, Delta_T_type3
				Delta_T_type3[q] = -0.05
				q += 1
				InsertPoints i, 1, Delta_T, ISI_pooled, ISI_no_spike
				Delta_T[i] = -0.05; ISI_pooled[i] = NaN; ISI_no_spike[i] = Occ1[i+1] - Occ1[i]
			endif
		else		// Occ1[i] - int < Occ2[N] < Occ1[i] + int
			if ( Occ2[N+1] < Occ1[i + 1] - int)
				appendtograph/L=Left1/B=Bottom4 $namedummy1
				appendtograph/L=Left2/B=Bottom4 $namedummy2
				//ModifyGraph rgb($namedummy2)=(52224,0,41728)
				//ModifyGraph rgb($namedummy1)=(52224,0,41728)
				InsertPoints r, 1, ISI_type4; InsertPoints r, 1, ISI_type4_X
				ISI_type4[r] = Occ1[i+1] - Occ1[i]; ISI_type4_X[r] = (Occ1[i] + Occ1[i+1])/2
				InsertPoints r, 1, Delta_T_type4
				Delta_T_type4[r] = Occ2[N] - Occ1[i]
				r += 1
				InsertPoints i, 1, Delta_T, ISI_pooled, ISI_no_spike
				Delta_T[i] = Occ2[N] - Occ1[i]; ISI_pooled[i] = Occ1[i+1] - Occ1[i]; ISI_no_spike[i] = NaN
			endif
			if ( Occ2[N+1] > Occ1[i+1] - int && Occ2[N+1] < Occ1[i+1] + int )
				appendtograph/L=Left1/B=Bottom5 $namedummy1
				appendtograph/L=Left2/B=Bottom5 $namedummy2
				//ModifyGraph rgb($namedummy2)=(52224,0,41728)
				//ModifyGraph rgb($namedummy1)=(52224,0,41728)
				InsertPoints s, 1, ISI_type5; InsertPoints s, 1, ISI_type5_X
				ISI_type5[s] = Occ1[i+1] - Occ1[i]; ISI_type5_X[s] = (Occ1[i] + Occ1[i+1])/2
				InsertPoints s, 1, Delta_T_type5
				Delta_T_type5[s] = Occ2[N] - Occ1[i]
				s += 1
				InsertPoints i, 1, Delta_T, ISI_pooled, ISI_no_spike
				Delta_T[i] = Occ2[N] - Occ1[i]; ISI_pooled[i] = Occ1[i+1] - Occ1[i]; ISI_no_spike[i] = NaN
			endif
			if ( Occ2[N+1] > Occ1[i+1] + int)
				appendtograph/L=Left1/B=Bottom6 $namedummy1
				appendtograph/L=Left2/B=Bottom6 $namedummy2
				//ModifyGraph rgb($namedummy2)=(52224,0,41728)
				//ModifyGraph rgb($namedummy1)=(52224,0,41728)
				InsertPoints t, 1, ISI_type6; InsertPoints t, 1, ISI_type6_X
				ISI_type6[t] = Occ1[i+1] - Occ1[i]; ISI_type6_X[t] = (Occ1[i] + Occ1[i+1])/2
				InsertPoints t, 1, Delta_T_type6
				Delta_T_type6[t] = Occ2[N] - Occ1[i]
				t += 1
				InsertPoints i, 1, Delta_T, ISI_pooled, ISI_no_spike
				Delta_T[i] = Occ2[N] - Occ1[i]; ISI_pooled[i] = Occ1[i+1] - Occ1[i]; ISI_no_spike[i] = NaN
			endif
		endif
	Endfor
	ModifyGraph axisEnab(Left1)={0,0.49}
	ModifyGraph axisEnab(Left2)={0.51,1}
	ModifyGraph axisEnab(bottom1)={0,0.15}
	ModifyGraph axisEnab(bottom2)={0.16,0.31}
	ModifyGraph axisEnab(bottom3)={0.32,0.47}
	ModifyGraph axisEnab(bottom4)={0.48,0.63}
	ModifyGraph axisEnab(bottom5)={0.64,0.79}
	ModifyGraph axisEnab(bottom6)={0.8,0.95}
	ModifyGraph margin=57
	DoWindow/K Window_ISIs
	Display ISI_type1 vs ISI_type1_X as "ISIs vs. time"; ModifyGraph mode(ISI_type1)=3,marker(ISI_type1)=1
	DoWindow/C Window_ISIs
	Appendtograph ISI_type2 vs ISI_type2_X; ModifyGraph mode(ISI_type2)=3,marker(ISI_type1)=2
	Appendtograph ISI_type3 vs ISI_type3_X; ModifyGraph mode(ISI_type3)=3,marker(ISI_type3)=3
	Appendtograph ISI_type4 vs ISI_type4_X; ModifyGraph mode(ISI_type4)=3,marker(ISI_type4)=4
	Appendtograph ISI_type5 vs ISI_type5_X; ModifyGraph mode(ISI_type5)=3,marker(ISI_type5)=5
	Appendtograph ISI_type6 vs ISI_type6_X; ModifyGraph mode(ISI_type6)=3,marker(ISI_type6)=6
//	display 
//	For (i = 0; i < (numpnts(Occ2)); i += 1)
//		namedummy = "Dummy" + num2str(i)
//		Duplicate/o/R=(Occ2[i]-0.02,Occ2[i]+ 0.1) $Filename1 $namedummy
//		SetScale/P x -0.02,SI1*1e-6,"s",$namedummy
//		appendtograph $namedummy
//	Endfor
	//DoWindow/K ISI_vs_DeltaT
	//Display ISI_pooled vs Delta_T as "ISI vs. delta T"
	//DoWindow/C ISI_vs_DeltaT
	Make/O/N=1 ISI_no_spike_avg, ISI_no_spike_avg_X, ISI_no_spike_SD
	Wavestats/Q ISI_no_spike
	ISI_no_spike_avg = V_avg; ISI_no_spike_SD = V_sdev; ISI_no_spike_avg_X = -0.05
	Appendtograph ISI_no_spike_avg vs ISI_no_spike_avg_X
	ModifyGraph mode=3,marker=8; ModifyGraph marker(ISI_no_spike_avg)=5,msize(ISI_no_spike_avg)=5
	ErrorBars ISI_no_spike_avg Y,wave=(ISI_no_spike_SD,ISI_no_spike_SD)

	Concatenate/o {ISI_type1, ISI_type4}, ISI_typeC
	Concatenate/o {ISI_type5, ISI_type6}, ISI_typeB
	Concatenate/o {ISI_type2, ISI_type3}, ISI_typeA
	Concatenate/o {Delta_T_type1, Delta_T_type4}, Delta_T_typeC
	Concatenate/o {Delta_T_type5, Delta_T_type6}, Delta_T_typeB
	Concatenate/o {Delta_T_type2, Delta_T_type3}, Delta_T_typeA
	
	DoWindow/K ISI_plot
	Display/W=(100,50,500,550) ISI_typeA vs Delta_T_typeA
	DoWindow/C ISI_plot
	appendtograph ISI_typeB vs Delta_T_typeB
	appendtograph ISI_typeC vs Delta_T_typeC
	ModifyGraph rgb(ISI_typeA)=(0,15872,65280); ModifyGraph rgb(ISI_typeB)=(0,65280,0)
	ModifyGraph mode=3,marker=8
	SetAxis left 0.04,0.18 
	
	Wavestats/Q ISI_typeA
	DrawText 0.058,0, "ISI_typeA = " + num2str(V_avg) + "  " + num2str(V_sdev) + " ms"
	Wavestats/Q ISI_typeB
	DrawText 0.058,0.05, "ISI_typeB = " + num2str(V_avg) + "  " + num2str(V_sdev) + " ms"
	Wavestats/Q ISI_typeC
	DrawText 0.058,0.1, "ISI_typeC = " + num2str(V_avg) + "  " + num2str(V_sdev) + " ms"
	
	DrawText 0.058,0.15, "Ecart rduits (Mann-Whitney U test)"
	
	MannWhitney (ISI_typeA, ISI_typeB); Nvar ecart_reduit_x
	DrawText 0.058,0.2, "ISI_typeA vs ISI_typeB: " + num2str(abs(ecart_reduit_x))
	MannWhitney (ISI_typeB, ISI_typeC)
	DrawText 0.058,0.25, "ISI_typeB vs ISI_typeC: " + num2str(abs(ecart_reduit_x))
	MannWhitney (ISI_typeA, ISI_typeC)
	DrawText 0.058,0.3, "ISI_typeA vs ISI_typeC: " + num2str(abs(ecart_reduit_x))
END

FUNCTION MannWhitney(wave1, wave2)
	Wave wave1, wave2
	Duplicate/o wave1, grouping1; grouping1 = 0
	Duplicate/o wave2, grouping2; grouping2 = 1
	Concatenate/o/NP {wave1, wave2}, combination
	Concatenate/o/NP {grouping1, grouping2}, grouping
	Sort combination combination, grouping
	Duplicate/o grouping grouping_wave2; Wave grouping_wave2
	Duplicate/o grouping_wave2 grouping_wave1
	grouping_wave1 = 1 - grouping_wave2
	Duplicate/o grouping_wave1 dummy; dummy = p+1
	grouping_wave1 = grouping_wave1 * dummy
	grouping_wave2 = grouping_wave2 * dummy
	Variable Wx // somme des rangs de grouping_wave1
	Variable Wy // somme des rangs de grouping_wave2
	Variable Null_Wx, Null_Wy
	Wx = sum(grouping_wave1, 0, numpnts(grouping_wave1))
	Wy = sum(grouping_wave2, 0, numpnts(grouping_wave2))
	//Killwaves grouping1, grouping2, combination, grouping, grouping_wave1, grouping_wave2, dummy
	Null_Wx = 0.5*numpnts(wave1)*(numpnts(wave1)+numpnts(wave2)+1)
	Null_Wy = 0.5*numpnts(wave2)*(numpnts(wave1)+numpnts(wave2)+1)
	Variable/G ecart_reduit_x
	ecart_reduit_x = abs( (Wx - Null_Wx) / sqrt( numpnts(wave1) * numpnts(wave2) * (numpnts(wave1)+numpnts(wave2)+1)/12 ) )
	print "Ecart reduit x = " + num2str(abs(ecart_reduit_x))
	Variable/G ecart_reduit_y
	ecart_reduit_y = abs( (Wy - Null_Wy) / sqrt( numpnts(wave1)*numpnts(wave2)*(numpnts(wave1)+numpnts(wave2)+1)/12 ) )
	//print "Ecart reduit y = " + num2str(abs(ecart_reduit_y))
	
END

FUNCTION spikes(realpeakposition_1,realpeakposition_2,Tags_1,Tags_2,Filename_1,Filename_2)
Wave realpeakposition_1, realpeakposition_2, Tags_1, Tags_2
String Filename_1, Filename_2

Nvar SI1, SI2
Variable/G before = -0.02
Variable/G after = 0.04
FindLevel/Q Tags_1, 1
if (V_flag == 1)
	abort
endif
Duplicate/O/R=(realpeakposition_1[V_levelX] + before, realpeakposition_1[V_levelX] + after) $Filename_1 TEMPWAVE1_ch1
setscale/P x before,SI1*1e-6,"s",TEMPWAVE1_ch1
Duplicate/O/R=(realpeakposition_1[V_levelX] + before, realpeakposition_1[V_levelX] + after) $Filename_1 TEMPWAVE1_ch2
setscale/P x before,SI2*1e-6,"s",TEMPWAVE1_ch2
TEMPWAVE1_ch1 = 0; TEMPWAVE1_ch2 = 0	// To have waves with the right size
Variable i; Variable k = 0
For (i = 0; i < numpnts(realpeakposition_1); i += 1)
	if (Tags_1[i] == 1)
		FindLevel/Q/P realpeakposition_2, realpeakposition_1[i]
		if (realpeakposition_2[floor(V_levelX)] < realpeakposition_1[i] + before && realpeakposition_2[floor(V_levelX)+1] > realpeakposition_1[i] + after)
			Duplicate/O/R=(realpeakposition_1[i] + before, realpeakposition_1[i] + after) $Filename_1 TEMPWAVE2_ch1
			setscale/P x before,SI1*1e-6,"s",TEMPWAVE2_ch1
			TEMPWAVE1_ch1 = TEMPWAVE1_ch1 + TEMPWAVE2_ch1
			Duplicate/O/R=(realpeakposition_1[i] + before, realpeakposition_1[i] + after) $Filename_2 TEMPWAVE2_ch2
			setscale/P x before,SI2*1e-6,"s",TEMPWAVE2_ch2
			TEMPWAVE1_ch2 = TEMPWAVE1_ch2 + TEMPWAVE2_ch2
			k+= 1
		endif
	endif
Endfor
TEMPWAVE1_ch1 = -TEMPWAVE1_ch1 / k
TEMPWAVE1_ch2 = -TEMPWAVE1_ch2 / k
display TEMPWAVE1_ch1, TEMPWAVE1_ch2
END

//*****************************************************************************************************************************************************************
Function SThrshld(number,x1value, x2value)	// ST for "Spike Threshold"
	Variable number,x1value, x2value	// number : channel number.
	// This function is used for cutting and concatenating segments of the recording.
	// Segment are taken from the trace around each detected event.
	// The limits of the segments are OccurrenceN[i] + x1value and OccurrenceN[i] + x2value...
	// ...with N being the channel number and i the event number.
	// Warning: there must be only the waves of interest in the subwindows MainWindow#Channel1 and MainWindow#Channel2
	// After concatenating the segments, the function calculates and displays the histogram of the values of the resulting wave.
	Variable i
	Make/o/N=1 TEMPWAVE_IN
	If (number == 1)
		Wave Occurrence1, DetectedLevels1
		Svar FileName1
		Nvar SI1
		SetScale/P x 0,SI1*1e-6,"s",TEMPWAVE_IN
		For (i = 0; i < numpnts(DetectedLevels1); i += 1)
			Duplicate/o/R=(Occurrence1[i]+x1value, Occurrence1[i]+x2value) $Filename1, FRAGMENT
			SetScale/P x 0,SI1*1e-6,"s",FRAGMENT
			Concatenate/o {TEMPWAVE_IN, FRAGMENT}, TEMPWAVE
			KillWaves TEMPWAVE_IN
			rename TEMPWAVE, TEMPWAVE_IN
		Endfor
	Endif
	If (number == 2)
		Wave Occurrence2, DetectedLevels2
		Svar FileName2
		Nvar SI2
		SetScale/P x 0,SI2*1e-6,"s",TEMPWAVE_IN
		For (i = 0; i < numpnts(DetectedLevels2); i += 1)
			Duplicate/o/R=(Occurrence2[i]+x1value, Occurrence2[i]+x2value) $Filename2, FRAGMENT
			SetScale/P x 0,SI2*1e-6,"s",FRAGMENT
			Concatenate/o {TEMPWAVE_IN, FRAGMENT}, TEMPWAVE
			KillWaves TEMPWAVE_IN
			rename TEMPWAVE, TEMPWAVE_IN
		Endfor
	Endif
	DeletePoints 0,1,TEMPWAVE_IN
	String name = "HISTO_" + num2str(abs(x1value)*1000)
	Make/N=250/D/O $name
	Histogram/B={-0.05,0.0001,250} TEMPWAVE_IN,$name
	Killwaves TEMPWAVE_IN
	DoWindow/K HistoST; Display $name; DoWindow/C HistoST; ModifyGraph mode=6
End

Function ST(number)
	Variable number
	SThrshld(number,-0.005,0.001); SThrshld(number,-0.01,0.001); SThrshld(number,-0.02,0.001)
	DoWindow/K HistoST
	Svar Filename1, Filename2
	Wave Histo_5, Histo_10, Histo_20
	String name1, name2, name3
	If (number == 1)
		name1 = "Histo_5_" + Filename1[0,strlen(Filename1)-2] + num2str(number)
		name2 = "Histo_10_" + Filename1[0,strlen(Filename1)-2] + num2str(number)
		name3 = "Histo_20_" + Filename1[0,strlen(Filename1)-2] + num2str(number)
	Endif
	If (number == 2)
		name1 = "Histo_5_" + Filename2[0,strlen(Filename2)-2] + num2str(number)
		name2 = "Histo_10_" + Filename2[0,strlen(Filename2)-2] + num2str(number)
		name3 = "Histo_20_" + Filename2[0,strlen(Filename2)-2] + num2str(number)
	Endif
	Duplicate/o Histo_5 $name1; Duplicate/o Histo_10 $name2; Duplicate/o Histo_20 $name3
	KillWaves Histo_5, Histo_10, Histo_20
	DoWindow/K HistoSTsup; Display/W=(50,50,500,400) $name1, $name2, $name3; DoWindow/C HistoSTsup
	ModifyGraph standoff(left)=0,font(left)="Arial"
	ModifyGraph standoff(bottom)=0,font(bottom)="Arial"
	ModifyGraph mirror(left)=2 ; ModifyGraph mirror(bottom)=2
	//SetAxis bottom -0.05,-0.02
	ModifyGraph rgb=(0,0,0); DrawText 0.0484210526315789,0.1,"(- 5, - 10, - 20 ms; + 1 ms)"
	String name 
	If (number == 1)
		name= Filename1[0,strlen(Filename1)-2] + num2str(number)
	Endif
	If (number == 2)
		name= Filename2[0,strlen(Filename2)-2] + num2str(number)
	Endif
	ModifyGraph rgb=(0,0,0); DrawText 0.0484210526315789,0.15,name
	ModifyGraph rgb=(0,0,0); DrawText 0.0484210526315789,0.2,"Seuil ="
	//SavePICT/C=0/E=-6/B=144 as "Vm_histo_" + name
	Save/C $name1, $name2, $name3
End

//*****************************************************************************************************************************************************************
// The following function is a quick programmed code for calculating the average detected event without aligning the baseline to zero.
// It makes sense for calculating average spikes and see in what range the potential is ramping.
Function average(number)
	Variable number	// the channel number
	Variable i, j
	Wave DetectedLevels1, DetectedLevels2, Occurrence1, Occurrence2, Tags1, Tags2
	Nvar Baseline1, Baseline2, Decay1, Decay2, SI1, SI2
	Svar Filename1, Filename2
	If (number == 1)
		FindLevel/Q Tags1, 1
		Duplicate/o/R=(Occurrence1[V_LevelX] - Baseline1*.001, Occurrence1[V_LevelX] + Decay1*.001) $Filename1, TEMPWAVE1
		SetScale/P x (-Baseline1*.001),SI1*1e-6,"s",TEMPWAVE1
		j = 1
		For (i = (V_LevelX + 1); i < numpnts(DetectedLevels1); i += 1)
			If (Tags1[i] == 1)
				Duplicate/o/R=(Occurrence1[i] - Baseline1*.001, Occurrence1[i] + Decay1*.001) $Filename1, TEMPWAVE2
				SetScale/P x (-Baseline1*.001),SI1*1e-6,"s",TEMPWAVE2
				TEMPWAVE1 = TEMPWAVE1 + TEMPWAVE2; j += 1
			Endif
		Endfor
		TEMPWAVE1 = TEMPWAVE1 / j
		Duplicate/o TEMPWAVE1, AverageEvent1
		AverageEvent1 = -AverageEvent1
	Endif
	If (number == 2)
		FindLevel/Q Tags2, 1
		Duplicate/o/R=(Occurrence2[V_LevelX] - Baseline2*.001, Occurrence2[V_LevelX] + Decay2*.001) $Filename2, TEMPWAVE1
		SetScale/P x (-Baseline2*.001),SI2*1e-6,"s",TEMPWAVE1
		j = 1
		For (i = (V_LevelX + 1); i < numpnts(DetectedLevels2); i += 1)
			If (Tags2[i] == 1)
				Duplicate/o/R=(Occurrence2[i] - Baseline2*.001, Occurrence2[i] + Decay2*.001) $Filename2, TEMPWAVE2
				SetScale/P x (-Baseline2*.001),SI2*1e-6,"s",TEMPWAVE2
				TEMPWAVE1 = TEMPWAVE1 + TEMPWAVE2; j += 1
			Endif
		Endfor
		TEMPWAVE1 = TEMPWAVE1 / j
		Duplicate/o TEMPWAVE1, AverageEvent2
		AverageEvent2 = -AverageEvent2
	Endif
	KillWaves TEMPWAVE1, TEMPWAVE2
End

// Calcul glissant de la variance
FUNCTION Var(x1, x2, N, thewave)
	Variable x1, x2, N
	Wave thewave
	Duplicate/o/R=(x1,x2) thewave TEMPWAVE0
	Duplicate/o TEMPWAVE0, TEMPWAVE1
	InsertPoints 0, N, TEMPWAVE1; DeletePoints numpnts(TEMPWAVE1)-N, N, TEMPWAVE1 
	Make/o/N=(numpnts(TEMPWAVE1)) TEMPWAVE2
	TEMPWAVE2 =  TEMPWAVE0 - TEMPWAVE1
	Duplicate/o TEMPWAVE2 VarWave
	KillWaves TEMPWAVE0, TEMPWAVE1, TEMPWAVE2
	Duplicate/o VarWave HistoVar
	Histogram/B={-0.004,0.0001,100} VarWave,HistoVar
	Display HistoVar
End

FUNCTION GGK2()
Variable x0, x1, x2, N
Prompt x0, "PMPulse_1_?_#_#"
Prompt x1, "PMPulse_1_#_?_# for start"
Prompt x2, "PMPulse_1_#_?_# for end"
Prompt N, "Number of sweeps to average ?"
Doprompt "parameters" x0,x1,x2, N
String Name1, Name2
Variable i, j, k
Make/o/N=0 Step1_1, Step1_2, Step2_1, Step2_2
k = 1
For (i = x1; i < x2-N+1; i += N)
	Name1 = "PMPulse_1_" + num2str(x0) + "_" + num2str(i) + "_1"
	Name2 = "PMPulse_1_" + num2str(x0) + "_" + num2str(i) + "_2"
	Duplicate/o $Name1 TEMPWAVE1
	Duplicate/o $Name2 TEMPWAVE3
	For (j = 1; j < N; j += 1)
		Name1 = "PMPulse_1_" + num2str(x0) + "_" + num2str(i+j) + "_1"
		Name2 = "PMPulse_1_" + num2str(x0) + "_" + num2str(i+j) + "_2"
		Duplicate/o $Name1 TEMPWAVE2
		Duplicate/o $Name2 TEMPWAVE4
		TEMPWAVE1 = TEMPWAVE1 + TEMPWAVE2
		TEMPWAVE3 = TEMPWAVE3 + TEMPWAVE4
	Endfor
	KillWaves TEMPWAVE2, TEMPWAVE4
	TEMPWAVE1 = TEMPWAVE1 / N; TEMPWAVE3 = TEMPWAVE3 / N
	Wavestats/Q/R=(1.4,1.5492) TEMPWAVE1; TEMPWAVE1 = TEMPWAVE1 - V_avg
	Wavestats/Q/R=(1.4,1.5492) TEMPWAVE3; TEMPWAVE3 = TEMPWAVE3 - V_avg
	Wavestats/Q/R=(1.75,2.05) TEMPWAVE1; InsertPoints k-1, 1, Step2_1; Step2_1[k-1] = V_avg
	Wavestats/Q/R=(1.75,2.05) TEMPWAVE3; InsertPoints k-1, 1, Step2_2; Step2_2[k-1] = V_avg
	Wavestats/Q/R=(0,0.34998) TEMPWAVE1; TEMPWAVE1 = TEMPWAVE1 - V_avg
	Wavestats/Q/R=(0,0.34998) TEMPWAVE3; TEMPWAVE3 = TEMPWAVE3 - V_avg
	Wavestats/Q/R=(0.55,0.85) TEMPWAVE1; InsertPoints k-1, 1, Step1_1; Step1_1[k-1] = V_avg
	Wavestats/Q/R=(0.55,0.85) TEMPWAVE3; InsertPoints k-1, 1, Step1_2; Step1_2[k-1] = V_avg
	Name1 = "Wave" + num2str(k) + "_1"; Name2 = "Wave" + num2str(k) + "_2"
	Duplicate/o TEMPWAVE1 $Name1; Duplicate/o TEMPWAVE3 $Name2
	k += 1
Endfor
KillWaves TEMPWAVE1, TEMPWAVE2, TEMPWAVE3, TEMPWAVE4
Duplicate/o Step1_1, RCell1; RCell1 = Step1_1 / (-200e-12)
Duplicate/o Step2_2, RCell2; RCell2 = Step2_2 / (-200e-12)
Duplicate/o Step1_1, K_12; K_12 = Step1_2 / Step1_1
Duplicate/o Step2_2, K_21; K_21 = Step2_1 / Step2_2
KillWaves Step1_1, Step1_2, Step2_1, Step2_2
END

Function env(wave_name)
	Wave wave_name
	Variable bin
	bin = 0.08
	Prompt bin, "interval (s)?"
	Doprompt "", bin
	Make/O/N=0 top, bottom
	Setscale/P x 0,deltaX(wave_name),"s",wave_name
	Variable i
	For (i = 0; i < (floor((deltaX(wave_name)*(numpnts(wave_name))/bin))) ; i += 1)
		wavestats/Q/R=(i*bin,i*bin + bin) wave_name
		InsertPoints i, 1, top; top[i] = V_max
		InsertPoints i, 1, bottom; bottom[i] = V_min
	Endfor
	Setscale/P x 0,bin,"s", top
	Setscale/P x 0,bin,"s", bottom
	DoWindow/K Window_Env
	display/W=(35,100,750,400) wave_name; ModifyGraph rgb=(65280,48896,48896)
	DoWindow/C Window_Env
	appendtograph top, bottom
	ModifyGraph rgb(top)=(0,0,0)
	ModifyGraph rgb(bottom)=(0,0,0)
	Duplicate/o top amp; amp = top - bottom
	wavestats/Q amp; DrawText 0.642,0.093,"amplitude = " + num2str(V_avg) + "  " + num2str(V_sdev) + " V"
End

Function trial()

Wave STO
	Duplicate/o STO wave_name
	Variable bin
	Make/O/N=0 dummy; display dummy
	Variable l; l = 0
for (bin = 0.05; bin < 1; bin += 0.01)	
	Make/O/N=0 top, bottom
	Setscale/P x 0,deltaX(wave_name),"s",wave_name
	Variable i
	For (i = 0; i < (floor((deltaX(wave_name)*(numpnts(wave_name))/bin))) ; i += 1)
		wavestats/Q/R=(i*bin,i*bin + bin) wave_name
		InsertPoints i, 1, top; top[i] = V_max
		InsertPoints i, 1, bottom; bottom[i] = V_min
	Endfor
	Duplicate/o top amp; amp = top - bottom
	wavestats/Q amp
	InsertPoints l, 1, dummy; dummy[l] = V_avg
	l += 1
	setscale/P x 0.05,0.01,"s",dummy
endfor

End

// ----------------------- BORIS : POUR CHARGER .abf -----------------------------
FUNCTION LOAD_FILE_PARAMETERS(file_name)
	string file_name

	// given file name, will extract many parameters from the header, placing the values, all as text, in
	// the wave sweep_params

	wave /t sweep_params = sweep_params
	
	variable numdata			
	variable counter = 0
	variable loopcount
	
	string chardata
	
	Nvar refnum
	
	Svar CurrentFile = CurrentFile
	
	open /r/t="????" refnum as CurrentFile

	counter = load_text(	"sFileType",				refNum, counter, 4)
	counter = load_float(	"fFileVersionNumber", 		refNum, counter)
	counter = load_int(		"nOperationMode", 			refNum, counter)
	counter = load_longint(	"lActualAcqLength", 			refNum, counter)
	counter = load_int(		"nNumPointsIgnored", 		refNum, counter)
	counter = load_longint(	"lActualEpisodes", 			refNum, counter)
	counter = load_longint(	"lFileStartDate", 			refNum, counter)
	counter = load_longint(	"lFileStartTime", 			refNum, counter)
	counter = load_longint(	"lStopwatchTime", 			refNum, counter)
	counter = load_float(	"fHeaderVersionNumber", 	refNum, counter)
	counter = load_int(		"nFileType", 				refNum, counter)
	counter = load_int(		"nMSBinFormat", 			refNum, counter)
	counter = load_longint(	"lDataSectionPtr", 			refNum, counter)
	counter = load_longint(	"lTagSectionPtr", 			refNum, counter)
	counter = load_longint(	"lNumTagEntries", 			refNum, counter)
	counter = load_longint(	"lLongDescriptionPtr", 		refNum, counter)
	counter = load_longint(	"lLongDescriptionLines", 	refNum, counter)
	counter = load_longint(	"lDACFilePtr", 				refNum, counter)
	counter = load_longint(	"lDACFileNumEpisodes", 	refNum, counter)

	FSetPos refNum, 72

	counter = load_longint(	"lDeltaArrayPtr", 			refNum, counter)
	counter = load_longint(	"lNumDeltas", 				refNum, counter)
	counter = load_longint(	"lNotebookPtr", 			refNum, counter)
	counter = load_longint(	"lNotebookManEntries", 	refNum, counter)
	counter = load_longint(	"lNotebookAutoEntries", 	refNum, counter)
	counter = load_longint(	"lSynchArrayPtr", 			refNum, counter)
	counter = load_longint(	"lSynchArrayPtr", 			refNum, counter)
	counter = load_longint(	"lSynchArraySize", 			refNum, counter)
	counter = load_longint(	"nDataFormat", 				refNum, counter)

	FSetPos refNum, 120

	counter = load_int(		"nADCNumChannels", 			refNum, counter)
	counter = load_float(	"fADCSampleInterval", 			refNum, counter)
	counter = load_float(	"fADCSecondSampleInterval", 	refNum, counter)
	counter = load_float(	"fSynchTimeUnit", 				refNum, counter)
	counter = load_float(	"fSecondsPerRun", 				refNum, counter)
	counter = load_longint(	"lNumSamplesPerEpisode", 		refNum, counter)
	counter = load_longint(	"lPreTriggerSamples", 			refNum, counter)
	counter = load_longint(	"lEpisodesPerRun", 				refNum, counter)
	counter = load_longint(	"lRunsPerTrial", 				refNum, counter)
	counter = load_longint(	"lNumberOfTrials", 			refNum, counter)
	counter = load_int(		"nAveragingMode", 				refNum, counter)
	counter = load_int(		"nUndoRunCount", 				refNum, counter)
	counter = load_int(		"nFirstEpisodeInRun", 			refNum, counter)
	counter = load_float(	"fTriggerThreshold", 			refNum, counter)
	counter = load_int(		"nTriggerSource", 				refNum, counter)
	counter = load_int(		"nTriggerAction", 				refNum, counter)
	counter = load_int(		"nTriggerPolarity", 			refNum, counter)
	counter = load_float(	"fScopeOutputInterval", 		refNum, counter)
	counter = load_float(	"fEpisodeStartToStart", 			refNum, counter)
	counter = load_float(	"fRunStartToStart", 			refNum, counter)
	counter = load_float(	"fTrialStartToStart", 			refNum, counter)
	counter = load_longint(	"lNumberOfTrials", 			refNum, counter)
	counter = load_longint(	"lAverageCount", 				refNum, counter)
	counter = load_longint(	"lClockChange", 				refNum, counter)

	FSetPos refNum, 244

	counter = load_float(		"fADCRange", 				refNum, counter)
	counter = load_float(		"fDACRange", 				refNum, counter)
	counter = load_longint(		"lADCResolution", 			refNum, counter)
	counter = load_longint(		"lDACResolution", 			refNum, counter)
	counter = load_int(			"nExperimentType", 		refNum, counter)
	counter = load_int(			"nAutoSampleEnable", 		refNum, counter)
	counter = load_int(			"nAutoSampleADCNum", 	refNum, counter)
	counter = load_int(			"nAutoSampleInstrument", 	refNum, counter)
	counter = load_float(		"fAutoSampleAdditGain", 	refNum, counter)
	counter = load_float(		"fAutoSampleFilter", 		refNum, counter)
	counter = load_float(		"fAutoSampleMembraneCap", refNum, counter)
	counter = load_int(			"nManualInfoStrategy", 		refNum, counter)
	counter = load_float(		"fCelIID1", 					refNum, counter)
	counter = load_float(		"fCelIID2", 					refNum, counter)
	counter = load_float(		"fCelIID3", 					refNum, counter)
	counter = load_text(		"sCreatorInfo",				refNum, counter, 16)
	counter = load_text(		"sFileComment",			refNum, counter, 56)

	FSetPos refNum, 378

	loopcount = 0
	do
		counter = load_int(	"nADCPtoLChannelMap" 		+ num2str(loopcount), refNum,counter)
		loopcount += 1
	while (loopcount < 16)

	loopcount = 0
	do
		counter = load_int(	"nADCSamplingSeq" 			+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 16)

	loopcount = 0
	do
		counter = load_text(	"sADCChannelName" 			+ num2str(loopcount), refNum, counter, 10)
		loopcount += 1
	while (loopcount < 16)

	loopcount = 0
	do
		counter = load_text(	"sADCUnits" 				+ num2str(loopcount), refNum, counter, 8)
		loopcount += 1
	while (loopcount < 16)

	loopcount = 0
	do
		counter = load_float(	"fADCProgrammableGain" 		+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 16)

	FSetPos refNum, 922

	loopcount = 0
	do
		counter = load_float(	"fInstrumentScaleFactor" 		+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 16)

	loopcount = 0
	do
		counter = load_float(	"fInstrumentOffset" 			+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 16)

	loopcount = 0
	do
		counter = load_float(	"fSignalGain" 				+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 16)

	loopcount = 0
	do
		counter = load_float(	"fSignalOffset" 				+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 16)

	loopcount = 0
	do
		counter = load_float(	"fSignalLowPassFilter" 		+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 16)

	loopcount = 0
	do
		counter = load_float(	"fSignalHighPassFilter" 		+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 16)

	loopcount = 0
	do
		counter = load_text(	"sDACChannelName" 			+ num2str(loopcount), refNum, counter, 10)
		loopcount += 1
	while (loopcount < 4)

	loopcount = 0
	do
		counter = load_text(	"sDACChannelsUnits"			+ num2str(loopcount), refNum, counter, 8)
		loopcount += 1
	while (loopcount < 4)

	loopcount = 0
	do
		counter = load_float(	"fDACScaleFactor" 			+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 4)

	loopcount = 0
	do
		counter = load_float(	"fDACHoldingLevel" 			+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 4)

	counter = load_int(		"nSignalType", 				refNum, counter)

	FSetPos refNum, 1436

	counter = load_int(		"nDigitalEnable", 				refNum, counter)
	counter = load_int(		"nWaveformSource", 			refNum, counter)
	counter = load_int(		"nActiveDACChannel", 		refNum, counter)
	counter = load_int(		"nInterEpisodeLevel", 			refNum, counter)

	loopcount = 0
	do
		counter = load_int(	"nEpochType" 				+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 10)

	loopcount = 0
	do
		counter = load_float(	"nEpochInitLevel" 			+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 10)

	loopcount = 0
	do
		counter = load_float(	"nEpochLevelInc" 			+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 10)

	loopcount = 0
	do
		counter = load_int(	"nEpochInitDuration" 			+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 10)

	loopcount = 0
	do
		counter = load_int(	"nEpochDurationInc" 			+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 10)

	counter = load_int(		"nDigitalHolding",				refNum, counter)
	counter = load_int(		"nDigitalInterEpisode", 		refNum, counter)

	loopcount = 0
	do
		counter = load_int(	"nDigitalValue" 				+ num2str(loopcount), refNum, counter)
		loopcount += 1
	while (loopcount < 10)

	counter = load_float(		"fWaveformOffset", 			refNum, counter)

	FSetPos refNum, 1620

	counter = load_float(		"fDACFileScale", 				refNum, counter)
	counter = load_float(		"fDACFileOffset", 				refNum, counter)

	FSetPos refNum, 1630

	counter = load_int(		"nDACFileEpisodeNum", 		refNum, counter)
	counter = load_int(		"nDACFileADCNum", 			refNum, counter)
counter = load_text(		"sDACFileName",				refNum, counter, 12)
counter = load_text(		"sDACFilePath",				refNum, counter, 60)

FSetPos refNum, 1718

counter = load_int(		"nConditEnable", 				refNum, counter)
counter = load_int(		"nConditChannel", 			refNum, counter)
counter = load_longint(	"nConditNumPulses", 			refNum, counter)
counter = load_float(		"fBaselineDuration", 			refNum, counter)
counter = load_float(		"fBaselineLevel", 				refNum, counter)
counter = load_float(		"fStepDuration", 				refNum, counter)
counter = load_float(		"fStepLevel", 					refNum, counter)
counter = load_float(		"fPostTrainPeriod", 			refNum, counter)
counter = load_float(		"fPostTrainLevel", 			refNum, counter)

FSetPos refNum, 1762

counter = load_int(		"nParamToVary", 			refNum, counter)
counter = load_text(		"sParamValueList",			refNum, counter, 80)

counter = load_int(		"nAutopeakEnable", 			refNum, counter)
counter = load_int(		"nAutopeakPolarity", 			refNum, counter)
counter = load_int(		"nAutopeakADCNum", 		refNum, counter)
counter = load_int(		"nAutopeakSearchMode", 		refNum, counter)
counter = load_longint(	"lAutoPeakStart", 			refNum, counter)
counter = load_longint(	"lAutoPeakEnd", 				refNum, counter)
counter = load_int(		"nAutopeakSmoothing", 		refNum, counter)
counter = load_int(		"nAutopeakBaseline", 		refNum, counter)
counter = load_int(		"nAutopeakAverage", 			refNum, counter)

FSetPos refNum, 1880

counter = load_int(		"nArithmeticEnable", 			refNum, counter)

FSetPos refNum, 1890

counter = load_int(		"nArithmeticADCNumA", 		refNum, counter)
counter = load_int(		"nArithmeticADCNumB", 		refNum, counter)
counter = load_float(		"fArithmeticK1", 				refNum, counter)
counter = load_float(		"fArithmeticK2", 				refNum, counter)
counter = load_float(		"fArithmeticK3", 				refNum, counter)
counter = load_float(		"fArithmeticK4", 				refNum, counter)
counter = load_text(		"sArithmeticOperator",		refNum, counter, 2)
counter = load_text(		"sArithmeticUnits",			refNum, counter, 8)
counter = load_float(		"fArithmeticK5", 				refNum, counter)
counter = load_float(		"fArithmeticK6", 				refNum, counter)
counter = load_int(		"nArithmeticExpression", 		refNum, counter)

FSetPos refNum, 1932

counter = load_int(		"nPNEnable", 				refNum, counter)
counter = load_int(		"nPNPosition", 				refNum, counter)
counter = load_int(		"nPNPolarity", 				refNum, counter)
counter = load_int(		"nPNNumPulses", 			refNum, counter)
counter = load_int(		"nPNADCNum", 				refNum, counter)
counter = load_float(		"fPNHoldingLevel",			refNum, counter)
counter = load_float(		"fPNSettlingTime",			refNum, counter)
counter = load_float(		"fPNInterPulse",				refNum, counter)

FSetPos refNum, 1966

counter = load_int(		"nListEnable", 				refNum, counter)

if (dimsize(sweep_params,0) > (counter + 1))
	deletepoints counter, dimsize(sweep_params,0)-counter, sweep_params
endif

end






FUNCTION 		LOAD_TEXT(parameter, refNum, counter, char_count)		// load required number of characters
string parameter
variable refNum
variable counter
variable char_count

string chardata
wave /t sweep_params = sweep_params

if (waveexists(sweep_params))
	wave /t sweep_params = sweep_params
	if (dimsize(sweep_params,0) < (counter + 1))
		insertpoints dimsize(sweep_params,0), 1, sweep_params
	endif
else
	make /t/n=(1,2) sweep_params
endif

FReadLine /n=(char_count) refNum, chardata
sweep_params[counter][0] = parameter
SetDimLabel 0, counter, $parameter sweep_params
sweep_params[counter][1] = chardata

counter += 1

return counter

end




FUNCTION 		LOAD_FLOAT(parameter, refNum, counter)					// load 4-byte float
string parameter
variable refNum
variable counter

variable numdata
string chardata
wave /t sweep_params = sweep_params

if (waveexists(sweep_params))
	wave /t sweep_params = sweep_params
	if (dimsize(sweep_params,0) < (counter + 1))
		insertpoints dimsize(sweep_params,0), 1, sweep_params
	endif
else
	make /t/n=(1,2) sweep_params
endif

FBinRead /b=3 /f=4 refNum, numdata				
sweep_params[counter][0] = parameter
SetDimLabel 0, counter, $parameter sweep_params
sprintf chardata ,"%9.6E", numdata
sweep_params[counter][1] = chardata

counter += 1

return counter

end



FUNCTION 		LOAD_INT(parameter, refNum, counter)					// load 2-byte integer
string parameter
variable refNum
variable counter

variable numdata
string chardata
variable places
wave /t sweep_params = sweep_params

if (waveexists(sweep_params))
	wave /t sweep_params = sweep_params
	if (dimsize(sweep_params,0) < (counter + 1))
		insertpoints dimsize(sweep_params,0), 1, sweep_params
	endif
else
	make /t/n=(1,2) sweep_params
endif

FBinRead /b=3 /f=2 refNum, numdata
sweep_params[counter][0] = parameter
SetDimLabel 0, counter, $parameter sweep_params
if ((numdata >= 10) && (numdata > 0))
	places = trunc(log(numdata))
else
	places = 1
endif
sprintf chardata ,"%1.*d", places, numdata
sweep_params[counter][1] = chardata

counter += 1

return counter

end



FUNCTION		LOAD_LONGINT(parameter, refNum, counter)					// load 4-byte integer
string parameter
variable refNum
variable counter

variable numdata
string chardata
variable places
wave /t sweep_params = sweep_params

if (waveexists(sweep_params))
	wave /t sweep_params = sweep_params
	if (dimsize(sweep_params,0) < (counter + 1))
		insertpoints dimsize(sweep_params,0), 1, sweep_params
	endif
else
	make /t/n=(1,2) sweep_params
endif


FBinRead /b=3 /f=3 refNum, numdata
sweep_params[counter][0] = parameter
SetDimLabel 0, counter, $parameter sweep_params
if ((numdata >= 10) && (numdata > 0))
	places = trunc(log(numdata))
else
	places = 1
endif
sprintf chardata ,"%1.*d", places, numdata
sweep_params[counter][1] = chardata

counter += 1

return counter

end

// ============================= Some other functions ==============================

// CONCATENATE TWO WAVES. THIS FUNCTION WAS WRITTEN FOR A SPECIFIC PURPOSE.
// CAN BE REMOVED FROM SPACAN.
FUNCTION Concatenate_waves(numero)
Variable numero
	Svar Filename1, Filename2
	Nvar Length1, Length2, Length
	concatenate/o {$Filename1,$Filename2}, data
	removefromgraph/W=MainWindow#Channel1 $Filename1
	removefromgraph/W=MainWindow#Channel2 $Filename2
	appendtograph/W=MainWindow#Channel1 data
	data = data * 1e-12
	Filename1 = "data"
	Length = Length1 + Length2
END
